# Update masterMetaData to include Order, along with some renaming if necessary

# Change "CellType" is a column, copy to "Class2"
if ("CellType"%in%colnames(masterMetaData)){
  colnames(masterMetaData)[colnames(masterMetaData)=="CellType"] <- "Class2"
}

# If Animal is a column name, rename to Species
names(masterMetaData)[names(masterMetaData)=="Animal"] <- "Species"

# Set any instance of "Cerebral cortex" brain region to "Neocortex"
masterMetaData[masterMetaData$BrainRegion=="Cerebral cortex","BrainRegion"] <- "Neocortex"

# Create order column from species
masterMetaData$Order <- rep("",nrow(masterMetaData))
masterMetaData[masterMetaData$Species%in%c("Human","Monkey","Ape"),"Order"] <- "Primate"
masterMetaData[masterMetaData$Species%in%c("Rat","Mouse","Guinea-pig","Agouti","Proechimys"),"Order"] <- "Rodent"
masterMetaData[masterMetaData$Species=="Elephant","Order"] <- "Proboscidae"
masterMetaData[masterMetaData$Species%in%c("Blowfly","Drosophila"),"Order"] <- "Diptera"
masterMetaData[masterMetaData$Species%in%c("Goldfish","Zebrafish"),"Order"] <- "Cypriniformes"
masterMetaData[masterMetaData$Species=="Rabbit","Order"] <- "Lagomorpha"
masterMetaData[masterMetaData$Species=="Cat","Order"] <- "Carnivora"
masterMetaData[masterMetaData$Species=="Cricket","Order"] <- "Orthoptera"
masterMetaData[masterMetaData$Species=="Salamander","Order"] <- "Caudata"
masterMetaData[masterMetaData$Species=="Spiny lobster","Order"] <- "Decapoda"
masterMetaData[masterMetaData$Species%in%c("Frog"),"Order"] <- "Anura"
masterMetaData[masterMetaData$Species%in%c("Turtle"),"Order"] <-"Testudines"
masterMetaData[masterMetaData$Species=="Chicken","Order"] <- "Galliformes"

# Create Layer column based on Region3, and set any Region3 value containing a layer to "Not reported"
layerInds <- with(masterMetaData,grepl("layer",Region3,ig=T) | grepl("Stratum",Region3,ig=T) |
  grepl("granul",Region3,ig=T) | Region3=="Hilus" | Region3=="Superficial")
if (sum(layerInds) > 0){
  masterMetaData$Layer <- masterMetaData$Region3
  masterMetaData[!layerInds,"Layer"] <- "Not reported"
  masterMetaData[layerInds,"Region3"] <- "Not reported"
}

# Set Layer for cases in which neuron name includes code for it
inds <- masterMetaData$Layer=="Not reported"&masterMetaData$Archive=="Bikson"&grepl(".+-L(\\d)(pyr|int)-.+-.+",rownames(masterMetaData),ignore.case=T)
masterMetaData[inds,"Layer"] <- gsub(".+-(cut)?L(\\d)(pyr|int)-.+-.+","Layer \\2",rownames(masterMetaData)[inds],ignore.case=T)

# Leave original metadata table alone. Assign to "usedMetaData" which can be manipulated
usedMetaData <- masterMetaData
# I use the following line to join Dendrite and BasalDendrite arbor types
usedMetaData[usedMetaData$ArborType=="BasalDendrite","ArborType"] <- "Dendrite"

# Merge layers
usedMetaData[usedMetaData$Layer%in%c("Layer 2","Layer 3","Supragranular"),"Layer"] <- "Layer 2/3"
#usedMetaData[usedMetaData$Layer=="Infragranular"),"Layer"] <- "Layer 5 and/or 6"

# Update specification of culture type for protocol
pos <- usedMetaData$Protocol=="Culture" & usedMetaData$Archive == "Ascoli"
usedMetaData[pos,"Protocol"] <- "Acute culture"
pos <- usedMetaData$Protocol=="Culture" & (usedMetaData$Archive == "Pozzo-Miller"|usedMetaData$Archive == "Bausch")
usedMetaData[pos,"Protocol"] <- "Organotypic slice culture"

# Get rid of any case of Acute culture from the metadata to be used (store in separate table)
acuteCultureData <- subset(usedMetaData,Protocol=="Acute culture")
usedMetaData <- subset(usedMetaData,Protocol!="Acute culture")

# Reorganize cortical regions, with Region2 and Region3 feeding into CorticalRegion and CorticalLobe
corticalSubregionR2Conversions <- list(c("Frontopolar","Prefrontal"),
                                       c("Inferior frontal gyrus","Prefrontal"),
                                       c("Prefrontal","Prefrontal"),
                                       c("Medial prefrontal cortex","Prefrontal"),
                                       c("Precentral gyri","Motor"),
                                       c("Occipital lobe","Visual"),
                                       c("Suprasylvian gyrus","Visual"),
                                       c("Visual","Visual"),
                                       c("Motor","Motor"),
                                       c("Postcentral gyri","Somatosensory"),
                                       c("Somatosensory","Somatosensory"),
                                       c("Anterior cingulate","Cingulate"),
                                       c("Perirhinal","Perirhinal"))
corticalSubregionR3Conversions <- list(c("Brodmann area 10","Prefrontal"),
                                       c("Brodmann area 11","Prefrontal"),
                                       c("Brodmann area 44","Prefrontal"),
                                       c("Prelimbic","Prefrontal"),
                                       c("Brodmann area 4","Motor"),
                                       c("Brodmann area 6","Motor"),
                                       c("Superior frontal gyrus","Motor"),
                                       c("Area 17","Visual"),
                                       c("Brodmann area 18","Visual"),
                                       c("Brodmann area 3,1,2","Somatosensory"),
                                       c("Barrel","Somatosensory"))

usedMetaData$CorticalRegion <- rep("Not reported",nrow(usedMetaData))
pos <- grepl("Neocortex",usedMetaData$BrainRegion) & usedMetaData$Region3 == "Not reported"
usedMetaData[pos,"CorticalRegion"] <- usedMetaData[pos,"Region2"]
for (subPair in corticalSubregionR2Conversions){
  pos <- grepl(subPair[1],usedMetaData$Region2)
  usedMetaData[pos,"CorticalRegion"] <- subPair[2]
}
for (subPair in corticalSubregionR3Conversions){
  pos <- grepl(subPair[1],usedMetaData$Region3)
  usedMetaData[pos,"CorticalRegion"] <- subPair[2]
}

lobeSubRegionConversions <- list(c("Prefrontal","Frontal"),
                                 c("Motor","Frontal"),
                                 c("Visual","Occipital"),
                                 c("Somatosensory","Parietal"),
                                 c("Insula","Insula"),
                                 c("Anterior long insular gyrus","Insula"),
                                 c("Middle short insular gyrus","Insula"),
                                 c("Posterior short insular gyrus","Insula"),
                                 c("Cingulate","Limbic"),
                                 c("Perirhinal","Limbic"))
lobeR2Conversions <- list(c("Temporal sulcus","Temporal"))
lobeR3Conversions <- list(c("Brodmann area 22","Temporal"),
                          c("Primary auditory","Temporal"),
                          c("Brodmann area 39","Parietal"),
                          c("Fronto-insula","Insula"))

usedMetaData$CorticalLobe <- rep("Not reported",nrow(usedMetaData))
usedMetaData[grepl("lobe",usedMetaData$Region2),"CorticalLobe"] <- gsub(" lobe","",usedMetaData[grepl("lobe",usedMetaData$Region2),"Region2"])
for (subPair in lobeSubRegionConversions){
  pos <- grepl(subPair[1],usedMetaData$CorticalRegion)
  usedMetaData[pos,"CorticalLobe"] <- subPair[2]
}
for (subPair in lobeR2Conversions){
  pos <- grepl(subPair[1],usedMetaData$Region2)
  usedMetaData[pos,"CorticalLobe"] <- subPair[2]
}
for (subPair in lobeR3Conversions){
  pos <- grepl(subPair[1],usedMetaData$Region3)
  usedMetaData[pos,"CorticalLobe"] <- subPair[2]
}

getColorsForMetaType <- function(neuriteNames,metaType,myMetaData){
  metaVals <- myMetaData[neuriteNames,metaType]
  uniqueVals <- unique(metaVals)
  groupIds <- rep(0,length(metaVals))
  for (i in 1:length(uniqueVals)){
    groupIds[metaVals==uniqueVals[i]] <- i
  }
  return(groupIds)
}

perisomaticTargetingInterneuronClasses <- c(
  "Basket cell","Parvalbumin (PV) containing cell","Chandelier cell","Non-fast spiking")
dendriticTargetingInterneuronClasses <- c(
  "Martinotti cell", "Somatostatin (SOM) containing cell",
  "Bipolar cell","Bitufted cell","Double bouquet cell",
  "Multipolar cell","Neurogliaform cell", "Non-basket",
  "Neuropeptide Y (NPY) containing cell", "Small axonless neuron",
  "Stellate cell","Tripolar cell")

#######
# Class comparison definitions
#######
SpecificationLists <- list()
SpecificationTitles <- c()

# Axon/Rodent/Pyramidal cell by BrainRegion
specification1 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex")
specification2 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Hippocampus")
specs <- list(Cortical=specification1,Hippocampal=specification2)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Axon/Rodent/Pyramidal cell by BrainRegion")

# Axon/Rodent/Pyramidal cell by Species
specification1 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",Species="Rat")
specification2 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",Species="Mouse")
specs <- list(Rat=specification1,Mouse=specification2)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Axon/Rodent/Pyramidal cell by Species")

# Axon/Rodent/Neocortex/Pyramidal cell by CorticalRegion
specification1 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex",CorticalRegion="Somatosensory")
specification2 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex",CorticalRegion="Visual")
specs <- list(Somatosensory=specification1,Visual=specification2)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Axon/Rodent/Neocortex/Pyramidal cell by CorticalRegion")

# Axon/Rodent/Neocortex/Pyramidal cell by Species
specification1 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex",Species="Mouse")
specification2 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex",Species="Rat")
specs <- list(Mouse=specification1,Rat=specification2)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Axon/Rodent/Neocortex/Pyramidal cell by Species")

# Axon/Rodent/Neocortex/Pyramidal cell by Layer
specification1 <- list(ArborType="Axon",ExperCond="Control",Order="Rodent",Class2="Pyramidal cell",BrainRegion="Neocortex",Layer="Layer 4")
specification2 <- list(ArborType="Axon",ExperCond="Control",Order="Rodent",Class2="Pyramidal cell",BrainRegion="Neocortex",Layer="Layer 5")
specification3 <- list(ArborType="Axon",ExperCond="Control",Order="Rodent",Class2="Pyramidal cell",BrainRegion="Neocortex",Layer="Layer 2/3")
specs <- list(L5=specification1,L4=specification2,L23=specification3)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Axon/Rodent/Neocortex/Pyramidal cell by Layer")

# Axon/Rodent/Young/Neocortex/Pyramidal cell by Layer
specification1 <- list(ArborType="Axon",ExperCond="Control",Order="Rodent",AgeClass="Young",Class2="Pyramidal cell",BrainRegion="Neocortex",Layer="Layer 4")
specification2 <- list(ArborType="Axon",ExperCond="Control",Order="Rodent",AgeClass="Young",Class2="Pyramidal cell",BrainRegion="Neocortex",Layer="Layer 5")
specification3 <- list(ArborType="Axon",ExperCond="Control",Order="Rodent",AgeClass="Young",Class2="Pyramidal cell",BrainRegion="Neocortex",Layer="Layer 2/3")
specs <- list(L5=specification1,L4=specification2,L23=specification3)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Axon/Rodent/Young/Neocortex/Pyramidal cell by Layer")

# Axon/Rodent/Young/Neocortex/Somatosensory/Pyramidal cell by Layer
specification1 <- list(ArborType="Axon",ExperCond="Control",Order="Rodent",AgeClass="Young",Class2="Pyramidal cell",BrainRegion="Neocortex",CorticalRegion="Somatosensory",Layer="Layer 4")
specification2 <- list(ArborType="Axon",ExperCond="Control",Order="Rodent",AgeClass="Young",Class2="Pyramidal cell",BrainRegion="Neocortex",CorticalRegion="Somatosensory",Layer="Layer 5")
specs <- list(L5=specification1,L4=specification2)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Axon/Rodent/Young/Neocortex/Somatosensory/Pyramidal cell by Layer")

# Axon/Rodent/Neocortex/Somatosensory/Pyramidal cell by Species
specification1 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",CorticalRegion="Somatosensory",Species="Rat")
specification2 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",CorticalRegion="Somatosensory",Species="Mouse")
specs <- list(Rat=specification1,Mouse=specification2)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Axon/Rodent/Neocortex/Somatosensory/Pyramidal cell by Species")

# Axon/Rodent/Interneuron by Class2
specification1 <- list(ArborType="Axon",ExperCond="Control",Class1="Interneuron",Order="Rodent",Class2="Basket cell")
specification2 <- list(ArborType="Axon",ExperCond="Control",Class1="Interneuron",Order="Rodent",Class2="Parvalbumin (PV) containing cell")
specification3 <- list(ArborType="Axon",ExperCond="Control",Class1="Interneuron",Order="Rodent",Class2=c("Martinotti cell","Somatostatin (SOM) containing cell"))
specs <- list(Basket=specification1,PV_Pos=specification2,Martinotti_U_SOM_Pos=specification3)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Axon/Rodent/Interneuron by Class2")

specification1 <- list(ArborType="Axon",ExperCond="Control",Class1="Interneuron",Order="Rodent",Class2=c("Basket cell","Parvalbumin (PV) containing cell"))
specification2 <- list(ArborType="Axon",ExperCond="Control",Class1="Interneuron",Order="Rodent",Class2=c("Martinotti cell","Somatostatin (SOM) containing cell"))
specs <- list(Basket_U_PV_Pos=specification1,Martinotti_U_SOM_Pos=specification2)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Axon/Rodent/Interneuron by Class2 (Basket+PV combined)")

# Axon/Drosophila/Olfactory bulb/iPN/Uniglomerular projection neuron by AgeClass
specification1 <- list(ArborType="Axon",ExperCond="Control",Class2="Uniglomerular projection neuron",Class3="iPN",AgeClass="Larval")
specification2 <- list(ArborType="Axon",ExperCond="Control",Class2="Uniglomerular projection neuron",Class3="iPN",AgeClass="Embryonic")
specs <- list(Larval=specification1,Embryonic=specification2)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Axon/Drosophila/Olfactory bulb/iPN/Uniglomerular projection neuron by AgeClass")

# ApicalDendrite/Neocortex/Pyramidal cell by Order
specification1 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Rodent")
specification2 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Primate")
specification3 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Species="Elephant")
specs <- list(Rodent=specification1,Primate=specification2,Elephant=specification3)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"ApicalDendrite/Neocortex/Pyramidal cell by Order")

# ApicalDendrite/Neocortex/Adult/Pyramidal cell by Order
specification1 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",AgeClass="Adult",BrainRegion="Neocortex",Order="Rodent")
specification2 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",AgeClass="Adult",BrainRegion="Neocortex",Order="Primate")
specification3 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",AgeClass="Adult",BrainRegion="Neocortex",Species="Elephant")
specs <- list(Rodent=specification1,Primate=specification2,Elephant=specification3)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"ApicalDendrite/Neocortex/Adult/Pyramidal cell by Order")

# ApicalDendrite/Neocortex/Young/Pyramidal cell by Order
specification1 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",AgeClass="Young",BrainRegion="Neocortex",Order="Rodent")
specification2 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",AgeClass="Young",BrainRegion="Neocortex",Order="Primate")
specs <- list(Rodent=specification1,Primate=specification2)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"ApicalDendrite/Neocortex/Young/Pyramidal cell by Order")

# ApicalDendrite/Rodent/Pyramidal cell by BrainRegion
specification1 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex")
specification2 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Hippocampus")
specification3 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Anterior olfactory nucleus")
specs <- list(Neocortex=specification1,Hippocampus=specification2,AnteriorOlfactoryNucleus=specification3)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"ApicalDendrite/Rodent/Pyramidal cell by BrainRegion")

# ApicalDendrite/Rodent/Young/Pyramidal cell by BrainRegion
specification1 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",AgeClass="Young",BrainRegion="Neocortex")
specification2 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",AgeClass="Young",BrainRegion="Hippocampus")
specification3 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",AgeClass="Young",BrainRegion="Anterior olfactory nucleus")
specs <- list(Neocortex=specification1,Hippocampus=specification2,AnteriorOlfactoryNucleus=specification3)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"ApicalDendrite/Rodent/Young/Pyramidal cell by BrainRegion")

# ApicalDendrite/Rodent/Neocortex/Pyramidal cell by CorticalRegion
specification1 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex",CorticalRegion="Somatosensory")
specification2 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex",CorticalLobe="Frontal")
specification3 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex",CorticalLobe="Occipital")
specs <- list(Somatosensory=specification1,Frontal=specification2,Occipital=specification3)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"ApicalDendrite/Rodent/Neocortex/Pyramidal cell by CorticalRegion")

# Too many have experimental condition of "Not reported"
#specification3 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex",Region2="Prefrontal")
# ApicalDendrite/Rodent/Neocortex/Layer 2,3/Pyramidal cell by CorticalRegion
# NOT ENOUGH WHEN CONSTRAINED BY SeqLen >= 15
specification1 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex",Layer="Layer 2/3",Region2="Somatosensory")
specification2 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex",Layer="Layer 2/3",Region2="Frontal lobe")
specs <- list(Somatosensory=specification1,Frontal=specification2)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"ApicalDendrite/Rodent/Neocortex/Layer 2,3/Pyramidal cell by CorticalRegion")

# ApicalDendrite/Rodent/Neocortex/Pyramidal cell by Layer
specification1 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex",Layer="Layer 5")
specification2 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex",Layer="Layer 4")
specification3 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex",Layer="Layer 2/3")
specs <- list(L5=specification1,L4=specification2,L23=specification3)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"ApicalDendrite/Rodent/Neocortex/Pyramidal cell by Layer")

# ApicalDendrite/Rodent/Neocortex/Somatosensory/Pyramidal cell by Layer
specification1 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex",Region2="Somatosensory",Layer="Layer 5")
specification2 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex",Region2="Somatosensory",Layer="Layer 4")
specification3 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex",Region2="Somatosensory",Layer="Layer 2/3")
specs <- list(L5=specification1,L4=specification2,L23=specification3)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"ApicalDendrite/Rodent/Neocortex/Somatosensory/Pyramidal cell by Layer")

# ApicalDendrite/Rodent/Neocortex/Pyramidal cell by AgeClass
specification1 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex",AgeClass="Young")
specification2 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex",AgeClass="Adult")
specs <- list(Young=specification1,Adult=specification2)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"ApicalDendrite/Rodent/Neocortex/Pyramidal cell by AgeClass")

# Primate ApicalDendrite
# ApicalDendrite/Primate/Neocortex/Pyramidal cell by Species
# Not enough Human cases when SeqLen >= 15
specification1 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Primate",Species="Monkey")
specification2 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Primate",Species="Human")
specs <- list(Monkey=specification1,Human=specification2)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"ApicalDendrite/Primate/Neocortex/Pyramidal cell by Species")

# ApicalDendrite/Primate/Neocortex/Pyramidal cell by AgeClass
specification1 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Primate",AgeClass="Adult")
specification2 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Primate",AgeClass="Young")
specification3 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Primate",AgeClass="Old")
specs <- list(Adult=specification1,Young=specification2,Old=specification3)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"ApicalDendrite/Primate/Neocortex/Pyramidal cell by AgeClass")

# ApicalDendrite/Primate/Neocortex/Layer 2,3/Pyramidal cell by CorticalRegion
specification1 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Primate",Layer="Layer 2/3",CorticalRegion="Prefrontal")
specification2 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Primate",Layer="Layer 2/3",CorticalRegion="Temporal sulcus")
specs <- list(Prefrontal=specification1,TemporalSulcus=specification2)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"ApicalDendrite/Primate/Neocortex/Layer 2,3/Pyramidal cell by CorticalRegion")

# ApicalDendrite/Primate/Neocortex/Prefrontal/Layer 2,3/Pyramidal cell by Species
specification1 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Layer="Layer 2/3",Species="Monkey")
specification2 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Layer="Layer 2/3",Species="Human")
specs <- list(Monkey=specification1,Human=specification2)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"ApicalDendrite/Primate/Neocortex/Prefrontal/Layer 2,3/Pyramidal cell by Species")

# Pyramidal cell subtype - small numbers in most groups and not clear distinctions between them
# ApicalDendrite/Monkey/Adult/Neocortex/Pyramidal cell by Hemisphere projecting
# Magnopyramidal(107),Long projecting(43),Callosal-projecting(37),Local projecting(31),Ipsilateral-projecting(30)
specification1 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Primate",CorticalRegion="Prefrontal",Class3="Ipsilateral-projecting")
specification2 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Primate",CorticalRegion="Prefrontal",Class3="Callosal-projecting")
specs <- list(Ipsilateral=specification1,Callosal=specification2)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"ApicalDendrite/Monkey/Adult/Neocortex/Pyramidal cell by Hemisphere projecting")

# Pyramidal Dendrite

# Dendrite/Neocortex/Pyramidal cell by Order
specification1 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Protocol="In vitro",Order="Primate")
specification2 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Protocol="In vitro",Order="Rodent")
specification3 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Protocol="In vitro",Species="Elephant")
specs <- list(Primate=specification1,Rodent=specification2,Elephant=specification3)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Dendrite/Neocortex/Pyramidal cell by Order")

# Dendrite/Neocortex/Pyramidal cell by Species
specification1 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Protocol="In vitro",Species="Human")
specification2 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Protocol="In vitro",Species="Monkey")
specification3 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Protocol="In vitro",Species="Mouse")
specification4 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Protocol="In vitro",Species="Rat")
specification5 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Protocol="In vitro",Species="Elephant")
specs <- list(Human=specification1,Monkey=specification2,Mouse=specification3,Rat=specification4,Elephant=specification5)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Dendrite/Neocortex/Pyramidal cell by Species")

# Dendrite/Neocortex/Layer 2,3/Pyramidal cell by Order
specification1 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Protocol="In vitro",Layer="Layer 2/3",Order="Primate")
specification2 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Protocol="In vitro",Layer="Layer 2/3",Order="Rodent")
specification3 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Protocol="In vitro",Layer="Layer 2/3",Species="Elephant")
specs <- list(Primate=specification1,Rodent=specification2,Elephant=specification3)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Dendrite/Neocortex/Layer 2,3/Pyramidal cell by Order")

# Dendrite/Neocortex/Layer 2,3/Pyramidal cell by Species
specification1 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Protocol="In vitro",Layer="Layer 2/3",Species="Human")
specification2 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Protocol="In vitro",Layer="Layer 2/3",Species="Monkey")
specification3 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Protocol="In vitro",Layer="Layer 2/3",Species="Mouse")
specification4 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Protocol="In vitro",Layer="Layer 2/3",Species="Rat")
specification5 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Protocol="In vitro",Layer="Layer 2/3",Species="Elephant")
specs <- list(Human=specification1,Monkey=specification2,Mouse=specification3,Rat=specification4,Elephant=specification5)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Dendrite/Neocortex/Layer 2,3/Pyramidal cell by Species")

# Dendrite/Human/Neocortex/Pyramidal cell by AgeClass
specification1 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",Species="Human",BrainRegion="Neocortex",Protocol="In vitro",AgeClass="Adult")
specification2 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",Species="Human",BrainRegion="Neocortex",Protocol="In vitro",AgeClass="Neonate")
specs <- list(Adult=specification1,Neonate=specification2)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Dendrite/Human/Neocortex/Pyramidal cell by AgeClass")

# Dendrite/Human/Neocortex/Pyramidal cell by Gender
specification1 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",Species="Human",BrainRegion="Neocortex",Protocol="In vitro",Gender="M")
specification2 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",Species="Human",BrainRegion="Neocortex",Protocol="In vitro",Gender="F")
specs <- list(Male=specification1,Female=specification2)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Dendrite/Human/Neocortex/Pyramidal cell Gender")

# Cortical Layers with multiple regions:
# Dendrite/Primate/Neocortex/Layer 2,3/Pyramidal cell by CorticalLobe
# Insula, Frontal, Temporal
specification1 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Primate",Layer="Layer 2/3",CorticalLobe="Insula")
specification2 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Primate",Layer="Layer 2/3",CorticalLobe="Frontal")
specification3 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Primate",Layer="Layer 2/3",CorticalLobe="Temporal")
specs <- list(Insula=specification1,Frontal=specification2,Temporal=specification3)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Dendrite/Primate/Neocortex/Layer 2,3/Pyramidal cell by CorticalLobe")

# Dendrite/Primate/Neocortex/Layer 2,3/Pyramidal cell by CorticalRegion
# Ant long insular gyr, Frontal lobe, Inferior frontal gyr, Mid short insular gyr,
#       Posterior short insular gyr, Prefrontal, Somatosensory, Temporal sulcus, Visual
specification1 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Primate",Layer="Layer 2/3",CorticalRegion="Anterior long insular gyrus")
specification2 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Primate",Layer="Layer 2/3",CorticalRegion="Posterior short insular gyrus")
specification3 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Primate",Layer="Layer 2/3",CorticalRegion="Middle short insular gyrus")
specification4 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Primate",Layer="Layer 2/3",CorticalRegion="Prefrontal")
specification5 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Primate",Layer="Layer 2/3",CorticalRegion="Temporal sulcus")
specs <- list(AntLongInsula=specification1,PosteriorShortInsula=specification2,MidShortInsula=specification3,Prefrontal=specification4,TemporalSulcus=specification5)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Dendrite/Primate/Neocortex/Layer 2,3/Pyramidal cell by CorticalRegion")

# Cortical regions with multiple layers: 
# Somatosensory: L2/3,4,5 (all Rodent)
# Dendrite/Rodent/Neocortex/Somatosensory/Pyramidal cell by Layer
specification1 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",CorticalRegion="Somatosensory",Layer="Layer 2/3")
specification2 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",CorticalRegion="Somatosensory",Layer="Layer 4")
specification3 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",CorticalRegion="Somatosensory",Layer="Layer 5")
specs <- list(L23=specification1,L4=specification2,L5=specification3)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Dendrite/Rodent/Neocortex/Somatosensory/Pyramidal cell by Layer")

# Dendrite/Rodent/Neocortex/Layer 2,3/Pyramidal cell by CorticalRegion
specification1 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Rodent",Layer="Layer 2/3",CorticalRegion="Somatosensory")
specification2 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Rodent",Layer="Layer 2/3",CorticalLobe="Frontal")
specification3 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Rodent",Layer="Layer 2/3",CorticalRegion="Visual")
specs <- list(Somatosensory=specification1,Frontal=specification2,Visual=specification3)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Dendrite/Rodent/Neocortex/Layer 2,3/Pyramidal cell by CorticalRegion")

# Dendrite/Rodent/Young/Neocortex/Somatosensory/Layer 4/Pyramidal cell by Species
specification1 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",CorticalRegion="Somatosensory",Layer="Layer 4",Species="Mouse")
specification2 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",CorticalRegion="Somatosensory",Layer="Layer 4",Species="Rat")
specs <- list(Mouse=specification1,Rat=specification2)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Dendrite/Rodent/Young/Neocortex/Somatosensory/Layer 4/Pyramidal cell by Species")

# Other (non-pyramidal) Dendrites
#  Dendrite/Retina/Ganglion cell by Species
specification1 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Ganglion cell",Species="Mouse")
specification2 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Ganglion cell",Species="Salamander")
specification3 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Ganglion cell",Species="Goldfish")
specs <- list(Mouse=specification1,Salamander=specification2,Goldfish=specification3)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Dendrite/Retina/Ganglion cell by Species")

# Dendrite/Rodent/Neocortex by Class2
specification1 <- list(ArborType="Dendrite",ExperCond="Control",Order="Rodent",BrainRegion="Neocortex",Class2="Pyramidal cell")
specification2 <- list(ArborType="Dendrite",ExperCond="Control",Order="Rodent",BrainRegion="Neocortex",Class2="Parvalbumin (PV) containing cell")
specification3 <- list(ArborType="Dendrite",ExperCond="Control",Order="Rodent",BrainRegion="Neocortex",Class2="Basket cell")
specification4 <- list(ArborType="Dendrite",ExperCond="Control",Order="Rodent",BrainRegion="Neocortex",Class2=c("Martinotti cell","Somatostatin (SOM) containing cell"))
specs <- list(Pyramidal=specification1,PVContaining=specification2,Basket=specification3,Martinotti=specification4)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Dendrite/Rodent/Neocortex by Class2")

# Not enough in control condition?
# Dendrite/Rat/Hippocampus/Granule cell by AgeClass 
# NOT ENOUGH WHEN SeqLen >= 15
specification1 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Granule cell",Species="Rat",AgeClass="Young")
specification2 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Granule cell",Species="Rat",AgeClass="Adult")
specs <- list(Young=specification1,Adult=specification2)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Dendrite/Rat/Hippocampus/Granule cell by AgeClass")

# Dendrite/Motoneuron by BrainRegion
specification1 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Motoneuron",BrainRegion="Spinal cord")
specification2 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Motoneuron",BrainRegion="Brainstem")
specs <- list(SpinalCord=specification1,Brainstem=specification2)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Dendrite/Motoneuron by BrainRegion")

# Hippocampus pyramidal cell neurites by Subregion
specification1 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Hippocampus",Region2="CA1")
specification2 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Hippocampus",Region2="CA3")
specs <- list(CA1=specification1,CA3=specification2)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Dendrite/Hippocampus/Pyramidal cell by Subregion")

specification1 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Hippocampus",Region2="CA1")
specification2 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Hippocampus",Region2="CA3")
specs <- list(CA1=specification1,CA3=specification2)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"ApicalDendrite/Hippocampus/Pyramidal cell by Subregion")

specification1 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Hippocampus",Region2="CA1")
specification2 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Hippocampus",Region2="CA3")
specification3 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Layer="Layer 2/3")
specification4 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Layer="Layer 4")
specification5 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Layer="Layer 5")
specs <- list(CA1=specification1,CA3=specification2,L23=specification3,L4=specification4,L5=specification5)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"ApicalDendrite/Pyramidal cell by Subregion/Layer")

#specification1 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Hippocampus",Region2="CA1")
#specification2 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Hippocampus",Region2="CA3")
#specs <- list(CA1=specification1,CA3=specification2)
#SpecificationLists <- append(SpecificationLists,list(specs))
#SpecificationTitles <- append(SpecificationTitles,"Axon/Hippocampus/Pyramidal cell by Subregion")

specification1 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Hippocampus",Region2="CA1")
specification2 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Hippocampus",Region2="CA3")
specification3 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Layer="Layer 2/3")
specification4 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Layer="Layer 4")
specification5 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Layer="Layer 5")
specs <- list(CA1=specification1,CA3=specification2,L23=specification3,L4=specification4,L5=specification5)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Axon/Pyramidal cell by Subregion/Layer")

specification1 <- list(ArborType="Axon",Class2="Granule cell",BrainRegion="Hippocampus",ExperCond="Control")
specification2 <- list(ArborType="Axon",Class2="Granule cell",BrainRegion="Hippocampus",ExperCond="D-2-amino-5-phosphonovaleric acid (D-APV)")
specification3 <- list(ArborType="Axon",Class2="Granule cell",BrainRegion="Hippocampus",ExperCond="Tetrodotoxin (TTX)")
specs <- list(Control=specification1,DAPV=specification2,TTX=specification3)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Axon/Dentate Gyrus/Granule cell by ExperimentalCondition")

specification1 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell")
specification2 <- list(ArborType="Axon",ExperCond="Control",Class1="Interneuron")
specs <- list(Pyramidal=specification1,Interneuron=specification2)
SpecificationLists <- append(SpecificationLists,list(specs))
SpecificationTitles <- append(SpecificationTitles,"Axon by Class")

names(SpecificationLists) <- SpecificationTitles

# Turn Specification Lists into pairwise specification lists
PairwiseSpecificationLists <- list()
for (i in 1:length(SpecificationLists)){
  specifications <- SpecificationLists[[i]]
  specTitle <- names(SpecificationLists)[i]
  for (j in 1:(length(specifications)-1)){
    for (k in (j+1):length(specifications)){
      specPair <- list(specifications[[j]],specifications[[k]])
      names(specPair) <- names(specifications)[c(j,k)]
      specTitle1 <- specifications[[j]][[length(specifications[[j]])]]
      if (length(specTitle1) > 1){
        specTitle1 <- paste(specTitle1,collapse="&")
      }
      specTitle2 <- specifications[[k]][[length(specifications[[k]])]]
      if (length(specTitle2) > 1){
        specTitle2 <- paste(specTitle2,collapse="&")
      }
      specPairTitle <- paste(specTitle,"-",specTitle1,"vs",specTitle2)
      PairwiseSpecificationLists <- append(PairwiseSpecificationLists,list(specPair))
      names(PairwiseSpecificationLists)[length(PairwiseSpecificationLists)] <- specPairTitle
    }
  }
}

#### Define the non-overlapping set of classes (split by arbor type) ####

#### All Neurites by Arbor Type ####

AllNeuriteSpecificationsByArbor <- list()

# Axons
specification <- list(ArborType="Axon",ExperCond="Control")
AllNeuriteSpecificationsByArbor <- append(AllNeuriteSpecificationsByArbor,list(specification))
names(AllNeuriteSpecificationsByArbor)[length(AllNeuriteSpecificationsByArbor)] <- "Axon"

# Dendrites
specification <- list(ArborType="Dendrite",ExperCond="Control")
AllNeuriteSpecificationsByArbor <- append(AllNeuriteSpecificationsByArbor,list(specification))
names(AllNeuriteSpecificationsByArbor)[length(AllNeuriteSpecificationsByArbor)] <- "Dendrite"

# Apical Dendrites
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell")
AllNeuriteSpecificationsByArbor <- append(AllNeuriteSpecificationsByArbor,list(specification))
names(AllNeuriteSpecificationsByArbor)[length(AllNeuriteSpecificationsByArbor)] <- "ApicalDendrite/Pyramidal cell"

#### Axons ####

AllAxonSpecifications <- list()

# Mouse/Neocortex/Pyramidal cell (N=138; Layer2/3=21, Layer4=16, Layer5=9, NotReported=92)
specification <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",Species="Mouse",BrainRegion="Neocortex")
AllAxonSpecifications <- append(AllAxonSpecifications,list(specification))
names(AllAxonSpecifications)[length(AllAxonSpecifications)] <- "Mouse/Neocortex/Pyramidal cell"

# Rat/Neocortex/Pyramidal cell (N=121; Layer2/3=18, Layer4=51, Layer5=51, NotReported=1)
specification <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",Species="Rat",BrainRegion="Neocortex")
AllAxonSpecifications <- append(AllAxonSpecifications,list(specification))
names(AllAxonSpecifications)[length(AllAxonSpecifications)] <- "Rat/Neocortex/Pyramidal cell"

# Rodent/Neocortex/Pyramidal cell by CorticalRegion instead (N=137,48 respectively)
#specification <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex",CorticalRegion="Somatosensory")
#specification <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex",CorticalRegion="Visual")

# Rodent/Neocortex/Pyramidal cell by CorticalLayer instead (N=39,67,60 respectively, NotReported=93)
#specification <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex",Layer="Layer 2/3")
#specification <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex",Layer="Layer 4")
#specification <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Neocortex",Layer="Layer 5")

# Only 12
#specification <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Hippocampus",Region2="CA1")
# Only 10
#specification <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Hippocampus",Region2="CA3")

# Axon/Rodent/Interneuron/Basket&PV+ (N=188, could restrict to Neocortex [N=153])
#specification <- list(ArborType="Axon",ExperCond="Control",Class1="Interneuron",Order="Rodent",Class2=c("Basket cell","Parvalbumin (PV) containing cell"))
#AllAxonSpecifications <- append(AllAxonSpecifications,list(specification))
#names(AllAxonSpecifications)[length(AllAxonSpecifications)] <- "Rodent/Interneuron/Basket&PV+"

# Axon/Rodent/Hippocampus/Interneuron/Basket (N=32)
#specification <- list(ArborType="Axon",ExperCond="Control",BrainRegion="Hippocampus",Class1="Interneuron",Order="Rodent",Class2="Basket cell")
#AllAxonSpecifications <- append(AllAxonSpecifications,list(specification))
#names(AllAxonSpecifications)[length(AllAxonSpecifications)] <- "Rodent/Hippocampus/Interneuron/Basket"

# Axon/Rodent/Neocortex/Interneuron/Basket (N=98)
#specification <- list(ArborType="Axon",ExperCond="Control",BrainRegion="Neocortex",Class1="Interneuron",Order="Rodent",Class2="Basket cell")
#AllAxonSpecifications <- append(AllAxonSpecifications,list(specification))
#names(AllAxonSpecifications)[length(AllAxonSpecifications)] <- "Rodent/Neocortex/Interneuron/Basket"

# Axon/Rodent/Neocortex/Interneuron/PV+ (N=56)
#specification <- list(ArborType="Axon",ExperCond="Control",BrainRegion="Neocortex",Class1="Interneuron",Order="Rodent",Class2="Parvalbumin (PV) containing cell")
#AllAxonSpecifications <- append(AllAxonSpecifications,list(specification))
#names(AllAxonSpecifications)[length(AllAxonSpecifications)] <- "Rodent/Interneuron/PV+"

# # Axon/Rodent/Neocortex/Interneuron/Basket&PV+ (N=153)
# specification <- list(ArborType="Axon",ExperCond="Control",BrainRegion="Neocortex",Class1="Interneuron",Order="Rodent",Class2=c("Basket cell","Parvalbumin (PV) containing cell"))
# AllAxonSpecifications <- append(AllAxonSpecifications,list(specification))
# names(AllAxonSpecifications)[length(AllAxonSpecifications)] <- "Rodent/Neocortex/Interneuron/Basket&PV+"
# 
# # Axon/Rodent/Neocortex/Interneuron/Martinotti&SOM+ (N=80=45+35)
# specification <- list(ArborType="Axon",ExperCond="Control",BrainRegion="Neocortex",Class1="Interneuron",Order="Rodent",Class2=c("Martinotti cell","Somatostatin (SOM) containing cell"))
# AllAxonSpecifications <- append(AllAxonSpecifications,list(specification))
# names(AllAxonSpecifications)[length(AllAxonSpecifications)] <- "Rodent/Neocortex/Interneuron/Martinotti&SOM+"
# 
# # Axon/Rodent/Neocortex/Interneuron/Other (N=203; NotReported=153, NPY+= 12, Bitufted=11, Neurogliaform=10,Stellate=7,Bipolar=6,Double bouquet=4)
# specification <- list(ArborType="Axon",ExperCond="Control",BrainRegion="Neocortex",Class1="Interneuron",Order="Rodent",Class2=c("Bipolar cell","Bitufted cell","Double bouquet cell","Neurogliaform cell","Neuropeptide Y (NPY) containing cell","Not reported","Stellate cell"))
# AllAxonSpecifications <- append(AllAxonSpecifications,list(specification))
# names(AllAxonSpecifications)[length(AllAxonSpecifications)] <- "Rodent/Neocortex/Interneuron/Other"
# # Non-rodent: 6 additional cells

# Axon/Rodent/Neocortex/Interneuron/Perisomatic Targeting (N=153)
specification <- list(ArborType="Axon",ExperCond="Control",BrainRegion="Neocortex",Class1="Interneuron",Order="Rodent",Class2=perisomaticTargetingInterneuronClasses)
AllAxonSpecifications <- append(AllAxonSpecifications,list(specification))
names(AllAxonSpecifications)[length(AllAxonSpecifications)] <- "Rodent/Neocortex/Interneuron/PerisomaticTargeting"

# Axon/Rodent/Neocortex/Interneuron/Dendritic Targeting (N=153)
specification <- list(ArborType="Axon",ExperCond="Control",BrainRegion="Neocortex",Class1="Interneuron",Order="Rodent",Class2=dendriticTargetingInterneuronClasses)
AllAxonSpecifications <- append(AllAxonSpecifications,list(specification))
names(AllAxonSpecifications)[length(AllAxonSpecifications)] <- "Rodent/Neocortex/Interneuron/DendriticTargeting"

# Axon/Rodent/Neocortex/Interneuron/Not Reported (N=153)
specification <- list(ArborType="Axon",ExperCond="Control",BrainRegion="Neocortex",Class1="Interneuron",Order="Rodent",Class2="Not reported")
AllAxonSpecifications <- append(AllAxonSpecifications,list(specification))
names(AllAxonSpecifications)[length(AllAxonSpecifications)] <- "Rodent/Neocortex/Interneuron/NotReported"

# Axon/Drosophila/Olfactory bulb/Uniglomerular projection neuron (N=114, if just Class3=iPN, N=96)
specification <- list(ArborType="Axon",ExperCond="Control",Class2="Uniglomerular projection neuron")
AllAxonSpecifications <- append(AllAxonSpecifications,list(specification))
names(AllAxonSpecifications)[length(AllAxonSpecifications)] <- "Drosophila/Olfactory bulb/Uniglomerular projection neuron"

#### Axons - Major Classes (Olf UPNs, Mouse NC Pyramids, Rat NC Pyramids, Rodent NC Interneurons) ####

AllAxonMajorSpecifications <- list()

# Mouse/Neocortex/Pyramidal cell (N=138; Layer2/3=21, Layer4=16, Layer5=9, NotReported=92)
specification <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",Species="Mouse",BrainRegion="Neocortex")
AllAxonMajorSpecifications <- append(AllAxonMajorSpecifications,list(specification))
names(AllAxonMajorSpecifications)[length(AllAxonMajorSpecifications)] <- "Mouse/Neocortex/Pyramidal cell"

# Rat/Neocortex/Pyramidal cell (N=121; Layer2/3=18, Layer4=51, Layer5=51, NotReported=1)
specification <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",Species="Rat",BrainRegion="Neocortex")
AllAxonMajorSpecifications <- append(AllAxonMajorSpecifications,list(specification))
names(AllAxonMajorSpecifications)[length(AllAxonMajorSpecifications)] <- "Rat/Neocortex/Pyramidal cell"

# # Axon/Rodent/Neocortex/Interneuron/Basket&PV+ (N=153)
# specification <- list(ArborType="Axon",ExperCond="Control",BrainRegion="Neocortex",Class1="Interneuron",Order="Rodent")
# AllAxonMajorSpecifications <- append(AllAxonMajorSpecifications,list(specification))
# names(AllAxonMajorSpecifications)[length(AllAxonMajorSpecifications)] <- "Rodent/Neocortex/Interneuron"

# Axon/Rodent/Neocortex/Interneuron/Perisomatic Targeting (N=153)
specification <- list(ArborType="Axon",ExperCond="Control",BrainRegion="Neocortex",Class1="Interneuron",Order="Rodent",Class2=perisomaticTargetingInterneuronClasses)
AllAxonMajorSpecifications <- append(AllAxonMajorSpecifications,list(specification))
names(AllAxonMajorSpecifications)[length(AllAxonMajorSpecifications)] <- "Rodent/Neocortex/Interneuron/PerisomaticTargeting"

# Axon/Rodent/Neocortex/Interneuron/Dendritic Targeting (N=153)
specification <- list(ArborType="Axon",ExperCond="Control",BrainRegion="Neocortex",Class1="Interneuron",Order="Rodent",Class2=dendriticTargetingInterneuronClasses)
AllAxonMajorSpecifications <- append(AllAxonMajorSpecifications,list(specification))
names(AllAxonMajorSpecifications)[length(AllAxonMajorSpecifications)] <- "Rodent/Neocortex/Interneuron/DendriticTargeting"

# Axon/Drosophila/Olfactory bulb/Uniglomerular projection neuron (N=114, if just Class3=iPN, N=96)
specification <- list(ArborType="Axon",ExperCond="Control",Class2="Uniglomerular projection neuron")
AllAxonMajorSpecifications <- append(AllAxonMajorSpecifications,list(specification))
names(AllAxonMajorSpecifications)[length(AllAxonMajorSpecifications)] <- "Drosophila/Olfactory bulb/Uniglomerular projection neuron"

#### Axons - Neocortical Interneurons ####

CorticalInterneuronAxonSpecifications <- list()

# Axon/Rodent/Neocortex/Interneuron/Basket&PV+ (N=153)
specification <- list(ArborType="Axon",ExperCond="Control",BrainRegion="Neocortex",Class1="Interneuron",Order="Rodent",Class2=c("Basket cell","Parvalbumin (PV) containing cell"))
CorticalInterneuronAxonSpecifications <- append(CorticalInterneuronAxonSpecifications,list(specification))
names(CorticalInterneuronAxonSpecifications)[length(CorticalInterneuronAxonSpecifications)] <- "Rodent/Neocortex/Interneuron/Basket&PV+"

# Axon/Rodent/Neocortex/Interneuron/Martinotti&SOM+ (N=80=45+35)
specification <- list(ArborType="Axon",ExperCond="Control",BrainRegion="Neocortex",Class1="Interneuron",Order="Rodent",Class2=c("Martinotti cell","Somatostatin (SOM) containing cell"))
CorticalInterneuronAxonSpecifications <- append(CorticalInterneuronAxonSpecifications,list(specification))
names(CorticalInterneuronAxonSpecifications)[length(CorticalInterneuronAxonSpecifications)] <- "Rodent/Neocortex/Interneuron/Martinotti&SOM+"

# Axon/Rodent/Neocortex/Interneuron/Other (N=203; NotReported=153, NPY+= 12, Bitufted=11, Neurogliaform=10,Stellate=7,Bipolar=6,Double bouquet=4)
specification <- list(ArborType="Axon",ExperCond="Control",BrainRegion="Neocortex",Class1="Interneuron",Order="Rodent",Class2=c("Bipolar cell","Bitufted cell","Double bouquet cell","Neurogliaform cell","Neuropeptide Y (NPY) containing cell","Not reported","Stellate cell"))
CorticalInterneuronAxonSpecifications <- append(CorticalInterneuronAxonSpecifications,list(specification))
names(CorticalInterneuronAxonSpecifications)[length(CorticalInterneuronAxonSpecifications)] <- "Rodent/Neocortex/Interneuron/Other"
# Non-rodent: 6 additional cells

#### Apical Dendrites by region - CA1, CA3, Neocortex
AllApicalSpecificationsByHPNC <- list()

# ApicalDendrite/Hippocampus/CA1 (N=129)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Hippocampus",Region2="CA1")
AllApicalSpecificationsByHPNC <- append(AllApicalSpecificationsByHPNC,list(specification))
names(AllApicalSpecificationsByHPNC)[length(AllApicalSpecificationsByHPNC)] <- "Rodent/Hippocampus/CA1"

# ApicalDendrite/Hippocampus/CA3 (N=71)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Hippocampus",Region2="CA3")
AllApicalSpecificationsByHPNC <- append(AllApicalSpecificationsByHPNC,list(specification))
names(AllApicalSpecificationsByHPNC)[length(AllApicalSpecificationsByHPNC)] <- "Rodent/Hippocampus/CA3"

# ApicalDendrite/Rodent/Neocortex/FrontalLobe (N=?)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Rodent")
AllApicalSpecificationsByHPNC <- append(AllApicalSpecificationsByHPNC,list(specification))
names(AllApicalSpecificationsByHPNC)[length(AllApicalSpecificationsByHPNC)] <- "Rodent/Neocortex"


#### Apical Dendrites by region ####
# Alternative would be to use cortical lobes to get larger N for each group

AllApicalSpecificationsByRegion <- list()

# ApicalDendrite/Hippocampus/CA1 (N=129)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Hippocampus",Region2="CA1")
AllApicalSpecificationsByRegion <- append(AllApicalSpecificationsByRegion,list(specification))
names(AllApicalSpecificationsByRegion)[length(AllApicalSpecificationsByRegion)] <- "Rodent/Hippocampus/CA1"

# ApicalDendrite/Hippocampus/CA3 (N=71)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Hippocampus",Region2="CA3")
AllApicalSpecificationsByRegion <- append(AllApicalSpecificationsByRegion,list(specification))
names(AllApicalSpecificationsByRegion)[length(AllApicalSpecificationsByRegion)] <- "Rodent/Hippocampus/CA3"


# ApicalDendrite/Primate/Neocortex/Prefrontal (N=50)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Primate",CorticalRegion="Prefrontal")
AllApicalSpecificationsByRegion <- append(AllApicalSpecificationsByRegion,list(specification))
names(AllApicalSpecificationsByRegion)[length(AllApicalSpecificationsByRegion)] <- "Primate/Neocortex/Prefrontal"

# ApicalDendrite/Rodent/Neocortex/Somatosensory (N=223; makes up all of Rodent Parietal data)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Rodent",CorticalRegion="Somatosensory")
AllApicalSpecificationsByRegion <- append(AllApicalSpecificationsByRegion,list(specification))
names(AllApicalSpecificationsByRegion)[length(AllApicalSpecificationsByRegion)] <- "Rodent/Neocortex/Somatosensory"

# ApicalDendrite/Rodent/Neocortex/Visual (N=43; makes up all of Rodent Occipital data)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Rodent",CorticalRegion="Visual")
AllApicalSpecificationsByRegion <- append(AllApicalSpecificationsByRegion,list(specification))
names(AllApicalSpecificationsByRegion)[length(AllApicalSpecificationsByRegion)] <- "Rodent/Neocortex/Visual"

# # ApicalDendrite/Rodent/Neocortex/Prefrontal (N=17; if Frontal lobe, N=42)
# specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Rodent",CorticalRegion="Prefrontal")
# AllApicalSpecificationsByRegion <- append(AllApicalSpecificationsByRegion,list(specification))
# names(AllApicalSpecificationsByRegion)[length(AllApicalSpecificationsByRegion)] <- "Rodent/Neocortex/Prefrontal"

# ApicalDendrite/Rodent/Neocortex/FrontalLobe (N=42)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Rodent",CorticalLobe="Frontal")
AllApicalSpecificationsByRegion <- append(AllApicalSpecificationsByRegion,list(specification))
names(AllApicalSpecificationsByRegion)[length(AllApicalSpecificationsByRegion)] <- "Rodent/Neocortex/FrontalLobe"

#### Apical Dendrites by Layer ####

AllApicalSpecificationsByLayer <- list()

# ApicalDendrite/Rodent/Hippocampus/CA1 (N=129)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Hippocampus",Region2="CA1")
AllApicalSpecificationsByLayer <- append(AllApicalSpecificationsByLayer,list(specification))
names(AllApicalSpecificationsByLayer)[length(AllApicalSpecificationsByLayer)] <- "Rodent/Hippocampus/CA1"

# ApicalDendrite/Rodent/Hippocampus/CA3 (N=71)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Hippocampus",Region2="CA3")
AllApicalSpecificationsByLayer <- append(AllApicalSpecificationsByLayer,list(specification))
names(AllApicalSpecificationsByLayer)[length(AllApicalSpecificationsByLayer)] <- "Rodent/Hippocampus/CA3"


# ApicalDendrite/Primate/Neocortex/Layer 2,3 (N=63)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Primate",Layer="Layer 2/3")
AllApicalSpecificationsByLayer <- append(AllApicalSpecificationsByLayer,list(specification))
names(AllApicalSpecificationsByLayer)[length(AllApicalSpecificationsByLayer)] <- "Primate/Neocortex/Layer 2,3"

# ApicalDendrite/Rodent/Neocortex/Layer 2,3 (N=41)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Rodent",Layer="Layer 2/3")
AllApicalSpecificationsByLayer <- append(AllApicalSpecificationsByLayer,list(specification))
names(AllApicalSpecificationsByLayer)[length(AllApicalSpecificationsByLayer)] <- "Rodent/Neocortex/Layer 2,3"

# ApicalDendrite/Rodent/Neocortex/Layer 4 (N=25)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Rodent",Layer="Layer 4")
AllApicalSpecificationsByLayer <- append(AllApicalSpecificationsByLayer,list(specification))
names(AllApicalSpecificationsByLayer)[length(AllApicalSpecificationsByLayer)] <- "Rodent/Neocortex/Layer 4"

# ApicalDendrite/Rodent/Neocortex/Layer 5 (N=83)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Rodent",Layer="Layer 5")
AllApicalSpecificationsByLayer <- append(AllApicalSpecificationsByLayer,list(specification))
names(AllApicalSpecificationsByLayer)[length(AllApicalSpecificationsByLayer)] <- "Rodent/Neocortex/Layer 5"

#### Cortical Apical Dendrites by Layer ####

#with(subset(usedMetaData,ExperCond=="Control"&ArborType=="ApicalDendrite"&BrainRegion=="Neocortex"),
#     aggregate(BrainRegion,by=list(Order,Layer),length))

CorticalApicalSpecificationsByLayer <- list()

# ApicalDendrite/Primate/Neocortex/Layer 2,3 (N=63)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Primate",Layer="Layer 2/3")
CorticalApicalSpecificationsByLayer <- append(CorticalApicalSpecificationsByLayer,list(specification))
names(CorticalApicalSpecificationsByLayer)[length(CorticalApicalSpecificationsByLayer)] <- "Primate/Neocortex/Layer 2,3"

# ApicalDendrite/Rodent/Neocortex/Layer 2,3 (N=41)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Rodent",Layer="Layer 2/3")
CorticalApicalSpecificationsByLayer <- append(CorticalApicalSpecificationsByLayer,list(specification))
names(CorticalApicalSpecificationsByLayer)[length(CorticalApicalSpecificationsByLayer)] <- "Rodent/Neocortex/Layer 2,3"

# ApicalDendrite/Rodent/Neocortex/Layer 4 (N=25)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Rodent",Layer="Layer 4")
CorticalApicalSpecificationsByLayer <- append(CorticalApicalSpecificationsByLayer,list(specification))
names(CorticalApicalSpecificationsByLayer)[length(CorticalApicalSpecificationsByLayer)] <- "Rodent/Neocortex/Layer 4"

# ApicalDendrite/Rodent/Neocortex/Layer 5 (N=83)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Rodent",Layer="Layer 5")
CorticalApicalSpecificationsByLayer <- append(CorticalApicalSpecificationsByLayer,list(specification))
names(CorticalApicalSpecificationsByLayer)[length(CorticalApicalSpecificationsByLayer)] <- "Rodent/Neocortex/Layer 5"

#### CA1 Apical Dendrites ####

CA1ApicalSpecificationsBySpecies <- list()

# Mouse (N=35)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Species="Mouse",BrainRegion="Hippocampus",Region2="CA1")
CA1ApicalSpecificationsBySpecies <- append(CA1ApicalSpecificationsBySpecies,list(specification))
names(CA1ApicalSpecificationsBySpecies)[length(CA1ApicalSpecificationsBySpecies)] <- "Mouse"

# Rat (N=134)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Species="Rat",BrainRegion="Hippocampus",Region2="CA1")
CA1ApicalSpecificationsBySpecies <- append(CA1ApicalSpecificationsBySpecies,list(specification))
names(CA1ApicalSpecificationsBySpecies)[length(CA1ApicalSpecificationsBySpecies)] <- "Rat"


# CA1ApicalSpecificationsBySpeciesAge <- list()
# 
# # Mouse/Young (N=35)
# specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Species="Mouse",AgeClass="Young",BrainRegion="Hippocampus",Region2="CA1")
# CA1ApicalSpecificationsBySpeciesAge <- append(CA1ApicalSpecificationsBySpeciesAge,list(specification))
# names(CA1ApicalSpecificationsBySpeciesAge)[length(CA1ApicalSpecificationsBySpeciesAge)] <- "Mouse/Young"
# 
# # Rat/Young (N=51)
# specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Species="Rat",AgeClass="Young",BrainRegion="Hippocampus",Region2="CA1")
# CA1ApicalSpecificationsBySpeciesAge <- append(CA1ApicalSpecificationsBySpeciesAge,list(specification))
# names(CA1ApicalSpecificationsBySpeciesAge)[length(CA1ApicalSpecificationsBySpeciesAge)] <- "Rat/Young"
# 
# # Rat/Adult&Old (N=71)
# specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Species="Rat",AgeClass=c("Adult","Old"),BrainRegion="Hippocampus",Region2="CA1")
# CA1ApicalSpecificationsBySpeciesAge <- append(CA1ApicalSpecificationsBySpeciesAge,list(specification))
# names(CA1ApicalSpecificationsBySpeciesAge)[length(CA1ApicalSpecificationsBySpeciesAge)] <- "Rat/Adult"
# 
# 
# CA1ApicalSpecificationsBySpeciesProtocol <-list()
# 
# # Mouse/In vitro (N=35)
# specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Species="Mouse",BrainRegion="Hippocampus",Region2="CA1")
# CA1ApicalSpecificationsBySpeciesProtocol <- append(CA1ApicalSpecificationsBySpeciesProtocol,list(specification))
# names(CA1ApicalSpecificationsBySpeciesProtocol)[length(CA1ApicalSpecificationsBySpeciesProtocol)] <- "Mouse/Young"
# 
# # Rat/Young (N=51)
# specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Species="Rat",Protocol="In vitro",BrainRegion="Hippocampus",Region2="CA1")
# CA1ApicalSpecificationsBySpeciesProtocol <- append(CA1ApicalSpecificationsBySpeciesProtocol,list(specification))
# names(CA1ApicalSpecificationsBySpeciesProtocol)[length(CA1ApicalSpecificationsBySpeciesProtocol)] <- "Rat/In vitro"
# 
# # Rat/Adult&Old (N=71)
# specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Species="Rat",Protocol="In vivo",BrainRegion="Hippocampus",Region2="CA1")
# CA1ApicalSpecificationsBySpeciesProtocol <- append(CA1ApicalSpecificationsBySpeciesProtocol,list(specification))
# names(CA1ApicalSpecificationsBySpeciesProtocol)[length(CA1ApicalSpecificationsBySpeciesProtocol)] <- "Rat/In vivo"

CA1ApicalSpecificationsBySpeciesStrain <-list()

# Mouse (N=35)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Species="Mouse",BrainRegion="Hippocampus",Region2="CA1")
CA1ApicalSpecificationsBySpeciesStrain <- append(CA1ApicalSpecificationsBySpeciesStrain,list(specification))
names(CA1ApicalSpecificationsBySpeciesStrain)[length(CA1ApicalSpecificationsBySpeciesStrain)] <- "Mouse/Young"

# Rat/Fischer (N=34)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Species="Rat",Strain="Fischer 344 Rat",BrainRegion="Hippocampus",Region2="CA1")
CA1ApicalSpecificationsBySpeciesStrain <- append(CA1ApicalSpecificationsBySpeciesStrain,list(specification))
names(CA1ApicalSpecificationsBySpeciesStrain)[length(CA1ApicalSpecificationsBySpeciesStrain)] <- "Rat/Fischer"

# Rat/Sprague-Dawley (N=52)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Species="Rat",Strain="Sprague-Dawley Rat",BrainRegion="Hippocampus",Region2="CA1")
CA1ApicalSpecificationsBySpeciesStrain <- append(CA1ApicalSpecificationsBySpeciesStrain,list(specification))
names(CA1ApicalSpecificationsBySpeciesStrain)[length(CA1ApicalSpecificationsBySpeciesStrain)] <- "Rat/Sprague-Dawley"

# Rat/Wistar (N=48)
specification <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Species="Rat",Strain="Wistar Rat",BrainRegion="Hippocampus",Region2="CA1")
CA1ApicalSpecificationsBySpeciesStrain <- append(CA1ApicalSpecificationsBySpeciesStrain,list(specification))
names(CA1ApicalSpecificationsBySpeciesStrain)[length(CA1ApicalSpecificationsBySpeciesStrain)] <- "Rat/Wistar"

#### Dendrites by Species/CellType ####

AllDendriteSpecifications <- list()

# Dendrite/Motoneuron (N=88)
specification <- list(ArborType="Dendrite",ExperCond="Control",Class2="Motoneuron")
AllDendriteSpecifications <- append(AllDendriteSpecifications,list(specification))
names(AllDendriteSpecifications)[length(AllDendriteSpecifications)] <- "Motoneuron"

# Dendrite/Carnivora(Cat)/Motoneuron (50; All principal cells N=83)
# specification <- list(ArborType="Dendrite",ExperCond="Control",Order="Carnivora",Class2="Motoneuron")
# AllDendriteSpecifications <- append(AllDendriteSpecifications,list(specification))
# names(AllDendriteSpecifications)[length(AllDendriteSpecifications)] <- "Cat/Motoneuron"
# 
# # Dendrite/Rodent/Motoneuron (N=38)
# specification <- list(ArborType="Dendrite",ExperCond="Control",Order="Rodent",Class2="Motoneuron")
# AllDendriteSpecifications <- append(AllDendriteSpecifications,list(specification))
# names(AllDendriteSpecifications)[length(AllDendriteSpecifications)] <- "Rodent/Motoneuron"

# Dendrite/Caudata(Salamander)/Ganglion cell (N=45)
#specification <- list(ArborType="Dendrite",ExperCond="Control",Order="Caudata",Class2="Ganglion cell")
#AllDendriteSpecifications <- append(AllDendriteSpecifications,list(specification))
#names(AllDendriteSpecifications)[length(AllDendriteSpecifications)] <- "Salamander/Ganglion cell"
# Could combine with Fish for just Ganglion cell (N=102)
# Combine w/ Fish and Rodent (N=532)

# Dendrite/Cypriniformes(Fish)/Ganglion cell (N=57)
#specification <- list(ArborType="Dendrite",ExperCond="Control",Order="Cypriniformes",Class2="Ganglion cell")
#AllDendriteSpecifications <- append(AllDendriteSpecifications,list(specification))
#names(AllDendriteSpecifications)[length(AllDendriteSpecifications)] <- "Fish/Ganglion cell"

# Dendrite/Other(Fish&Salamander)/Ganglion cell (N=102)
#specification <- list(ArborType="Dendrite",ExperCond="Control",Order=c("Cypriniformes","Caudata"),Class2="Ganglion cell")
#AllDendriteSpecifications <- append(AllDendriteSpecifications,list(specification))
#names(AllDendriteSpecifications)[length(AllDendriteSpecifications)] <- "Other/Retinal Ganglion cell"

# Dendrite/Rodent/Ganglion cell (N=430)
specification <- list(ArborType="Dendrite",ExperCond="Control",Order="Rodent",Class2="Ganglion cell")
AllDendriteSpecifications <- append(AllDendriteSpecifications,list(specification))
names(AllDendriteSpecifications)[length(AllDendriteSpecifications)] <- "Rodent/Retinal Ganglion cell"

# Dendrite/Diptera(Fly)/Interneuron/Tangential cell (N=54)
specification <- list(ArborType="Dendrite",ExperCond="Control",Order="Diptera",Class2="Tangential cell")
AllDendriteSpecifications <- append(AllDendriteSpecifications,list(specification))
names(AllDendriteSpecifications)[length(AllDendriteSpecifications)] <- "Fly/Tangential cell"

# Dendrite/Diptera(Fly)/Sensory neuron DA1 (N=139)
#specification <- list(ArborType="Dendrite",Order="Diptera",Class2="Sensory neuron",Class3="Class I")
#AllDendriteSpecifications <- append(AllDendriteSpecifications,list(specification))
#names(AllDendriteSpecifications)[length(AllDendriteSpecifications)] <- "LarvalFly/Sensory DA1"

# Dendrite/Diptera(Fly)/Sensory neuron DA2-4 (N=139)
#specification <- list(ArborType="Dendrite",Order="Diptera",Class2="Sensory neuron",Class3=c("Class II","Class III","Class IV"))
#AllDendriteSpecifications <- append(AllDendriteSpecifications,list(specification))
#names(AllDendriteSpecifications)[length(AllDendriteSpecifications)] <- "LarvalFly/Sensory DA2-4"

# Dendrite/Diptera(Fly)/Sensory neuron/Control (N=69)
specification <- list(ArborType="Dendrite",ExperCond="Control",Order="Diptera",Class2="Sensory neuron")
AllDendriteSpecifications <- append(AllDendriteSpecifications,list(specification))
names(AllDendriteSpecifications)[length(AllDendriteSpecifications)] <- "LarvalFly/Sensory neuron"

# Dendrite/Diptera(Fly)/Sensory neuron/Mutant tutl expression (N=70)
#specification <- list(ArborType="Dendrite",ExperCond="Mutant tutl expression",Order="Diptera",Class2="Sensory neuron")
#AllDendriteSpecifications <- append(AllDendriteSpecifications,list(specification))
#names(AllDendriteSpecifications)[length(AllDendriteSpecifications)] <- "Fly/Sensory neuron/Mutant tutl"

# Dendrite/Proboscidae(Elephant)/Pyramidal cell (N=46)
specification <- list(ArborType="Dendrite",ExperCond="Control",Order="Proboscidae",Class2="Pyramidal cell")
AllDendriteSpecifications <- append(AllDendriteSpecifications,list(specification))
names(AllDendriteSpecifications)[length(AllDendriteSpecifications)] <- "Elephant/Pyramidal cell/Neocortex"

# Dendrite/Primate/Pyramidal cell
# Regions: Prefrontal=532, Visual=254, Somatosensory=101, MidShortInsularGyrus=186, PostShortInsularGyrus=175, AntLongInsularGyrus=174
# Layers: Layer2/3=693, Layer5=15, Layer6=9
# Archive: Jacobs=1707,Lewis=136,Wearn_Hof=68,  Vuksic=4,Allman=2,Conzelez-Burgos=1

# Dendrite/Primate/Pyramidal cell/Layer2,3 (N=693, All Layers N=1918, 1201 Not Reported)
specification <- list(ArborType="Dendrite",ExperCond="Control",Order="Primate",Class2="Pyramidal cell",Layer="Layer 2/3")
AllDendriteSpecifications <- append(AllDendriteSpecifications,list(specification))
names(AllDendriteSpecifications)[length(AllDendriteSpecifications)] <- "Primate/Pyramidal cell/Neocortex/Layer2,3"

# Dendrite/Rodent/Pyramidal cell (N=739)
#specification <- list(ArborType="Dendrite",ExperCond="Control",Order="Rodent",Class2="Pyramidal cell")
# Regions: CA1=106, CA3=71, FrontalLobe=40, Motor=30, Somatosensory=302, Visual=50
# Layers: Layer5=100, Layer2/3=165,Layer4=73

# Dendrite/Rodent/Neocortex/Pyramidal cell (N=338+)
specification <- list(ArborType="Dendrite",ExperCond="Control",Order="Rodent",BrainRegion="Neocortex",Class2="Pyramidal cell")
AllDendriteSpecifications <- append(AllDendriteSpecifications,list(specification))
names(AllDendriteSpecifications)[length(AllDendriteSpecifications)] <- "Rodent/Pyramidal cell/Neocortex"

# Dendrite/Rodent/Pyramidal cell/Layer 2,3 (N=165)
#specification <- list(ArborType="Dendrite",ExperCond="Control",Order="Rodent",Class2="Pyramidal cell",Layer="Layer 2/3")
#AllDendriteSpecifications <- append(AllDendriteSpecifications,list(specification))
#names(AllDendriteSpecifications)[length(AllDendriteSpecifications)] <- "Rodent/Pyramidal cell/Neocortex/Layer2,3"

# Dendrite/Rodent/Pyramidal cell/Layer 4 (N=73)
#specification <- list(ArborType="Dendrite",ExperCond="Control",Order="Rodent",Class2="Pyramidal cell",Layer="Layer 4")
#AllDendriteSpecifications <- append(AllDendriteSpecifications,list(specification))
#names(AllDendriteSpecifications)[length(AllDendriteSpecifications)] <- "Rodent/Pyramidal cell/Neocortex/Layer4"

# Dendrite/Rodent/Pyramidal cell/Layer 5 (N=100)
#specification <- list(ArborType="Dendrite",ExperCond="Control",Order="Rodent",Class2="Pyramidal cell",Layer="Layer 5")
#AllDendriteSpecifications <- append(AllDendriteSpecifications,list(specification))
#names(AllDendriteSpecifications)[length(AllDendriteSpecifications)] <- "Rodent/Pyramidal cell/Neocortex/Layer5"

# Dendrite/Rodent/Stellate cell (N=46)
#specification <- list(ArborType="Dendrite",ExperCond="Control",Order="Rodent",Class1="Principal cell",Class2="Stellate cell")
#AllDendriteSpecifications <- append(AllDendriteSpecifications,list(specification))
#names(AllDendriteSpecifications)[length(AllDendriteSpecifications)] <- "Rodent/Principal cell/Stellate cell"

# Dendrite/Rodent/Principal cell (N=1379)
# Granule cell 18
# PGN 20

# Dendrite/Rodent/Neocortex/Interneuron (N=485)
specification <- list(ArborType="Dendrite",ExperCond="Control",Order="Rodent",Class1="Interneuron",BrainRegion="Neocortex")
AllDendriteSpecifications <- append(AllDendriteSpecifications,list(specification))
names(AllDendriteSpecifications)[length(AllDendriteSpecifications)] <- "Rodent/Neocortex/Interneuron"

# Dendrite/Rodent/Interneuron (N=557)
#specification <- list(ArborType="Dendrite",ExperCond="Control",Order="Rodent",Class1="Interneuron")
#AllDendriteSpecifications <- append(AllDendriteSpecifications,list(specification))
#names(AllDendriteSpecifications)[length(AllDendriteSpecifications)] <- "Rodent/Interneuron"
# Basket cell 83
# PV+ 45
# Martinotti cell 40
# SOM+ 26
# Neurogliaform 20
#

#### New Pairwise Specifications ####
NewPairwiseSpecifications <- list()

## Apicals

# ApicalDendrite/Rodent/Hippocampus/CA1 vs CA3
spec1 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Hippocampus",Region2="CA1")
spec2 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Hippocampus",Region2="CA3")
NewPairwiseSpecifications <- append(NewPairwiseSpecifications,list(list(CA1=spec1,CA3=spec2)))
names(NewPairwiseSpecifications)[length(NewPairwiseSpecifications)] <- "Apical/Rodent: CA1 vs CA3"

# ApicalDendrite/Rodent/Hippocampus/CA1 vs Neocortex/Layer 2,3
spec1 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Hippocampus",Region2="CA1")
spec2 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Layer="Layer 2/3")
NewPairwiseSpecifications <- append(NewPairwiseSpecifications,list(list(CA1=spec1,Layer23=spec2)))
names(NewPairwiseSpecifications)[length(NewPairwiseSpecifications)] <- "Apical: Rodent/CA1 vs Layer 2,3"

# ApicalDendrite/Rodent/Hippocampus/CA1 vs Neocortex/Layer 5
spec1 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Hippocampus",Region2="CA1")
spec2 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Layer="Layer 5")
NewPairwiseSpecifications <- append(NewPairwiseSpecifications,list(list(CA1=spec1,Layer5=spec2)))
names(NewPairwiseSpecifications)[length(NewPairwiseSpecifications)] <- "Apical: Rodent/CA1 vs Layer 5"

# ApicalDendrite/Rodent/Hippocampus/CA3 vs Neocortex/Layer 2,3
spec1 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Hippocampus",Region2="CA3")
spec2 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Layer="Layer 2/3")
NewPairwiseSpecifications <- append(NewPairwiseSpecifications,list(list(CA3=spec1,Layer23=spec2)))
names(NewPairwiseSpecifications)[length(NewPairwiseSpecifications)] <- "Apical: Rodent/CA3 vs Layer 2,3"

# ApicalDendrite/Rodent/Hippocampus/CA3 vs Neocortex/Layer 5
spec1 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",Order="Rodent",BrainRegion="Hippocampus",Region2="CA3")
spec2 <- list(ArborType="ApicalDendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Layer="Layer 5")
NewPairwiseSpecifications <- append(NewPairwiseSpecifications,list(list(CA3=spec1,Layer5=spec2)))
names(NewPairwiseSpecifications)[length(NewPairwiseSpecifications)] <- "Apical: Rodent/CA3 vs Layer 5"

## Axons

# Axon/Neocortex/Pyramidal Mouse vs Rat
spec1 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Species="Mouse")
spec2 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Species="Rat")
NewPairwiseSpecifications <- append(NewPairwiseSpecifications,list(list(CA1=spec1,Layer5=spec2)))
names(NewPairwiseSpecifications)[length(NewPairwiseSpecifications)] <- "Axon: Neocortex/Pyramidal cell Mouse vs Rat"

# Axon/Mouse/Neocortex/Pyramidal cell  vs Rodent/Neocortex/Basket cell
spec1 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Species="Mouse")
spec2 <- list(ArborType="Axon",ExperCond="Control",Class2="Basket cell",BrainRegion="Neocortex",Order="Rodent")
NewPairwiseSpecifications <- append(NewPairwiseSpecifications,list(list(MousePyramidal=spec1,RodentBasket=spec2)))
names(NewPairwiseSpecifications)[length(NewPairwiseSpecifications)] <- "Axon: Mouse/Neocortex/Pyramidal cell  vs Rodent/Neocortex/Basket cell"

# Axon/Rat/Neocortex/Pyramidal cell  vs Rodent/Neocortex/Basket cell
spec1 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Species="Rat")
spec2 <- list(ArborType="Axon",ExperCond="Control",Class2="Basket cell",BrainRegion="Neocortex",Order="Rodent")
NewPairwiseSpecifications <- append(NewPairwiseSpecifications,list(list(RatPyramidal=spec1,RodentBasket=spec2)))
names(NewPairwiseSpecifications)[length(NewPairwiseSpecifications)] <- "Axon: Rat/Neocortex/Pyramidal cell  vs Rodent/Neocortex/Basket cell"

# Axon/Fly/UPN  vs Rodent/Neocortex/Basket cell
spec1 <- list(ArborType="Axon",ExperCond="Control",Class2="Uniglomerular projection neuron",Species="Drosophila")
spec2 <- list(ArborType="Axon",ExperCond="Control",Class2="Basket cell",BrainRegion="Neocortex",Order="Rodent")
NewPairwiseSpecifications <- append(NewPairwiseSpecifications,list(list(FlyUPN=spec1,RodentBasket=spec2)))
names(NewPairwiseSpecifications)[length(NewPairwiseSpecifications)] <- "Axon: Fly/UPN  vs Rodent/Neocortex/Basket cell"

# Axon/Fly/UPN  vs Rat/Neocortex/Pyramidal cell
spec1 <- list(ArborType="Axon",ExperCond="Control",Class2="Uniglomerular projection neuron",Species="Drosophila")
spec2 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Species="Rat")
NewPairwiseSpecifications <- append(NewPairwiseSpecifications,list(list(FlyUPN=spec1,RodentBasket=spec2)))
names(NewPairwiseSpecifications)[length(NewPairwiseSpecifications)] <- "Axon: Fly/UPN  vs Rat/Neocortex/Pyramidal cell"

## Dendrites

# Dendrite/Rodent/Interneuron  vs Fly/Sensory
spec1 <- list(ArborType="Dendrite",ExperCond="Control",Class1="Interneuron",BrainRegion="Neocortex",Order="Rodent")
spec2 <- list(ArborType="Dendrite",ExperCond="Control",Order="Diptera",Class2="Sensory neuron")
NewPairwiseSpecifications <- append(NewPairwiseSpecifications,list(list(RodentInterneuron=spec1,FlySensory=spec2)))
names(NewPairwiseSpecifications)[length(NewPairwiseSpecifications)] <- "Dendrite: Rodent/Neocortex/Interneuron vs Fly/Sensory"

# Dendrite/Primate/Neocortex/Pyramidal/Layer 2,3  vs Fly/Sensory
spec1 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Primate",Layer="Layer 2/3")
spec2 <- list(ArborType="Dendrite",ExperCond="Control",Order="Diptera",Class2="Sensory neuron")
NewPairwiseSpecifications <- append(NewPairwiseSpecifications,list(list(PrimatePyrL23=spec1,FlySensory=spec2)))
names(NewPairwiseSpecifications)[length(NewPairwiseSpecifications)] <- "Dendrite: Primate/Pyramidal/Layer2,3 vs Fly/Sensory"

# Dendrite/Rodent/Ganglion cell  vs Fly/Sensory
spec1 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Ganglion cell",Order="Rodent")
spec2 <- list(ArborType="Dendrite",ExperCond="Control",Order="Diptera",Class2="Sensory neuron")
NewPairwiseSpecifications <- append(NewPairwiseSpecifications,list(list(RodentGanglion=spec1,FlySensory=spec2)))
names(NewPairwiseSpecifications)[length(NewPairwiseSpecifications)] <- "Dendrite: Rodent/Ganglion cell vs Fly/Sensory"

# Dendrite/Rodent/Interneuron  vs Rodent/Ganglion cell
spec1 <- list(ArborType="Dendrite",ExperCond="Control",Class1="Interneuron",BrainRegion="Neocortex",Order="Rodent")
spec2 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Ganglion cell",Order="Rodent")
NewPairwiseSpecifications <- append(NewPairwiseSpecifications,list(list(RodentInterneuron=spec1,RodentGanglion=spec2)))
names(NewPairwiseSpecifications)[length(NewPairwiseSpecifications)] <- "Dendrite: Rodent/Neocortex/Interneuron vs Rodent/Ganglion cell"

# Dendrite/Primate/Neocortex/Pyramidal/Layer 2,3  vs Rodent/Interneuron
spec1 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Primate",Layer="Layer 2/3")
spec2 <- list(ArborType="Dendrite",ExperCond="Control",Class1="Interneuron",BrainRegion="Neocortex",Order="Rodent")
NewPairwiseSpecifications <- append(NewPairwiseSpecifications,list(list(PrimatePyrL23=spec1,RodentInterneuron=spec2)))
names(NewPairwiseSpecifications)[length(NewPairwiseSpecifications)] <- "Dendrite: Primate/Pyramidal/Layer2,3 vs Rodent/Interneuron"

# Dendrite/Primate/Neocortex/Pyramidal/Layer 2,3  vs Rodent/Ganglion cell
spec1 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Primate",Layer="Layer 2/3")
spec2 <- list(ArborType="Dendrite",ExperCond="Control",Class2="Ganglion cell",Order="Rodent")
NewPairwiseSpecifications <- append(NewPairwiseSpecifications,list(list(PrimatePyrL23=spec1,RodentGanglion=spec2)))
names(NewPairwiseSpecifications)[length(NewPairwiseSpecifications)] <- "Dendrite: Primate/Pyramidal/Layer2,3 vs Rodent/Ganglion cell"
