#########################################################################
#
# ClusterAnalysisFromGlobalAlign.r
#
# Reads score or correlation data from pairwise alignments performed by
# Parsa's Matlab alignment code.
#
#########################################################################

############# *SET UP ################

systemType <- "PC"
systemType <- "KrasClust"

# Set up directories
if (systemType == "PC"){
  clusterDir <- "C:/Documents and Settings/gillette/My Documents/research/ClusterAnalysis/"
  baseCodeDir <- "G:/Dropbox/RCode/"
  clusterGlobalDir <- "C:/Documents and Settings/gillette/My Documents/research/GlobalAlignment/"
  globalAlignDataDir <- "G:/AlignmentData/"
}else{
  clusterDir <- "/homes/tgillett/ClusterAnalysis/"
  baseCodeDir <- "/homes/tgillett/RUtils/"
  clusterGlobalDir <- "/homes/tgillett/PairwiseAlignment/"
  globalAlignDataDir <- clusterGlobalDir
  baselineDir <- "/homes/tgillett/PairwiseAlignment/Baseline/"
}
#motifAnalysisDir <- "C:/Documents and Settings/gillette/My Documents/My Dropbox/Motif Analysis/Motif Analysis R Files/"

# Load Utils
source(paste(baseCodeDir,"Utils.r",sep=""))
source(paste(baseCodeDir,"StringUtils.r",sep=""))
source(paste(baseCodeDir,"StatisticsUtils.r",sep=""))

# Get sequence names
sequenceNamesAll <- read.table(paste(clusterGlobalDir,"/fastaSequenceNames.txt",sep=""),stringsAsFactors=F)[,1]

# Get metadata and load related functions
masterMetaData <- read.csv(paste(clusterDir,"arborMetaData-NM-MapType2.csv",sep=""),na.strings="",stringsAsFactors=FALSE,row.names=1)
if ("CellType"%in%colnames(masterMetaData)){
  colnames(masterMetaData)[colnames(masterMetaData)=="CellType"] <- "Class2"
}
source(paste(clusterDir,"ClusterStatistics.r",sep=""))
source(paste(clusterDir,"ClusterAnalysisFunctions.r",sep=""))
source(paste(clusterDir,"ClassComparisonDefs.r",sep=""))
missingSeqInds <- which(!sequenceNamesAll%in%rownames(usedMetaData)) # Acute culture seqs removed from meta data
missingSeqNames <- sequenceNamesAll[missingSeqInds]
sequenceNames <- intersect(sequenceNamesAll,rownames(usedMetaData))
clusterMetaData <- usedMetaData[sequenceNames,]

library(parallel)
library(MASS)
library(mclust)
library(car)



###### *Specific Run Settings ########

# Read correlation matrix
travOrder <- "STL"
# If this is a random dataset rather than NMO
isRandom <- T
baseline <- 10
randStr <- ""
scoreParamStr <- ""
scoreParamStr <- "Blosum50G6E2"
scoreParamStr <- "Custom1G3E3"
scoreParamStr <- "TreeIdentityG0E1"
scoreParamStr <- "TreeIdentityG3E1"

if (isRandom){
  randStr <- paste("Rand",baseline,sep="")
}

matrixExt <- ".csv"
matrixExt <- ".tab"
rownamesVal <- 1 # Could be NULL


########### *Data loading ###########


# Old score matrix
oldScoreMatFilename <- paste("~/PairwiseAlignment/scoreMatrix",travOrder,".csv",sep="")
oldScoreMatDF <- read.csv(oldScoreMatFilename,header=F)
oldScoreMatDF <- oldScoreMatDF[-missingSeqInds,-missingSeqInds]
rownames(oldScoreMatDF) <- sequenceNames
colnames(oldScoreMatDF) <- sequenceNames

spearmanFilename <- paste("~/PairwiseAlignment/NeuriteLTS/spearman",travOrder,".csv",sep="")
corrMatDF <- read.csv(spearmanFilename,header=F)
corrMatDF <- corrMatDF[-missingSeqInds,-missingSeqInds]


#oldScoreMatDF <- rbind(oldScoreMatDF,c(1,oldScoreMatDF[,1]))
#oldScoreMatDF <- rbind(oldScoreMatDF[nrow(oldScoreMatDF),],corrMatDF[-nrow(oldScoreMatDF),])

if (isRandom){
  scoreMatFilename <- paste(globalAlignDataDir,"/Rand",baseline,travOrder,"/scoreMatrix",randStr,travOrder,".",".",scoreParamStr,matrixExt,sep="")
} else {
  scoreMatFilename <- paste(globalAlignDataDir,"/Neurite",travOrder,"/scoreMatrix",travOrder,".",".",scoreParamStr,matrixExt,sep="")
}
scoreMatDF <- read.csv(scoreMatFilename,sep="\t",check.names=F,row.names=rownamesVal,na.strings="None",stringsAsFactors=F)
if (ncol(scoreMatDF) > nrow(scoreMatDF)){
  scoreMatDF <- scoreMatDF[,-ncol(scoreMatDF)]
}

if (isRandom){
  normScoreMatFilename <- paste(globalAlignDataDir,"/Rand",baseline,travOrder,"/normScores",randStr,travOrder,".",scoreParamStr,matrixExt,sep="")
} else {
  normScoreMatFilename <- paste(globalAlignDataDir,"/Neurite",travOrder,"/normScores",travOrder,".",scoreParamStr,matrixExt,sep="")
  #normScoreMatFilename <- paste(globalAlignDataDir,"/",scoreParamStr,"/normScores",travOrder,".",scoreParamStr,matrixExt,sep="")
}
normScoreMatDF <- read.csv(normScoreMatFilename,sep="\t",check.names=F,row.names=rownamesVal,na.strings="None",stringsAsFactors=F)
if (ncol(normScoreMatDF) > nrow(normScoreMatDF)){
  normScoreMatDF <- normScoreMatDF[,-ncol(normScoreMatDF)]
}

if (isRandom){
  jcDistFilename <- paste(globalAlignDataDir,"/Rand",baseline,travOrder,"/jcDist",randStr,travOrder,".",scoreParamStr,matrixExt,sep="")
}else{
  jcDistFilename <- paste(globalAlignDataDir,"/Neurite",travOrder,"/jcDist",travOrder,".",scoreParamStr,matrixExt,sep="")
}
jcDistDF <- read.csv(jcDistFilename,sep="\t",check.names=F,row.names=rownamesVal)
if (ncol(jcDistDF) > nrow(jcDistDF)){
  jcDistDF <- jcDistDF[,-ncol(jcDistDF)]
}

if (isRandom){
  spearmanFilename <- paste(globalAlignDataDir,"/Rand",baseline,travOrder,"/spearman",randStr,travOrder,".",scoreParamStr,matrixExt,sep="")
} else {
  spearmanFilename <- paste(globalAlignDataDir,"/Neurite",travOrder,"/spearman",travOrder,".",scoreParamStr,matrixExt,sep="")
}
corrMatDF <- read.csv(spearmanFilename,sep="\t",check.names=F,row.names=rownamesVal)
if (ncol(corrMatDF) > nrow(corrMatDF)){
  corrMatDF <- corrMatDF[,-ncol(corrMatDF)]
}



########## *Data Cleaning ###########

# Removes header meta-data information, leaving only the neurite sequence name
cleanNames <- function(myDF){
  colnames(myDF) <- gsub(" \\|.*","",colnames(myDF))
  rownames(myDF) <- colnames(myDF)
  return(myDF)
}

# Was used when check.names was true in read.csv (it was by default, and changed the colnames substantially)
fixColAndRowNames <- function(myDF,sequenceNames){
  colnames(myDF) <- gsub("(^.+)\\.(Axon|ApicalDendrite|BasalDendrite|Dendrite).*","\\1-\\2",colnames(myDF))
  colnames(myDF) <- gsub("\\.","-",colnames(myDF))
  xstarts <- grepl("^X",sequenceNames)
  colnames(myDF)[!xstarts] <- gsub("^X","",colnames(myDF)[!xstarts])
  rownames(myDF) <- colnames(myDF)
  return(myDF)
}

fillDistMatrix <- function(distMat){
  for (i in 1:nrow(distMat)){
    naPos <- is.na(distMat[i,])
    distMat[i,naPos] <- distMat[naPos,i]
  }
  return(distMat)
}

# Remove sequences not in meta data
# Assign col and row names
jcDistDF <- jcDistDF[-missingSeqInds,-missingSeqInds]
jcDistDF <- cleanNames(jcDistDF)

corrMatDF <- corrMatDF[-missingSeqInds,-missingSeqInds]
corrMatDF <- cleanNames(corrMatDF)

scoreMatDF <- scoreMatDF[-missingSeqInds,-missingSeqInds]
scoreMatDF <- cleanNames(scoreMatDF)

corrMatDF <- corrMatDF[sequenceNames,sequenceNames]

sequenceNames <- rownames(scoreMatDF)[rownames(scoreMatDF)%in%sequenceNames]
scoreMatDF <- scoreMatDF[sequenceNames,sequenceNames]

sequenceNames <- rownames(normScoreMatDF)[rownames(normScoreMatDF)%in%sequenceNames]
normScoreMatDF <- normScoreMatDF[sequenceNames,sequenceNames]
normScoreMatDF <- fillDistMatrix(normScoreMatDF)

#scoreMatDF <- fillDistMatrix(scoreMatDF)

removeGapPenaltiesAndNormalizeSimple <- function(scoreMat,sequenceLengths=as.numeric(usedMetaData[rownames(scoreMat),"SeqLen"]),gapPenalty=-1){
  numSeqs <- nrow(scoreMat)
  for (i in 1:(numSeqs-1)){
    indices <- (i+1):numSeqs
    smallerLength <- sapply(sequenceLengths[indices],min,sequenceLengths[i])
    scoreMat[i,indices] <- (as.numeric(scoreMat[i,indices]) + -gapPenalty*abs(sequenceLengths[i]-sequenceLengths[indices]))/smallerLength
    scoreMat[indices,i] <- t(scoreMat[i,indices])
    scoreMat[i,i] <- scoreMat[i,i]/sequenceLengths[i]
  }
  return(scoreMat)
}
removeGapPenaltiesAndNormalize <- removeGapPenaltiesAndNormalizeSimple

lengthIndScoreMat <- removeGapPenaltiesAndNormalize(scoreMatDF,gapPenalty=-1)
lengthIndCorMat <- cor(lengthIndScoreMat,method="spearman")

corrMatDF <- cor(scoreMatDF,method="spearman")

#corrMatDF <- fixColAndRowNames(corrMatDF,sequenceNames)
#jcDistDF <- fixColAndRowNames(jcDistDF,sequenceNames)
#scoreMatDF <- fixColAndRowNames(scoreMatDF,sequenceNames)

#rownames(corrMatDF) <- sequenceNames
#colnames(corrMatDF) <- sequenceNames



############### *Distance matrix calculation ###################

# Convert correlations to distances (using simple conversion, 0.5-cor/2)
#distMatSimple <- 0.5-corrMatDF/2

# Make negative values go toward infinity with some sort of exponential formula
# x = 0.75-corrMatDF/4; Dist = log((1-x)/x)
x <- 0.75-corrMatDF/4
distMatStretch <- -log((1-x)/x)

# Square the shifted values
#x <- ((0.5-corrMatDF/2)^2)/2 + 0.5
x <- ((1-corrMatDF)^2)/8 + 0.5
#x <- ((1-lengthIndCorMat)^2)/8 + 0.5
distMatStretchSq <- -log((1-x)/x)
rm(x)

# Turn score values into distance values. Original scale 0-max(scores). Shift and stretch.
maxScore <- max(lengthIndScoreMat)

#corrMat <- as.dist(corrMatDF)
#rm(corrMatDF)

# Calculate normalized distances based on baseline distributions for given length pairs
source(paste(clusterGlobalDir,"BaselineHandler.r",sep=""))
baselineKey <- paste("G3E1_e3",travOrder)
loadBaselineAlignmentStats(baselineKey,travOrder=travOrder,directory=paste(baselineDir,"GapOpen3_1000/",sep=""))
before <- date()
distMatDF <- baselineNormalizedDistancesFromScores(baselineKey,normScoreMatDF,usedMetaData[colnames(normScoreMatDF),"SeqLen"])
after <- date()

# distMatDF <- matrix(0,nrow=nrow(normScoreMatDF),ncol=nrow(normScoreMatDF))
# colnames(distMatDF) <- colnames(normScoreMatDF)
# rownames(distMatDF) <- rownames(normScoreMatDF)
# numSeqs <- nrow(distMatDF)
# allSeqLengths <- usedMetaData[colnames(normScoreMatDF),"SeqLen"]
# Go through each row
# for (i in 1:(numSeqs-1)){
#   currRowLength <- usedMetaData[colnames(normScoreMatDF)[i],"SeqLen"]
#   uniqueLengths <- unique(usedMetaData[colnames(normScoreMatDF)[(i+1):numSeqs],"SeqLen"])
#   # Go through all unique lengths for pairs that need to be calculated
#   for (j in 1:length(uniqueLengths)){
#     currColLength <- uniqueLengths[j]
#     # Determine which columns to run based on the current length that have not yet been run
#     currCols <- 1:numSeqs>i & allSeqLengths==currColLength
#     distMatDF[i,currCols] <- as.numeric(getBaselineNormalizedDistance(baselineKey,normScoreMatDF[i,currCols],currRowLength,currColLength))
#     distMatDF[currCols,i] <- distMatDF[i,currCols]
#   }
# }
  
#distMatDF <- distMatSimple
#distMatDF <- distMatStretch
distMatDF <- distMatStretchSq
distMatDF <- jcDistDF

#distMatFilename <- paste(clusterDir,"/alignmentDist",travOrder,".",scoreParamStr,".csv",sep="")
#write.csv(distMatDF,distMatFilename)

#### Setup variables ####
# Choose a subset to cluster on
numSamples <- 20
maxSubsampleSize <- 100
numPairs <- length(PairwiseSpecificationLists)

#avgRIs <- rep(NA,numPairs)
wardRIs <- rep(NA,numPairs)
ward3RIs <- rep(NA,numPairs)
kmeansRIs <- rep(NA,numPairs)
kmeans3RIs <- rep(NA,numPairs)
kmediodsRIs <- rep(NA,numPairs)
kmediods3RIs <- rep(NA,numPairs)
emRIs <- rep(NA,numPairs)
emPlus1RIs <- rep(NA,numPairs)

#avgRISDs <- rep(NA,numPairs)
wardRISDs <- rep(NA,numPairs)
kmeansRISDs <- rep(NA,numPairs)
kmediodsRISDs <- rep(NA,numPairs)
emRISDs <- rep(NA,numPairs)

#avgBest <- rep(0,numPairs)
wardBest <- rep(0,numPairs)
kmeansBest <- rep(0,numPairs)
kmediodsBest <- rep(0,numPairs)
emBest <- rep(0,numPairs)
numDimensions <- 3

subsampleSizes <- rep(0,numPairs)

# Known from other runs, no reason to waste time running these
#poorPerformers <- c(2,3,4,5,10,12,13,14,15,16,17,24,34,35,36,37,39,40,49,54,58,62,67,71,77,79,80,82,85,87,89,90,91,92,96,99,100,102,104)
poorPerformers <- sort(c(118,16,68,97,75,40,34,79,128,90,1,7,66,4,102,107,83,58,8,15,
                    60,116,36,91,56,89,59,80,92,5,81,55,13,86,103,70,126,84,10,
                    50,6,74,88,14,76,78,73,95,9,99))
tooFew <- c(105,119,47,43,22,99,94,77,67,66,64,46,41,28,26,25,21,19,18)

veryStrongCases <- c(129,115,114,113,109,3,110,11)
weakCases <- c(117, 112,87,72,69,27,24)
interestingCases <- c(111, 110, 86, 76, 75, 66, 53, 29)
somewhatInteresting <- c(64,54,52,51,41,12)
# Looked interesting, but actually not: 26

allWorthRunning <- union(veryStrongCases,union(weakCases))

library(cluster)

indicesToRun <- 1:length(PairwiseSpecificationLists)
indicesToRun <- indicesToRun[!indicesToRun%in%poorPerformers]
indicesToRun <- allWorthRunning

clusterImDir <- "/homes/tgillett/PairwiseAlignment/ClusterImages/"
metaTypes <- c("AgeClass","Archive","Class2","Class3","Region2","CorticalRegion","Layer","Region3","Species","Protocol","Strain")

#### Generate figures ####
# Density, Known classes, Kmeans, EM, and potential confounds
for (listIndex in indicesToRun){
  specifications <- PairwiseSpecificationLists[[listIndex]]
  numDimensions <- bestNumDims[[listIndex]]
  print(paste(listIndex,names(PairwiseSpecificationLists)[listIndex]))
  subsetSeqs <- c()
  groupSeqs <- list()
  groupIds <- c()
  for (specInd in 1:length(specifications)){
    specs <- specifications[[specInd]]
    groupSeqs[[specInd]] <- intersect(rownames(distMatDF),rownames(getSubset2(clusterMetaData,specs,typeMetaData=clusterMetaData)))
    subsetSeqs <- append(subsetSeqs,groupSeqs[[specInd]])
    groupIds <- append(groupIds,rep(specInd,length(groupSeqs[[specInd]])))
  }
  
  setDists <- distMatDF[subsetSeqs,subsetSeqs]
  subsetMDSFull <- cmdscale(setDists,k=numDimensions,eig=T)
  subsetMDS <- as.data.frame(subsetMDSFull$points)
  
  # Remove outliers
  if (removeOutliers){
    Q1 <- quantile(subsetMDS[,1],.25)
    Q3 <- quantile(subsetMDS[,1],.75)
    iqr <- Q3-Q1
    notOutliers <- subsetMDS[,1]>(Q1-3*iqr)&subsetMDS[,1]<(Q3+3*iqr)
    subsetSeqs <- subsetSeqs[notOutliers]
    groupSeqs[[1]] <- intersect(groupSeqs[[1]],subsetSeqs)
    groupSeqs[[2]] <- intersect(groupSeqs[[2]],subsetSeqs)
    subsetMDS <- subsetMDS[subsetSeqs,,drop=F]
  }
  
  #emModel <- Mclust(subsetMDS,G=2)
  #emModel <- Mclust(subsetMDS,G=1:6)
  
#   png(paste(clusterImDir,"Ind",listIndex,"MDS_Density.png",sep=""))
#   k <- kde2d(subsetMDS[,1],subsetMDS[,2])
#   filled.contour(k)
#   dev.off()
#   
#   png(paste(clusterImDir,"Ind",listIndex,"EM_BIC.png",sep=""))
#   plot(mclustBIC(subsetMDS,G=1:6))
#   dev.off()
# 
#   png(paste(clusterImDir,"Ind",listIndex,"EM_Clusters.png",sep=""))
#   plot(emModel,what="classification")
#   dev.off()
# 
#   png(paste(clusterImDir,"Ind",listIndex,"EM+1_Clusters.png",sep=""))
#   plot(Mclust(subsetMDS,G=emModel$G+1),what="classification")
#   dev.off()
#   
#   png(paste(clusterImDir,"Ind",listIndex,"_KnownClasses.png",sep=""))
#   plot(subsetMDS,col=groupIds,cex=.43)
#   dev.off()
# 
#   png(paste(clusterImDir,"Ind",listIndex,"_KmeansCluster.png",sep=""))
#   kmeansClust <- kmeans(subsetMDS,2)$cluster
#   score <- adjustedRandIndex(kmeansClust,groupIds)
#   plot(subsetMDS,col=kmeansClust,cex=.43,main=paste("Ind:",listIndex," Score:",score))
  # plot(subsetMDS[,1:2],col=kmeansClust,cex=.43,main=paste("Ind:",listIndex," Score:",score))
#   dev.off()

  # Combine 1st dimension with a morphometric
  metricName <- "Depth"
     png(paste(clusterImDir,"Ind",listIndex,"_KnownClasses_GA&",metricName,".png",sep=""))
     plot(subsetMDS[,1],morphometricsXformed[subsetSeqs,metricName],col=groupIds,cex=.43,xlab="First MDS Dimension",ylab=paste("Normalized",metricName),cex.lab=1.3)
     dev.off()
  #
   png(paste(clusterImDir,"Ind",listIndex,"_KmeansCluster_GA1&",metricName,".png",sep=""))
     comboDF <- data.frame(scale(subsetMDS[,1]),scale(morphometricsXformed[subsetSeqs,metricName]))
     kmeansClust <- kmeans(comboDF,2)$cluster
     plot(comboDF[,1],scale(morphometricsXformed[subsetSeqs,metricName]),col=kmeansClust,cex=.43,xlab="First MDS Dimension",ylab=paste("Normalized",metricName),cex.lab=1.3)
     dev.off()
   png(paste(clusterImDir,"Ind",listIndex,"_KmeansCluster_GAAll&",metricName,".png",sep=""))
     comboDF <- data.frame(subsetMDS[,1]/sd(subsetMDS[,1]),scale(morphometricsXformed[subsetSeqs,metricName]))
     kmeansClust <- kmeans(comboDF,2)$cluster
     plot(comboDF[,1],scale(morphometricsXformed[subsetSeqs,metricName]),col=kmeansClust,cex=.43,xlab="First MDS Dimension",ylab=paste("Normalized",metricName),cex.lab=1.3)
     dev.off()
  
  availMetaTypes <- metaTypes[!metaTypes %in% names(specifications[[1]])]
  for (metaType in availMetaTypes){
    filename <- paste(clusterImDir,"Ind",listIndex,"_",metaType,".png",sep="")
    if (!file.exists(filename)){
      png(filename)
      metaColors <- getColorsForMetaType(subsetSeqs,metaType,usedMetaData)
      plot(subsetMDS,col=metaColors,cex=.43)
      dev.off()
    }
  }
}


############## *Cluster method comparison ################

# Function to enable algorithm parallelization
runSample <- function(sampleNum,subsetMDS,distMatDF,groupSeqs,subsampleSize){
  subsample1 <- groupSeqs[[1]][sample(1:length(groupSeqs[[1]]),subsampleSize,replace=F)]
  subsample2 <- groupSeqs[[2]][sample(1:length(groupSeqs[[2]]),subsampleSize,replace=F)]
  subsampleSeqs <- c(subsample1,subsample2)
  sampleGroupIds <- c(rep(1,subsampleSize),rep(2,subsampleSize))
  
  subsetSampleDist <- as.dist(distMatDF[subsampleSeqs,subsampleSeqs])
  subsetSampleMDS <- subsetMDS[subsampleSeqs,]
  
  # Cluster data
  wardRI <- hclustAndScore(subsetSampleDist,sampleGroupIds,2,method="ward")
  ward3RI <- hclustAndScore(subsetSampleDist,sampleGroupIds,3,method="ward",permute=T)
  
  # Kmeans and Kmediods clustering
  kmeansRI <- kmeansAndScore(subsetSampleMDS,sampleGroupIds,2,adjustedRandIndex,F)
  kmeans3RI <- kmeansAndScore(subsetSampleMDS,sampleGroupIds,3,adjustedRandIndex,T)
  
  kmediodsRI <- kmediodsAndScore(subsetSampleMDS,sampleGroupIds,2,adjustedRandIndex,F)
  kmediods3RI <- kmediodsAndScore(subsetSampleMDS,sampleGroupIds,3,adjustedRandIndex,T)
  
  # Run EM clustering
  emModel <- Mclust(subsetSampleMDS,G=1:6)
  emRI <- emAndScore(subsetSampleMDS,sampleGroupIds,1:5,permute=T)
  emPlus1RI <- emAndScore(subsetSampleMDS,sampleGroupIds,emModel$G+1,permute=T)
  
  return(c(ward=wardRI,ward3=ward3RI,kmeans=kmeansRI,kmeans3=kmeans3RI,kmediods=kmediodsRI,kmediods3=kmediods3RI,em=emRI,emPlus1=emPlus1RI))
}

### Iteration over pairs to run of actual cluster scoring

removeOutliers<-T
counts <- c()
#indicesToRun <- which(counts>2)
for (listIndex in indicesToRun){
  specifications <- PairwiseSpecificationLists[[listIndex]]
  print(paste(listIndex,names(PairwiseSpecificationLists)[listIndex]))
  subsetSeqs <- c()
  groupSeqs <- list()
  groupIds <- c()
  for (specInd in 1:length(specifications)){
    specs <- specifications[[specInd]]
    groupSeqs[[specInd]] <- intersect(rownames(distMatDF),rownames(getSubset2(clusterMetaData,specs,typeMetaData=clusterMetaData)))
    subsetSeqs <- append(subsetSeqs,groupSeqs[[specInd]])
    groupIds <- append(groupIds,rep(specInd,length(groupSeqs[[specInd]])))
  }
  
  setDists <- distMatDF[subsetSeqs,subsetSeqs]
  subsetMDSFull <- cmdscale(setDists,k=numDimensions,eig=T)
  subsetMDS <- subsetMDSFull$points
  
  # Remove outliers
  if (removeOutliers){
    Q1 <- quantile(subsetMDS[,1],.25)
    Q3 <- quantile(subsetMDS[,1],.75)
    iqr <- Q3-Q1
    notOutliers <- subsetMDS[,1]>(Q1-3*iqr)&subsetMDS[,1]<(Q3+3*iqr)
    subsetMDS <- subsetMDS[notOutliers,]
    setDists <- setDists[notOutliers,notOutliers]
    groupSeqs[[1]] <- groupSeqs[[1]][groupSeqs[[1]]%in%rownames(subsetMDS)]
    groupSeqs[[2]] <- groupSeqs[[2]][groupSeqs[[2]]%in%rownames(subsetMDS)]
  }
  
  emModel <- Mclust(subsetMDS,G=1:6)

  numSeqs <- length(subsetSeqs)
  groupMinimum <- numSeqs/6
  if (emModel$G > 1){
    count <- 0
    for (i in 1:emModel$G){
      if (sum(emModel$classification==i) > groupMinimum){
        count <- count+1
      }
    }
    counts[listIndex] <- count
  }else{
    counts[listIndex] <- 1
  }
  
  wardSampleRIs <- c()
  ward3SampleRIs <- c()
  kmeansSampleRIs <- c()
  kmeans3SampleRIs <- c()
  kmediodsSampleRIs <- c()
  kmediods3SampleRIs <- c()
  emSampleRIs <- c()
  emPlus1SampleRIs <- c()
  subsampleSize <- min(min(floor(0.8*length(groupSeqs[[1]])),floor(0.8*length(groupSeqs[[2]]))),maxSubsampleSize)
  if (subsampleSize >= 20){
     subsampleSizes[listIndex] <- subsampleSize
     
     #for (i in 1:numSamples){
     #  runSample(setDists,groupSeqs,subsampleSize,numDimensions)
     #}
     numCores <- detectCores()
     sampleResults <- mclapply(1:numSamples,runSample,subsetMDS,setDists,groupSeqs,subsampleSize,numDimensions,mc.cores=numCores)
     for (i in 1:numSamples){
       wardSampleRIs[i] <- sampleResults[[i]]["ward"]
       ward3SampleRIs[i] <- sampleResults[[i]]["ward3"]
       kmeansSampleRIs[i] <- sampleResults[[i]]["kmeans"]
       kmeans3SampleRIs[i] <- sampleResults[[i]]["kmeans3"]
       kmediodsSampleRIs[i] <- sampleResults[[i]]["kmediods"]
       kmediods3SampleRIs[i] <- sampleResults[[i]]["kmediods3"]
       emSampleRIs[i] <- sampleResults[[i]]["em"]
       emPlus1SampleRIs[i] <- sampleResults[[i]]["emPlus1"]
       #emSampleRIs[i] <- sampleResults[[i]]
     }
     
     #avgRIs[listIndex] <- mean(avgSampleRIs)
     wardRIs[listIndex] <- mean(wardSampleRIs)
     ward3RIs[listIndex] <- mean(ward3SampleRIs)
     kmeansRIs[listIndex] <- mean(kmeansSampleRIs)
     kmeans3RIs[listIndex] <- mean(kmeans3SampleRIs)
     kmediodsRIs[listIndex] <- mean(kmediodsSampleRIs)
     kmediods3RIs[listIndex] <- mean(kmediods3SampleRIs)
     emRIs[listIndex] <- mean(emSampleRIs)
     emPlus1RIs[listIndex] <- mean(emPlus1SampleRIs)
 
     #avgRISDs[listIndex] <- sd(avgSampleRIs)
     #wardRISDs[listIndex] <- sd(wardSampleRIs)
     #kmeansRISDs[listIndex] <- sd(kmeansSampleRIs)
     #emRISDs[listIndex] <- sd(emSampleRIs)
     
     # Statistical comparison to determine which if any method if better for this comparison
#      avgWardComp <- 0
#      avgKmeansComp <- 0
#      wardKmeansComp <- 0
#      emWardComp <- 0
#      emKmeansComp <- 0
#      if (emRIs[listIndex] > .1 || wardRIs[listIndex] > .1 || kmeansRIs[listIndex] > .1){
#        if (wilcox.test(emSampleRIs,wardSampleRIs,paired=T)$p.value < .001){
#          if (emRIs[listIndex] > wardRIs[listIndex]){
#            emWardComp <- 1
#          }else{
#            emWardComp <- -1
#          }
#        }
#        if (wilcox.test(emSampleRIs,kmeansSampleRIs,paired=T)$p.value < .001){
#          if (emRIs[listIndex] > kmeansRIs[listIndex]){
#            emKmeansComp <- 1
#          }else{
#            emKmeansComp <- -1
#          }
#        }
#        if (wilcox.test(wardSampleRIs,kmeansSampleRIs,paired=T)$p.value < .001){
#          if (wardRIs[listIndex] > kmeansRIs[listIndex]){
#            wardKmeansComp <- 1
#          }else{
#            wardKmeansComp <- -1
#          }
#        }
#  
#        # Cases in which avg is best
#       if ((emWardComp==1&&emKmeansComp!=-1)||(emKmeansComp==1&&emWardComp!=-1)){
#         emBest[listIndex] <- 1
#       }
#  
#       # Cases in which ward is best
#       if ((emWardComp==-1&&wardKmeansComp!=-1)||(wardKmeansComp==1&&emWardComp!=1)){
#         wardBest[listIndex] <- 1
#       }
# 
#       # Cases in which kmeans is best
#       if ((emKmeansComp==-1&&wardKmeansComp!=1)||(wardKmeansComp==-1&&emKmeansComp!=1)){
#         kmeansBest[listIndex] <- 1
#       }
#     }
  }
}

# Save results to file
validPairs <- subsampleSizes > 0
RISampleMeanResults <- data.frame(Index=1:length(subsampleSizes),subsampleSize=subsampleSizes,em=emRIs,emPlus1=emPlus1RIs,ward=wardRIs,ward3=ward3RIs,kmeans=kmeansRIs,kmeans3=kmeans3RIs,kmediods=kmediodsRIs,kmediods3=kmediods3RIs,name=names(PairwiseSpecificationLists))
RISampleMeanResults <- RISampleMeanResults[validPairs,]

RISampleMeanResults_NormAlignDistM20NWDef <- RISampleMeanResults
write.csv(RISampleMeanResults_NormAlignDistM20NWDef,paste("PairClust_NormAlignDist_Min20",randStr,travOrder,".",scoreParamStr,"_Samp",numSamples,"_TreeNW_NoOutliers.csv",sep=""),row.names=F)

ks.test(mahalanobis(subsetMDS[groupSeqs[[1]],],emModel$parameters$mean[,1],emModel$parameters$variance$sigma[,,1]),pchisq,df=ncol(subsetMDS))
ks.test(mahalanobis(subsetMDS[groupSeqs[[2]],],emModel$parameters$mean[,2],emModel$parameters$variance$sigma[,,2]),pchisq,df=ncol(subsetMDS))

RISampleMeanResults_NormAlignDistM20NWDef <- RISampleMeanResults
write.csv(RISampleMeanResults_NormAlignDistM20NWDef,paste("PairClust_NormAlignDist_Min20",randStr,travOrder,".",scoreParamStr,"_Samp",numSamples,"_TreeNW.csv",sep=""),row.names=F)

RISampleMeanResults_SpearStretch2M20NWDef <- RISampleMeanResults
write.csv(RISampleMeanResults_SpearStretch2M20NWDef,paste("PairClust_SpearmanStrSq_Min20",randStr,travOrder,".",scoreParamStr,"_Samp",numSamples,"_NWDefault.csv",sep=""),row.names=F)

RISampleMeanResults_LISpearStretch2M20NWDef <- RISampleMeanResults
write.csv(RISampleMeanResults_LISpearStretch2M20NWDef,paste("PairClust_LI-SpearmanStrSq_Min20",randStr,travOrder,".",scoreParamStr,"_Samp",numSamples,"_NWDefault.csv",sep=""),row.names=F)

RISampleMeanResults_jcDistM20NWDef <- RISampleMeanResults
write.csv(RISampleMeanResults_jcDistM20NWDef,paste("PairClust_jcDist_Min20",randStr,travOrder,".",scoreParamStr,"_Samp",numSamples,"_NWDefault.csv",sep=""),row.names=F)

RISampleMeanResults_SpearStretch1M20NWDef <- RISampleMeanResults
write.csv(RISampleMeanResults_SpearStretch1M20NWDef,paste("PairClust_SpearmanStr_Min20",randStr,travOrder,".",scoreParamStr,"_Samp",numSamples,"_NWDefault.csv",sep=""),row.names=F)

RISampleMeanResults_SpearM20NWDef <- RISampleMeanResults
write.csv(RISampleMeanResults_SpearM20NWDef,paste("PairClust_Spearman_Min20",randStr,travOrder,".",scoreParamStr,"_Samp",numSamples,"_NWDefault.csv",sep=""),row.names=F)

bestResults <- subset(RISampleMeanResults,subsampleSize>=25 & avg>.2 & ward>.2)

### Looking at subsample-based Rand Index distributions to determine what
#    statistical test to use
listIndex <- 113
specifications <- PairwiseSpecificationLists[[listIndex]]
print(paste(listIndex,names(PairwiseSpecificationLists)[listIndex]))
subsetSeqs <- c()
groupSeqs <- list()
groupIds <- c()
for (specInd in 1:length(specifications)){
  specs <- specifications[[specInd]]
  groupSeqs[[specInd]] <- intersect(rownames(distMatDF),rownames(getSubset2(clusterMetaData,specs,typeMetaData=clusterMetaData)))
  subsetSeqs <- append(subsetSeqs,groupSeqs[[specInd]])
  groupIds <- append(groupIds,rep(specInd,length(groupSeqs[[specInd]])))
}

setDists <- distMatDF[subsetSeqs,subsetSeqs]
#subsetMDSFull <- cmdscale(setDists,k=numDimensions,eig=T)
#subsetMDS <- subsetMDSFull$points
subsetMDS <- apicalMDS[subsetSeqs,]

# Remove outliers
Q1 <- quantile(subsetMDS[,1],.25)
Q3 <- quantile(subsetMDS[,1],.75)
iqr <- Q3-Q1
notOutliers <- subsetMDS[,1]>(Q1-3*iqr)&subsetMDS[,1]<(Q3+3*iqr)
subsetMDS <- subsetMDS[notOutliers,]
setDists <- setDists[notOutliers,notOutliers]
groupSeqs[[1]] <- groupSeqs[[1]][groupSeqs[[1]]%in%rownames(subsetMDS)]
groupSeqs[[2]] <- groupSeqs[[2]][groupSeqs[[2]]%in%rownames(subsetMDS)]

sampleResults <- mclapply(1:numSamples,runSample,subsetMDS,setDists,groupSeqs,subsampleSize,numDimensions,mc.cores=numCores)


#### *Comparing clustering results given num MDS dimensions ######

source(paste(baseCodeDir,"BMDS/bmds.Open.R",sep=""))

removeOutliers<-T
counts <- c()
bestNumDims <- list()
#indicesToRun <- which(counts>2)
####
# Produces list of optimal numDimensions for a each pair
# Also can produce graphs of clustering performance by number of dimensions
for (listIndex in indicesToRun){
  specifications <- PairwiseSpecificationLists[[listIndex]]
  print(paste(listIndex,names(PairwiseSpecificationLists)[listIndex]))
  
  subsetSeqsList <- loadSubsetSeqs(specifications,clusterMetaData,candidates=rownames(distMatDF))
  subsetSeqs <- subsetSeqsList$Sequences
  groupSeqs <- subsetSeqsList$SeqsByGroup
  groupIds <- subsetSeqsList$GroupIds
  
  # Remove outliers (first run basic MDS)
  setDists <- distMatDF[subsetSeqs,subsetSeqs]
  subsetMDSFull <- isoMDS(setDists,k=3)
  subsetMDS <- subsetMDSFull$points
  if (removeOutliers){
    Q1 <- quantile(subsetMDS[,1],.25)
    Q3 <- quantile(subsetMDS[,1],.75)
    iqr <- Q3-Q1
    notOutliers <- subsetMDS[,1]>(Q1-3*iqr)&subsetMDS[,1]<(Q3+3*iqr)
    subsetMDS <- subsetMDS[notOutliers,]
    setDists <- setDists[notOutliers,notOutliers]
    groupSeqs[[1]] <- groupSeqs[[1]][groupSeqs[[1]]%in%rownames(subsetMDS)]
    groupSeqs[[2]] <- groupSeqs[[2]][groupSeqs[[2]]%in%rownames(subsetMDS)]
    subsetSeqs <- c(groupSeqs[[1]],groupSeqs[[2]])
  }
  
  subsampleSize <- min(min(floor(0.8*length(groupSeqs[[1]])),floor(0.8*length(groupSeqs[[2]]))),maxSubsampleSize)
  
  #imFilename <- paste("Cluster",listIndex,"_CompareMDSDims.png",sep="")
  imFilename <- NA
  testResults <- testNumMDSDims(1:12,distMatDF[subsetSeqs,subsetSeqs],groupSeqs,imageFilename=imFilename)
  bestNumDims[[listIndex]] <- which.max(testResults$mean)
}

# For a given data subset, calculates MDS for each numDimensions given, calculates
# clustering performance given subsamples, and returns the results (and can produce a plot)
testNumMDSDims <- function(numDimensions,dists,groupSeqs,plotData=F,imageFilename=NA){
  means <- c()
  sds <- c()
  require(MASS)
  
  for (i in 1:length(numDimensions)){
    numDims <- numDimensions[i]
    mdsData <- isoMDS(dists,k=numDims)
    #bmds.res <- bmds(dists,maxp=12,n.iter=25000,n.burnin=5000)
    subsetMDS <- mdsData$points

    subsampleSize <- min(min(floor(0.8*length(groupSeqs[[1]])),floor(0.8*length(groupSeqs[[2]]))),maxSubsampleSize)
    
    numCores <- detectCores()
    sampleResults <- mclapply(1:numSamples,sampleAndKmeans,subsetMDS,groupSeqs,subsampleSize,mc.cores=numCores)
    for (i in 1:numSamples){
      kmeansSampleRIs[i] <- sampleResults[[i]]
    }
    means <- append(means,mean(kmeansSampleRIs))
    sds <- append(sds,sd(kmeansSampleRIs))
  }
  
  if (!is.na(imageFilename)){
    png(imageFilename)
    plot(numDimensions,means)
    error.bar(numDimensions,means,sds/sqrt(numSamples))
    dev.off()    
  }else if (plotData){
    plot(numDimensions,means)
    error.bar(numDimensions,means,sds/sqrt(numSamples))
  }
  return(list(means=means,sds=sds))
}

sampleAndKmeans <- function(sampleNum,subsetMDS,groupSeqs,subsampleSize){
  subsample1 <- groupSeqs[[1]][sample(1:length(groupSeqs[[1]]),subsampleSize,replace=F)]
  subsample2 <- groupSeqs[[2]][sample(1:length(groupSeqs[[2]]),subsampleSize,replace=F)]
  subsampleSeqs <- c(subsample1,subsample2)
  sampleGroupIds <- c(rep(1,subsampleSize),rep(2,subsampleSize))
  subsetSampleMDS <- subsetMDS[subsampleSeqs,]
  kmeansRI <- kmeansAndScore(subsetSampleMDS,sampleGroupIds,2,AdjustedRandIndex,F)
  return(kmeansRI)
}


#### *Statistical anlysis wrt random of all class comparisons ####

removeOutliers<-T
counts <- c()

####
# Produces list of optimal numDimensions for a each pair
# Also can produce graphs of clustering performance by number of dimensions
means <- c()
SEMs <- c()
pVals <- c()
indicesToRun <- 19:length(PairwiseSpecificationLists)
for (listIndex in indicesToRun){
  specifications <- PairwiseSpecificationLists[[listIndex]]
  print(paste(listIndex,names(PairwiseSpecificationLists)[listIndex]))
  
  subsetSeqsList <- loadSubsetSeqs(specifications,clusterMetaData,candidates=rownames(distMatDF))
  subsetSeqs <- subsetSeqsList$Sequences
  groupSeqs <- subsetSeqsList$SeqsByGroup
  groupIds <- subsetSeqsList$GroupIds
  
  # Remove outliers (first run basic MDS)
  setDists <- distMatDF[subsetSeqs,subsetSeqs]
  subsetMDSFull <- isoMDS(setDists,k=3)
  subsetMDS <- subsetMDSFull$points
  if (removeOutliers){
    Q1 <- quantile(subsetMDS[,1],.25)
    Q3 <- quantile(subsetMDS[,1],.75)
    iqr <- Q3-Q1
    notOutliers <- subsetMDS[,1]>(Q1-3*iqr)&subsetMDS[,1]<(Q3+3*iqr)
    subsetMDS <- subsetMDS[notOutliers,]
    groupIds <- groupIds[notOutliers]
    setDists <- setDists[notOutliers,notOutliers]
    groupSeqs[[1]] <- groupSeqs[[1]][groupSeqs[[1]]%in%rownames(subsetMDS)]
    groupSeqs[[2]] <- groupSeqs[[2]][groupSeqs[[2]]%in%rownames(subsetMDS)]
    subsetSeqs <- c(groupSeqs[[1]],groupSeqs[[2]])
  }
  
  subsampleSize <- min(min(floor(0.8*length(groupSeqs[[1]])),floor(0.8*length(groupSeqs[[2]]))),maxSubsampleSize)
  if (subsampleSize >= 20){
    sampleSizes <- rep(subsampleSize,2)
    
    realRIs <- as.numeric(sampleClusterRIs(subsetMDS,groupIds,sampleSizes=sampleSizes,numSamples=40,randomizeGroups=F))
    randomRIs <- as.numeric(sampleClusterRIs(subsetMDS,groupIds,sampleSizes=sampleSizes,numSamples=40,randomizeGroups=T))
    
    means[listIndex] <- mean(realRIs)
    SEMs[listIndex] <- sem(realRIs)
    pVals[listIndex] <- wilcox.test(realRIs,randomRIs,alternative="greater")$p.value
  }
}

newP <- c()
for (listIndex in indicesToRun){
  if (!is.null(pVals[[listIndex]])){
    newP[listIndex] <- pVals[[listIndex]]
  }
}


sampleAndKmeans <- function(sampleNum,subsetMDS,groupSeqs,subsampleSize){
  subsample1 <- groupSeqs[[1]][sample(1:length(groupSeqs[[1]]),subsampleSize,replace=F)]
  subsample2 <- groupSeqs[[2]][sample(1:length(groupSeqs[[2]]),subsampleSize,replace=F)]
  subsampleSeqs <- c(subsample1,subsample2)
  sampleGroupIds <- c(rep(1,subsampleSize),rep(2,subsampleSize))
  subsetSampleMDS <- subsetMDS[subsampleSeqs,]
  kmeansRI <- kmeansAndScore(subsetSampleMDS,sampleGroupIds,2,AdjustedRandIndex,F)
  return(kmeansRI)
}


######## *Comparing global alignment and morphometrics ##########

# Morphometrics loaded in ClusterAnalysis.r, use morphometricsXformed

require(MASS)
require(parallel)
require(mclust)

indicesToRun <- 1:length(NewPairwiseSpecifications)

numFeatures <- ncol(morphometricsXformed)
numResultColumns <- 2*numFeatures+3
metricNames <- colnames(morphometricsXformed)
means <- matrix(NA,ncol=numResultColumns,nrow=length(indicesToRun))
means[,1] <- indicesToRun
colnames(means) <- c("Index","Name","GlobalAlignment",metricNames,paste("GA",metricNames,sep="&"))
SEMs <- matrix(NA,ncol=numResultColumns,nrow=length(indicesToRun))
SEMs[,1] <- indicesToRun
colnames(SEMs) <- c("Index","Name","GlobalAlignment",metricNames,paste("GA",metricNames,sep="&"))
mergePVals <- matrix(NA,ncol=numFeatures+2,nrow=length(indicesToRun))
mergePVals[,1] <- indicesToRun
colnames(mergePVals) <- c("Index","Name",metricNames)
gaVsMetricPVals <- matrix(NA,ncol=numFeatures+2,nrow=length(indicesToRun))
gaVsMetricPVals[,1] <- indicesToRun
colnames(gaVsMetricPVals) <- c("Index","Name",metricNames)

#means[,2] <- names(PairwiseSpecificationLists)[indicesToRun]
#SEMs[,2] <- names(PairwiseSpecificationLists)[indicesToRun]
#mergePVals[,2] <- names(PairwiseSpecificationLists)[indicesToRun]
#gaVsMetricPVals[,2] <- names(PairwiseSpecificationLists)[indicesToRun]
means[,2] <- names(NewPairwiseSpecifications)[indicesToRun]
SEMs[,2] <- names(NewPairwiseSpecifications)[indicesToRun]
mergePVals[,2] <- names(NewPairwiseSpecifications)[indicesToRun]
gaVsMetricPVals[,2] <- names(NewPairwiseSpecifications)[indicesToRun]

maxGroupSize <- 80

#for (listIndex in indicesToRun){
proportion<-0.8
for (i in 1:length(indicesToRun)){
  listIndex <- indicesToRun[i]
  specifications <- NewPairwiseSpecifications[[listIndex]]
  print(paste(listIndex,names(NewPairwiseSpecifications)[listIndex]))
  #specifications <- PairwiseSpecificationLists[[listIndex]]
  #print(paste(listIndex,names(PairwiseSpecificationLists)[listIndex]))
  givenArborType <- specifications[[1]]$ArborType
  if (givenArborType == "ApicalDendrite"){
    subsetMDS <- apicalMDS
  }else if (givenArborType == "Axon"){
    subsetMDS <- axonMDS
  }else{
    subsetMDS <- dendriteMDS
  }
  
  #numDims <- bestNumDims[[listIndex]]
  #RIs <- runMergeSampleWrapper(specifications,usedMetaData,distMatDF,morphometricsXformed,numSamples,numDims)

  # Load group/sequences
  #subsetSeqsList <- loadSubsetSeqs(specifications,usedMetaData,candidates=rownames(distMatDF))
  subsetSeqsList <- loadSubsetSeqs(specifications,usedMetaData,candidates=rownames(subsetMDS))
  subsetSeqs <- subsetSeqsList$Sequences
  groupSeqs <- subsetSeqsList$SeqsByGroup
  groupIds <- subsetSeqsList$GroupIds
  groupSizes <- aggregate(groupIds,by=list(groupIds),length)[unique(groupIds),2]
  
  # Take subset of alignment distance matrix and get MDS
  #subsetMDS <- isoMDS(distMatDF[subsetSeqs,subsetSeqs],k=bestNumDims[[listIndex]])
  # Standardize each dimension by the standard deviation of the first dimension
  #subsetMDS <- subsetMDS/sd(subsetMDS[,1])
  
  # Generate random samples
  if (sampleProportionally){
    sampleSizes <- getSampleSizesProportionally(groupSizes,proportion=proportion)
  }else{
    sampleSizes <- getSampleSizesEvenly(groupSizes,proportionOfSmallest=proportion,maxSize=maxGroupSize)
  }
  subsamples <- generateSubsamples(groupIds,sampleSizes,numSamples)

  # Set up RI matrix
  RIs <- matrix(0,nrow=numSamples,ncol=2*ncol(morphometrics)+1)
  rownames(RIs) <- 1:numSamples
  colnames(RIs) <- c("GlobalAlign",colnames(morphometrics),paste("GA&",colnames(morphometrics),sep=""))
  
  # Cluster alignment and get RIs
  alignmentRIs <- as.numeric(sampleClusterRIs(subsetMDS[subsetSeqs,],groupIds,sampleSizes,subsamples=subsamples,clusterMethod="kmeans",randomizeGroups=F))
  alignmentRandomRIs <- as.numeric(sampleClusterRIs(subsetMDS[subsetSeqs,],groupIds,sampleSizes,subsamples=subsamples,clusterMethod="kmeans",randomizeGroups=T))
  RIs[,1] <- alignmentRIs

  numFeatures <- ncol(morphometricsXformed)
  morphometricsSubset <- morphometricsXformed[subsetSeqs,]
  for (featInd in 1:numFeatures){
    RIs[,1+featInd] <- 
      as.numeric(sampleClusterRIs(morphometricsSubset[,featInd,drop=F],groupIds,sampleSizes,subsamples=subsamples,clusterMethod="kmeans",randomizeGroups=F))
    RIs[,1+numFeatures+featInd] <- 
      as.numeric(sampleClusterRIs(cbind(subsetMDS[subsetSeqs,],morphometricsSubset[,featInd,drop=F]),groupIds,sampleSizes,subsamples=subsamples,clusterMethod="kmeans",randomizeGroups=F))
  }
    
  # Loop through features to produce mean, SEM, and p-value results
  means[i,3:numResultColumns] <- apply(RIs,2,mean)
  SEMs[i,3:numResultColumns] <- apply(RIs,2,sem)
  for (featInd in 1:numFeatures){
    gaVsMetricPVals[i,2+featInd] <- wilcox.test(RIs[,1],RIs[,1+featInd],paired=T,alternative="greater")$p.value
    metricIsBetter <- means[i,3+featInd] > means[i,3]
    if (metricIsBetter){
      mergePVals[i,2+featInd] <- wilcox.test(RIs[,1+numFeatures+featInd],RIs[,1+featInd],paired=T,alternative="greater")$p.value
    }else{
      mergePVals[i,2+featInd] <- wilcox.test(RIs[,1+numFeatures+featInd],RIs[,1],paired=T,alternative="greater")$p.value
    }
  }
}

# Save results to file
if (sampleProportionally){
  # For proportionally sampled results
  write.csv(means,paste(clusterDir,"kmeansClusterRI_proportion_Means.csv",sep=""),row.names=F)
  write.csv(SEMs,paste(clusterDir,"kmeansClusterRI_proportion_SEMs.csv",sep=""),row.names=F)
  write.csv(gaVsMetricPVals,paste(clusterDir,"kmeansClusterRI_GA-vs-Metric_proportion_pVals.csv",sep=""),row.names=F)
  write.csv(mergePVals,paste(clusterDir,"kmeansClusterRI_Merge_proportion_pVals.csv",sep=""),row.names=F)
}else{
  # Evenly sample results
  write.csv(means,paste(clusterDir,"NewKmeansClusterRI_Means.csv",sep=""),row.names=F)
  write.csv(SEMs,paste(clusterDir,"NewKmeansClusterRI_SEMs.csv",sep=""),row.names=F)
  write.csv(gaVsMetricPVals,paste(clusterDir,"NewKmeansClusterRI_GA-vs-Metric_pVals.csv",sep=""),row.names=F)
  write.csv(mergePVals,paste(clusterDir,"NewKmeansClusterRI_Merge_pVals.csv",sep=""),row.names=F)
}


runSubsample <- function(subsamplePair,mydata){
  subsample1 <- subsamplePair[[1]]
  subsample2 <- subsamplePair[[2]]
  subsampleSeqs <- c(subsample1,subsample2)
  sampleGroupIds <- c(rep(1,length(subsample1)),rep(2,length(subsample2)))
  require(mclust)
  return(kmeansAndScore(mydata[subsampleSeqs,],sampleGroupIds,2,adjustedRandIndex,F))
}

gaMergeRIMat113 <- runMergeSampleWrapper(PairwiseSpecificationLists[[113]],distMatDF,morphometricsXformed,100,3)


runMergeSampleWrapper <- function(specifications,clusterMetaData,distMatDF,morphometrics,numSamples,numDimensions){
  subsetSeqs <- c()
  groupSeqs <- list()
  groupIds <- c()
  for (specInd in 1:length(specifications)){
    specs <- specifications[[specInd]]
    groupSeqs[[specInd]] <- intersect(rownames(distMatDF),rownames(getSubset2(clusterMetaData,specs,typeMetaData=clusterMetaData)))
    subsetSeqs <- append(subsetSeqs,groupSeqs[[specInd]])
  }
  subsampleSize <- min(min(floor(0.8*length(groupSeqs[[1]])),floor(0.8*length(groupSeqs[[2]]))),maxSubsampleSize)
  
  require(MASS)
  
  # Subset distance matrix
  setDists <- distMatDF[subsetSeqs,subsetSeqs]
  # Multidimensional scale
  subsetMDS <- isoMDS(setDists,k=numDimensions)$points
  # Scale dimensions so that 1st dimension standard deviation is 1 (to match Morphometrics, which should be standardized)
  subsetMDS <- subsetMDS/sd(subsetMDS[,1])

  # Standardize Morphometrics just in case they aren't already
  morphometrics <- scale(morphometrics[subsetSeqs,])
  
  #RIs <- runMergeSamples(subsetMDS,morphometrics,groupSeqs,numSamples,subsampleSize,numDimensions)
  subsamplePairs <- mclapply(1:numSamples,function(sampleNum,groupSeqs,subsampleSize){
    l1 <- groupSeqs[[1]][sample(1:length(groupSeqs[[1]]),subsampleSize,replace=F)]
    l2 <- groupSeqs[[2]][sample(1:length(groupSeqs[[2]]),subsampleSize,replace=F)]
    return(list(l1,l2))
  },groupSeqs,subsampleSize)
  
  RIs <- matrix(0,nrow=numSamples,ncol=2*ncol(morphometrics)+1)
  rownames(RIs) <- 1:numSamples
  colnames(RIs) <- c("GlobalAlign",colnames(morphometrics),paste("GA&",colnames(morphometrics),sep=""))
  
  require(parallel)
  
  # Global alignment MDS
  globalAlignRIs <- mclapply(subsamplePairs,runSubsample,subsetMDS)
  #kmeansAndScore(subsetSampleMDS,sampleGroupIds,2,adjustedRandIndex,F)
  for (j in 1:numSamples){
    RIs[j,1] <- globalAlignRIs[[j]]
  }
  
  for (i in 1:ncol(morphometrics)){
    # Morphometrics
    morphomRIs <- mclapply(subsamplePairs,runSubsample,morphometrics[,i,drop=F])
    
    # Combination
    mergedRIs <- mclapply(subsamplePairs,runSubsample,cbind(subsetMDS,morphometrics[,i]))
    
    for (j in 1:numSamples){
      RIs[j,1+i] <- morphomRIs[[j]]
      RIs[j,1+ncol(morphometrics)+i] <- mergedRIs[[j]]
    }
  }
  
  return(RIs)
}

# Function to enable algorithm parallelization - generates list of sample RIs for various inputs
runMergeSamples <- function(subsetMDS,morphometrics,groupSeqs,numSamples,subsampleSize,numDimensions){
  subsamplePairs <- mclapply(1:numSamples,function(sampleNum,groupSeqs,subsampleSize){
      l1 <- groupSeqs[[1]][sample(1:length(groupSeqs[[1]]),subsampleSize,replace=F)]
      l2 <- groupSeqs[[2]][sample(1:length(groupSeqs[[2]]),subsampleSize,replace=F)]
      return(list(l1,l2))
    },groupSeqs,subsampleSize)
  
  RIs <- matrix(0,nrow=numSamples,ncol=2*ncol(morphometrics)+1)
  rownames(RIs) <- 1:numSamples
  colnames(RIs) <- c("GlobalAlign",colnames(morphometrics),paste("GA&",colnames(morphometrics),sep=""))

  require(parallel)
  
  # Global alignment MDS
  globalAlignRIs <- mclapply(subsamplePairs,runSubsample,subsetMDS)
    #kmeansAndScore(subsetSampleMDS,sampleGroupIds,2,adjustedRandIndex,F)
  for (j in 1:numSamples){
    RIs[j,1] <- globalAlignRIs[[j]]
  }

  for (i in 1:ncol(morphometrics)){
    # Morphometrics
    morphomRIs <- mclapply(subsamplePairs,runSubsample,morphometrics[,i,drop=F])
    
    # Combination
    mergedRIs <- mclapply(subsamplePairs,runSubsample,cbind(subsetMDS,morphometrics[,i]))
    
    for (j in 1:numSamples){
      RIs[j,1+i] <- morphomRIs[[j]]
      RIs[j,1+ncol(morphometrics)+i] <- mergedRIs[[j]]
    }
  }
  
  return(RIs)
}


#### *Comparing global alignment and morphometrics explicitly ####

predictionSuccess <- list()
allKlarResults <- list()
## Try with klaR:stepclass (10/28/13) - classification models by adding/removing features
# Get data and sequences from selected groups
#specifications <- NewPairwiseSpecifications[[]]
# Dendrite/Rodent/L23/Pyramidal vs Dendrite/Rodent/Neocortex/Interneuron
specifications <- list(list(ArborType="Dendrite",Order="Rodent",Layer="Layer 2/3",Class2="Pyramidal cell"),
                       list(ArborType="Dendrite",Order="Rodent",BrainRegion="Neocortex",Class1="Interneuron"))
# Dendrite/Primate/Neocortex/Pyramidal vs Dendrite/Rodent/Neocortex/Pyramidal
specifications <- list(list(ArborType="Dendrite",Order="Primate",BrainRegion="Neocortex",Class2="Pyramidal cell"),
                       list(ArborType="Dendrite",Order="Rodent",BrainRegion="Neocortex",Class2="Pyramidal cell"))

arborType <- "Axon"
specifications <- AllAxonSpecifications

arborType <- "Dendrite"
specifications <- AllDendriteSpecifications

arborType <- "Apical"
specifications <- AllApicalSpecificationsByLayer

subsetMDS <- arborMDS[[arborType]]
seqs <- intersect(rownames(subsetMDS),rownames(morphometricsXformed))

subsetMDS <- subsetMDS[seqs,]
subsetMetrics <- morphometricsXformed[seqs,-c(3,7,9,10,15)] # Remove Branches, Diameter, Surface, Volume, Fragmentation

# Use specifications as the basis for "ground truth" classification
subsetSeqsList <- loadSubsetSeqs(specifications,usedMetaData,candidates=seqs)
seqs <- subsetSeqsList$Sequences

# Alternatively, use the classifications created by Mclust
featEM <- emResults[[paste("STL",arborType,"0 G3E1Identity",numDimsInitList[[arborType]],numDimsList[[arborType]])]]
seqs <- names(featEM$classification)

klarData <- cbind(subsetMDS,subsetMetrics)[seqs,]
klarTopo <- cbind(subsetMDS,subsetMetrics[,c("Stems","Bifurcations","BranchOrder","PartitionAsymmetry")])[subsetSeqsList$Sequences,]

if (arborType != "Dendrite"){
  # Remove stems
  klarData <- klarData[,colnames(klarData)!="Stems"]
  klarTopo <- klarTopo[,colnames(klarTopo)!="Stems"]
}

klarRes <- stepclass(x=klarData,grouping=subsetSeqsList$GroupIds,method="lda",direction="both",improvement=.01)
klarTopoRes <- stepclass(x=klarTopo,grouping=subsetSeqsList$GroupIds,method="lda",direction="both",improvement=.01)
klarResSA <- stepclass(x=subsetMDS[subsetSeqsList$Sequences,],grouping=subsetSeqsList$GroupIds,method="lda",direction="both",improvement=.01)

allKlarResults[[paste(arborType,"AllFeats")]] <- klarRes
allKlarResults[[paste(arborType,"TopoFeats")]] <- klarTopoRes
allKlarResults[[paste(arborType,"SASpace")]] <- klarResSA

predictionSuccess[[paste(arborType,"AllFeats")]] <- klarRes$result.pm[1]
predictionSuccess[[paste(arborType,"TopoFeats")]] <- klarTopoRes$result.pm[1]
predictionSuccess[[paste(arborType,"SASpace")]] <- klarResSA$result.pm[1]

ldsRes <- lda(klarTopoRes$formula,klarTopo)
prediction <- predict(ldsRes,klarData)

# See all classification performances
sapply(allKlarResults,function(x){print(x$result.pm[1])})
# See all features selected
for (case in names(allKlarResults)){
  print(case)
  print(allKlarResults[[case]]$result.pm[1])
  print(allKlarResults[[case]]$model)
}
sapply(allKlarResults,function(x){print(x$model)})

x11()
plot(klarData$V1,klarData$V2,col=subsetSeqsList$GroupIds,pch=as.numeric(prediction$class))

chars <- c("A","B","C","D","E","F")
logdata <- log(Morphometrics[rownames(klarData),"Bifurcations"])
numTics <- 5
axpos <- seq(from=min(logdata),to=max(logdata),by=diff(range(logdata))/numTics)
axval <- exp(axpos)
plot(klarData$V1,logdata,col=mycolors[subsetSeqsList$GroupIds],pch=chars[as.numeric(prediction$class)],xlab="Alignment Dimension 1",ylab="#Bifurcations (log scale)",yaxt="n")
axis(side=2,at=axpos,labels=round(axval,0))

plot(klarData$V4,Morphometrics[rownames(klarData),"PartitionAsymmetry"],col=mycolors[subsetSeqsList$GroupIds],pch=chars[as.numeric(prediction$class)],xlab="Alignment Dimension 4",ylab="Partition Asymmetry")
plot(klarData$V4,klarData$V1,col=mycolors[subsetSeqsList$GroupIds],pch=chars[as.numeric(prediction$class)],xlab="Alignment Dimension 4",ylab="Alignment Dimension 1")


rawMetrics <- Morphometrics[seqs,]

classCols <- c("black","red","blue","orange","green","purple","gray","cyan","magenta")
labs <- c("A","B","C","D","E","F","G","H")
labs <- as.character(1:max(subsetSeqsList$GroupIds))
plot(klarData$V1,log(rawMetrics$Bifurcations),col="white")
text(klarData$V1,log(rawMetrics$Bifurcations),col=classCols[subsetSeqsList$GroupIds],labels=labs[as.numeric(prediction$class)])
#text(klarData$V1,klarData$V2,col=classCols[as.numeric(prediction$class)],labels=labs[subsetSeqsList$GroupIds])
legend(-2.56,7.065,legend=names(specifications),col=classCols[1:max(subsetSeqsList$GroupIds)],pch=1,bg="white",cex=.8)

subsetMetrics <- scale(subsetMetrics)
subsetMDS <- subsetMDS/sd(subsetMDS[,1])

klarDatPCs <- prcomp(klarData)

# Older method

listIndex <- 1
specifications <- NewPairwiseSpecifications[[listIndex]]
print(paste(listIndex,names(NewPairwiseSpecifications)[listIndex]))
givenArborType <- specifications[[1]]$ArborType
if (givenArborType == "ApicalDendrite"){
  subsetMDS <- apicalMDS
}else if (givenArborType == "Axon"){
  subsetMDS <- axonMDS
}else{
  subsetMDS <- dendriteMDS
}


#numDims <- bestNumDims[[listIndex]]
#RIs <- runMergeSampleWrapper(specifications,usedMetaData,distMatDF,morphometricsXformed,numSamples,numDims)
candidates <- intersect(rownames(subsetMDS),rownames(Morphometrics))

# Loading group/sequence data explicitly
subsetSeqsList <- loadSubsetSeqs(specifications,usedMetaData,candidates=candidates)

# Load group/sequences
subsetSeqs <- subsetSeqsList$Sequences
groupSeqs <- subsetSeqsList$SeqsByGroup
groupIds <- subsetSeqsList$GroupIds
groupSizes <- aggregate(groupIds,by=list(groupIds),length)[unique(groupIds),2]

# Take subset of alignment distance matrix and get MDS
subsetMDS <- subsetMDS[subsetSeqs,]
# Standardize each dimension by the standard deviation of the first dimension
subsetMDS <- subsetMDS/sd(subsetMDS[,1])
# Standardize morphometrics given current subset as well
morphometricsSubset <- scale(morphometricsXformed[subsetSeqs,])
numFeatures <- ncol(morphometricsSubset)

# Generate random samples
if (sampleProportionally){
  sampleSizes <- getSampleSizesProportionally(groupSizes,proportion=proportion)
}else{
  #sampleSizes <- getSampleSizesEvenly(groupSizes,proportionOfSmallest=proportion)
  sampleSizes <- getSampleSizesEvenly(groupSizes,proportionOfSmallest=1)
}
subsamples <- generateSubsamples(groupIds,sampleSizes,numSamples)
subsamples <- list(1:length(groupIds))

# Set up RI matrix
RIs <- matrix(0,nrow=numSamples,ncol=2*numFeatures+1)
rownames(RIs) <- 1:numSamples
colnames(RIs) <- c("GlobalAlign",colnames(morphometricsSubset),paste("GA&",colnames(morphometricsSubset),sep=""))

clusterColors <- c("black","blue")

clusterGA <- kmeans(subsetMDS[subsamples[[1]],],centers=2)
gaRI <- AdjustedRandIndex(clusterGA$cluster,groupIds[subsamples[[1]]])
x11(height=5,width=5)
dims <- 1:2
plot(subsetMDS[subsamples[[1]],dims],col="white",xlab="Global Alignment Dimension 1",ylab="Global Alignment Dimension 2",main="Cluster on Global Alignment")
text(subsetMDS[subsamples[[1]],dims],col=clusterColors[groupIds[subsamples[[1]]]],labels=as.character(clusterGA$cluster),font=2)
for (i in 1:length(clusterCombo$cluster)){
  if (clusterCombo$cluster[i]==clusterGA$cluster[i]){
    ellipse(as.numeric(subsetMDS[i,1:2]),radius=.1,shape=matrix(c(1,0,0,1),2.1,2.1),center.pch=F,font=1)
  }
}
x11(height=5,width=5)
plot(subsetMDS[subsamples[[1]],1],morphometricsSubset[subsamples[[1]],bestMetric],col="white",xlab="Global Alignment Dimension 1",ylab=paste("Normalized Branch",bestMetricLabel),main="Cluster on Global Alignment")
text(subsetMDS[subsamples[[1]],1],morphometricsSubset[subsamples[[1]],bestMetric],labels=as.character(clusterGA$cluster),font=2,col=clusterColors[groupIds[subsamples[[1]]]])
for (i in 1:length(clusterGA$cluster)){
  if (clusterMetricBest$cluster[i]==clusterGA$cluster[i]){
    if (clusterGA$cluster[i] == groupIds[i]){ ringCol<-3 }else{ ringCol<-2}
    ellipse(c(subsetMDS[i,1],morphometricsSubset[i,bestMetric]),radius=.1,shape=matrix(c(1,0,0,1),2.1,2.1),center.pch=F,col=ringCol,font=1)
  }
}

bestMetric <- "Depth"
bestMetricLabel <- "Depth"
bestMetric <- "BranchOrder"
bestMetricLabel <- "Branch Order"
bestMetric <- "RemoteBifurcationAmpl"
bestMetricLabel <- "Remote Bifurcation Angle"
bestMetric <- "Fragmentation"
bestMetricLabel <- "Fragmentation"
#clusterMetricBest <- kmeans(morphometricsSubset[,"BranchOrder"],centers=2)
clusterMetricBest <- kmeans(morphometricsSubset[,bestMetric],centers=2)
MetricBestRI <- AdjustedRandIndex(clusterMetricBest$cluster,groupIds)
x11(height=5,width=5)
plot(subsetMDS[,1:2],col="white",xlab="Global Alignment Dimension 1",ylab="Global Alignment Dimension 2",main=paste("Cluster on",bestMetricLabel))
text(subsetMDS[,1:2],labels=as.character(3-clusterMetricBest$cluster),font=2,col=clusterColors[groupIds])
for (i in 1:length(clusterMetricBest$cluster)){
  if (clusterMetricBest$cluster[i]==clusterGA$cluster[i]){
    if (clusterGA$cluster[i] != groupIds[i]){ ringCol<-3 }else{ ringCol<-2}
    ellipse(as.numeric(subsetMDS[i,1:2]),radius=.1,shape=matrix(c(1,0,0,1),2.1,2.1),center.pch=F,col=ringCol,font=1)
  }
}
x11(height=5,width=5)
plot(subsetMDS[,1],morphometricsSubset[,bestMetric],col="white",xlab="Global Alignment Dimension 1",ylab=paste("Normalized Branch",bestMetricLabel),main=paste("Cluster on",bestMetricLabel))
text(subsetMDS[,1],morphometricsSubset[,bestMetric],labels=as.character(3-clusterMetricBest$cluster),font=2,col=clusterColors[groupIds])
for (i in 1:length(clusterMetricBest$cluster)){
  if (clusterMetricBest$cluster[i]==clusterGA$cluster[i]){
    if (clusterGA$cluster[i] != groupIds[i]){ ringCol<-3 }else{ ringCol<-2}
    ellipse(c(subsetMDS[i,1],morphometricsSubset[i,bestMetric]),radius=.1,shape=matrix(c(1,0,0,1),2.1,2.1),center.pch=F,col=ringCol,font=1)
  }
}

clusterCombo <- kmeans(cbind(subsetMDS,morphometricsSubset[,bestMetric,drop=F]),centers=2)
comboRI <- AdjustedRandIndex(clusterCombo$cluster,groupIds)
x11(height=5,width=5)
plot(subsetMDS[,1:2],col="white",xlab="Global Alignment Dimension 1",ylab="Global Alignment Dimension 2",main=paste("Cluster on",bestMetricLabel,"and Global Alignment"))
text(subsetMDS[,1:2],col=clusterColors[groupIds],labels=as.character(clusterCombo$cluster),font=2)
for (i in 1:length(clusterGA$cluster)){
  #if (clusterCombo$cluster[i]==clusterGA$cluster[i]){
  if (clusterCombo$cluster[i]==clusterMetricBest$cluster[i]){
    if (clusterCombo$cluster[i] == groupIds[i]){ ringCol<-3 }else{ ringCol<-2}
    ellipse(as.numeric(subsetMDS[i,1:2]),radius=.1,shape=matrix(c(1,0,0,1),2.1,2.1),center.pch=F,col=ringCol,font=1)
  }
}
x11(height=5,width=5)
plot(subsetMDS[,1],morphometricsSubset[,bestMetric],col="white",xlab="Global Alignment Dimension 1",ylab=paste("Normalized",bestMetricLabel),main=paste("Cluster on",bestMetricLabel,"and Global Alignment"))
text(subsetMDS[,1],morphometricsSubset[,bestMetric],col=clusterColors[groupIds],labels=as.character(clusterCombo$cluster),font=2)
for (i in 1:length(clusterGA$cluster)){
  if (clusterCombo$cluster[i]!=clusterGA$cluster[i]){
    if (clusterCombo$cluster[i] == groupIds[i]){ ringCol<-3 }else{ ringCol<-2}
    ellipse(c(subsetMDS[i,1],morphometricsSubset[i,bestMetric]),radius=.1,shape=matrix(c(1,0,0,1),2.1,2.1),center.pch=F,col=ringCol,font=1)
  }
}

# Cluster alignment and get RIs
alignmentRIs <- as.numeric(sampleClusterRIs(subsetMDS,groupIds,sampleSizes,subsamples=subsamples,clusterMethod="kmeans",randomizeGroups=F))
alignmentRandomRIs <- as.numeric(sampleClusterRIs(subsetMDS,groupIds,sampleSizes,subsamples=subsamples,clusterMethod="kmeans",randomizeGroups=T))
RIs[,1] <- alignmentRIs

morphometricsSubset <- morphometrics[subsetSeqs,]
for (featInd in 1:numFeatures){
  RIs[,1+featInd] <- 
    as.numeric(sampleClusterRIs(morphometricsSubset[,featInd,drop=F],groupIds,sampleSizes,subsamples=subsamples,clusterMethod="kmeans",randomizeGroups=F))
  RIs[,1+numFeatures+featInd] <- 
    as.numeric(sampleClusterRIs(cbind(subsetMDS,morphometricsSubset[,featInd,drop=F]),groupIds,sampleSizes,subsamples=subsamples,clusterMethod="kmeans",randomizeGroups=F))
}

# Loop through features to produce mean, SEM, and p-value results
means[i,3:numResultColumns] <- apply(RIs,2,mean)
SEMs[i,3:numResultColumns] <- apply(RIs,2,sem)
for (featInd in 1:numFeatures){
  gaVsMetricPVals[i,2+featInd] <- wilcox.test(RIs[,1],RIs[,1+featInd],paired=T,alternative="greater")$p.value
  metricIsBetter <- means[i,3+featInd] > means[i,3]
  if (metricIsBetter){
    mergePVals[i,2+featInd] <- wilcox.test(RIs[,1+numFeatures+featInd],RIs[,1+featInd],paired=T,alternative="greater")$p.value
  }else{
    mergePVals[i,2+featInd] <- wilcox.test(RIs[,1+numFeatures+featInd],RIs[,1],paired=T,alternative="greater")$p.value
  }
}



#### Export MDS data for each class comparison ###
for (listIndex in bestIndices){
  chosenMorphometrics <- morphometricsToCombine[[listIndex]]
  
  specifications <- PairwiseSpecificationLists[[listIndex]]
  print(paste(listIndex,names(PairwiseSpecificationLists)[listIndex]))
  
  # Load group/sequences
  subsetSeqsList <- loadSubsetSeqs(specifications,clusterMetaData,candidates=rownames(distMatDF))
  subsetSeqs <- subsetSeqsList$Sequences
  groupSeqs <- subsetSeqsList$SeqsByGroup
  groupIds <- subsetSeqsList$GroupIds
  groupSizes <- aggregate(groupIds,by=list(groupIds),length)[unique(groupIds),2]

  # Take subset of alignment distance matrix and get MDS
  subsetMDS <- isoMDS(distMatDF[subsetSeqs,subsetSeqs],k=bestNumDims[[listIndex]])$points
  # Standardize each dimension by the standard deviation of the first dimension
  subsetMDS <- subsetMDS/sd(subsetMDS[,1])
  
  write.csv(subsetMDS,paste("ClusterAnalysis/MDS-Data-Ind",listIndex,".csv",sep=""))
}

#### Exploring MDS and metric data on PC using tourr package ####

library(tourr)

listIndex <- 3
specifications <- PairwiseSpecificationLists[[listIndex]]
print(paste(listIndex,names(PairwiseSpecificationLists)[listIndex]))

subsetMDS <- read.csv(paste(clusterDir,"MDS-Data-Ind",listIndex,".csv",sep=""),row.names=1)

# Load group/sequences
candidates <- intersect(rownames(morphometricsXformed),rownames(subsetMDS))
subsetSeqsList <- loadSubsetSeqs(specifications,clusterMetaData,candidates=candidates)
subsetSeqs <- subsetSeqsList$Sequences
groupSeqs <- subsetSeqsList$SeqsByGroup
groupIds <- subsetSeqsList$GroupIds
groupSizes <- aggregate(groupIds,by=list(groupIds),length)[unique(groupIds),2]

subsetMorphometrics <- morphometricsXformed[subsetSeqs,]
subsetMDS <- subsetMDS[subsetSeqs,,drop=F]

chosenMorphometrics <- morphometricsToCombine[[listIndex]]

animate_depth(subsetMorphometrics,grand_tour(d=3))
animate_xy(subsetMorphometrics,grand_tour(d=3),col=groupIds)
animate_xy(subsetMDS,grand_tour(d=3),col=groupIds)
animate_xy(cbind(subsetMDS,subsetMorphometrics[,chosenMorphometrics]),grand_tour(d=3),col=groupIds)

#### *Look at specific morphometric combinations based on best comparison pairs ####

require(MASS)
require(parallel)
bestIndices <- c(3,109,110,113,115)
sampleProportionally <- F
proportion <- 0.8

morphometricsToCombine <- list()
morphometricsToCombine[[3]] <- c("Depth","Contraction","PartitionAsymmetry","Height","EuclideanDistance","BranchOrder")
morphometricsToCombine[[109]] <- c("BranchOrder","RemoteBifurcationAmpl","PartitionAsymmetry")
morphometricsToCombine[[110]] <- c("BranchOrder","Length","EuclideanDistance","PathDistance","Branches")
morphometricsToCombine[[113]] <- c("Length","RemoteBifurcationAmpl","EuclideanDistance","Branches")
morphometricsToCombine[[115]] <- c("RemoteBifurcationAmpl","BranchOrder","LocalBifurcationAmpl")

# For the new ones
bestIndices <- c(1,4,3,6,11,16)
morphometricsToCombine <- list()
morphometricsToCombine[[1]] <- c("BranchOrder","RemoteBifurcationAmpl")
morphometricsToCombine[[4]] <- c("Length","RemoteBifurcationAmpl","EuclideanDistance")
morphometricsToCombine[[3]] <- c("Height","Diameter","EuclideanDistance")
#morphometricsToCombine[[6]] <- c("Depth","Fragmentation","Contraction")
#morphometricsToCombine[[11]] <- c("Depth","BranchOrder","Branches","Fragmentation")
#morphometricsToCombine[[16]] <- c("Fragmentation","Depth","BranchOrder","LocalBifurcationAmpl")

morphometricsToCombine[[6]] <- c("Depth","Fragmentation","Contraction")
morphometricsToCombine[[11]] <- c("Depth","BranchOrder","Branches","Fragmentation")
morphometricsToCombine[[16]] <- c("Fragmentation","Depth","BranchOrder","LocalBifurcationAmpl")

listIndex <- 113

for (listIndex in bestIndices){
  chosenMorphometrics <- morphometricsToCombine[[listIndex]]
  #chosenMorphometrics <- c("BranchOrder","PartitionAsymmetry","Branches")
  
#  specifications <- PairwiseSpecificationLists[[listIndex]]
#  print(paste(listIndex,names(PairwiseSpecificationLists)[listIndex]))
  specifications <- NewPairwiseSpecifications[[listIndex]]
  print(paste(listIndex,names(NewPairwiseSpecifications)[listIndex]))
  
  # Load group/sequences
  #subsetSeqsList <- loadSubsetSeqs(specifications,usedMetaData,candidates=rownames(distMatDF))
  givenArborType <- specifications[[1]]$ArborType
  if (givenArborType == "ApicalDendrite"){
    subsetMDS <- apicalMDS
  }else if (givenArborType == "Axon"){
    subsetMDS <- axonMDS
  }else{
    subsetMDS <- dendriteMDS
  }
  subsetSeqsList <- loadSubsetSeqs(specifications,usedMetaData,candidates=rownames(subsetMDS))
  
  subsetSeqs <- subsetSeqsList$Sequences
  groupSeqs <- subsetSeqsList$SeqsByGroup
  groupIds <- subsetSeqsList$GroupIds
  groupSizes <- aggregate(groupIds,by=list(groupIds),length)[unique(groupIds),2]
  
  # Take subset of alignment distance matrix and get MDS
  #subsetMDS <- isoMDS(distMatDF[subsetSeqs,subsetSeqs],k=bestNumDims[[listIndex]])$points
  # Standardize each dimension by the standard deviation of the first dimension
  subsetMDS <- subsetMDS/sd(subsetMDS[,1])
  subsetMDS <- subsetMDS[subsetSeqs,]
  
  # Generate random samples
  if (sampleProportionally){
    sampleSizes <- getSampleSizesProportionally(groupSizes,proportion=proportion)
  }else{
    sampleSizes <- getSampleSizesEvenly(groupSizes,proportionOfSmallest=proportion,maxSize=150)
  }
  subsamples <- generateSubsamples(groupIds,sampleSizes,numSamples)
  
  # Set up RI matrix
  numFeatures <- length(chosenMorphometrics)
  RIs <- matrix(0,nrow=numSamples,ncol=2*numFeatures+1)
  rownames(RIs) <- 1:numSamples
  colnames(RIs) <- c("GlobalAlign",chosenMorphometrics,paste("GA&",chosenMorphometrics,sep=""))
  
  # Cluster alignment and get RIs
  alignmentRIs <- as.numeric(sampleClusterRIs(subsetMDS,groupIds,sampleSizes,subsamples=subsamples,clusterMethod="kmeans",randomizeGroups=F))
  alignmentRandomRIs <- as.numeric(sampleClusterRIs(subsetMDS,groupIds,sampleSizes,subsamples=subsamples,clusterMethod="kmeans",randomizeGroups=T))
  RIs[,1] <- alignmentRIs
  
  morphometricsSubset <- morphometricsXformed[subsetSeqs,]
  for (featInd in 1:numFeatures){
    RIs[,1+featInd] <- 
      as.numeric(sampleClusterRIs(morphometricsSubset[,chosenMorphometrics[1:featInd],drop=F],groupIds,sampleSizes,subsamples=subsamples,clusterMethod="kmeans",randomizeGroups=F))
    RIs[,1+numFeatures+featInd] <- 
      as.numeric(sampleClusterRIs(cbind(subsetMDS,morphometricsSubset[,chosenMorphometrics[1:featInd],drop=F]),groupIds,sampleSizes,subsamples=subsamples,clusterMethod="kmeans",randomizeGroups=F))
  }
  
  # Loop through features to produce mean, SEM, and p-value results
  means <- apply(RIs,2,mean)
  SEMs <- apply(RIs,2,sem)
  gaVsMetricPVals <- c()
  mergePVals <- c()
  pImprovement <- c()
  for (featInd in 1:numFeatures){
    gaVsMetricPVals[featInd] <- wilcox.test(RIs[,1],RIs[,1+featInd],paired=T,alternative="greater")$p.value
    metricIsBetter <- means[1+featInd] > means[1]
    if (metricIsBetter){
      mergePVals[featInd] <- wilcox.test(RIs[,1+numFeatures+featInd],RIs[,1+featInd],paired=T,alternative="greater")$p.value
    }else{
      mergePVals[featInd] <- wilcox.test(RIs[,1+numFeatures+featInd],RIs[,1],paired=T,alternative="greater")$p.value
    }
    if (featInd > 1){
      bestComboYet <- which.max(means[2:featInd])
      pImprovement[featInd] <- wilcox.test(RIs[,1+featInd],RIs[,1+bestComboYet],paired=T,alternative="greater")$p.value
      bestComboYet <- which.max(means[numFeatures+(2:featInd)])
      pImprovement[numFeatures+featInd] <- wilcox.test(RIs[,1+numFeatures+featInd],RIs[,1+numFeatures+bestComboYet],paired=T,alternative="greater")$p.value
    }
  }
  
  results <- data.frame(Mean=means,SEM=SEMs,GA_vs_Metrics=c(NA,gaVsMetricPVals,rep(NA,numFeatures)),Merge_vs_Better=c(rep(NA,1+numFeatures),mergePVals),Curr_vs_Best=c(NA,pImprovement))
  rownames(results) <- colnames(RIs)
  
  if (sampleProportionally){
    write.csv(results,paste(clusterDir,"NewKmeansCumulMetricRI_Ind",listIndex,"_Stats_Proportional.csv",sep=""),row.names=T)
  }else{
    write.csv(results,paste(clusterDir,"NewKmeansCumulMetricClusterRI_Ind",listIndex,"_Stats.csv",sep=""),row.names=T)
    #write.csv(results,paste(clusterDir,"NewKmeansCumulTopoMetricsClusterRI_Ind",listIndex,"_Stats.csv",sep=""),row.names=T)
  }
}

##################################################################
##### *Experimenting with MDS, choosing number of dimensions #####
library(MASS)

listIndex <- 107
specifications <- PairwiseSpecificationLists[[listIndex]]
subsetSeqs <- c()
groupSeqs <- list()
groupIds <- c()
for (specInd in 1:length(specifications)){
  specs <- specifications[[specInd]]
  groupSeqs[[specInd]] <- rownames(getSubset2(clusterMetaData,specs,typeMetaData=clusterMetaData))
  subsetSeqs <- append(subsetSeqs,groupSeqs[[specInd]])
  groupIds <- append(groupIds,rep(specInd,length(groupSeqs[[specInd]])))
}
subsetInds <- which(subsetSeqs%in%sequenceNames)

# If setting the groups to the same size
#subsample1 <- groupSeqs[[1]][sample(1:length(groupSeqs[[1]]),subsampleSize,replace=F)]
#subsample2 <- groupSeqs[[2]][sample(1:length(groupSeqs[[2]]),subsampleSize,replace=F)]
# For comparison between machines/instances
subsample1 <- groupSeqs[[1]][1:subsampleSize]
subsample2 <- groupSeqs[[2]][1:subsampleSize]
subsampleSeqs <- c(subsample1,subsample2)
sampleGroupIds <- c(rep(1,subsampleSize),rep(2,subsampleSize))

subsetSampleDist <- as.dist(distMatDF[subsampleSeqs,subsampleSeqs])
#subsetSampleDist <- as.dist(jcDistDF[subsampleSeqs,subsampleSeqs])
#subsetSampleDist <- as.dist(distMatSimple[subsampleSeqs,subsampleSeqs])
#subsetSampleDist <- as.dist(distMatStretchSq[subsampleSeqs,subsampleSeqs])
distType <- "JC"

# using all data
subsetSampleDist <- as.dist(distMatDF[subsetSeqs,subsetSeqs])
sampleGroupsIds <- groupIds

editDistSubset <- editDistDF[subsetSeqs,subsetSeqs]+.001
subsetSampleDist <- as.dist(editDistSubset)

# Continue
#mds2 <- isoMDS(subsetSampleDist,k=2)
#mds3 <- isoMDS(subsetSampleDist,k=3)
#mds4 <- isoMDS(subsetSampleDist,k=4)
#mds5 <- isoMDS(subsetSampleDist,k=5)

title <- paste(randStr,travOrder,"_",distType,"_Ind",listIndex,"Subsample",sep="")

mdsK <- 4
#mds2 <- mdsComparer(subsetSampleDist,k=2,title)
#mds3 <- mdsComparer(subsetSampleDist,k=3,title)
#mds4 <- mdsComparer(subsetSampleDist,k=4,title)
#mds5 <- mdsComparer(subsetSampleDist,k=5,title)

mds <- mdsComparer(subsetSampleDist,mdsK,title)

mdsKmeans <- kmeans(mds$points,2)

# Score clusters compared to known groups
AdjustedRandIndex(mdsKmeans$cluster,sampleGroupIds)
png(paste("MDS",mdsK,"_Plot_",title,"_Cluster.png",sep=""),width=800,height=800)
#plot(mds$points[,1],mds$points[,2],pch=mdsKmeans$cluster,col=sampleGroupIds,cex=.5,asp=1)
plot(as.data.frame(mds$points),pch=mdsKmeans$cluster,col=sampleGroupIds,cex=1,asp=1)
dev.off()

plotMDSStress <- function(dists,ks,tol=2,subtitle=NA,imageTitle=NA){
  mdsResults <- mclapply(ks,function(k,dists,tol){return(isoMDS(dists,k=k,tol=tol))},dists,tol,mc.cores=detectCores())
  #strainMat <- matrix(0,nrow=length(mdsResults),ncol=2)
  stress <- c()
  for (i in 1:length(ks)){
    #k <- ks[i]
    res <- mdsResults[[i]]
    #strainMat[i,1] <- k
    #strainMat[i,1] <- res$stress
    if (is.null(res)){
      stress <- append(stress,NA)
    }else{
      stress <- append(stress,res$stress)
    }
  }
  plotTitle <- "MDS Scree Plot"
  if (!is.na(subtitle)){
    plotTitle <- paste(plotTitle,subtitle,sep="\n")
  }
  if (is.na(imageTitle)){
    plot(ks,stress,type="l",main=plotTitle)
  }else{
    imageFilename <- paste("MDSScreePlot_",imageTitle,".png",sep="")
    png(imageFilename)
    plot(ks,stress,type="l",main=plotTitle)
    dev.off()
  }
  names(mdsResults) <- ks
  return(mdsResults)
}

mdsResults <- plotMDSStress(distMatDF,ks=2:9,tol=1,imageTitle="AllNeurites")

mdsResultsInd113 <- plotMDSStress(distMatDF[subsetSeqs,subsetSeqs],ks=2:13,tol=.001,imageTitle="Pair113")
mdsResults <- list()
listIndex <- 86
subsetSeqsObj<-loadSubsetSeqs(PairwiseSpecificationLists[[listIndex]],usedMetaData,rownames(distMatDF))
subsetSeqs <- subsetSeqsObj$Sequences
mdsResults[[listIndex]]<-plotMDSStress(distMatDF[subsetSeqs,subsetSeqs],ks=1:13,tol=.001,imageTitle=paste("Pair",listIndex,sep=""))

mdsComparer <- function(dists,k,subtitle){
  mdsObj <- isoMDS(dists,k=k)
  mdsDists <- as.numeric(dist(mdsObj$points))
  mdsDists <- mdsDists[mdsDists!=0]
  
  dists <- as.numeric(dists)
  dists <- dists[dists!=0]
  
  print(mdsObj$stress)
  #mdsDistCor <- mdsDistanceCorrelation(dists,mdsDists,T)
  mdsDistCor <- cor(dists,mdsDists)
  print(paste("MDS",k," r: ",mdsDistCor,"; R^2: ",mdsDistCor^2,sep=""))
  #mdsDistCor <- mdsDistanceCorrelation(log(dists+1),log(mdsDists+1),T)
  mdsDistCor <- cor(log(dists+1),log(mdsDists+1))
  print(paste("MDS",k,"(log) r: ",mdsDistCor,"; R^2: ",mdsDistCor^2,sep=""))
  png(paste("DistCompare_",subtitle,"_MDS",k,"_log.png",sep=""))
  plot(log(dists+1),log(mdsDists+1),xlab="log edit distance",ylab=paste("log MDS",k," distance",sep=""),cex=.5,asp=1)
  dev.off()
  png(paste("DistCompare_",subtitle,"_MDS",k,".png",sep=""))
  plot(dists,mdsDists,xlab="edit distance",ylab=paste("MDS",k," distance",sep=""),cex=.5,asp=1)
  dev.off()
  return(mdsObj)
}


mdsDistanceCorrelation <- function(originalDist,mdsMatrix,plotDistances=F){
  mOrig <- data.frame(t(combn(labels(originalDist),2)), as.numeric(originalDist))
  mdsDist <- dist(mdsMatrix)
  mMds <- data.frame(t(combn(labels(originalDist),2)), as.numeric(mdsDist))
  if (plotDistances){
    plot(mOrig[,3],mMds[,3])
  }
  return(cor(mOrig[,3],mMds[,3]))
}


#write.csv(RISampleMeanResults,"SpearmanPairwiseClusterResults_Min20_NWDefault.csv")

# Run MDS via PCA, then k-means

for (listIndex in 1:length(PairwiseSpecificationLists)){
  specifications <- PairwiseSpecificationLists[[listIndex]]
  subsetSeqs <- c()
  groupSeqs <- list()
  groupIds <- c()
  for (specInd in 1:length(specifications)){
    specs <- specifications[[specInd]]
    groupSeqs[[specInd]] <- rownames(getSubset2(clusterMetaData,specs,typeMetaData=clusterMetaData))
    subsetSeqs <- append(subsetSeqs,groupSeqs[[specInd]])
    groupIds <- append(groupIds,rep(specInd,length(groupSeqs[[specInd]])))
  }
  #subsetInds <- which(subsetSeqs%in%sequenceNames)
  subsetInds <- which(sequenceNames%in%subsetSeqs)
  
  # Take subset and convert to dist type
  #subsetDist <- as.dist(distMatDF[subsetInds,subsetInds])

  # Run MDS via PCA
  subsetMDS <- cmdscale(distMatDF[subsetInds,subsetInds],k=4)
  
  # Cluster data
  mdsKmeans <- kmeans(subsetMDS,2)
  
  # Score clusters compared to known groups
  kmeansRIs[listIndex] <- AdjustedRandIndex(mdsKmeans$cluster,groupIds)
}
kmeansRIResults <- data.frame(Index=1:length(PairwiseSpecificationLists),RandIndex=kmeansRIs,name=names(PairwiseSpecificationLists))

# Plotting distribution and fits
png(paste("Ind",listIndex,"_MDS1Hist.png",sep=""))
histObj <- hist(subsetMDS[,1],breaks=20,freq=F)
lines(density(subsetMDS[,1]))
x<-diff(range(histObj$breaks))*(0:100)/100 + min(histObj$breaks)

em1D <- Mclust(subsetMDS[,1],3:5,modelName="V")
params <- em1D$parameters
y1 <- params$pro[1]*dnorm(x,params$mean[1],sqrt(params$variance$scale[1]))
y2 <- params$pro[2]*dnorm(x,params$mean[2],sqrt(params$variance$scale[2]))
y3 <- params$pro[3]*dnorm(x,params$mean[3],sqrt(params$variance$scale[3]))
lines(x,y1,col="red")  
lines(x,y2,col="blue")
lines(x,y3,col="green")
lines(x,y1+y2,col="purple")
dev.off()

y1 <- params$pro[1]*dnorm(x,params$mean[1],sqrt(params$variance$sigmasq))
y2 <- params$pro[2]*dnorm(x,params$mean[2],sqrt(params$variance$sigmasq))

#Same but with increased kurtosis
png(paste("Ind",listIndex,"_IncKurt_MDS1Hist.png",sep=""))
png(paste("Ind",listIndex,"_DecKurt_MDS1HistEM2.png",sep=""))
subsetMDSXformed <- sinh(.3*asinh(subsetMDS[,1]))
histObj <- hist(subsetMDSXformed,breaks=20,freq=F)
lines(density(subsetMDSXformed))
x<-diff(range(histObj$breaks))*(0:100)/100 + min(histObj$breaks)

em1D <- Mclust(subsetMDSXformed,2,modelName="V")
params <- em1D$parameters
y1 <- params$pro[1]*dnorm(x,params$mean[1],sqrt(params$variance$scale[1]))
y2 <- params$pro[2]*dnorm(x,params$mean[2],sqrt(params$variance$scale[2]))
y3 <- params$pro[3]*dnorm(x,params$mean[3],sqrt(params$variance$scale[3]))
lines(x,y1,col="red")
lines(x,y2,col="blue")
lines(x,y3,col="green")
lines(x,y1+y2+y3,col="purple")
dev.off()

