####################################FileInfo###############################################
#
# Paper1FigureScripts.r
# Organized by figure number and including non-R instructions.
#
#
###########################################################################################

#### Setup ####

require(ape)
require(MASS)
require(ggplot2)
require(grid)
require(fields)

systemType <- "KrasnowCluster"
if (systemType == "PC"){
  homeDir <- "C:/"
  clusterDir <- "C:/Documents and Settings/gillette/My Documents/research/ClusterAnalysis/"
  baseCodeDir <- "G:/Dropbox/RCode/"
  clusterGlobalDir <- "C:/Documents and Settings/gillette/My Documents/research/GlobalAlignment/"
  globalAlignDataDir <- "G:/AlignmentData/"
  motifCodeDir <- 'G:/Dropbox/Motif Analysis/Motif Analysis R Files/'
  motifDataDirectory <- "C:/Documents and Settings/gillette/My Documents/research/MotifAnalysis/"
  figDir <- "C:/Documents and Settings/gillette/My Documents/research/papers & presentations/PASTA Paper/Paper1 - Motifs/figures/"
}else{
  homeDir <- "/home/tgillett/"
  clusterDir <- "/home/tgillett/ClusterAnalysis/"
  baseCodeDir <- "/home/tgillett/RUtils/"
  clusterGlobalDir <- "/home/tgillett/PairwiseAlignment/"
  globalAlignDataDir <- clusterGlobalDir
  baselineDir <- "/home/tgillett/PairwiseAlignment/Baseline/"
  motifCodeDir <- '/home/tgillett/ClusterAnalysis/'
  motifDataDirectory <- "/home/tgillett/results/"
  figDir <- "/home/tgillett/figures/Pasta1/"
}
prDataDirectory <- paste(motifDataDirectory,"/PercentileRanks/",sep="")

# Load Utils
source(paste(baseCodeDir,"Utils.r",sep=""))
source(paste(baseCodeDir,"VisualizationUtils.r",sep=""))
source(paste(baseCodeDir,"StringUtils.r",sep=""))
source(paste(baseCodeDir,"StatisticsUtils.r",sep=""))

prDataDirectory <- paste(motifDataDirectory,"/PercentileRanks/",sep="")

# Load metadata
masterMetaData <- read.csv(paste(homeDir,"arborMetaData-NM-MapType2.csv",sep=""),na.strings="",stringsAsFactors=FALSE,row.names=1)

# Load ClassComparisonDefs.r
source(paste(clusterDir,"ClassComparisonDefs.r",sep=""))
motifMetaData <- usedMetaData
motifMetaData[motifMetaData$ArborType=="BasalDendrite","ArborType"] <- "Dendrite"

# Load Sequence lengths
SeqLenData <- read.csv(paste(homeDir,"SequenceLengths-NM-MapType2.csv",sep=""),row.names=1)
SeqLenData2D <- read.csv(paste(homeDir,"SequenceLengths-NM_SpurLen2D-MapType2.csv",sep=""),row.names=1)
sharedSeqs <- intersect(rownames(motifMetaData),rownames(SeqLenData2D))
SeqLenData <- SeqLenData[sharedSeqs,,drop=F]
SeqLenData2D <- SeqLenData2D[sharedSeqs,,drop=F]
motifMetaData[sharedSeqs,"SeqLen"] <- SeqLenData2D[sharedSeqs,"SeqLen"]

# Load flagged neurons
flaggedNeurons <- read.table(paste(projectConfigDir,"FlaggedNeurons.txt",sep=""),stringsAsFactors = F)[,1]
flaggedNeurites <- flaggedNeurons[grep("-(Axon|ApicalDendrite|Dendrite|BasalDendrite)",flaggedNeurons)]
flaggedNeurons <- flaggedNeurons[!flaggedNeurons%in%flaggedNeurites]
flaggedNeurites <- append(flaggedNeurites,paste(flaggedNeurons,"-Axon",sep=""))
flaggedNeurites <- append(flaggedNeurites,paste(flaggedNeurons,"-ApicalDendrite",sep=""))
flaggedNeurites <- append(flaggedNeurites,paste(flaggedNeurons,"-Dendrite",sep=""))

paper1Theme <- theme_bw()+
  theme(panel.grid.major.x = element_blank(), panel.grid.minor.x = element_blank(),
        panel.grid.major.y = element_blank(), panel.grid.minor.y = element_blank(),
        axis.line = element_line(colour="black"),
        panel.border = element_blank())
gridLines <- theme(panel.grid.major.x = element_line(colour = "grey40"),
                   panel.grid.major.y = element_line(colour = "grey40"))

#### Load Data ####

surrogateSetNum <- 1
numSets <- 100
prDataB8STL <- list()
prDataB8LTS <- list()
prDataB8Combo <- list()
for (k in 1:5){
  prFilename <- getPRFilename(k,dataset=dataset,sets=numSets,surrogateSet=surrogateSetNum,order="STL",baselineMethod=8,threshold="0.0",thresholdTypeString="Whole",directory=prDataDirectory,sourceTraversal="STL")
  prDataB8STL[[k]] <- read.csv(prFilename,stringsAsFactors=F,row.names=1)
  prDataB8STL[[k]] <- prDataB8STL[[k]][,colnames(prDataB8STL[[k]])!="N",drop=F]
  
  prDataB8Combo[[k]] <- prDataB8STL[[k]]
  
  if (k >= 3){
    # Read in LTS k-mers
    prDataB8LTS[[k]] <- read.csv(getPRFilename(k,dataset=dataset,sets=numSets,surrogateSet=surrogateSetNum,order="LTS",baselineMethod=8,threshold="0.0",thresholdTypeString="Whole",directory=prDataDirectory,sourceTraversal="STL"),stringsAsFactors=F,row.names=1)
    prDataB8LTS[[k]] <- prDataB8LTS[[k]][,colnames(prDataB8LTS[[k]])!="N",drop=F]
    tmp <- prDataB8LTS[[k]]
    # For merging, get rid of LTS k-mers that have an STL counterpart
    inds <- grep("[ACT]C+[ACT]",colnames(tmp),perl=T)
    colnames(tmp) <- paste("L-",colnames(tmp),sep="")
    prDataB8Combo[[k]] <- cbind(prDataB8STL[[k]],tmp[,-inds])
    colnames(prDataB8Combo[[k]]) <- c(colnames(prDataB8STL[[k]]),colnames(tmp)[-inds])
  }
}
if (exists("tmp")) rm(tmp)

prSurrogateDataB8STL <- list()
prSurrogateDataB8LTS <- list()
prSurrogateDataB8Combo <- list()
for (k in 1:5){
  prSurrogateDataB8STL[[k]] <- read.csv(getPRFilename(k,sets=40,surrogateSet=surrogateSetNum,order="STL",baselineMethod=8,threshold="0.0",thresholdTypeString="Whole",directory=prDataDirectory,sourceTraversal="STL",useRandomSet=1,useRandomK="Plus1"),stringsAsFactors=F,row.names=1)
  prSurrogateDataB8STL[[k]] <- prSurrogateDataB8STL[[k]][,colnames(prSurrogateDataB8STL[[k]])!="N",drop=F]
  
  prSurrogateDataB8Combo[[k]] <- prSurrogateDataB8STL[[k]]
  
  if (k >= 3){
    # Read in LTS k-mers
    prSurrogateDataB8LTS[[k]] <- read.csv(getPRFilename(k,sets=40,surrogateSet=surrogateSetNum,order="LTS",baselineMethod=8,threshold="0.0",thresholdTypeString="Whole",directory=prDataDirectory,sourceTraversal="STL",useRandomSet=1,useRandomK="Plus1"),stringsAsFactors=F,row.names=1)
    prSurrogateDataB8LTS[[k]] <- prSurrogateDataB8LTS[[k]][,colnames(prSurrogateDataB8LTS[[k]])!="N",drop=F]
    tmp <- prSurrogateDataB8LTS[[k]]
    # For merging, get rid of LTS k-mers that have an STL counterpart
    inds <- grep("[ACT]C+[ACT]",colnames(tmp),perl=T)
    colnames(tmp) <- paste("L-",colnames(tmp),sep="")
    prSurrogateDataB8Combo[[k]] <- cbind(prSurrogateDataB8STL[[k]],tmp[,-inds])
    colnames(prSurrogateDataB8Combo[[k]]) <- c(colnames(prSurrogateDataB8STL[[k]]),colnames(tmp)[-inds])
  }
}
rm(tmp)

# Raw %C data
realCFreqs <- getKmerFreqs(1,dataset=dataset,travOrder="STL")
# Remove anything from Protocol:Culture or !ExperCond:Control
realCFreqs <- realCFreqs[rownames(subset(motifMetaData[rownames(realCFreqs),],Protocol!="Culture"&Protocol!="Acute culture"&ExperCond=="Control")),,drop=F]

# Terminal growth
termGrowCFreqs <- getKmerFreqs(1,travOrder="LTS",useRandomSet=9,useRandomK=0,method=2,sourceTraversal="")
termGrowCFreqs <- cbind(termGrowCFreqs,SeqLen=masterMetaData[rownames(termGrowCFreqs),"SeqLen"])

# Terminal & segmental growth
segmentGrowCFreqs <- getKmerFreqs(1,travOrder="LTS",useRandomSet=1,useRandomK=0,method=9,sourceTraversal="")
segmentGrowCFreqs <- cbind(segmentGrowCFreqs,SeqLen=masterMetaData[rownames(segmentGrowCFreqs),"SeqLen"])
#interstGrowCFreqs <- getKmerFreqs(1,useRandomSet=1,useRandomK=0,method=9)

# Uniform distribution of tree shapes
treeShapeCFreqs <- getKmerFreqs(1,travOrder="LTS",useRandomSet=21,useRandomK=0,method=8,sourceTraversal="")
treeShapeCFreqs <- cbind(treeShapeCFreqs,SeqLen=masterMetaData[rownames(treeShapeCFreqs),"SeqLen"])

#### Creating figures for Motif Analysis paper ####
## ggplot themes
getTheme <- function(size=20,xangle=0,yangle=0,x.hjust=NULL){
  return(theme(axis.title=element_text(size=size,face="bold"),axis.text.y=element_text(size=size,angle=yangle),axis.text.x = element_text(size=size, angle = xangle, hjust = x.hjust)))
}

arborTypeColors <- c(Axon="#DD00DD",ApicalDendrite="#00CC00",Apical="#00CC00",Dendrite="#0000EE")

#### Fig1: Encoding and traversing ####

## Panels A and B are done entirely in PowerPoint

## Panels C and D: Swc files with Neuromantic and GIMP, dendrograms here in R

# Create newick files for example neuron
#java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/figures/Pasta1/ca3b-N2-l51-spliced.swc" -o "/home/tgillett/figures/Pasta1/" -a 1 -L NODE_TYPE -b
#java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/figures/Pasta1/ca3b-N2-l51-spliced.swc" -o "/home/tgillett/figures/Pasta1/" -a 3 -L NODE_TYPE -b
# For apical, a subtree is highlighted (done in Neuromantic), so a separate file was created and the dendrogram is created on the entire SWC file (-a 0)
#java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/figures/Pasta1/ca3b-N2-l51-Apical.swc" -o "/home/tgillett/figures/Pasta1/" -a 0 -L NODE_TYPE -b

fig1Colors <- c("#6e7dff",Axon="#ff46ff",Dendrite="#6e7dff",Apical="#12c40e",ApicalHighlight="#17ff9a")
names(fig1Colors)[1] <- "0"

filename <- paste(figDir,"ca3b-N2-l51-spliced-AxonLabelType.newick",sep="")
tree <- read.tree(filename)
#png(paste(figDir,"Fig1-ca3b-N2-l51-spliced-Axon.png",sep=""),width=10,height=6,units="in",res = 300)
png(paste(figDir,"Fig1-ca3b-N2-l51-spliced-Axon-alternating.png",sep=""),width=10,height=6,units="in",res = 300)
plotTree(tree,treePalette=fig1Colors,colorByType=T,edge.width=2,no.margin=T)
dev.off()

filename <- paste(figDir,"ca3b-N2-l51-spliced-DendriteLabelType.newick",sep="")
tree <- read.tree(filename)
png(paste(figDir,"Fig1-ca3b-N2-l51-spliced-Dendrite.png",sep=""),width=500,height=500,units="px")
plotTree(tree,treePalette=fig1Colors,colorByType=T,edge.width=3,no.margin=T)
dev.off()

filename <- paste(figDir,"ca3b-N2-l51-Apical-BinaryLabelType.newick",sep="")
tree <- read.tree(filename)
png(paste(figDir,"Fig1-ca3b-N2-l51-Apical.png",sep=""),width=600,height=500,units="px")
#svg(paste(figDir,"Fig1-ca3b-N2-l51-Apical.svg",sep=""),width=600,height=500)
plotTree(tree,treePalette=fig1Colors,colorByType=T,edge.width=3,no.margin=T)
dev.off()

# ca3b-N2-l51-Apical.swc


ca3b-N2-l51-Apical-Binary.newick

#### Fig2c: Num Shapes vs Num Bifurcations ####
#shapesVsBifs <- read.csv("C:/Documents and Settings/gillette/My Documents/research/TreeSizeVsShapes.csv")
shapesVsBifs <- read.csv(paste(figDir,"TreeSizeVsShapes.csv",sep=""))
svbExpModel <- lm(log(Shapes,10) ~ Bifurcations,shapesVsBifs[60:120,])
shapesVsBifs$ExpModel <- svbExpModel$coefficients[1]+svbExpModel$coefficients[2]*shapesVsBifs$Bifurcations
shapesVsBifs$ExpModelNoOffset <- svbExpModel$coefficients[2]*shapesVsBifs$Bifurcations

# Calculate values in form of Fit = a * 3^(b*n)
a <- 3^(svbExpModel$coefficients[1]/log(3,10))
b <- svbExpModel$coefficients[2]/log(3,10)

## Include number of possible unique sequences
shapesVsBifs$LogNumSeqs <- shapesVsBifs$Bifurcations*log(3,10)

gg_shapesVsBifs <- ggplot(shapesVsBifs,aes(Bifurcations,log(Shapes,10)))+geom_path(size=2.3,color="#666666")+
  geom_path(aes(Bifurcations,ExpModel),size=1.8,color="#ff0000",linetype="dashed")+
  #geom_path(aes(Bifurcations,ExpModelNoOffset),size=0.9,color="#ff0000",linetype="dashed")+
  #geom_path(aes(Bifurcations,LogNumSeqs),size=1.1,color="#888888")+
  geom_path(aes(Bifurcations,LogNumSeqs),size=1.5,color="#999999")+
  labs(x="Number of Bifurcations", y="Number of Shapes")+
  theme_bw()+theme(panel.border=element_blank(),axis.line=element_line())
#ggsave(gg_shapesVsBifs,file="C:/Documents and Settings/gillette/My Documents/research/papers & presentations/PASTA Paper/Paper1 - Motifs/figures/Components/ShapesVsBifs-051514_6.pdf")
ggsave(gg_shapesVsBifs,file=paste(figDir,"ShapesVsBifsWithOffset-111214.png",sep=""))

#### Figure 3: Monomers ####
# %C
plotdf <- rbind(rbind(axonCFreq,apicalCFreq[apicalCFreq$SeqLen < 70,]),dendriteCFreq)
arborGroup <- c(rep("Axon",nrow(axonCFreq)),rep("Apical",sum(apicalCFreq$SeqLen < 70)),rep("Dendrite",nrow(dendriteCFreq)))
plotdf$ArborType <- factor(arborGroup,levels=c("Apical","Axon","Dendrite"),ordered=T)

# %C without apical dendrites
plotdf <- rbind(axonCFreq,dendriteCFreq)
arborGroup <- c(rep("Axon",nrow(axonCFreq)),rep("Dendrite",nrow(dendriteCFreq)))
plotdf$ArborType <- factor(arborGroup,levels=c("Apical","Axon","Dendrite"),ordered=T)

gg_percCDensity <- ggplot(plotdf,aes(C,fill=ArborType))+
  geom_density(linetype=0,size=.9,alpha=.35)+
  scale_fill_manual(values=arborTypeColors)+
  coord_flip()+
  labs(colour="Arbor Type",x="%C", y="Density")
ggsave(gg_percCDensity,file=paste(figDir,"PercentC_Density-Axons&Dendrites-Flip.png",sep=""))
#ggsave(gg_percCDensity,file=paste(figDir,"PercentC_Density-Axons&Dendrites-Flip.pdf",sep=""))


# %C of only pyramidal cells
plotdf <- rbind(rbind(pyrAxonCFreq,pyrDendriteCFreq),apicalCFreq[apicalCFreq$SeqLen < 70,])
arborGroup <- c(rep("Axon",nrow(pyrAxonCFreq)),rep("Dendrite",nrow(pyrDendriteCFreq)),rep("Apical",sum(apicalCFreq$SeqLen < 70)))
plotdf$Group <- factor(arborGroup,levels=c("Axon","Dendrite","Apical"),ordered=T)

gg_percCPyramidDensity <- ggplot(plotdf,aes(C,fill=Group))+
  geom_density(linetype=0,size=.9,alpha=.35)+
  scale_fill_manual(values=arborTypeColors)+
  geom_density(data=subset(plotdf,Group=="Apical"),linetype=0,alpha=1)+
  coord_flip()+
  labs(colour="Arbor Type",x="%C", y="Density")
ggsave(gg_percCPyramidDensity,file=paste(figDir,"PercentCPyramidal_Density-Flip.png",sep=""))

#### Fig 3c: %C by SeqLen ####

# %C without apical dendrites
plotdf <- rbind(axonCFreq,dendriteCFreq)
arborGroup <- c(rep("Axon",nrow(axonCFreq)),rep("Dendrite",nrow(dendriteCFreq)))
plotdf$Group <- factor(arborGroup,levels=c("Apical","Axon","Dendrite"),ordered=T)

## Load special growth model comparison
extremeQSGrowthKmers <- read.csv(paste(motifDataDirectory,"/MotifAnalysis/ExtremeQSModelsKmerPRs.csv",sep=""),stringsAsFactors=F)
#selectQSGrowthKmers <- read.csv(paste(motifDataDirectory,"/MotifAnalysis/SelectQSModelsKmerPRs.csv",sep=""),stringsAsFactors=F)
#selectQSGrowthKmers <- rbind(selectQSGrowthKmers,read.csv(paste(motifDataDirectory,"/MotifAnalysis/SelectQSModelsLargeKmerPRs.csv",sep=""),stringsAsFactors=F))

## Mean & variance (SD and/or SEM) of growth and tree shapes groups
# Start by weighting by size (though it likely doesn't matter - run both ways to find out)
dendLengths <- plotdf$SeqLen
dendLengthCounts <- aggregate(dendLengths,by=list(dendLengths),length)
rownames(dendLengthCounts) <- dendLengthCounts[,1]

# Get data of biased terminal growth and pure segmental growth
biasedTermPC <- extremeQSGrowthKmers[extremeQSGrowthKmers$Growth=="Q0S0.4",c("PercentC","Length","Growth")]
colnames(biasedTermPC)[1:2] <- c("C","SeqLen")
#biasedTermPC <- biasedTermPC[sample(1:nrow(biasedTermPC),nrow(2*plotdf),prob=dendLengthCounts[as.character(biasedTermPC$Length),"weight"]),]
# Add in lengths in biasedTermPC not in neurite lengths (dendLengthCounts) with 'x' (weight) of 0
addLengths <- unique(biasedTermPC$SeqLen)[!unique(biasedTermPC$SeqLen)%in%rownames(dendLengthCounts)]
if (!is.null(addLengths)){
  dendLengthCounts <- rbind(dendLengthCounts,data.frame(Group.1=addLengths,x=rep(0,length(addLengths))))
  rownames(dendLengthCounts) <- dendLengthCounts[,1]
}
# Sample growth tree data
biasedTermPC <- sample.rows(biasedTermPC,2*nrow(plotdf),prob=dendLengthCounts[as.character(biasedTermPC$SeqLen),"x"],replace=T)

# Segmental growth
segGrowthPC <- segmentGrowCFreqs
segGrowthPC$Growth <- "Segmental"
# Add in lengths in segGrowthPC not in neurite lengths (dendLengthCounts) with 'x' (weight) of 0
addLengths <- unique(segGrowthPC$SeqLen)[!unique(segGrowthPC$SeqLen)%in%rownames(dendLengthCounts)]
if (!is.null(addLengths)){
  dendLengthCounts <- rbind(dendLengthCounts,data.frame(Group.1=addLengths,x=rep(0,length(addLengths))))
  rownames(dendLengthCounts) <- dendLengthCounts[,1]
}
# Sample growth tree data
segGrowthPC <- sample.rows(segGrowthPC,2*nrow(plotdf),prob=dendLengthCounts[as.character(segGrowthPC$SeqLen),"x"],replace=T)

# Tree Shapes
treeShapePC <- treeShapeCFreqs
treeShapePC$Growth <- "Tree Shapes"
# Add in lengths in biasedSegPC not in neurite lengths (dendLengthCounts) with 'x' (weight) of 0
addLengths <- unique(treeShapePC$SeqLen)[!unique(treeShapePC$SeqLen)%in%rownames(dendLengthCounts)]
if (!is.null(addLengths)){
  dendLengthCounts <- rbind(dendLengthCounts,data.frame(Group.1=addLengths,x=rep(0,length(addLengths))))
  rownames(dendLengthCounts) <- dendLengthCounts[,1]
}
# Sample growth tree data
treeShapePC <- sample.rows(treeShapePC,2*nrow(plotdf),prob=dendLengthCounts[as.character(treeShapePC$SeqLen),"x"],replace=T)

# Mix biased terminal growth with pure segmental growth
plotdf2 <- extremeQSGrowthKmers[extremeQSGrowthKmers$Growth=="Q0S0.4",c("PercentC","Length","Growth")]
colnames(plotdf2) <- c("C","SeqLen","Group")
plotdf2$Group <- as.character(plotdf2$Group)
plotdf2 <- rbind(plotdf2,cbind(segmentGrowCFreqs,Group=rep("Segmental Growth",nrow(termGrowCFreqs))))
plotdf2[plotdf2$Group == "Q0S0.4","Group"] <- "Low-order Biased Terminal Growth"
tmp <- treeShapeCFreqs
tmp$Group <- "Tree Shapes"
plotdf2 <- rbind(plotdf2,tmp)
#pcVsSeqLenLevels <- c("Axon","Dendrite","Segmental Growth","Tree Shapes","Low-order Biased Terminal Growth")
pcVsSeqLenLevels <- c("Axon","Dendrite","Low-order Biased Terminal Growth","Segmental Growth","Tree Shapes")
plotdf2$Group <- factor(plotdf2$Group,levels=pcVsSeqLenLevels,ordered=T)
plotdf$Group <- factor(as.character(plotdf$Group),levels=pcVsSeqLenLevels,ordered=T)

## %C by Sequence Length

#groupColors <- c("#DD00DD","#2222CC", "#666666", "#999999", "#333333")
groupColors <- c("#DD00DD","#0000EE", "#666666", "#999999", "#333333")
names(groupColors) <- pcVsSeqLenLevels

# Plot C frequency smooths: by arbor type and with randoms
gg_cByLen <- ggplot(subset(plotdf,SeqLen<250),aes(x = SeqLen, y = C, colour = Group, group=Group)) + 
#   stat_smooth(method = "loess", span=.3, se = TRUE,level = 0.95, aes(fill = Group),size=1)+
#   stat_smooth(data=plotdf2, method = "loess", span=.3, se = FALSE,size=0.7)+
  stat_smooth(aes(fill = Group),size=1)+
  stat_smooth(data=plotdf2, se = FALSE,size=0.7)+
  scale_colour_manual(values=groupColors,guide=F)+
  scale_fill_manual(values=groupColors,guide=F)+
  coord_cartesian(xlim=c(0,180), ylim = c(.2, .7))+
  labs(colour="Arbor Type",fill="Arbor Type",x="Sequence Length", y="Percent C")+
  getTheme(25)
ggsave(gg_cByLen,file=paste(figDir,"PercentCByLen-022215.png",sep=""))
#ggsave(gg_cByLen,file=paste(figDir,"PercentCByLen-0.3-103014.pdf",sep=""))

## Add sequence length distributions to %C vs SeqLen plot
seqLenDF <- motifMetaData[rownames(dendriteCFreq),c("ArborType","SeqLen"),drop=F]
seqLenDF$ArborType <- "Dendrite"
seqLenDF <- rbind(seqLenDF,motifMetaData[rownames(apicalCFreq),c("ArborType","SeqLen"),drop=F])
seqLenDF <- rbind(seqLenDF,motifMetaData[rownames(axonCFreq),c("ArborType","SeqLen"),drop=F])

gg_seqLen <- ggplot(subset(seqLenDF,ArborType!="ApicalDendrite"&SeqLen<=250),aes(SeqLen,fill=ArborType,alpha=.05))+geom_density(linetype=0)+
  scale_fill_manual(values=groupColors)+
  scale_y_continuous(breaks=c(0,.05),minor_breaks=0.023)
ggsave(gg_seqLen,file=paste(figDir,"SeqLenDistributions_AxDend.png",sep=""))
#ggsave(gg_seqLen,file=paste(figDir,"SeqLenDistributions_AxDend.pdf",sep=""))


#### Figure 3d: Percent C by position ####

datasetTitle <- "NeuroMorpho.Org"
cPosFilename <- paste(motifDataDirectory,"RawKmerFreqs/CPercentPerPos-NM_SpurLen2D-SizeSTL.csv",sep='')
percentCByPos <- read.csv(cPosFilename,stringsAsFactor=F)
percentCByPos$Group <- motifMetaData[percentCByPos$Sequence,"ArborType"]
percentCByPos$Group[percentCByPos$Group=="BasalDendrite"] <- "Dendrite"
#datasetTitle <- "Segmental Growth"
cPosFilename <- paste(motifDataDirectory,"/CPercentPerPos-SegmentalGrowth-SizeSTL.csv",sep='')
percentCByPosTmp <- read.csv(cPosFilename,stringsAsFactor=F)
percentCByPosTmp$Group <- "Segmental Growth"
percentCByPos <- rbind(percentCByPos,percentCByPosTmp)
#datasetTitle <- "Terminal Growth"
#cPosFilename <- paste(motifDataDirectory,"/CPercentPerPos-TerminalGrowth-SizeSTL.csv",sep='')
cPosFilename <- paste(motifDataDirectory,"/CPercentPerPos-BiasedTerminalGrowth-SizeSTL.csv",sep='')
percentCByPosTmp <- read.csv(cPosFilename,stringsAsFactor=F)
percentCByPosTmp$Group <- "Terminal Growth"
percentCByPos <- rbind(percentCByPos,percentCByPosTmp)
rm(percentCByPosTmp)

datasetTitle <- "Combined"

# Get rows that are dendrites or growth from a particular range of lengths
percentCByPosSubset <- subset(percentCByPos,Group=="Dendrite"&Length<=150)
percentCByPosSubset$Label <- "18-50"
percentCByPosSubset$Label[percentCByPosSubset$Length>50] <- "51-90"
percentCByPosSubset$Label[percentCByPosSubset$Length>90&percentCByPosSubset$Length<=150] <- "91-150"

# Generate weights for each sequence length so growth sets seq length distribution
# match dendrite length distribution
dendLengths <- subset(percentCByPosSubset,Position==18)$Length
dendLengthCounts <- aggregate(dendLengths,by=list(dendLengths),length)
rownames(dendLengthCounts) <- dendLengthCounts[,1]
dendLengthCounts$weight <- dendLengthCounts$x*max(dendLengths)/dendLengthCounts[,1]

percentCByPosTmp <- subset(percentCByPos,Group%in%c("Terminal Growth","Segmental Growth")&Length%in%dendLengths)
percentCByPosTmp <- percentCByPosTmp[sample(1:nrow(percentCByPosTmp),nrow(percentCByPosTmp)/6,prob=dendLengthCounts[as.character(percentCByPosTmp$Length),"weight"]),]
percentCByPosTmp$Label <- percentCByPosTmp$Group
percentCByPosSubset <- rbind(percentCByPosSubset,percentCByPosTmp)

percentCByPosSubset$Label <- factor(percentCByPosSubset$Label,levels=c("18-50","51-90","91-150","Terminal Growth","Segmental Growth"),ordered=T)
percentCByPosSubset$RelPosition <- percentCByPosSubset$Position/percentCByPosSubset$Length
pcPalette <- c("#321093","#2222CC","#204AFF","#666666","#999999")

cByPosPlot <- ggplot(sample.rows(percentCByPosSubset,nrow(percentCByPosSubset)/4),aes(x=RelPosition,y=IsC,group=Label,colour=Label,fill=Label))+
  stat_smooth(method = "loess", span=.4, se = T,level = 0.95)+
  scale_colour_manual(values=pcPalette)+
  scale_fill_manual(values=pcPalette)+
  labs(colour="Length Range",fill="Length Range",x="Relative Position", y="Percent C", title=paste("%C Over Positions - ",datasetTitle))+
  getTheme()
ggsave(cByPosPlot,file=paste(motifDataDirectory,"PercentCByRelPos-wSE-DendGrowthCombo-Span.4_CV7_2.pdf",sep=""))


#### Supp Fig x: Dendrite %C vs SeqLen by Cell Type ####

# %C for dendrites
plotdf <- dendriteCFreq
cellGroup <- motifMetaData[rownames(dendriteCFreq),"Class1"]
cellGroup[arborGroup=="Principal cell"] <- motifMetaData[rownames(dendriteCFreq)[cellGroup=="Principal cell"],"Class2"]
#plotdf$Group <- factor(arborGroup,levels=c("Apical","Axon","Dendrite"),ordered=T)
plotdf$Group <- cellGroup


# Plot C frequency smooths: by arbor type and with randoms
gg_cByLen <- ggplot(subset(plotdf,SeqLen<250),aes(x = SeqLen, y = C, colour = Group, group=Group)) + 
  stat_smooth(method = "loess", span=.3, se = TRUE,level = 0.95, aes(fill = Group),size=1)+
#  scale_colour_manual(values=groupColors,guide=F)+
#  scale_fill_manual(values=groupColors,guide=F)+
  coord_cartesian(xlim=c(0,180), ylim = c(.2, .7))+
  labs(colour="Arbor Type",fill="Cell Type",x="Sequence Length", y="Percent C")+
  getTheme(25)
ggsave(gg_cByLen,file=paste(figDir,"PercentCByLen-DendriteSubtype-0.3.png",sep=""))
#ggsave(gg_cByLen,file=paste(figDir,"PercentCByLen-DendriteSubtype-0.3.pdf",sep=""))

## Add sequence length distributions to %C vs SeqLen plot - INCOMPLETE
# seqLenDF <- motifMetaData[rownames(dendriteCFreq),c("ArborType","SeqLen"),drop=F]
# seqLenDF$CellType <- cellGroup
# 
# gg_seqLen <- ggplot(subset(seqLenDF,SeqLen<=250),aes(SeqLen,fill=CellType,alpha=.05))+geom_density(linetype=0)+
#   scale_fill_manual(values=groupColors)+
#   scale_y_continuous(breaks=c(0,.05),minor_breaks=0.023)
# ggsave(gg_seqLen,file=paste(figDir,"SeqLenDistributions_AxDend.png",sep=""))
# #ggsave(gg_seqLen,file=paste(figDir,"SeqLenDistributions_AxDend.pdf",sep=""))


#### Figure 4c: Num kmers by k ####
numSoloKmers <- c(3,9,25,71,199)
numSoloUnconstrainedKmers <- c(1,9,24,69,194)
numUniqueComboKmers <- c(3,9,41,98,246)
numUniqueComboUnconstrainedKmers <- c(1,9,39,93,236)

numKmers <- data.frame(Solo=numSoloKmers,
                       SoloUnconstrained=numSoloUnconstrainedKmers,
                       UniqueCombo=numUniqueComboKmers,
                       UniqueComboUnconstrained=numUniqueComboUnconstrainedKmers)
rownames(numKmers) <- 1:5
numKmersLong <- reshape(numKmers,direction="long",varying=list(colnames(numKmers)),v.names="NumberOfKmers",timevar="Group",times=colnames(numKmers),idvar="k",ids=rownames(numKmers))
CairoWin()
ggplot(numKmersLong,aes(x=k,y=NumberOfKmers,group=Group,colour=Group))+
  geom_point(size=3.2)+
  geom_line(size=.9)+
  getTheme()

#STL+LTS unconstrained
ggplot(subset(numKmersLong,Group=="UniqueComboUnconstrained"),aes(x=k,y=NumberOfKmers,group=Group))+
  geom_point(size=3.2)+
  geom_line(size=.9)+
  scale_y_continuous(trans=log_trans(), breaks=c(1,10,20,40,80,160))+
  getTheme()

#### Figure 4d: Kmers and Surrogates ####

## Generate surrogate distribution histograms and/or CDFs
# Get distributions using linux commands on surrogates file
# grep -A 100 "MD070408-ApicalDendrite" surrogatesSTL-100sets_B8_Set13.dat | awk -F "AC" '{print NF-1}' | paste -d, -s
acDist <- c(4,1,4,4,4,3,4,4,2,4,2,4,3,3,3,4,2,2,2,5,3,4,2,1,2,2,4,4,4,3,2,2,6,1,2,4,3,3,4,5,3,3,4,3,3,3,5,2,2,4,3,3,2,3,4,3,3,6,2,3,3,1,4,1,3,2,4,1,4,3,3,3,4,1,5,2,3,2,3,2,3,1,1,3,3,4,2,1,2,3,3,6,4,4,3,1,2,3,3,3)
atDist <- c(2,4,2,1,2,3,2,2,3,2,3,2,3,2,3,2,4,3,3,1,3,2,3,4,3,3,2,2,2,2,4,3,0,4,3,2,3,2,2,1,2,2,2,2,2,2,1,4,3,1,3,2,3,2,2,3,3,0,3,3,2,4,1,3,2,3,2,3,1,3,2,3,1,5,1,4,2,3,3,3,2,4,3,3,3,1,3,5,4,3,1,0,2,2,3,4,2,2,3,2)

atCDFPlot <- ggplot(data.frame(ATCount=atDist2),aes(x=ATCount))+
  stat_ecdf()+
  scale_x_continuous(breaks=(0:5))+
  coord_cartesian(xlim=c(-.5,5.5))
ggsave(paste(figDir,"AT_CDF.png",sep="",atCDFPlot)

atCount <- 4
atRank <- sum(atDist < atCount) + (1+sum(atDist == atCount))/2
atPR <- (atRank-.5)/100

atRank2 <- sum(atDist2 < atCount) + (1+sum(atDist2 == atCount))/2
atPR2 <- (atRank2-.5)/100
(sum(atDist2 < atCount) + (1+sum(atDist2 == atCount))/2 - .5)/100

acCDFPlot <- ggplot(data.frame(ACCount=acDist),aes(x=ACCount))+
  stat_ecdf()+
  scale_x_continuous(breaks=(0:6))+
  coord_cartesian(xlim=c(-.5,6.5))
ggsave(paste(figDir,"AC_CDF.png",sep="",acCDFPlot)
       

acCount <- 2
acRank <- sum(acDist < acCount) + (1 + sum(acDist == acCount))/2
acPR <- (acRank-.5)/100

x11(width=2.5,height=3)
ggplot(data.frame(ATCount=atDist),aes(ATCount))+
  geom_histogram(binwidth=.5)+
  scale_y_continuous(breaks=4*(0:4))+
  scale_x_continuous(breaks=(0:4))+
  coord_cartesian(xlim=c(-.5,4.75))

## Example Neurite Dendrogram

# For AC, AT
treePalette <- c("black","orange","magenta","red","green2","red","pink")

# Surrogate dendrograms with kmer highlights
# See org.cn3.ns.scripts.FigureDataManager - main function creates AC&AT-colored SWC and Newick files
#   - currently writes to /home/tgillett/results/
newickDir <- "/home/tgillett/results/Newick/"
neuriteName <- "MD070408-ApicalDendrite"
kmer <- "AC"
kmer2 <- "AT"

labelType <- "MONOMER_AND_TYPE"
filename <- paste(newickDir,"/NeuriteDendrogram_",neuriteName,"_",kmer,"&",kmer2,"_",labelType,".newick",sep="")
tree <- read.tree(filename)
png(paste(figDir,"/NeuriteDendrogram_",neuriteName,"_",kmer,"&",kmer2,"-CD.png",sep=""),1120,425)
plotTreeByLabels(tree,labelSplit="-",distinctColors=F,treePalette=treePalette,colorDown=T,edge.width=3,font=2,label.offset=-0.7,cex=1.7)
dev.off()

## Example Surrogate Dendrograms

numFigs <- 6
for (i in 1:numFigs){
  #filename <- paste("C:/Documents and Settings/gillette/My Documents/research/papers & presentations/PASTA Paper/Paper1 - Motifs/figures/Components/SurrogateDendrograms_",neuriteName,"_",kmer,"&",kmer2,"_",i,"_",labelType,".newick",sep="")
  filename <- paste(figDir,"/SurrogateDendrograms_",neuriteName,"_",kmer,"&",kmer2,"_",i,"_",labelType,".newick",sep="")
  tree <- read.tree(filename)
  #png(paste("C:/Documents and Settings/gillette/My Documents/research/papers & presentations/PASTA Paper/Paper1 - Motifs/figures/Components/SurrogateDendrogramsCD_",kmer,"_",i,".png",sep=""),400,300)
  png(paste(figDir,"/SurrogateDendrogramsCD_",kmer,"_",i,".png",sep=""),410,300)
  #plotTreeByLabels(tree,distinctColors=T,treePalette=treePalette,edge.width=2,font=2,label.offset=-.5,srt=-90,cex=1.7,no.margin=T)
  plotTreeByLabels(tree,labelSplit="-",colorDown=T,distinctColors=F,show.node.label=T,treePalette=treePalette,edge.width=2,font=2,label.offset=-.5,srt=-90,cex=1.7,no.margin=T)
  dev.off()
  #plotTreeByLabels(tree,show.node.label=F,distinctColors=T,treePalette=treePalette)
}

#### K-mer PR distributions ####

## AT, AA, ACT
# Separately
CairoWin()
ggplot(prDataB8Combo_40[[2]][allNeuriteNames,],aes(AT,fill=usedMetaData[allNeuriteNames,"ArborType"]))+
  geom_density(linetype=0,size=.9,alpha=.35)+
  scale_fill_manual(values=samplePalette)+
  labs(colour="Arbor Type",x="AT Percentile Rank", y="Density")

CairoWin()
ggplot(prDataB8Combo_40[[3]][allNeuriteNames,],aes(ACT,fill=usedMetaData[allNeuriteNames,"ArborType"]))+
  #geom_histogram(linetype=0,size=.9,alpha=.35)+
  geom_density(linetype=0,size=.9,alpha=.35)+
  scale_fill_manual(values=samplePalette)+
  labs(colour="Arbor Type",x="ACT Percentile Rank", y="Density")

# Together
plotdf <- cbind(prDataB8Combo_40[[2]][allNeuriteNames,"AT",drop=F],usedMetaData[allNeuriteNames,"ArborType",drop=F])
colnames(plotdf)[1] <- "PercentileRank"
plotdf$Kmer <- "AT"
plotdftmp <- cbind(prDataB8Combo_40[[2]][allNeuriteNames,"AA",drop=F],usedMetaData[allNeuriteNames,"ArborType",drop=F])
colnames(plotdftmp)[1] <- "PercentileRank"
plotdftmp$Kmer <- "AA"
plotdf <- rbind(plotdf,plotdftmp)
plotdftmp <- cbind(prDataB8Combo_40[[3]][allNeuriteNames,"ACT",drop=F],usedMetaData[allNeuriteNames,"ArborType",drop=F])
colnames(plotdftmp)[1] <- "PercentileRank"
plotdftmp$Kmer <- "ACT"
plotdf <- rbind(plotdf,plotdftmp)

plotdftmp$Kmer <- factor(plotdftmp$Kmer,levels=c("AT","AA","ACT"),ordered=T)

ggplot(plotdf,aes(x=Kmer,y=PercentileRank,fill=ArborType))+
  geom_boxplot(notch=T)+
  scale_fill_manual(values=samplePalette)+
  labs(fill="Arbor Type",x="Kmer", y="Percentile Ranks")

#### Figure 5a: Percent Captured ####

## Core analysis: All neurites, and neurites by arbor type (axon, dendrite, apical dendrite)
allNeuritesSpec <- list(ExperCond="Control")
axonsSpec <- list(ExperCond="Control",ArborType="Axon")
dendritesSpec <- list(ExperCond="Control",ArborType=c("Dendrite","BasalDendrite"))
apicalsSpec <- list(ExperCond="Control",ArborType="ApicalDendrite")

pyrAxonsSpec <- list(ExperCond="Control",ArborType="Axon",Class2="Pyramidal cell")
pyrDendritesSpec <- list(ExperCond="Control",ArborType=c("Dendrite","BasalDendrite"),Class2="Pyramidal cell")
pyrApicalsSpec <- list(ExperCond="Control",ArborType="ApicalDendrite",Class2="Pyramidal cell")

specifications <- list(allNeuritesSpec,axonsSpec,apicalsSpec,dendritesSpec)
names(specifications) <- c("All Neurites","Axons","ApicalDendrites","Dendrites")

pyrSpecs <- list(pyrAxonsSpec,pyrApicalsSpec,pyrDendritesSpec)
names(pyrSpecs) <- c("Axons","ApicalDendrites","Dendrites")

#offsets <- c(0.6,0.5,0.65,0.75)
offsets <- c(0.5,0.5,0.5,0.5)

currOrder <- "STL"
kRange <- 1:5
myResults <- matrix(0,nrow=2*length(specifications),ncol=length(kRange))
colnames(myResults) <- kRange
rownames(myResults) <- 1:nrow(myResults)
diffSEs <- matrix(0,nrow=length(specifications),ncol=length(kRange))
colnames(diffSEs) <- kRange
rownames(diffSEs) <- 1:nrow(diffSEs)
resultOffset <- 0 # 8
minSize <- 18
maxSize <- NaN
removeATTFlag <- TRUE
ltsOnly <- FALSE
useAlternate <- F

# Set data source
myData <- prDataB8Combo
mySurrogateData <- prSurrogateDataB8Combo
resultTitle <- "With LTS"

myData <- prDataB8STL
mySurrogateData <- prSurrogateDataB8STL
resultTitle <- ""

for (k in kRange){
  sharedSeqs <- intersect(rownames(myData[[k]]),rownames(mySurrogateData[[k]]))
  myData[[k]] <- myData[[k]][sharedSeqs,,drop=F]
  mySurrogateData[[k]] <- mySurrogateData[[k]][sharedSeqs,,drop=F]
}

#for (i in 1:length(specifications)){
#specification <- specifications[[i]]
specification <- specifications[[1]] # All Arbors
realMeans <- kRange
names(realMeans) <- kRange
surrogateMeans <- kRange
realSDs <- kRange
surrogateSDs <- kRange
diffSE <- kRange
# Get the spec name
if (is.null(names(specifications))){
  specName <- getSetString(specification)
}else{
  specName <- names(specifications)[i]
}
for (k in kRange){
  # Get the data subset
  subData <- getSubset2(myData[[k]],specification,motifMetaData,minLength=minSize,maxLength=maxSize)
  subSurrogateData <- getSubset2(mySurrogateData[[k]],specification,motifMetaData,minLength=minSize,maxLength=maxSize)
  
  # Remove ATT-related kmers
  if (removeATTFlag && k >= 3){
    subData <- subData[,removeATT(currOrder,colnames(subData)),drop=F]
    subSurrogateData <- subSurrogateData[,removeATT(currOrder,colnames(subSurrogateData)),drop=F]
  }
  # Determine which columns to run
  if (ltsOnly){
    inds <- grep("L-",colnames(subData))
    subData <- subData[,inds,drop=F]
    subSurrogateData <- subSurrogateData[,inds,drop=F]
  }
  
  #outlierData <- calculatePROutlierRatio(subData,k=k,alpha=0.05)
  #surrogateOutlierData <- calculatePROutlierRatio(subSurrogateData,k=k,alpha=0.05)
  captured <- apply(subData,1,function(x){mean(x > .025 & x < .975)})
  capturedSurrogate <- apply(subSurrogateData,1,function(x){mean(x >= .025 & x <= .975)})
  
  kInd <- k-min(kRange)+1
  realMeans[kInd] <- mean(captured)
  realSDs[kInd] <- sd(captured)
  
  surrogateMeans[kInd] <- mean(capturedSurrogate)
  surrogateSDs[kInd] <- sd(capturedSurrogate)
  diffSE[kInd] = sqrt((var(captured)+var(capturedSurrogate)+cov(captured,capturedSurrogate))/length(captured))
}

myResults[resultOffset + 2*i-1,] <- realMeans
rownames(myResults)[resultOffset + 2*i-1] <- paste(resultTitle,specName,"Real")

myResults[resultOffset + 2*i,] <- surrogateMeans-realMeans
rownames(myResults)[resultOffset + 2*i] <- paste(resultTitle,specName,"Diff")

diffSEs[resultOffset+i,] <- diffSE
rownames(diffSEs)[resultOffset+i] <- paste(resultTitle,specName,"Diff SEs")

realSEs <- realSDs/sqrt(nrow(subData))
surrogateSEs <- surrogateSDs/sqrt(nrow(subData))
  
# Bar plot of results
if (is.nan(maxSize)){
  title <- paste(specName," (Min #bifs: ",minSize,")",sep="")
}else{
  title <- paste(specName," (Min: ",minSize,"; Max: ",maxSize,")",sep="")
}
title <- specName
if (useAlternate){
  realFlip <- 1-realMeans
  surrFlip <- 1-surrogateMeans
  vals <- realFlip/surrFlip
  barplot(vals,main=title,xlab="k",ylab="Real-Baseline captured ratio")
  realSE <- realSDs/sqrt(nrow(subData))
  surrSE <- surrogateSDs/sqrt(nrow(subData))
  errs <- sqrt((surrSE*realFlip/surrFlip^2)^2 + (realSE/surrFlip)^2)
  #errs <- sqrt(realSDs^2/nrow(subData) + surrogateSDs^2/nrow(subData))
  error.bar(1.2*(kRange-min(kRange))+.7,vals,errs)
}else{
  #offset <- floor(20*min(realMeans)-.5)/20
  offset <- offsets[i]
  png(paste("/home/tgillett/results/PercentCaptured_",gsub(" ","",specName),"_SurrogateN100_Min",offset,".png",sep=""),width=800,height=800)
  barplot(realMeans-offset,ylim=c(offset,1),offset=offset,main=title,xlab="k",ylab="Average proportion of k-mers captured by Baseline")
  error.bar(1.2*(kRange-min(kRange))+.7,realMeans,realSDs/sqrt(nrow(subData)))
  barplot(1-surrogateMeans,offset=surrogateMeans,col="gray50",add=T)
  error.bar(1.2*(kRange-min(kRange))+.7,surrogateMeans,surrogateSDs/sqrt(nrow(subData)))
  dev.off()
}

#### Figures 5&6: Dimers and Trimers ####

## Load k-mer PR data
kData <- prDataB8Combo[[k]]
#kSurrogateData <- mySurrogateData[[k]]

validKmers <- removeATT("STL",colnames(kData))
kData <- kData[,validKmers]
#kData$Group <- usedMetaData[rownames(kData),"ArborType"]

# Convert from wide to long
myDataLong <- reshape(kData,direction="long",varying=list(colnames(kData)),times=colnames(kData),idvar="SeqName",ids=rownames(kData),timevar="Kmer",v.names="PercentileRank")
myDataLong <- myDataLong[!myDataLong$SeqName%in%flaggedNeurites&myDataLong$SeqName%in%rownames(usedMetaData),]
myDataLong$Group <- usedMetaData[myDataLong$SeqName,"ArborType"]
myDataLong[myDataLong$Group=="BasalDendrite","Group"] <- "Dendrite"

# Determine kmer averages to sort in ascending order
kmerMeans <- with(subset(myDataLong,Group!="ApicalDendrite"),aggregate(PercentileRank,by=list(Kmer),FUN=mean))
kmerFactor <- factor(myDataLong$Kmer,levels=kmerMeans[order(kmerMeans[,2]),1])
myDataLong$Kmer <- kmerFactor

lowsem <- function(x){ return(mean(x) - sem(x)) }
highsem <- function(x){ return(mean(x) + sem(x)) }

# Dimers for all axons and dendrites
dimerPlot <- ggplot(subset(myDataLong,Group!="ApicalDendrite"),aes(Kmer,PercentileRank,width=0.56))+
  stat_summary(fun.y=mean, geom="bar")+
  stat_summary(fun.y=mean, fun.ymin=lowsem, fun.ymax=highsem, geom="linerange",color = 'black')+
  coord_cartesian(ylim=ylimsByK[[k]])+
  paper1Theme+theme(
    panel.grid.major.y = element_line(colour = "grey40"),
    axis.text.x = element_text(angle = 90, hjust = 0),
    axis.line = element_line(size = 1), 
    axis.ticks=element_line(size=1), 
    axis.ticks.length=unit(3,"mm"))+
  scale_y_continuous(breaks=(1:4/5))
ggsave(paste(figDir,"DimerPercentileRanks_3.png",sep=""),dimerPlot,width=3.5,height=2,units="in",dpi=600)

# Dimers by arbor type (all 3 types)
dimerPlot <- ggplot(myDataLong,aes(Kmer,PercentileRank,width=0.56,fill=Group))+
  stat_summary(fun.y=mean, geom="bar", position=position_dodge(width = 0.65))+
  scale_fill_manual(values=arborTypeColors, guide=F)+
  stat_summary(fun.y=mean, fun.ymin=lowsem, fun.ymax=highsem, geom="linerange",color = 'black', position=position_dodge(width = 0.65))+
  coord_cartesian(ylim=ylimsByK[[k]])+
  paper1Theme+theme(
    panel.grid.major.y = element_line(colour = "grey40"),
    axis.text.x = element_text(angle = 90, hjust = 0),
    axis.line = element_line(size = 1), 
    axis.ticks=element_line(size=1), 
    axis.ticks.length=unit(3,"mm"))+
  scale_y_continuous(breaks=(1:4/5))
ggsave(paste(figDir,"DimerPercentileRanks-ByArborType.png",sep=""),dimerPlot,width=3.5,height=2,units="in",dpi=600)

# Add low-order biased terminal and segmental grown tree dimers, length matched to axon and dendrite data
kDataNames <- rownames(subset(usedMetaData[rownames(kData),],ArborType!="ApicalDendrite"))
lengths <- usedMetaData[kDataNames,"SeqLen"]
dimers <- c("AA","AC","AT","CA","CC","CT","TA","TC","TT")
growthSub <- subset(selectQSGrowthKmers,Growth%in%c("Q0S0.4","Segmental"))[,c("Growth","Length",dimers)]
colnames(growthSub)[1:2] <- c("Group","SeqLen")
#require(data.table)
require(plyr)
# Make length map
lengths <- aggregate(rownames(kData),by=list(length=subset(usedMetaData,Protocol!="Acute culture"&Protocol!="Culture"&ExperCond=="Control")[rownames(kData),"SeqLen"]),length)
lengths <- subset(lengths,length <= max(growthSub$SeqLen))

sampleFromGrowthSet <- function(lengths,growthSet){
  growthDF <- NULL
  for (len in lengths$length){
    tmpG <- sample.rows(growthSet,lengths[lengths$len==len,"x"],replace=T)
    if (is.null(growthDF)){
      growthDF <- tmpG
    }else{
      growthDF <- rbind(growthDF,tmpG)
    }
  }
  return(growthDF)
}
growthSegmentals <- sampleFromGrowthSet(lengths,subset(growthSub,Group=="Segmental"))
growthTerms <- sampleFromGrowthSet(lengths,subset(growthSub,Group=="Q0S0.4"))
growthFinal <- rbind(growthSegmentals,growthTerms)

growthFinalLong <- reshape(growthFinal,direction="long",varying=list(dimers),times=dimers,timevar="Kmer",v.names="PercentileRank")
growthFinalLong <- growthFinalLong[,c("Kmer","PercentileRank","id","Group")]
growthFinalLong$Group[growthFinalLong$Group=="Q0S0.4"] <- "Terminal (order biased)"
colnames(myDataLong)[3:4] <- c("id","Group")

# Determine kmer averages to sort in ascending order
#kmerMeans <- aggregate(myDataLong$PercentileRank,by=list(myDataLong$Kmer),FUN=mean)
#kmerFactor <- factor(myDataLong$Kmer,levels=kmerMeans[order(kmerMeans[,2]),1],ordered=T)
#myDataLong$Kmer <- kmerFactor

  
png("/home/tgillett/results/MotifAnalysis/DimerPercentileRanks_withGrownTrees.png",width=800,height=400)
# Bar plot
ggplot(subset(rbind(myDataLong,growthFinalLong),Group!="ApicalDendrite"),aes(Kmer,PercentileRank,fill=Group))+
  stat_summary(fun.y=mean, geom="bar",position="dodge",colour="white")+
#  stat_summary(fun.y=mean, fun.ymin=lowsd, fun.ymax=highsd, geom="linerange", position="dodge",color = 'black')+
  scale_fill_manual(values=noApicalPal[c(1,2,5,3)])+
  theme(axis.text.x = element_text(angle = 90, hjust = 0))+
  getTheme()
dev.off()

# Dimers by arbor type (all 3 types)
dimerPlot <- ggplot(myDataLong,aes(Kmer,PercentileRank,width=0.56,fill=Group))+
  stat_summary(fun.y=mean, geom="bar", position=position_dodge(width = 0.65))+
  scale_fill_manual(values=arborTypeColors, guide=F)+
  stat_summary(fun.y=mean, fun.ymin=lowsem, fun.ymax=highsem, geom="linerange",color = 'black', position=position_dodge(width = 0.65))+
  coord_cartesian(ylim=ylimsByK[[k]])+
  paper1Theme+theme(
    panel.grid.major.y = element_line(colour = "grey40"),
    axis.text.x = element_text(angle = 90, hjust = 0),
    axis.line = element_line(size = 1), 
    axis.ticks=element_line(size=1), 
    axis.ticks.length=unit(3,"mm"))+
  scale_y_continuous(breaks=(1:4/5))
ggsave(paste(figDir,"DimerPercentileRanks-ByArborType.png",sep=""),dimerPlot,width=3.5,height=2,units="in",dpi=600)

# Trimers for all neurites (axons and dendrites)
# Bar plot
#trimerPlot <- ggplot(subset(myDataLong,Group!="ApicalDendrite"&Kmer%in%c("AAT","ATA","CTC","TCT","CCA","AAC","CAC","L-CAC","AAA","CCT","ATC","CTA","CAT")),aes(Kmer,PercentileRank,width=0.65))+
trimerPlot <- ggplot(subset(myDataLong,Group!="ApicalDendrite"),aes(Kmer,PercentileRank,width=0.65))+
  stat_summary(fun.y=mean, geom="bar",colour="white")+
  stat_summary(fun.y=mean, fun.ymin=lowsd, fun.ymax=highsd, geom="linerange",color = 'black')+
  coord_cartesian(ylim=ylimsByK[[k]])+
  paper1Theme+theme(
    panel.grid.major.y = element_line(colour = "grey40"),
    axis.text.x = element_text(angle = 90, hjust = 0),
    axis.line = element_line(size = 1), 
    axis.ticks=element_line(size=1), 
    axis.ticks.length=unit(3,"mm"))+
  scale_y_continuous(breaks=((2:7)/10))
ggsave(paste(figDir,"TrimerPercentileRanks-All-AxonsAndDendritesCombined.png",sep=""),trimerPlot,width=6,height=5,units="in",dpi=200)

trimerPlot <- ggplot(subset(myDataLong,Group!="ApicalDendrite"),aes(Kmer,PercentileRank,width=0.65,fill=Group))+
  stat_summary(fun.y=mean, geom="bar",colour="white", position=position_dodge(width = 0.65))+
  scale_fill_manual(values=arborTypeColors, guide=F)+
  stat_summary(fun.y=mean, fun.ymin=lowsd, fun.ymax=highsd, geom="linerange",color = 'black', position=position_dodge(width = 0.65))+
  coord_cartesian(ylim=ylimsByK[[k]])+
  paper1Theme+theme(
    panel.grid.major.y = element_line(colour = "grey40"),
    axis.text.x = element_text(angle = 90, hjust = 0),
    axis.line = element_line(size = 1), 
    axis.ticks=element_line(size=1), 
    axis.ticks.length=unit(3,"mm"))+
  scale_y_continuous(breaks=((2:7)/10))
ggsave(paste(figDir,"TrimerPercentileRanks-All-AxonsAndDendrites.png",sep=""),trimerPlot,width=10.5,height=5,units="in",dpi=200)

samplePalette <- c("#CC22CC","#22CCCC","#2222CC", "#999999", "#555555", "#444477", "#AA88AA", "#BA99AA")
# Trimer motifs focusing on axons and dendrites
png("/home/tgillett/results/MotifAnalysis/TrimerAxonDendriteMotifs.pdf",width=800,height=400)
# Bar plot
ggplot(subset(myDataLong,Group!="ApicalDendrite"&Kmer%in%c("TCT","ACC","ACT","TCC","TCA")),aes(Kmer,PercentileRank,fill=ArborType,width=0.56))+
  stat_summary(fun.y=mean, geom="bar",colour="white", position=position_dodge(width = 0.65))+
  #stat_summary(fun.y=mean, fun.ymin=lowsd, fun.ymax=highsd, geom="errorbar",color = 'black')+
  stat_summary(fun.y=mean, fun.ymin=lowsd, fun.ymax=highsd, position=position_dodge(width = 0.65), geom="linerange",color = 'black')+
  scale_fill_manual(values=samplePalette[-1])+
  coord_cartesian(ylim=ylimsByK[[k]])+
  theme(axis.text.x = element_text(angle = 90, hjust = 0))
dev.off()


# Trimer motifs focusing on only axons
png("/home/tgillett/results/MotifAnalysis/TrimerAxonMotifs.pdf",width=800,height=400)
# Bar plot
ggplot(subset(myDataLong,!Group%in%c("Dendrite","ApicalDendrite")&Kmer%in%c("TCT","ACC","ACT","TCC","TCA")),aes(Kmer,PercentileRank,fill=ArborType,width=0.56))+
  stat_summary(fun.y=mean, geom="bar",colour="white", position=position_dodge(width = 0.65))+
  #stat_summary(fun.y=mean, fun.ymin=lowsd, fun.ymax=highsd, geom="errorbar",color = 'black')+
  stat_summary(fun.y=mean, fun.ymin=lowsd, fun.ymax=highsd, position=position_dodge(width = 0.65), geom="linerange",color = 'black')+
  scale_fill_manual(values=samplePalette[-1])+
  coord_cartesian(ylim=ylimsByK[[k]])+
  theme(axis.text.x = element_text(angle = 90, hjust = 0))
dev.off()

# Trimer motifs with bars for axons and pointrange style for dendrites
#png("/home/tgillett/results/MotifAnalysis/TrimerAxon&DendBoxMotifs.png",width=800,height=400)
# Bar plot
gg_specTrimerAxDend<-ggplot(subset(myDataLong,Group=="Axon"&Kmer%in%c("TCT","ACC","ACT","TCC","TCA")),aes(Kmer,PercentileRank,fill=Group,width=0.56))+
  stat_summary(fun.y=mean, geom="bar",colour="white", position=position_dodge(width = 0.65))+
  #stat_summary(fun.y=mean, fun.ymin=lowsd, fun.ymax=highsd, geom="errorbar",color = 'black')+
  stat_summary(fun.y=mean, fun.ymin=lowsd, fun.ymax=highsd, position=position_dodge(width = 0.65), geom="linerange",color = 'black')+
  stat_summary(
    data=subset(myDataLong,Group=="Dendrite"&Kmer%in%c("TCT","ACC","ACT","TCC","TCA")),
    geom="linerange", fun.y=mean,fun.ymin=lowsd, fun.ymax=highsd)+
#  stat_summary(
#    data=subset(myDataLong,Group=="Dendrite"&Kmer%in%c("TCT","ACC","ACT","TCC","TCA")),
#    geom="point", fun.y=mean,color="#2222CC")+
  #scale_fill_manual(values=samplePalette[-1])+
  scale_fill_manual(values=noApicalPal)+
  coord_cartesian(ylim=ylimsByK[[k]])+
  theme(axis.text.x = element_text(angle = 90, hjust = 0))
ggsave(gg_specTrimerAxDend,file="/home/tgillett/results/MotifAnalysis/TrimerAxon&DendLineMotifs_Magenta.pdf")
#dev.off()

## Determine surrogate bounds for baseline region
k<-2
kSurrogateData <- mySurrogateData[[k]][rownames(kData),]
# Remove apicals
kSurrogateData <- getSubset2(kSurrogateData,list(ArborType=c("Axon","Dendrite","BasalDendrite")),usedMetaData)
# Determine means and SEs for each kmer
kSurrogateSummaries <- apply(kSurrogateData,2,summary)
kSurrogateSEMs <- apply(kSurrogateData,2,sem)
# Determine max and mins across kmers
maxVal <- max(kSurrogateSummaries["Mean",]+kSurrogateSEMs)
minVal <- min(kSurrogateSummaries["Mean",]-kSurrogateSEMs)

#### Figure 5d&e: Dendrograms ####
kmer <- "CT"
labelType <- "MONOMER_AND_TYPE"

neuriteName <- "C190898A-I1-Axon"
treePalette <- c("black","","","",rgb(.91,0,.91))
filename <- paste(figDir,"/NeuriteDendrogram_",neuriteName,"_",kmer,"_",labelType,".newick",sep="")
tree <- read.tree(filename)
png(paste(figDir,"/NeuriteDendrogram_",neuriteName,"_",kmer,"-CD.png",sep=""),580,700)
plotTreeByLabels(tree,labelSplit="-",show.node.label=F,distinctColors=F,treePalette=treePalette,colorDown=T,edge.width=4,no.margin=T)
dev.off()

neuriteName <- "C190898A-I1-Dendrite"
treePalette <- c("black","","","",rgb(0,0,1))
filename <- paste(figDir,"/NeuriteDendrogram_",neuriteName,"_",kmer,"_",labelType,".newick",sep="")
tree <- read.tree(filename)
png(paste(figDir,"/NeuriteDendrogram_",neuriteName,"_",kmer,"-CD.png",sep=""),440,440)
plotTreeByLabels(tree,labelSplit="-",show.node.label=F,distinctColors=F,treePalette=treePalette,colorDown=T,edge.width=5,no.margin=T)
dev.off()

