#library("nortest")

if (systemType == "PC"){
  motifDataDirectory <- 'C:/Documents and Settings/gillette/My Documents/research/MotifAnalysis/'
}else{
  motifDataDirectory <- "/home/tgillett/results/"
}
#researchDir <- "C:/Documents and Settings/gillette/My Documents/research/"
#motifSourceDirectory <- 'C:/Documents and Settings/gillette/My Documents/My Dropbox/Motif Analysis/Motif Analysis R Files/'
#source('C:/Documents and Settings/gillette/My Documents/My Dropbox/RCode/Utils.r')
#percentileRankDataDirectory <- 'C:/Documents and Settings/gillette/My Documents/research/MotifAnalysis/PercentileRanks/'

defaultOrder <- "STL"
defaultMethod <- 8
defaultThreshold <- "0.0"
#defaultThreshType <- "Whole"

actualNumKmers <- c(1,9,25,68,183)

#metaData <- read.csv(paste(motifDataDirectory,"arborMetaData-Map2.csv",sep=""),row.names="Sequence")

getPRFilename <- function(kPlus,dataset='NM',sets=20,surrogateSet=-1,order=defaultOrder,baselineMethod=defaultMethod,threshold=defaultThreshold,thresholdTypeString=defaultThreshType,directory=percentileRankDataDirectory,useRandomSet=0,useRandomK=0,useRandomN=1,sourceTraversal=""){
  filename <- paste(directory,"PR-",dataset,"-",sets,"Sets",sep="")
  if (surrogateSet > -1){
    filename <- paste(filename,"-",surrogateSet,sep="")
  }
  if (useRandomSet > 0){
    filename <- paste(filename,"-RandSet",useRandomSet,"_",useRandomK,sep="")
  }
  filename <- paste(filename,"-Size",order,"-B",baselineMethod,sourceTraversal,"-T",threshold,"-",thresholdTypeString,"_K",(kPlus-1),"-",kPlus,".csv",sep="")
  return(filename)
}

getFreqDiffFilename <- function(kPlus,sets=20,surrogateSet=-1,order=defaultOrder,baselineMethod=defaultMethod,threshold=defaultThreshold,thresholdTypeString=defaultThreshType,directory=percentileRankDataDirectory,useRandomSet=0,useRandomK=0,sourceTraversal=""){
  filename <- paste(directory,"FreqDiff-",sets,"Sets",sep="")
  if (surrogateSet > -1){
    filename <- paste(filename,"-",surrogateSet,sep="")
  }
  if (useRandomSet > 0){
    filename <- paste(filename,"-RandSet",useRandomSet,"_",useRandomK,sep="")
  }
  filename <- paste(filename,"-Size",order,"-B",baselineMethod,sourceTraversal,"-T",threshold,"-",thresholdTypeString,"_K",(kPlus-1),"-",kPlus,".csv",sep="")
  return(filename)
}

getPR_KPlus_Filename <- function(k,kPlus,sets=20,surrogateSet=-1,order=defaultOrder,method=defaultMethod,threshold=defaultThreshold,thresholdTypeString=defaultThreshType,directory=percentileRankDataDirectory,useRandomSet=0,useRandomK=0){
  filename <- paste(directory,"PR-",sets,"Sets",sep="")
  if (surrogateSet > -1){
    filename <- paste(filename,"-",surrogateSet,sep="")
  }
  if (useRandomSet > 0){
    filename <- paste(filename,"-RandSet",useRandomSet,"_",useRandomK,sep="")
  }
  filename <- paste(filename,"-Size",order,"-B",method,"-T",threshold,"-",thresholdTypeString,"_K",(k),"-",kPlus,".csv",sep="")
  return(filename)
}

getVarFilename <- function(kPlus,sets=20,surrogateSet=-1,order=defaultOrder,method=defaultMethod,threshold=defaultThreshold,thresholdTypeString=defaultThreshType,directory=motifDataDirectory,useRandomSet=0,useRandomK=0){
  filename <- paste(directory,"BaselineVariance-",sets,"Sets",sep="")
  if (surrogateSet > -1){
    filename <- paste(filename,"-",surrogateSet,sep="")
  }
  if (useRandomSet > 0){
    filename <- paste(filename,"-RandSet",useRandomSet,"_",useRandomK,sep="")
  }
  filename <- paste(filename,"-Size",order,"-B",method,"-T",threshold,"-",thresholdTypeString,"_K",(kPlus-1),"-",kPlus,".csv",sep="")
  return(filename)
}

removeATT <- function(order,columnNames,prefix=""){
  if (order == "STL"){
    # For STL, k-1 will contain TT[seq], which captures ATT[seq]; For LTS, k-1 will contain [seq]AT, which captures [seq]ATT
    excludePat <- "(^ATT[ACT]*|L-[ACT]*ATT$)"
    #excludeList <- paste(prefix,c("ATT","ATTA","ATTC","ATTAA","ATTAC","ATTAT","ATTCA","ATTCC"),sep="")
  }else{
    excludePat <- "([ACT]*ATT|S-ATT[ACT]*)"
    #excludeList <- paste(prefix,c("ATT","AATT","CATT","TATT","AAATT","ACATT","CAATT","CCATT","TAATT","TCATT","TTATT"),sep="")
  }
  inds <- grep(excludePat,columnNames)
  if (length(inds) > 0){
    return(columnNames[-inds])
  }else{
    return(columnNames)
  }
  #return(columnNames[!columnNames%in%excludeList])
}

calculatePROutlierRatio <- function(prRanks,k,alpha=0.05){
  kmers <- generateKmers(k)
  n <- actualNumKmers[k]
  lowerBound = alpha/2
  upperBound = 1-lowerBound
  if (is.data.frame(prRanks)){
    columns <- colnames(prRanks) %in% kmers
    tmpDF <- apply(prRanks[,columns,drop=F],2,isOutlier,lowerBound=lowerBound,upperBound=upperBound)
    OutlierRatioArr <- apply(tmpDF,1,sum)
    if ("OutlierRatio" %in% colnames(prRanks)){
      output <- prRanks
      output$OutlierRatio <- OutlierRatioArr/n
    }
    else{
      output <- cbind(prRanks,data.frame(OutlierRatio=OutlierRatioArr/n))
    }
  }
  else{
    columns <- names(prRanks) %in% kmers
    subdata <- prRanks[columns]
    if ("OutlierRatio" %in% names(prRanks)){
      output <- prRanks
      output["OutlierRatio"] <- sum(prRanks[i,columns] <= lowerBound | prRanks[i,columns] >= upperBound)/n
    }
    else{
      output <- c(prRanks,OutlierRatio=sum(prRanks[i,columns] <= lowerBound | prRanks[i,columns] >= upperBound)/n)
    }
  }
  return(output)
}

outlierBinomialFit <- function(prRanks,k=2){
  kmers <- generateKmers(k)
  numKmers <- length(kmers)
  ecdfData <- ecdf(prRanks$OutlierRatio*numKmers)
  x <- 0:numKmers
  y <- ecdfData(x)
  fit <- nls(y ~ pbinom(x,numKmers,p),start=c(p=.05))
  return(fit)
}

plotBinomialDataAndFits <- function(real,random,k){
  kmers <- generateKmers(k)
  numKmers <- actualNumKmers[k]

  realFit<-outlierBinomialFit(real,k)
  randFit<-outlierBinomialFit(random,k)

  ecdfData <- ecdf(real$OutlierRatio*numKmers)
  ecdfRandData <- ecdf(random$OutlierRatio*numKmers)
  x11()
  x <- 0:numKmers
  plot(x,ecdfData(x),main="Cumulative Distribution",xlab="Number of kmer outliers in a sequence",ylab="Cumulative Frequency")
  lines(x,pbinom(x,numKmers,coef(realFit)["p"]),col=1)
  points(x,ecdfRandData(x),col=2,pch=2)
  lines(x,pbinom(x,numKmers,coef(randFit)["p"]),col=2)
  leg.txt <- c("Real CDF and Fit","Random CDF and Fit")
  legend(numKmers*0.6,0.9,leg.txt,bg="white",col=c(1,2),pch=c(1,2),lty=c(1,1))

  par(ask=T)
  plot(x,ecdfRandData(x)-c(0,ecdfRandData(0:(numKmers-1))),col=2,xlab="Number of kmer outliers in a sequence",ylab="Frequency")
  lines(x,dbinom(x,numKmers,coef(randFit)["p"]),col=2)
  points(x,ecdfData(x)-c(0,ecdfData(0:(numKmers-1))),col=1,pch=2)
  lines(x,dbinom(x,numKmers,coef(realFit)["p"]),col=1)
  leg.txt <- c("Real PDF and Fit","Random PDF and Fit")
  legend(numKmers*0.6,0.9,leg.txt,bg="white",col=c(1,2),pch=c(1,2),lty=c(1,1))

  par(ask=F)
}

isOutlier <- function(val,lowerBound=0.025,upperBound=0.975){
  val > upperBound | val < lowerBound
}

getCountsFilename <- function(kPlus,order=defaultOrder,method=defaultMethod,threshold=defaultThreshold,thresholdTypeString=defaultThreshType,directory=defaultDirectory){
  filename <- paste(directory,"VirtKmerCounts-Size",order,"-B",method,"-T",threshold,"-",thresholdTypeString,"-K",kPlus,".csv",sep="")
  filename
}

getSubsetTitle <- function(titletxt="",arborType=NA,cellType=NA,brainRegion=NA,minLength=NaN,maxLength=NaN,randomSet=0,randomK=0){
  if (randomSet > 0){
    titletxt = paste(titletxt," (RandomSet",randomSet," K",randomK,")",sep="")
  }
  if (!is.na(arborType) || !is.na(cellType) || !is.na(brainRegion)){
    titletxt = paste(titletxt,"\n",sep="")
    divider = ""
    if (!is.na(arborType)){
      titletxt = paste(titletxt,"Arbor Type: ",arborType,sep="")
      divider = "; "
    }
    if (!is.na(cellType)){
      titletxt = paste(titletxt,divider,"Cell Type: ",cellType,sep="")
      divider = "; "
    }
    if (!is.na(brainRegion)){
      titletxt = paste(titletxt,divider,"Brain Region: ",brainRegion,sep="")
      divider = "; "
    }
  }
  else {
    titletxt = paste(titletxt,"\nAll neurites",sep="")
  }
  if (!is.nan(minLength) && !is.nan(maxLength)){
    titletxt = paste(titletxt,"\nLengths ",minLength," through ",maxLength,sep="")
  }
  else if (!is.nan(minLength) && is.nan(maxLength)){
    titletxt = paste(titletxt,"\nLengths equal to or greater than ",minLength,sep="")
  }
  else if (is.nan(minLength) && !is.nan(maxLength)){
    titletxt = paste(titletxt,"\nLengths less than or equal to ",maxLength,sep="")
  }
  return(titletxt)
}

findSignificantKmers <- function(mydata,k,varData=NULL,alpha=0.05,arborType=NA,cellType=NA,brainRegion=NA,minLength=NaN,maxLength=NaN,histogram=F){
  kmers <- generateKmers(k)
  numSignificant <- 0
  totalPossible <- 0
  for (kmer in kmers){
    if (kmer %in% colnames(mydata)){
      totalPossible <- totalPossible + 1
      subData = getSubset(mydata,arborType,cellType,brainRegion,minLength,maxLength)
      if (!is.null(varData)){
        sequences = as.character(varData[varData[,kmer]!=0,"Sequence"])
        subData = subset(subData,Sequence%in%sequences)
      }
      # Using the Anderson-Darling test for normality on a normalized version of the data
      normedData <- qnorm(subData[,kmer])
      testStat <- ad.test(normedData)
      if (testStat$p.value <= alpha){
        numSignificant <- numSignificant + 1
        print(paste(kmer,testStat$p.value))
        if (histogram){
          percentileRanksHistogram(subData,kmer,arborType=arborType,cellType=cellType,brainRegion=brainRegion,minLength=minLength,maxLength=maxLength)
        }
      }
    }
  }
  print(paste(numSignificant," significant ",k,"-mers out of ",totalPossible,sep=""))
}

analyzePercentileRanksForKmer <- function(mydata,kmer,varData=NULL,arborType=NA,cellType=NA,brainRegion=NA,minLength=NaN,maxLength=NaN,histogram=F,randomSet=0,randomK=0){
  subData = getSubset(mydata,arborType,cellType,brainRegion,minLength,maxLength)
  if (!is.null(varData)){
    sequences = as.character(varData[varData[,kmer]!=0,"Sequence"])
    subData = subset(subData,Sequence%in%sequences)
  }
  # Using the Anderson-Darling test for normality on a normalized version of the data
  normedData <- qnorm(subData[,kmer])
  ad.test(normedData)
  if (histogram){
    percentileRanksHistogram(subData,kmer,arborType=arborType,cellType=cellType,brainRegion=brainRegion,minLength=minLength,maxLength=maxLength,randomSet=randomSet,randomK=randomK)
  }
}

percentileRanksHistogram <- function(mydata,kmer,breaks=20,arborType=NA,cellType=NA,brainRegion=NA,minLength=NaN,maxLength=NaN,randomSet=0,randomK=0){
  x11()
  titletxt = paste("Percentile frequency distibution for ",kmer,sep="")
  titletxt <- getSubsetTitle(titletxt,arborType,cellType,brainRegion,minLength,maxLength,randomSet,randomK)
  hist(mydata[,kmer],breaks=breaks,main=titletxt,xlab="Percentile Rank of real kmer within surrogate distribution")
}

histZeroVariancePRs <- function(prData,varData,kmer,breaks=20,newWindow=T){
  if (newWindow){
    x11()
  }
  prSubset <- prData[varData[,kmer]==0,kmer]
  hist(prSubset,breaks=breaks,main=paste("Percentile ranks of ",kmer," given 0 surrogate variance",sep=""))
}

# Correlation of zero variance cases with total kmer count
modelVarianceKmerCount <- function(varData,countData,kmer,maxLength=300,plotData=FALSE,averages=TRUE,binSize=0){
  xs <- c() #kmer Count
  ys <- c() #kmer variance
  if (maxLength > 0){
    sequences <- subset(metaData,SeqLen<=maxLength)$Sequence
  }else{
    sequences <- metaData$Sequence
  }

  #Make sure varSub and countSub share sequence set
  sequences <- as.character(subset(varData,Sequence%in%sequences)$Sequence)
  sequences <- as.character(subset(countData,Sequence%in%sequences)$Sequence)
  varSub <- subset(varData,Sequence%in%sequences)
  countSub <- subset(countData,Sequence%in%sequences)

  #Make sure countSub and varSub are in the same order
  rownames(varSub) <- varSub$Sequence
  rownames(countSub) <- countSub$Sequence
  countSub <- countSub[sequences,]
  varSub <- varSub[sequences,]
  
  if (plotData){
    x11()
    if (averages){
      plotVarianceVSKmerCountAverages(varData,countData,kmer,newWindow=F,binSize=binSize)
    }
    else{
      mainLab <- paste("Variance as a function of kmer count for ",kmer,sep="")
      plot(countSub[,kmer],varSub[,kmer],main=mainLab,xlab="Kmer Count",ylab="Kmer Variance")
    }
  }
  print(summary(lm(varSub[,kmer]~countSub[,kmer])))
}

# Average variance per count to fix overplotting issue
plotVarianceVSKmerCountAverages <- function(varData,countData,kmer,newWindow=T,binSize=0){
  if (newWindow){
    x11()
  }
  xs <- unique(countData[,kmer])
  if (binSize > 0){
    bins <- max(xs)/binSize
    xs <- ((1:bins)-.5)*binSize
  }
  dists <- list()
  for (i in 1:dim(countData)[1]){
    x <- as.character(countData[i,kmer])
    if (binSize > 0){
      x <- as.character(floor(as.integer(x)/binSize))
    }
    if (is.null(dists[[x]])){
      dists[x] <- c(varData[i,kmer])
    }
    else{
      dists[[x]] <- c(dists[[x]],varData[i,kmer])
    }
  }
  ys <- c()
  for (i in xs){
    x <- as.character(i)
    if (binSize > 0){
      x <- as.character(floor(as.integer(x)/binSize))
    }
    ys <- c(ys,mean(dists[[x]]))
  }
  mainLab <- paste("Variance as a function of kmer count for ",kmer,sep="")
  plot(xs,ys,main=mainLab,xlab="Kmer Count",ylab="Kmer Variance")
}

generateSubclassSummary <- function(mydata,specifications,filename=NA,myMetaData=usedMetaData){
  # Columns are mean and sd for each class
  dataSummary <- matrix("",2+ncol(mydata),1+2*length(specifications))
  dataSummary[1,1] <- "K-mer"
  dataSummary[3:(2+ncol(mydata)),1] <- t(colnames(mydata))
  for (i in 1:length(specifications)){
    specification <- specifications[[i]]
    # Get the data subset
    subData <- getSubset2(mydata,specification,myMetaData)
    # Get the spec name
    if (is.null(names(specifications))){
      specName <- getSetString(specification)
    }
    else{
      specName <- names(specifications)[i]
    }
    # Include class name and N
    dataSummary[1,2+2*(i-1)] <- paste(specName," (N=",nrow(subData),")",sep="")
    dataSummary[1,3+2*(i-1)] <- nrow(subData)
    dataSummary[2,2+2*(i-1)] <- "Mean"
    dataSummary[2,3+2*(i-1)] <- "SD"
    # Calculate and include means and SDs
    means <- apply(subData,2,mean)
    dataSummary[3:(2+length(means)),2+2*(i-1)] <- t(means)
    sds <- apply(subData,2,sd)
    dataSummary[3:(2+length(means)),3+2*(i-1)] <- t(sds)
  }

  # Write to file if one is provided
  if (!is.na(filename)){
    write.csv(dataSummary,filename,row.names=F,col.names=F)
  }
  return(dataSummary)
}
