
eDeriv <- function(x,y,width=.05*diff(range(x)),kernelWeights=c("uniform","gaussian")){
  kernelWeights <- match.arg(kernelWeights)
  y <- y[order(x)]
  x <- sort(x)
  if (diff(range(x)) == 0){
    return(NULL)
  }
  uniqueX <- unique(x)
  newX <- uniqueX
  newX[1] <- mean(newX[1:2])
  xLen <- length(newX)
  newX[length(newX)] <- mean(newX[(xLen-1):xLen])
  dy <- rep(0,length(uniqueX))
  for (i in seq(length(uniqueX))){
    thisX <- uniqueX[i]
    lower <- thisX - width/2
    upper <- thisX + width/2
    xs <- x[x >= lower & x <= upper]
    ys <- y[x >= lower & x <= upper]
    currWidth <- width
    while (sum(xs!=thisX) == 0){
      currWidth <- 1.5*currWidth
      lower <- thisX - currWidth/2
      upper <- thisX + currWidth/2
      xs <- x[x >= lower & x <= upper]
      ys <- y[x >= lower & x <= upper]
    }

    if (kernelWeights=="guassian"){
      flm <- lm(ys ~ xs, weights=dnorm(xs-thisX,0,width*2))
    }
    else{
      flm <- lm(ys ~ xs)
    }
    dy[i] <- flm$coefficients[[2]]
  }
  data.frame(x=newX,dy)
}


sem <- function(x,na.rm=F){
  if (na.rm){
    return(sqrt(var(x,na.rm=T)/sum(!is.na(x))))
  }
  return(sqrt(var(x,na.rm=F)/length(x)))
}
 

bin2dec <- function(binaryvector) {   sum(2^(which(rev(binaryvector)==TRUE)-1)) }
dec2bin <- function(dec) { return(integer.base.b(dec,b=2)) }
integer.base.b <- function(x, b=2){ 
  xi <- as.integer(x) 
  if(any(is.na(xi) | ((x-xi)!=0))) 
  print(list(ERROR="x not integer", x=x)) 
  N <- length(x) 
  xMax <- max(x)  
  ndigits <- (floor(logb(xMax, base=2))+1) 
  Base.b <- array(NA, dim=c(N, ndigits)) 
  for(i in 1:ndigits){#i <- 1 
    Base.b[, ndigits-i+1] <- (x %% b) 
    x <- (x %/% b) 
  }
  if(N ==1) Base.b[1, ] else Base.b 
}

distdex<-function(i,j,n) #given row, column, and n, return index
  n*(i-1) - i*(i-1)/2 + j-i

rowcol<-function(ix,n) { #given index, return row and column
  nr=ceiling(n-(1+sqrt(1+4*(n^2-n-2*ix)))/2)
  nc=n-(2*n-nr+1)*nr/2+ix+nr
  cbind(nr,nc)
}

# Produces means (row 1) and standard deviations (row 2) of each column of the data set 
# constrained by the meta type parameters
summaryByClass <- function(mydata,arborType=NA,cellType=NA,brainRegion=NA,minLength=NaN,maxLength=NaN){
  subData = getSubset(mydata,arborType,cellType,brainRegion,minLength,maxLength)
  result <- matrix(0,2,ncol(subData))
  rownames(result) <- c("Mean","SD")
  colnames(result) <- colnames(subData)
  # Should not contain NA, but if there is some error it may, so I am not removing NAs in
  # order to increase the chances of catching it if there is an NA
  result[1,] <- mean(subData)
  result[2,] <- sd(subData)
  result
}

# Plot (bar or box) (assuming each data frame has the same columns)
distributionComparison <- function(dataFrames,title=NA,ylab="",dataLabels=NA,error=c(NA,"SD","SEM"),plotType=c("bar","box"),appendNs=F,las=3,minX=NULL,ylim=NULL,markSignificance=F){
  # Validate input parameters
  error <- match.arg(error)
  plotType <- match.arg(plotType)
  numSets <- length(dataFrames)
  if (is.null(names(dataFrames))){
    names(dataFrames) <- paste("Group",1:numSets)
  }
  
  subtypeSummaries <- list()
  # Get mean and sd summary of columns in each data frame
  for (i in 1:numSets){
    subtypeSummaries[[i]] <- summaryByClass(dataFrames[[i]])
  }
  # Get number of columns. Assumes each data frame has the same columns
  dataCols <- ncol(subtypeSummaries[[1]])
  #sqrtN <- sqrt(nrow(getSubset(dataFrames[[1]],arborType=arborType)))
  summaryMatrix <- t(matrix(0,dataCols,numSets))
  for (i in 1:numSets){
    summaryMatrix[i,] <- subtypeSummaries[[i]][1,]
  }
  colnames(summaryMatrix) <- colnames(dataFrames[[1]])
  if (length(dataLabels) > 1 || !is.na(dataLabels)){
    rownames(summaryMatrix) <- dataLabels
  }
  else{
    rownames(summaryMatrix) <- paste("Group",1:numSets,sep="")
  }
  if (appendNs){
    newrownames <- rep("",numSets)
    for (i in 1:numSets){
      newrownames[i] <- paste(rownames(summaryMatrix)[i]," (N=",nrow(dataFrames[[i]]),")",sep="")
    }
    rownames(summaryMatrix) <- newrownames
  }
  if (plotType=="bar"){
    # Round 80% of min value to nearest .05 by doubling, rounding to .1, and halving
    if (is.null(ylim)){
      myRange <- range(summaryMatrix)
      numDecimals <- ceiling(-log(diff(myRange),10))
      minY <- round(2*1.15*min(summaryMatrix),numDecimals)/2
      maxY <- round(2*1.15*max(summaryMatrix),numDecimals)/2
      barplot(summaryMatrix,beside=T,main=title,ylab=ylab,las=las)
    }
    else{
      minY <- ylim[1]
      maxY <- ylim[2]
      barplot(summaryMatrix,beside=T,main=title,ylab=ylab,las=las,ylim=ylim)
    }
    yRange <- maxY-minY
    #barplot(summaryMatrix-minY,beside=T,ylim=c(minY,maxY),offset=minY,main=title,ylab=ylab,legend=T,args.legend=list(x=0,y=maxY-.11*yRange,y.intersp=.2,xjust=0,yjust=0,bty="o",horiz=T,cex=.75),las=las)
    #barplot(summaryMatrix,beside=T,main=title,ylab=ylab,legend=T,args.legend=list(x=0,y=maxY-.11*yRange,y.intersp=.2,xjust=0,yjust=0,bty="o",horiz=T,cex=.75),las=las)
    if (numSets > 1){
      legend(legend=names(dataFrames),x=0,y=maxY-.11*yRange,y.intersp=.2,xjust=0,yjust=0,bty="o",horiz=T,cex=.75)
    }
    if (!is.na(error)){
      for (i in 1:ncol(summaryMatrix)){
        for (j in 1:numSets){
          if (error == "SD"){
            errVal <- subtypeSummaries[[j]][2,i]
          }
          else if (error == "SEM"){
            errVal <- subtypeSummaries[[j]][2,i]/sqrt(nrow(dataFrames[[j]]))
          }
          lines(((numSets+1)*(i-1)+j+.5)*c(1,1),subtypeSummaries[[j]][1,i]+errVal*c(-1,1))
        }
      }
    }
  }
  else if (plotType=="box"){
    require(ggplot2)
    numCols <- ncol(dataFrames[[1]])
    origCols <- colnames(dataFrames[[1]])[2:numCols]
    for (i in 1:numSets){
      colnames(dataFrames[[i]])[2:numCols] <- paste("pr",origCols,sep=".")
    }
    cols <- colnames(dataFrames[[1]])[2:numCols]
    combinedObj <- NULL
    for (i in 1:numSets){
      tmpObj <- reshape(dataFrames[[i]],varying=cols,v.names="pr",timevar="Kmer",times=origCols,idvar="Sequence",direction="long",drop="N")
      tmpObj$Baseline <- rep(dataLabels[i],nrow(tmpObj))
      if (is.null(combinedObj)){
        combinedObj <- tmpObj
      }
      else{
        combinedObj <- rbind(combinedObj,tmpObj)
      }
    }
    if (is.null(ylim)){
      myRange <- range(summaryMatrix)
      numDecimals <- ceiling(-log(diff(myRange),10))
      minY <- round(2*1.15*min(summaryMatrix),numDecimals)/2
      maxY <- round(2*1.15*max(summaryMatrix),numDecimals)/2
      qplot(Kmer,pr,fill=factor(Baseline),data=combinedObj,geom="boxplot",main=title,ylab="Percentile Rank Distribution",xlab="")
    }
    else{
      minY <- ylim[1]
      maxY <- ylim[2]
      qplot(Kmer,pr,fill=factor(Baseline),data=combinedObj,geom="boxplot",main=title,ylab="Percentile Rank Distribution",xlab="",ylim=ylim)
    }
    yRange <- maxY-minY
    if (numSets > 1){
      legend(legend=names(dataFrames),x=0,y=maxY-.11*yRange,y.intersp=.2,xjust=0,yjust=0,bty="o",horiz=T,cex=.75)
    }
  }
  if (markSignificance && length(dataFrames) > 1){
    for (i in 1:ncol(dataFrames[[1]])){
      testResult <- wilcox.test(classPRs1[,i],classPRs2[,i])
      adjPVal <- ncol(classPRs1)*testResult$p.value
      if (adjPVal < 0.05){
        val <- "*"
        if (adjPVal < 0.01){
          val <- "**"
          if (adjPVal < 0.001){
            val <- "***"
          }
        }
        mtext(side=1,at=2+(1+length(dataFrames))*(i-1),text=val)
      }
    }
  }
  return(summaryMatrix)
}

getSubset2 <- function(mydata,specifications=NULL,typeMetaData=masterMetaData,minLength=NaN,maxLength=NaN){
  sequences <- rownames(typeMetaData)
  if (!is.null(specifications)){
    for (i in 1:length(specifications)){
      specTypes <- names(specifications)[i]
      if (is.list(specifications)){
        specValues <- specifications[[i]]
      }
      else{
        specValues <- specifications[i]
      }
      if (specTypes == "Class3"){
        specTypes <- c("Class3","Class3b","Class3c")
      }
      theseSeqs <- c()
      for (specType in specTypes){
        for (specValue in specValues){
          theseSeqs <- union(theseSeqs,as.character(rownames(typeMetaData[typeMetaData[,specType]==specValue,])))
        }
      }
      sequences <- intersect(sequences,theseSeqs)
    }
  }
  if (!is.nan(minLength)){
    sequences <- intersect(sequences,as.character(rownames(subset(typeMetaData,SeqLen>=minLength))))
  }
  if (!is.nan(maxLength)){
    sequences <- intersect(sequences,as.character(rownames(subset(typeMetaData,SeqLen<=maxLength))))
  }
  if ("Sequence"%in%colnames(mydata)){
    return(subset(mydata,Sequence%in%sequences))
  }
  else{
    sequences <- intersect(sequences,rownames(mydata))
    return(subset(mydata,rownames(mydata)%in%sequences))
  }
}

loadSubsetSeqs <- function(specifications,metaData,candidates=rownames(metaData)){
  subsetSeqs <- c()
  groupSeqs <- list()
  groupIds <- c()
  for (specInd in 1:length(specifications)){
    specs <- specifications[[specInd]]
    groupSeqs[[specInd]] <- intersect(candidates,rownames(getSubset2(metaData,specs,typeMetaData=metaData)))
    subsetSeqs <- append(subsetSeqs,groupSeqs[[specInd]])
    groupIds <- append(groupIds,rep(specInd,length(groupSeqs[[specInd]])))
  }
  return(list(Sequences=subsetSeqs,SeqsByGroup=groupSeqs,GroupIds=groupIds))
}

getSubset <- function(mydata,typeMetaData=masterMetaData,arborType=NA,cellType=NA,brainRegion=NA,region2=NA,minLength=NaN,maxLength=NaN){
  subData <- mydata
  if (!is.na(arborType)){
    if (arborType == "Dendrite" || arborType == "BasalDendrite"){
      sequences <- as.character(subset(typeMetaData,ArborType=="Dendrite"|ArborType=="BasalDendrite")$Sequence)
    }
    else{
      sequences <- as.character(subset(typeMetaData,ArborType==arborType)$Sequence)
    }
    subData <- subset(subData,Sequence%in%sequences)
  }
  if (!is.na(cellType)){
    sequences <- as.character(subset(typeMetaData,Class1==cellType|Class2==cellType)$Sequence)
    subData <- subset(subData,Sequence%in%sequences)
  }
  if (!is.na(brainRegion)){
    sequences <- as.character(subset(typeMetaData,BrainRegion==brainRegion)$Sequence)
    subData <- subset(subData,Sequence%in%sequences)
  }
  if (!is.na(region2)){
    sequences <- as.character(subset(typeMetaData,Region2==region2)$Sequence)
    subData <- subset(subData,Sequence%in%sequences)
  }
  if (!is.nan(minLength)){
    sequences <- as.character(subset(typeMetaData,SeqLen>=minLength)$Sequence)
    subData <- subset(subData,Sequence%in%sequences)
  }
  if (!is.nan(maxLength)){
    sequences <- as.character(subset(typeMetaData,SeqLen<=maxLength)$Sequence)
    subData <- subset(subData,Sequence%in%sequences)
  }
  subData
}

object.sizes <- function()
{
  return(rev(sort(sapply(ls(envir=.GlobalEnv), function (object.name)
                         object.size(get(object.name))))))
}

stringsToColorIndices <- function(strings){
  unStr <- unique(strings)
  indices <- rep(1,length(strings))
  for (i in 2:length(unStr)){
    indices[strings==unStr[i]] <- i
  }
  return(indices)
}

sample.rows = function(x,size,replace=FALSE,prob=NULL){
  if (size < 1){
    if (size <= 0){
      stop('Parameter \'size\' must be greater than 0')
    }
    size <- size*nrow(x)
  }
  if (!replace & nrow(x) < size){
    stop(paste("If 'replace' is FALSE, 'size' must be less than or equal to nrow(x): nrow(x) is ",nrow(x),", size is ",size,sep=""))
  }
  return(x[sample(nrow(x),size,replace,prob),])
}
