####################################################################
#
# VisualizationUtils.r
#
####################################################################

require(MASS)
require(ape)
source(paste(baseCodeDir,"scatboot.r",sep=""))

# From Yasir Kahei, found at https://stat.ethz.ch/pipermail/r-help/2008-May/161202.html
rgl.plot3d<-function(z, x, y, cols="red",axes=T,new=T,size=0.01,preserveAspectRatio=F){
  require(rgl)
  xr<-range(x)
  xmin <- xr[1]
  xr <- xr[2]-xr[1]
  yr<-range(y)
  ymin <- yr[1]
  yr <- yr[2]-yr[1]
  zr<-range(z)
  zmin <- zr[1]
  zr <- zr[2]-zr[1]
  
  if (preserveAspectRatio){
    allr <- max(c(xr,yr,zr))
    x01<-(x-xmin)/allr
    y01<-(y-ymin)/allr
    z01<-(z-zmin)/allr
  }else{
    x01<-(x-xmin)/xr
    y01<-(y-ymin)/yr
    z01<-(z-zmin)/zr
  }
 
 if(new) rgl.clear()
 if(axes)
 {xlab<-pretty(x)
  ylab<-pretty(y)
  zlab<-pretty(z)
  if (preserveAspectRatio){
    xat<-(xlab-xmin)/allr
    yat<-(ylab-ymin)/allr
    zat<-(zlab-zmin)/allr
  }else{
    xat<-(xlab-xmin)/xr
    yat<-(ylab-ymin)/yr
    zat<-(zlab-zmin)/zr
  }
  rgl.lines(c(0,1.1),0,0)
  rgl.lines(0,c(0,1.1),0)
  rgl.lines(0,0,c(0,1.1))	
  rgl.texts(xat,-.05,-.05,xlab)
  rgl.texts(-.05,yat,-.05,ylab)
  rgl.texts(-.05,-.05,zat,zlab)
  rgl.texts(c(0.5,-.15,-.15),c(-.15,.5,-.15),c(-.15,-.15,.5),
            c(deparse(substitute(x)),deparse(substitute(y)),deparse(substitute(z))))
 }
 
 rgl.spheres(x01,y01,z01,size,color=cols)
}

kde2dplot <- function(d,                # a 2d density computed by kde2D
                      ncol=50,          # the number of colors to use
                      zlim=c(0,max(z)), # limits in z coordinates
                      nlevels=20,       # see option nlevels in contour
                      theta=30,         # see option theta in persp
                      phi=30,           # see option phi in persp
                      zlab="density")           
{
  z   <- d$z
  nrz <- nrow(z)
  ncz <- ncol(z)
  
  couleurs  <- tail(topo.colors(trunc(1.4 * ncol)),ncol)
  fcol      <- couleurs[trunc(z/zlim[2]*(ncol-1))+1]
  dim(fcol) <- c(nrz,ncz)
  fcol      <- fcol[-nrz,-ncz]
  
  par(mfrow=c(1,2),mar=c(0.5,0.5,0.5,0.5))
  persp(d,col=fcol,zlim=zlim,theta=theta,phi=phi,zlab=zlab)
  
  par(mar=c(2,2,2,2))
  image(d,col=couleurs)
  contour(d,add=T,nlevels=nlevels)
  box()
  par(mfrow=c(1,1),mar=c(5.1,4.1,4.1,2.1))
}

plotOverLengths <- function(mydata,column,metaData=masterMetaData,minValues=1,maxValues=0,plotXRange=F,yRangeType=c("None","SD","SEM","Range","Quartiles"),new=T,title="",ylab="",fcol=1,fxlim=NA,fylim=NA,plotXHistogram=F){
  metaDataSub <- metaData[rownames(mydata),]
  allLengths <- sort(metaDataSub$SeqLen)
  currDataPos <- 1
  currLength <- allLengths[1]
  currLengthPos <- 1
  index <- 1
  xGroups <- list()
  yGroups <- list()
  values <- 0
  while (currDataPos < nrow(mydata)){
    #    print(paste("CurrDataPos:",currDataPos))
    #    if (index > 1){
    #      print(paste("Values:",values,"| Index:",index,"| xs:",str(xGroups[[index]]),"| ys:",str(yGroups[[index]])))
    #    }
    if (values == 0){
      # new x/y
      index <- length(xGroups)+1
    }
    # Get all values of the current length
    newValues <- allLengths[allLengths==currLength]
    valuesOfLength <- length(newValues)
    
    # Now take only the ones that haven't already been used
    newValues <- newValues[currLengthPos:length(newValues)]
    # If max values condition, only take up to maxValues number of the current length set
    if (maxValues > 0 && values + length(newValues) > maxValues){
      if (length(xGroups) < index){
        xGroups[[index]] <- newValues[1:maxValues]
        yGroups[[index]] <- subset(mydata,rownames(mydata)%in%rownames(subset(metaDataSub,SeqLen==currLength)))[currLengthPos:valuesOfLength,column]
      }else{
        xGroups[[index]] <- c(xGroups[[index]],newValues[1:maxValues])
        yGroups[[index]] <- c(yGroups[[index]],subset(mydata,rownames(mydata)%in%rownames(subset(metaDataSub,SeqLen==currLength)))[currLengthPos:valuesOfLength,column])
      }
      currDataPos <- currDataPos + maxValues
      currLengthPos <- currLengthPos + maxValues
      values <- 0
    }else{
      # Take all of the current length set
      if (length(xGroups) < index){
        xGroups[[index]] <- newValues
        yGroups[[index]] <- subset(mydata,rownames(mydata)%in%rownames(subset(metaDataSub,SeqLen==currLength)))[currLengthPos:valuesOfLength,column]
      }else{
        xGroups[[index]] <- c(xGroups[[index]],newValues)
        yGroups[[index]] <- c(yGroups[[index]],subset(mydata,rownames(mydata)%in%rownames(subset(metaDataSub,SeqLen==currLength)))[currLengthPos:valuesOfLength,column])
      }
      currDataPos <- currDataPos + length(newValues)
      currLengthPos <- 1
      currLength <- allLengths[currDataPos]
      values <- values + length(newValues)
      if (values >= minValues){
        values <- 0
      }
    }
  }
  
  # Plot data
  xs <- rep(0,length(xGroups))
  ys <- xs
  for (i in 1:length(xGroups)){
    xs[i] <- mean(xGroups[[i]])
    ys[i] <- mean(yGroups[[i]])
  }
  if (new){
    x11()
    if (length(fxlim) != 2){
      fxlim <- round(range(xs)*c(.9,1.1),digits=-1)
    }
    if (length(fylim) != 2){
      fylim <- round(range(ys)*c(.9,1.1),digits=-1)
    }
    plot(xs,ys,main=title,xlab="Sequence Length",ylab=ylab,col=fcol,xlim=fxlim,ylim=fylim)
  }
  for (i in 1:length(xGroups)){
    if (length(xGroups[[i]]) > 1){
      if (plotXRange){
        lines(c(min(xGroups[[i]]),max(xGroups[[i]])),c(ys[i],ys[i]),col="gray80")
      }
      if (yRangeType != "None"){
        if (yRangeType == "SD"){
          lines(c(xs[i],xs[i]),ys[i]+sd(yGroups[[i]])*c(-1,1),col="gray80")
        }
        else if (yRangeType == "SEM"){
          lines(c(xs[i],xs[i]),ys[i]+(sd(yGroups[[i]])/sqrt(length(yGroups[[i]])))*c(-1,1),col="gray80")
        }
        else if (yRangeType == "Range"){
          lines(c(xs[i],xs[i]),c(min(yGroups[[i]]),max(yGroups[[i]])),col="gray80")
        }
        else if (yRangeType == "Quartiles"){
          groupLen <- length(yGroups[[i]])
          lines(c(xs[i],xs[i]),c(yGroups[[i]][round(.25*groupLen)],yGroups[[i]][round(.75*groupLen)]),col="gray80")
        }
      }
    }
  }
  points(xs,ys,col=fcol)
  
  # Add smooth to plot
  sequencesInOrder <- rownames(metaDataSub)[order(metaDataSub$SeqLen)]
  rownames(mydata) <- as.character(rownames(mydata))
  #print(sort(metaDataSub$SeqLen))
  #print(mydata[sequencesInOrder,column])
  #lowessSmooth <- lowess(sort(metaDataSub$SeqLen),mydata[sequencesInOrder,column])
  #lines(lowessSmooth$x,lowessSmooth$y,col=col)
  
  xs <- sort(metaDataSub$SeqLen)
  ys <- mydata[sequencesInOrder,column]
  loessfit  <- loess(ys ~ xs)
  plx <- predict(loessfit, se=TRUE)
  lines(xs,plx$fit,col=fcol) 
  lines(xs,plx$fit+2*plx$s,lty=2,col=fcol) #rough & ready CI 
  lines(xs,plx$fit-2*plx$s,lty=2,col=fcol) 
  
  if (plotXHistogram){
    seqLenHist <- hist(xs,breaks=200,plot=F)
    histMod <- .05/max(seqLenHist$density)
    histMin <- fxlim[1]
    lines(seqLenHist$mids,histMod*seqLenHist$density+.24,col=fcol)
  }
  
}

plotSmooths2 <- function(mydata,column,metaData=masterMetaData,plotXRange=F,yRangeType=c("None","SD","SEM","Range","Quartiles"),new=T,title="",ylab="",fcol=1,fxlim=NA,fylim=NA){
  require(KernSmooth)
  rowNames <- intersect(rownames(mydata),rownames(metaData))
  mydata <- mydata[rowNames,]
  metaDataSub <- metaData[rowNames,]
  xs <- sort(metaDataSub$SeqLen)
  sequencesInOrder <- rownames(metaDataSub)[order(metaDataSub$SeqLen)]
  #ys <- mydata[sequencesInOrder,column]
  ys <- mydata[,column]
  loessfit  <- loess(ys ~ xs)
  plx <- predict(loessfit, se=TRUE)
  if (new){
    x11()
    if (length(fxlim) != 2){
      fxlim <- round(range(xs)*c(.9,1.1),digits=-1)
    }
    if (length(fylim) != 2){
      fylim <- round(range(c(plx$fit+2*plx$s,plx$fit-2*plx$s)),digits=1)
    }
    plot(xs,plx$fit,col=fcol,type="l",xlab="Sequence Length",ylab=ylab,xlim=fxlim,ylim=fylim,main=title)
  }else{
    lines(xs,plx$fit,col=fcol) 
  }
  lines(xs,plx$fit+2*plx$s,lty=2,col=fcol) #rough & ready CI 
  lines(xs,plx$fit-2*plx$s,lty=2,col=fcol) 
}

plotSmooths <- function(mydata,column,metaData=masterMetaData,minValues=1,maxValues=0,new=T,confidence=0,title="",ylab="",fcol=1,fxlim=NA,fylim=NA,plotXHistogram=F,lwd=1,kernelWidth=diff(range(xs))/50){
  metaDataSub <- metaData[rownames(mydata),]
  
  sequencesInOrder <- rownames(metaDataSub)[order(metaDataSub$SeqLen)]
  #rownames(mydata) <- as.character(mydata$Sequence)
  
  xs <- sort(metaDataSub$SeqLen)
  ys <- mydata[sequencesInOrder,column]
  
  # Calculate smooth
  #  loessfit  <- loess(ys ~ xs)
  smoothObj <- locpoly(xs,ys,bandwidth=kernelWidth)
  
  if (new){
    x11()
    if (length(fxlim) != 2){
      fxlim <- round(range(xs)*c(.9,1.1),digits=-1)
    }
    if (length(fylim) != 2){
      fylim <- round(range(ys)*c(.9,1.1),digits=-1)
    }
    #plot(xs,loessfit$fit,type="l",main=title,xlab="Sequence Length",ylab=ylab,col=fcol,xlim=fxlim,ylim=fylim,lwd=lwd)
    plot(smoothObj,type="l",main=title,xlab="Sequence Length",ylab=ylab,col=fcol,xlim=fxlim,ylim=fylim,lwd=lwd)
  }
  else{
    #lines(xs,loessfit$fit,col=fcol,lwd=lwd)
    lines(smoothObj,col=fcol,lwd=lwd)
  }
  
  if (confidence > 0){
    sb <- scatboot(xs,ys,confidence=confidence)
    lines(sb$fit$x,sb$fit$up.lim,lty=2,col=fcol)
    lines(sb$fit$x,sb$fit$low.lim,lty=2,col=fcol)
    #plx <- predict(loessfit, se=TRUE)
    #lines(xs,plx$fit+2*plx$s,lty=2,col=fcol) #rough & ready CI 
    #lines(xs,plx$fit-2*plx$s,lty=2,col=fcol) 
  }
  
  if (plotXHistogram && !is.na(fylim)){
    seqLenHist <- hist(xs,breaks=200,plot=F)
    histMod <- .05/max(seqLenHist$density)
    histMin <- fylim[1]
    
    lines(seqLenHist$mids,histMod*seqLenHist$density+fylim[1]+.01*diff(fylim),col=fcol)
  }
}

# From "Monkey's Uncle" [http://monkeysuncle.stanford.edu/?p=485]
error.bar <- function(x, y, upper, lower=upper, length=0.1,...){
  if(length(x) != length(y) | length(y) !=length(lower) | length(lower) != length(upper))
    stop("vectors must be same length")
  arrows(x,y+upper, x, y-lower, angle=90, code=3, length=length, ...)
}

# Plotting dendrograms
plotTree <- function(tree,colorByType=T,equalizeEdgeLengths=T,treePalette=c("black","orange","cyan","magenta","green2","blue","red","yellow2","gray","pink"),modulateEdgeWidths=F,show.tip.label=F,colorDown=F,...){
  nodeEdges <- tree$edge[,2] > tree$Nnode+1
  cols <- rep(treePalette[1],tree$Nnode*2)
  if (equalizeEdgeLengths){
    tree$edge.length <- rep(1,length(tree$edge.length))
    tree$root.edge <- 1
  }
  if (colorByType){
    if ("0"%in%tree$node.label){
      if (unique(tree$node.label) > 1){
        tree$node.label[tree$node.label=="0"] <- unique(tree$node.label)[2]
      }else{
        tree$node.label[tree$node.label=="0"] <- 1
      }
    }

    # label[indexed by edge child - number of leaf nodes]
    rootCol <- treePalette[as.numeric(tree$node.label[1])]
    cols[nodeEdges] <- treePalette[as.numeric(tree$node.label[tree$edge[nodeEdges,2]-(tree$Nnode+1)])]
    if (colorDown){
      # Propagate the colors down a level
      cols <- sapply(tree$edge[,1],
                     function(x,tree,cols,rootCol){
                       if (x%in%tree$edge[,2]){
                         return(cols[which(x==tree$edge[,2])])
                       }else{
                         return(rootCol)
                       }
                     },tree,cols,rootCol)
    }
    plot.phylo(tree,edge.color=cols,show.tip.label=show.tip.label,...)
  }else{
    plot.phylo(tree,show.tip.label=show.tip.label,edge.color=cols,...)
  }
}
plotTreeByLabels <- function(tree,labelSplit=NA,distinctColors=F,equalizeEdgeLengths=T,treePalette=c("black","orange","cyan","magenta","green2","blue","red","yellow2","gray","pink"),colorLabel=T,colorDown=F,modulateEdgeWidths=F,root.edge=T,show.node.label=T,show.tip.label=F,...){
  nodeEdges <- tree$edge[,2] > tree$Nnode+1
  tree$node.label[tree$node.label==""] <- " "
  tree$tip.label[tree$tip.label==""] <- " "
  cols <- rep(1,tree$Nnode*2)
  if (!is.na(labelSplit)){
    # Root
    rootColIndex <- as.numeric(strsplit(tree$node.label[1],labelSplit)[[1]][2])
    if (is.na(rootColIndex)){
      rootCol <- treePalette[1]
    }else{
      rootCol <- treePalette[rootColIndex]
    }
    
    tree$node.label[1] <- strsplit(tree$node.label[1],labelSplit)[[1]][1]
    # Bifurcations
    orderedInds <- tree$edge[nodeEdges,2]-(tree$Nnode+1)
    colorIndex <- as.numeric(sapply(tree$node.label[orderedInds],function(str,spl){return(strsplit(str,spl)[[1]][2])},labelSplit))
    colorIndex[is.na(colorIndex)] <- 1
    cols[nodeEdges] <- treePalette[colorIndex]
    tree$node.label[orderedInds] <- sapply(tree$node.label[orderedInds],function(str,spl){return(strsplit(str,spl)[[1]][1])},labelSplit)
    # Leaves
    orderedInds <- tree$edge[!nodeEdges,2]
    colorIndex <- as.numeric(sapply(tree$tip.label[orderedInds],function(str,spl){return(strsplit(str,spl)[[1]][2])},labelSplit))
    colorIndex[is.na(colorIndex)] <- 1
    cols[!nodeEdges] <- treePalette[colorIndex]
    tree$tip.label[orderedInds] <- sapply(tree$tip.label[orderedInds],function(str,spl){return(strsplit(str,spl)[[1]][1])},labelSplit)
  }else{
    uniqueLabels <- sort(unique(tree$node.label))
    if (distinctColors){
      labelColors <- treePalette[1:length(uniqueLabels)]
    }else{
      labelColors <- treePalette[c(1,rep(2,length(uniqueLabels)-1))]
    }
    names(labelColors) <- uniqueLabels
    # Root
    rootCol <- labelColors[tree$node.label[1]]
    # label[indexed by edge child - number of leaf nodes]
    cols[nodeEdges] <- labelColors[tree$node.label[tree$edge[nodeEdges,2]-(tree$Nnode+1)]]
    cols[!nodeEdges] <- labelColors[tree$tip.label[tree$edge[!nodeEdges,2]]]
  }
  print(cols)
  if (colorDown){
    # Propagate the colors down a level
    cols <- sapply(tree$edge[,1],
                   function(x,tree,cols,rootCol){
                     if (x%in%tree$edge[,2]){
                       return(cols[which(x==tree$edge[,2])])
                     }else{
                       return(rootCol)
                     }
                   },tree,cols,rootCol)
    print(cols)
  }
  if (equalizeEdgeLengths){
    tree$edge.length <- rep(1,length(tree$edge.length))
    tree$root.edge <- 1
  }
  plot.phylo(tree,edge.color=cols,show.node.label=show.node.label,root.edge=root.edge,show.tip.label=show.tip.label,...)
}

plotConsensusDendrogram <- function(tree,labelSplit="-",distinctColors=F,equalizeEdgeLengths=F,primaryColor,secondaryColor="black",rescaleConservation=F,colorLabel=T,colorDown=F,modulateEdgeWidths=F,edge.widths=1,root.edge=T,show.node.label=T,show.tip.label=F,label.offset=.3,colorLegend=F,widthLegend=F,legendStart.x=0.85,legendStart.y=0.82,legendWidth=0.1,legendHeight=0.16,nlevels=60,maxWidth=10,...){
  require(fields)
  # nodeEdges signifies bifurcations (rather than leaf nodes)
  nodeEdges <- tree$edge[,2] > tree$Nnode+1
  tree$node.label[tree$node.label==""] <- " "
  tree$tip.label[tree$tip.label==""] <- " "
  cols <- rep(1,tree$Nnode*2)
  widths <- rep(edge.widths,tree$Nnode*2)
  hsvDat1 <- rgb2hsv(col2rgb(primaryColor))
  hsvDat2 <- rgb2hsv(col2rgb(secondaryColor))
  
  # If the label can be split to provide label and conservation %
  if (!is.na(labelSplit)){
    # Root
    rootConservation <- as.numeric(strsplit(tree$node.label[1],labelSplit)[[1]][2])
    rootHsv <- calculateConservationColor(hsvDat1,rootConservation,hsvDat2)
    rootCol <- hsv(rootHsv[1],rootHsv[2],rootHsv[3])
    
    rootWidth <- floor((maxWidth-1)*rootConservation)+1
    tree$node.label[1] <- strsplit(tree$node.label[1],labelSplit)[[1]][1]
    
    # Bifurcations
    orderedInds <- tree$edge[nodeEdges,2]-(tree$Nnode+1)
    conservation <- as.numeric(sapply(tree$node.label[orderedInds],function(str,spl){return(strsplit(str,spl)[[1]][2])},labelSplit))
    proportion <- conservation
    #Rescale conservation given min value
    if (rescaleConservation){
      proportion <- (conservation - min(conservation))/(1-min(conservation))
    }
    
    # Modify core color by conservation
    modHsv <- calculateConservationColor(hsvDat1,proportion,hsvDat2)

    # Set colors based on conservation of each node
    cols[nodeEdges] <- mapply(hsv,modHsv[1,],modHsv[2,],modHsv[3,])
    
    #colorIndex[is.na(colorIndex)] <- 1
    #cols[nodeEdges] <- treePalette[colorIndex]
    if (modulateEdgeWidths){
      widths[nodeEdges] <- floor((maxWidth-1)*proportion)+1
    }
    
    if (colorDown){
      # Propagate the colors down a level
      cols <- sapply(tree$edge[,1],
                                 function(x,tree,cols,rootCol){
                                   if (x%in%tree$edge[,2]){
                                     return(cols[which(x==tree$edge[,2])])
                                   }else{
                                     return(rootCol)
                                   }
                                 },tree,cols,rootCol)
      
      # Propagate the edge widths down a level
      widths <- 
        sapply(tree$edge[,1],
               function(x,tree,widths,rootWidth){
                 if (x%in%tree$edge[,2]){
                   return(widths[which(x==tree$edge[,2])])
                 }else{
                   return(rootWidth)
                 }
               },tree,widths,rootWidth)
    }else{
      # Set colors of leaves
      cols[!nodeEdges] <- sapply(tree$edge[!nodeEdges,1],
                               function(x,tree,cols,rootCol){
                                 if (x%in%tree$edge[,2]){
                                   return(cols[which(x==tree$edge[,2])])
                                 }else{
                                   return(rootCol)
                                 }
                               },tree,cols,rootCol)
      
      widths[!nodeEdges] <- 
        sapply(tree$edge[!nodeEdges,1],
               function(x,tree,widths,rootWidth){
                 if (x%in%tree$edge[,2]){
                   return(widths[which(x==tree$edge[,2])])
                 }else{
                   return(rootWidth)
                 }
               },tree,widths,rootWidth)
      
    }
    

    # Set node label after having split off additional info
    tree$node.label[orderedInds] <- sapply(tree$node.label[orderedInds],function(str,spl){return(strsplit(str,spl)[[1]][1])},labelSplit)
    
    # Leaves
    orderedInds <- tree$edge[!nodeEdges,2]
    #tree$tip.label[orderedInds] <- sapply(tree$tip.label[orderedInds],function(str,spl){return(strsplit(str,spl)[[1]][1])},labelSplit)
  }else{
    uniqueLabels <- sort(unique(tree$node.label))
    if (distinctColors){
      labelColors <- treePalette[1:length(uniqueLabels)]
    }else{
      labelColors <- treePalette[c(1,rep(2,length(uniqueLabels)-1))]
    }
    names(labelColors) <- uniqueLabels
    # label[indexed by edge child - number of leaf nodes]
    cols[nodeEdges] <- labelColors[tree$node.label[tree$edge[nodeEdges,2]-(tree$Nnode+1)]]
    cols[!nodeEdges] <- labelColors[tree$tip.label[tree$edge[!nodeEdges,2]]]
  }
  
  if (equalizeEdgeLengths){
    tree$edge.length <- rep(1,length(tree$edge.length))
    tree$root.edge <- 1
  }else{
    tree$edge.length[nodeEdges] <- tree$edge.length[nodeEdges]+1
    tree$root.edge <- tree$root.edge+1
  }
  phyloObj <- plot.phylo(tree,edge.color=cols,show.node.label=show.node.label,root.edge=tree$root.edge,show.tip.label=show.tip.label,edge.width=widths,label.offset=label.offset,...)
  
  # Color legend
  if (colorLegend){
    xspan <- diff(phyloObj$x.lim)
    xmin <- phyloObj$x.lim[1]
    yspan <- diff(phyloObj$y.lim)
    ymin <- phyloObj$y.lim[1]
    nlevels <- 50
    yinc <- diff(phyloObj$y.lim)*legendHeight/nlevels
    
    legendcolsHSV <- calculateConservationColor(hsvDat1,(0:nlevels)/nlevels,hsvDat2)
    legendcols <- mapply(hsv,legendcolsHSV[1,],legendcolsHSV[2,],legendcolsHSV[3,])
    xleft <- 0.85*xspan+xmin
    ybottom <- 0.82*yspan+ymin
    xright <- 0.95*xspan+xmin
    ytop <- nlevels*yinc + ybottom
    segments(
      rep(xleft,nlevels),
      (0:nlevels)*yinc + ybottom,
      #      (0:nlevels)*.4 + ybottom,
      rep(0.95*xspan+xmin,nlevels),
      #      (0:nlevels)*.4 + ybottom,
      (0:nlevels)*yinc + ybottom,
      col=legendcols,lty=1,lwd=3)
    rect(xleft-0.001*xspan,ybottom-0.0025*yspan,xright+0.001*xspan,ytop+0.0025*yspan,border="black",lwd=2)
    if (rescaleConservation){
      text(xleft-0.018*xspan,ytop,labels=paste(round(max(conservation),2),"\u2013"),adj=c(0,0.5))
      text(xleft-0.018*xspan,ybottom,labels=paste(round(min(conservation),2),"\u2013"),adj=c(0,0.5))
    }else{
      text(xleft-0.018*xspan,ytop,labels="1 \u2013",adj=c(1,0.5))
      text(xleft-0.018*xspan,ybottom,labels="0 \u2013",adj=c(1,0.5))
    }
  }
  
  if (widthLegend){
    xspan <- diff(phyloObj$x.lim)
    xmin <- phyloObj$x.lim[1]
    yspan <- diff(phyloObj$y.lim)
    ymin <- phyloObj$y.lim[1]
    nlevels <- 50
    yinc <- diff(phyloObj$y.lim)*legendHeight/nlevels
    
    xleft <- 0.85*xspan+xmin
    ybottom <- 0.82*yspan+ymin
    xright <- 0.95*xspan+xmin
    ytop <- nlevels*yinc + ybottom
    segments(
      rep(xleft,2),
      c(ybottom,ytop),
      rep(0.95*xspan+xmin,2),
      c(ybottom,ytop),
      col=primaryColor,lty=1,lwd=c(1,maxWidth))

    if (rescaleConservation){
      text(xleft-0.02*xspan,ytop,labels=paste(round(max(conservation),2),"\u2013"),adj=c(1,0.5))
      text(xleft-0.02*xspan,ybottom+0.001*yspan,labels=paste(round(min(conservation),2),"\u2013"),adj=c(1,0.5))
    }else{
      text(xleft-0.02*xspan,ytop,labels="1 \u2013",adj=c(1,0.5))
      text(xleft-0.02*xspan,ybottom+0.001*yspan,labels="0 \u2013",adj=c(1,0.5))
    }
  }

  return(invisible(phyloObj))
}

calculateIntermediateColor <- function(colorHsv1,proportion,colorHsv2=NULL){
  if (is.null(colorHsv2)){
    if (colorHsv1[3,] > .5){
      colorHsv2 <- matrix(c(colorHsv1[1,1],colorHsv1[2,1],0),nrow=3,ncol=1,dimnames=list(c("h","s","v")))
    }else{
      colorHsv2 <- matrix(c(colorHsv1[1,1],0,1),nrow=3,ncol=1,dimnames=list(c("h","s","v")))
    }
  }
  modHsv <- colorHsv1%*%rep(1,length(proportion))
  modHsv[1,] <- proportion*(colorHsv1[1,]-colorHsv2[1,]) + colorHsv2[1,]
  modHsv[2,] <- proportion*(colorHsv1[2,]-colorHsv2[2,]) + colorHsv2[2,]
  modHsv[3,] <- proportion*(colorHsv1[3,]-colorHsv2[3,]) + colorHsv2[3,]
  return(modHsv)
}
calculateConservationColor <- calculateIntermediateColor

plotConsensusDendrogram2 <- function(tree,labelSplit="-",lengthRatio=5,rescaleConservation=T,distinctColors=F,equalizeEdgeLengths=F,invert.length=F,treePalette=c("black","orange","cyan","magenta","green2","blue","red","yellow2","gray","pink"),colorLabel=T,maxVariability=max(tree$edge.length),edge.width=1,root.edge=T,show.node.label=T,show.tip.label=F,label.offset=.3,...){
  require(fields)
  # nodeEdges signifies bifurcations (rather than leaf nodes)
  nodeEdges <- tree$edge[,2] > tree$Nnode+1
  tree$node.label[tree$node.label==""] <- " "
  tree$tip.label[tree$tip.label==""] <- " "
  cols <- rep(1,tree$Nnode*2)
  widths <- rep(edge.width,tree$Nnode*2)
  hsvDat <- rgb2hsv(col2rgb(treePalette[1]))
  
  rootConservation <- as.numeric(strsplit(tree$node.label[1],labelSplit)[[1]][2])
  orderedInds <- tree$edge[nodeEdges,2]-(tree$Nnode+1)
  conservation <- as.numeric(sapply(tree$node.label[orderedInds],function(str,spl){return(strsplit(str,spl)[[1]][2])},labelSplit))
  
  # If the label can be split to provide label and conservation %
  if (!is.na(labelSplit)){
    # Root
    rootVariability <- 1- tree$root.edge/maxVariability
    if (rootVariability < 0) rootVariability <- 0
    rootHsv <- calculateConservationColor(hsvDat,rootVariability)
    rootCol <- hsv(rootHsv[1],rootHsv[2],rootHsv[3])
    rootWidth <- 0.5+2*rootVariability
    
    # Bifurcations
    # Convert variability (length value) to color
    #variability <- tree$edge.length[orderedInds]
    variability <- tree$edge.length[nodeEdges]
    variability[variability > maxVariability] <- maxVariability
    variability <- 1 - variability/maxVariability

    # Modify core color by conservation
    modHsv <- calculateConservationColor(hsvDat,variability)
    
    # Set colors based on conservation of each node
    cols[nodeEdges] <- mapply(hsv,modHsv[1,],modHsv[2,],modHsv[3,])
    #cols <- mapply(hsv,modHsv[1,],modHsv[2,],modHsv[3,])
    cols[!nodeEdges] <- rep(treePalette[1],sum(!nodeEdges))
    
    # Set node label after having split off additional info
    tree$node.label <- sapply(tree$node.label,function(str,spl){return(strsplit(str,spl)[[1]][1])},labelSplit)
    
  }else{
    uniqueLabels <- sort(unique(tree$node.label))
    if (distinctColors){
      labelColors <- treePalette[1:length(uniqueLabels)]
    }else{
      labelColors <- treePalette[c(1,rep(2,length(uniqueLabels)-1))]
    }
    names(labelColors) <- uniqueLabels
    # label[indexed by edge child - number of leaf nodes]
    cols[nodeEdges] <- labelColors[tree$node.label[tree$edge[nodeEdges,2]-(tree$Nnode+1)]]
    cols[!nodeEdges] <- labelColors[tree$tip.label[tree$edge[!nodeEdges,2]]]
  }
  
  if (equalizeEdgeLengths){
    tree$edge.length <- rep(1,length(tree$edge.length))
    tree$root.edge <- 1
  }else{
    # Convert conservation to length
    if (rescaleConservation){
      minConservation <- min(c(rootConservation,conservation))
      conservation <- (conservation - minConservation)/(1-minConservation)
      rootConservation <- (rootConservation - minConservation)/(1-minConservation)
    }

    if (invert.length){
      tree$edge.length[nodeEdges] <- lengthRatio*(1 - conservation) + 1
      tree$root.edge <- lengthRatio*(1 - rootConservation) + 1
    }else{
      tree$edge.length[nodeEdges] <- lengthRatio*(conservation) + 1
      tree$root.edge <- lengthRatio*(rootConservation) + 1
    }
  }
  phyloObj <- plot.phylo(tree,edge.color=cols,show.node.label=show.node.label,root.edge=tree$root.edge,show.tip.label=show.tip.label,edge.width=widths,label.offset=label.offset,...)
  return(invisible(phyloObj))
}


plotDendrogramAsymmetry <- function(tree,primaryColor,secondaryColor=NULL,labelSplit="-",rescaleColorRange=F,colorLabel=T,colorDown=F,edge.widths=1,root.edge=T,show.node.label=T,show.tip.label=F,label.offset=.3,colorLegend=F,legendStart.x=0.85,legendStart.y=0.82,legendWidth=0.1,legendHeight=0.16,nlevels=60,...){
  require(fields)
  # nodeEdges signifies bifurcations (rather than leaf nodes)
  nodeEdges <- tree$edge[,2] > tree$Nnode+1
  tree$node.label[tree$node.label==""] <- " "
  tree$tip.label[tree$tip.label==""] <- " "
  cols <- rep(1,tree$Nnode*2)
  widths <- rep(edge.widths,tree$Nnode*2)
  hsvDat1 <- rgb2hsv(col2rgb(primaryColor))
  if (is.null(secondaryColor)){
    hsvDat2 <- NULL    
  }else{
    hsvDat2 <- rgb2hsv(col2rgb(secondaryColor))
  }
  
  subtreeList <- subtrees(tree)
  subtreeDegrees <- sapply(subtreeList,function(x){return(x$Nnode)})+1
  
  asymmetry <- mapply(function(ind,subtreeDegrees){
    if (subtreeDegrees[ind] == 2) return(0)
    if (subtreeDegrees[ind] == 3) return(1)
    sub1Deg <- subtreeDegrees[ind+1]
    if (subtreeDegrees[ind]-sub1Deg==1){
      return(1)
    }
    sub2Deg <- subtreeDegrees[ind+sub1Deg]
    return(abs(sub1Deg-sub2Deg)/(subtreeDegrees[ind]-2))
  },
  1:length(subtreeDegrees),MoreArgs=list(subtreeDegrees))
  
  # Root
  rootAsymmetry <- asymmetry[1]
  rootHsv <- calculateIntermediateColor(hsvDat1,rootAsymmetry,hsvDat2)
  rootCol <- hsv(rootHsv[1],rootHsv[2],rootHsv[3])
  
  tree$node.label[1] <- strsplit(tree$node.label[1],labelSplit)[[1]][1]
  
  # Bifurcations
  orderedInds <- tree$edge[nodeEdges,2]-(tree$Nnode+1)
  
  # Modify core color by asymmetry
  modHsv <- calculateIntermediateColor(hsvDat1,asymmetry[orderedInds],hsvDat2)
  
  # Set colors based on conservation of each node
  cols[nodeEdges] <- mapply(hsv,modHsv[1,],modHsv[2,],modHsv[3,])
  
  if (colorDown){
    # Propagate the colors down a level
    cols <- sapply(tree$edge[,1],
                   function(x,tree,cols,rootCol){
                     if (x%in%tree$edge[,2]){
                       return(cols[which(x==tree$edge[,2])])
                     }else{
                       return(rootCol)
                     }
                   },tree,cols,rootCol)
  }else{
    # Set colors of leaves
    cols[!nodeEdges] <- sapply(tree$edge[!nodeEdges,1],
                               function(x,tree,cols,rootCol){
                                 if (x%in%tree$edge[,2]){
                                   return(cols[which(x==tree$edge[,2])])
                                 }else{
                                   return(rootCol)
                                 }
                               },tree,cols,rootCol)
  }
  
  # Set node label after having split off additional info
  tree$node.label[orderedInds] <- sapply(tree$node.label[orderedInds],function(str,spl){return(strsplit(str,spl)[[1]][1])},labelSplit)
  
  # Leaves
  orderedInds <- tree$edge[!nodeEdges,2]
  #tree$tip.label[orderedInds] <- sapply(tree$tip.label[orderedInds],function(str,spl){return(strsplit(str,spl)[[1]][1])},labelSplit)
  
  tree$edge.length <- rep(1,length(tree$edge.length))
  tree$root.edge <- 1

  phyloObj <- plot.phylo(tree,edge.color=cols,show.node.label=show.node.label,root.edge=tree$root.edge,show.tip.label=show.tip.label,edge.width=widths,label.offset=label.offset,...)
  
  if (colorLegend){
    xspan <- diff(phyloObj$x.lim)
    xmin <- phyloObj$x.lim[1]
    yspan <- diff(phyloObj$y.lim)
    ymin <- phyloObj$y.lim[1]
    yinc <- diff(phyloObj$y.lim)*legendHeight/nlevels
      
    legendcolsHSV <- calculateConservationColor(hsvDat1,(0:nlevels)/nlevels,hsvDat2)
    legendcols <- mapply(hsv,legendcolsHSV[1,],legendcolsHSV[2,],legendcolsHSV[3,])
    xleft <- 0.85*xspan+xmin
    ybottom <- 0.82*yspan+ymin
    xright <- 0.95*xspan+xmin
    ytop <- nlevels*yinc + ybottom
    segments(
      rep(xleft,nlevels),
      (0:nlevels)*yinc + ybottom,
#      (0:nlevels)*.4 + ybottom,
      rep(0.95*xspan+xmin,nlevels),
#      (0:nlevels)*.4 + ybottom,
      (0:nlevels)*yinc + ybottom,
      col=legendcols,lty=1,lwd=3)
    rect(xleft-0.001*xspan,ybottom-0.0025*yspan,xright+0.001*xspan,ytop+0.0025*yspan,border="black",lwd=2)
  }
  
  return(invisible(phyloObj))
}
