####################################FileInfo###############################################
#
# Paper2FigureScripts.r
# Organized by figure number and including non-R instructions.
#
#
###########################################################################################

#### Setup ####

require(ape)
require(MASS)
require(ggplot2)
require(grid)
require(mclust)
require(fields)

if (systemType == "PC"){
  homeDir <- "C:/Documents and Settings/gillette/My Documents/research/"
  clusterDir <- "C:/Documents and Settings/gillette/My Documents/research/ClusterAnalysis/"
  baseCodeDir <- "G:/Dropbox/RCode/"
  clusterGlobalDir <- "C:/Documents and Settings/gillette/My Documents/research/GlobalAlignment/"
  globalAlignDataDir <- "G:/AlignmentData/"
  motifCodeDir <- 'G:/Dropbox/Motif Analysis/Motif Analysis R Files/'
  motifDataDirectory <- "C:/Documents and Settings/gillette/My Documents/research/MotifAnalysis/"
}else{
  homeDir <- "/home/tgillett/"
  clusterDir <- "/home/tgillett/ClusterAnalysis/"
  baseCodeDir <- "/home/tgillett/RUtils/"
  clusterGlobalDir <- "/home/tgillett/PairwiseAlignment/"
  globalAlignDataDir <- clusterGlobalDir
 #  baselineDir <- "/home/tgillett/PairwiseAlignment/Baseline/"
  motifCodeDir <- '/home/tgillett/ClusterAnalysis/'
  motifDataDirectory <- "/home/tgillett/results/"
 consensusCompareDir <- "/home/tgillett/Domains/"
}
prDataDirectory <- paste(motifDataDirectory,"/PercentileRanks/",sep="")

# Load Utils
source(paste(baseCodeDir,"Utils.r",sep=""))
source(paste(baseCodeDir,"VisualizationUtils.r",sep=""))
source(paste(baseCodeDir,"StringUtils.r",sep=""))
source(paste(baseCodeDir,"StatisticsUtils.r",sep=""))

# Load Metadata
masterMetaData <- read.csv(paste(clusterDir,"arborMetaData-NM-MapType2.csv",sep=""),na.strings="",stringsAsFactors=FALSE,row.names=1)
if ("CellType"%in%colnames(masterMetaData)){
  colnames(masterMetaData)[colnames(masterMetaData)=="CellType"] <- "Class2"
}

# Load flagged neurons
flaggedNeurons <- read.table(paste(projectConfigDir,"FlaggedNeurons.txt",sep=""),stringsAsFactors = F)[,1]
flaggedNeurites <- flaggedNeurons[grep("-(Axon|ApicalDendrite|Dendrite|BasalDendrite)",flaggedNeurons)]
flaggedNeurons <- flaggedNeurons[!flaggedNeurons%in%flaggedNeurites]
flaggedNeurites <- append(flaggedNeurites,paste(flaggedNeurons,"-Axon",sep=""))
flaggedNeurites <- append(flaggedNeurites,paste(flaggedNeurons,"-ApicalDendrite",sep=""))
flaggedNeurites <- append(flaggedNeurites,paste(flaggedNeurons,"-Dendrite",sep=""))

# Load cluster functions
source(paste(clusterDir,"ClusterStatistics.r",sep=""))
source(paste(clusterDir,"ClusterAnalysisFunctions.r",sep=""))
# Load metadata class definitions
source(paste(clusterDir,"ClassComparisonDefs.r",sep=""))

clusterMetaData <- clusterMetaData[!rownames(clusterMetaData)%in%flaggedNeurites,]

paper2Theme <- theme_bw()+
  theme(panel.grid.major.x = element_blank(), panel.grid.minor.x = element_blank(),
                   panel.grid.major.y = element_blank(), panel.grid.minor.y = element_blank(),
                   axis.line = element_line(colour="black"),
                   panel.border = element_blank())
gridLines <- theme(panel.grid.major.x = element_line(colour = "grey40"),
                   panel.grid.major.y = element_line(colour = "grey40"))

margins <- c(0.1,0.05,2.4,0.05)
figDir <- "/home/tgillett/figures/Pasta2/"
clusterColors <- c(Gray="#999999",C1="#6666FF",C2="#00CC00",C3="#EE0000",C4="#00CCCC",C5="#DD00DD",C6="#E68800",C7="#555555",PlainCircle="#333333",White="#FFFFFF")
#1:select=2.1, 2:background=1.7, 3:selectEl=1.6, 4:backgroundEl=0.8
scatterClusterSizes <- c(2.1,1.7,1.6,0.8)
apicalColors <- c(CA1="#00B000",CA3="#839800",Neocortex="#00AAAB",
                  #CA1Gray="#99BB99",CA3Gray="#A0B688",NeocortexGray="#88DCDD",
                  CA1Gray="#99CC99",CA3Gray="#A8BB83",NeocortexGray="#92D8D9",
                  CA1Circle="#00B000",CA3Circle="#839800",NeocortexCircle="#00AAAB")

topoFeats <- c("Branches","BranchOrder","Caulescence","PartitionAsymmetry")


# Arbor-specific parameters for clustering
clusterParams <- list(Axon=list(G=4,modelName="VII"),Apical=list(G=3,modelName="EII"),Dendrite=list(G=6,modelName="VEV"))

# Function for generating ellipse points based on cluster model
generateEllipse <- function(parameters,group,points=20){
  sigma <- parameters$variance$sigma[dims,dims,group]
  mu <- parameters$mean[dims,group]
  ev <- eigen(sigma, symmetric = TRUE)
  s <- sqrt(rev(sort(ev$values)))
  V <- t(ev$vectors[, rev(order(ev$values))])
  theta <- (0:points) * (pi/(2 * points))
  x <- s[1] * cos(theta)
  y <- s[2] * sin(theta)
  xy <- cbind(c(x, rev(-x), -x, rev(x)), c(y, rev(y), -y, rev(-y)))
  xy <- xy %*% V
  xy <- sweep(xy, MARGIN = 2, STATS = mu, FUN = "+")
  return(xy)
}


#### Creating figures for Cluster/Consensus/Domain Analysis paper ####
## ggplot themes
getTheme <- function(size=20,xangle=0,yangle=0,x.hjust=NULL){
  return(theme(axis.title=element_text(size=size,face="bold"),axis.text.y=element_text(size=size,angle=yangle),axis.text.x = element_text(size=size, angle = xangle, hjust = x.hjust)))
}

if (!exists("numDimsList")){
  numDimsList <- list()
}
numDimsList[["Apical"]] <- 4
numDimsList[["Axon"]] <- 2
numDimsList[["Dendrite"]] <- 4

#### Figure 1b: Select and align neurites for alignment figure ####
# Use length of 28
Fig1BaselineLength <- 28
rownames(subset(usedMetaData,SeqLen==Fig1BaselineLength&Class2=="Pyramidal cell"&
                  ArborType=="ApicalDendrite"&BrainRegion=="Hippocampus"&
                  ExperCond=="Control"&Protocol!="Acute culture"&Protocol!="Culture"))

# Generate alignment
# python3 spaghetti.py -f /home/tgillett/PairwiseAlignment/SeqList-Fig1b.fasta --gap -1 --gapopen 0 --custom identity.tab -n 12 -o /home/tgillett/PairwiseAlignment/ScoreMat_Fig1bSelect_G0E1.tab -a /home/tgillett/PairwiseAlignment/Alignments_Fig1bSelect_G0E1.tab
# python3 spaghetti.py -f /home/tgillett/PairwiseAlignment/SeqList-Fig1b.fasta --gap -1 --gapopen -3 --custom identity.tab -n 12 -o /home/tgillett/PairwiseAlignment/ScoreMat_Fig1bSelect_G3E1.tab -a /home/tgillett/PairwiseAlignment/Alignments_Fig1bSelect_G3E1.tab
# Currently using "cell49-CA3-ApicalDendrite" and "4s4-CA1-06-11-23-ApicalDendrite"
# grep "cell49-CA3" /home/tgillett/PairwiseAlignment/Alignments_Fig1bSelect_G0E1.tab
# grep "cell49-CA3" /home/tgillett/PairwiseAlignment/Alignments_Fig1bSelect_G3E1.tab
# grep "CA2-W-P15-3-ApicalDendrite" /home/tgillett/PairwiseAlignment/Alignments_Fig1bSelect_G3E1.tab
# apicalDistances["cell49-CA3-ApicalDendrite","4s4-CA1-06-11-23-ApicalDendrite"]
# Chose 01-03-c1-ApicalDendrite cell49-CA3-ApicalDendrite
#apicalDistances["cell49-CA3-ApicalDendrite","01-03-c1-ApicalDendrite"]

# Load baseline
source(paste(clusterGlobalDir,"BaselineHandler.r",sep=""))
baselineKey <- paste("G3E1_e5",travOrder)
loadBaselineAlignmentStats(baselineKey,directory=paste(baselineDir,"GapOpen3_e5/",sep=""),travOrder=travOrder)
#baselineKey <- paste("G3E1_e3",travOrder)
#loadBaselineAlignmentStats(baselineKey,directory=paste(baselineDir,"GapOpen3_1000/",sep=""),travOrder=travOrder)

# Get normalized distance value using baseline shift from Fig1c inset
#globalBaselineSD <- 0.15
globalBaselineSD <- 0.15
#perCharScore <- -1.04 # Gapped score using ungapped alignment
#perCharScore <- -0.82 # from previous pair
perCharScore <- -0.75
zScore <- getBaselineZScore(baselineKey,perCharScore,28,28)
pairDistance <- 1 - zScore*globalBaselineSD

#### Figure 1c: Selecting Neurites ####
# Get featMat as apical dendrite alignment space data (from MDS) using G3E1
featMat <- read.csv(paste(globalAlignDataDir,"Apical_STL_G3E1Identity_MDS8.csv",sep=""),row.names=1)
apicColors <- rep(1,nrow(featMat))
apicColors[rownames(featMat)%in%rownames(subset(usedMetaData,Region2=="CA3"))] <- 2
apicColors[rownames(featMat)%in%rownames(subset(usedMetaData,Region2=="CA1"))] <- 3
png("/home/tgillett/figures/Pasta2/ApicSpaceByRegion.png")
plot(featMat[,1:2],col=apicColors)
dev.off()
#apicalColors <- 
#plot(featMat,col=)
require(mclust)
featEM <- Mclust(featMat[,1:4],G=3)
#plot(featEM,what="classification")
  
# Neocortical
#candidates1 <- featMat[featMat[,1] > -.08 & featMat[,1] < 0.02 & featMat[,2] > .08 & featMat[,2] < .12,]
candidates1 <- featMat[featMat[,1] > -.05 & featMat[,1] < 0.05 & featMat[,2] > .05 & featMat[,2] < .11,]
candidates1 <- candidates1[usedMetaData[rownames(candidates1),"BrainRegion"]=="Neocortex"&usedMetaData[rownames(candidates1),"ExperCond"]=="Control",]
subset(usedMetaData[rownames(candidates1),],SeqLen < 40)[,"SeqLen", drop=F]
# selected short (20): 206211nieuw-ApicalDendrite, 0712LS-2-ApicalDendrite, 
# selected short (22): 30308L1-ApicalDendrite, 0712LS-1-ApicalDendrite, ACELL321-ApicalDendrite
# selected mid   (27): MD070417-3-2-ApicalDendrite, AM-111-1-1-AP-ApicalDendrite, g0437P1-ApicalDendrite 

# CA1
#candidates2 <- featMat[featMat[,1] > -.1 & featMat[,1] < 0.15 & featMat[,2] > -.12 & featMat[,2] < -.06,]
candidates2 <- featMat[featMat[,1] > -0.22 & featMat[,1] < -0.05 & featMat[,2] > -0.14 & featMat[,2] < -0.04 & featMat[,2] > -0.2,]
candidates2 <- candidates2[usedMetaData[rownames(candidates2),"Region2"]=="CA1"&usedMetaData[rownames(candidates2),"ExperCond"]=="Control",]
subset(usedMetaData[rownames(candidates2),],SeqLen < 40)[,"SeqLen", drop=F]
# Selected short (20): 4s1-CA1-08-07-03-ApicalDendrite, 1s3a-CA1-08-05-22-ApicalDendrite
# Selected mid   (27): 
# Selected long  (31): pc1c-ApicalDendrite, 2s2-CA1-08-06-20-ApicalDendrite, 3s1-CA1-08-07-03-ApicalDendrite

# CA3
#candidates3 <- featMat[featMat[,1] > -.25 & featMat[,1] < -0.1 & featMat[,2] > -.15 & featMat[,2] < 0,]
candidates3 <- featMat[featMat[,1] > 0.1 & featMat[,1] < 0.27 & featMat[,2] > -0.15 & featMat[,2] < 0,]
candidates3 <- candidates3[usedMetaData[rownames(candidates3),"Region2"]=="CA3"&usedMetaData[rownames(candidates3),"ExperCond"]=="Control",]
subset(usedMetaData[rownames(candidates3),],SeqLen < 40)[,"SeqLen", drop=F]
# selected short (22): 960824c-ApicalDendrite, cell1-8b-CA3-ApicalDendrite
# selected mid   (27): c60463-ApicalDendrite, cell3zr-ApicalDendrite
# selected long  (31): c53063-ApicalDendrite, c80764-ApicalDendrite 

# Get number of neurites in each candidate set (shorter than 40) by sequence length
with(subset(usedMetaData[rownames(candidates1),],SeqLen < 40)[,"SeqLen", drop=F],aggregate(SeqLen,list(SeqLen),length))
with(subset(usedMetaData[rownames(candidates2),],SeqLen < 40)[,"SeqLen", drop=F],aggregate(SeqLen,list(SeqLen),length))
with(subset(usedMetaData[rownames(candidates3),],SeqLen < 40)[,"SeqLen", drop=F],aggregate(SeqLen,list(SeqLen),length))

# Create sequence list and create fasta file from it
# java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.CreateGroupFastaFile -f "/home/tgillett/fastaFromSwc/NM/Arborization/Mapping2/Size/NMArborSizeSTLMap2.fasta" -g "/home/tgillett/PairwiseAlignment/SeqList-ApicalSelect_Fig1.txt" -o "/home/tgillett/PairwiseAlignment/"

# Run pairwise alignment and record results in ApicalComparisonsWorksheet.docx
# python3 spaghetti.py -f /home/tgillett/PairwiseAlignment/SeqList-ApicalSelect_Fig1.fasta -E -1 -G 0 --custom identity.tab -n 12 -o /home/tgillett/PairwiseAlignment/ScoreMat_ApicalSelect_G0E1_Fig1.tab -a /home/tgillett/PairwiseAlignment/Alignments_ApicalSelect_G0E1_Fig1.tab
# python3 spaghetti.py -f /home/tgillett/PairwiseAlignment/SeqList-ApicalSelect_Fig1.fasta -E -1 -G -3 --custom identity.tab -n 12 -o /home/tgillett/PairwiseAlignment/ScoreMat_ApicalSelect_G3E1_Fig1.tab -a/home/tgillett/PairwiseAlignment/Alignments_ApicalSelect_G3E1_Fig1.tab
# grep "pc1c" Alignments_ApicalSelect_G0E1_Fig1.tab
# grep "pc1c" Alignments_ApicalSelect_G3E1_Fig1.tab

## Convert files to Newick and create dendrograms
# Selected: Smit-Rigter/206211nieuw-ApicalDendrite (NC), Lewis/0712LS-2-ApicalDendrite (NC), korte/4s1-CA1-08-07-03-ApicalDendrite (CA1), korte/1s3a-CA1-08-05-22-ApicalDendrite (CA1)
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/smit-rigter/CNG version/206211nieuw.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 4 -L NONE -b
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/lewis/CNG version/0712LS-2.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 4 -L NONE -b
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/korte/CNG version/4s1-CA1-08-07-03.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 4 -L NONE -b
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/korte/CNG version/1s3a-CA1-08-05-22.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 4 -L NONE -b

# Selected: Gulyas/pc1c-ApicalDendrite (CA1), Korte/2s2-CA1-08-06-20-ApicalDendrite (CA1), Amaral/c53063-ApicalDendrite (CA3), Amaral/c80764-ApicalDendrite (CA1)
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/gulyas/CNG version/pc1c.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 4 -L NONE -b
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/korte/CNG version/2s2-CA1-08-06-20.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 4 -L NONE -b
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/amaral/CNG version/c53063.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 4 -L NONE -b
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/amaral/CNG version/c80764.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 4 -L NONE -b

newickDir <- "/home/tgillett/figures/Pasta2/"
figDir <- newickDir
# Neocortical and CA1
#neurons <- c("206211nieuw","0712LS-2","4s1-CA1-08-07-03","1s3a-CA1-08-05-22")
#dendColors <- c("#DD0088","#DD0088","#C07000","#C07000")
# CA1="#00B000",CA3="#839800"
neurons <- c("pc1c","2s2-CA1-08-06-20","c53063","c80764")
dendColors <- c(apicalColors["CA1"],apicalColors["CA1"],apicalColors["CA3"],apicalColors["CA3"])
for (i in 1:length(neurons)){
  neuron <- neurons[i]
  apicalTree <- read.tree(paste(figDir,neuron,".CNG-Apical.newick",sep=""))
  apicalTree$edge.length <- rep(1,length(apicalTree$edge.length))
  apicalTree$root.edge <- 1
  png(paste(figDir,neuron,"-EqLen.png",sep=""),height=500,width=800)
  plot.phylo(apicalTree,root.edge = T,no.margin = T, edge.color=dendColors[i],edge.width=3)
  dev.off()
}

perCharScores <- c(-29,-35,-23,-14,-17,-30)/31
sapply(perCharScores,function(x,key,l1,l2){return(1-globalBaselineSD*getBaselineZScore(key,x,l1,l2))},baselineKey,31,31)


## Fig 1c Inset - Distribution of scores

gapOpenCost <- -3
if (abs(gapOpenCost)==3){
  gapParamStr <- "G3E1Identity"
  baselineKey <- "G3E1_e5 STL"
}else {
  gapParamStr <- paste("G",abs(gapOpenCost),"E1Identity",sep="")
  baselineKey <- paste("G",abs(gapOpenCost),"E1_e3 STL",sep="")
}
# Load second normalization scores of apical dendrites
apicalDistances <- read.csv(paste(globalAlignDataDir,"ScoringV2Data/ApicalDistances_STL_",gapParamStr,".csv",sep=""),row.names=1,check.names=F)
#apicalDistances <- read.csv(paste(globalAlignDataDir,"ApicalDistances_STL_G3E1Identity.csv",sep=""),row.names=1,check.names=F)
#apicalDistances <- read.csv(paste(globalAlignDataDir,"ScoringV2Data/ApicalDistances_STL_G3E1Identity.csv",sep=""),row.names=1,check.names=F)
#apicalDistances <- read.csv(paste(globalAlignDataDir,"ScoringV2Data/ApicalDistances_STL_G0E1Identity.csv",sep=""),row.names=1,check.names=F)
#apicalDistances <- read.csv(paste(globalAlignDataDir,"ScoringV2Data/ApicalDistances_STL_G1E1Identity.csv",sep=""),row.names=1,check.names=F)

# Removed flagged neurons
flaggedNeurites <- paste(flaggedNeurons,"ApicalDendrite",sep="-")
apicalDistances <- apicalDistances[
  !rownames(apicalDistances)%in%flaggedNeurites,
  !colnames(apicalDistances)%in%flaggedNeurites]
# Get rows and columns in the same order
apicalDistances <- apicalDistances[rownames(apicalDistances),rownames(apicalDistances)]


# Rescale so mean is at 1, sd = 
baselineLength <- getBaselineLength()
#baselineNSMean <- getBaselineMean(baselineKey,baselineLength,baselineLength)
baselineMean <- getBaselineMean(baselineKey,baselineLength,baselineLength)
baselineSD <- getBaselineSD(baselineKey,baselineLength,baselineLength)

# Original: zScore((val-lenPairMean)/lenPairSD)>putInBaselineSpace(zScore*baseSD+baseMean)>CreateDistance(1-inBaselineSpace)
# Reverse:
zscores <- ((1-apicalDistances)-baselineMean)/baselineSD
# Create distance where baseline mean = 1 and SD = globalBaselineSD
figDists <- 1 - zscores*globalBaselineSD

# Get CA1 apicals and CA3 apicals
CA1Apicals <- intersect(rownames(figDists),rownames(subset(usedMetaData,ArborType=="ApicalDendrite"&Region2=="CA1"&Class2=="Pyramidal cell")))
CA3Apicals <- intersect(rownames(figDists),rownames(subset(usedMetaData,ArborType=="ApicalDendrite"&Region2=="CA3"&Class2=="Pyramidal cell")))

CA1CA1 <- c()
CA1CA3 <- c()
CA3CA3 <- c()
for (i in 1:length(CA3Apicals)){
  neurite <- CA3Apicals[i]
  if (i < length(CA3Apicals)){
    CA3CA3 <- append(CA3CA3,as.numeric(figDists[neurite,CA3Apicals[(i+1):length(CA3Apicals)]]))
  }
  CA1CA3 <- append(CA1CA3,as.numeric(figDists[neurite,CA1Apicals]))
}

for (i in 1:length(CA1Apicals)){
  neurite <- CA1Apicals[i]
  if (i < length(CA1Apicals)){
    CA1CA1 <- append(CA1CA1,as.numeric(figDists[neurite,CA1Apicals[(i+1):length(CA1Apicals)]]))
  }
}

CA1vsCA3DF <- data.frame(
  Class=c(rep("CA1vsCA1",length(CA1CA1)),rep("CA1vsCA3",length(CA1CA3)),rep("CA3vsCA3",length(CA3CA3))),
  NormScore=c(CA1CA1,CA1CA3,CA3CA3)
)

distHistPlot <- ggplot()+stat_density(data=data.frame(x=CA1CA3),aes(x=x),color="red",alpha=0.25,fill=NA,size=1.6)+
  stat_density(data=data.frame(x=CA1CA1),aes(x=x),color=apicalColors["CA1"],alpha=0.25,fill=NA,size=1.6)+
  stat_density(data=data.frame(x=CA3CA3),aes(x=x),color=apicalColors["CA3"],alpha=0.25,fill=NA,size=1.6)
ggsave(filename="/home/tgillett/figures/Pasta2/CA1vsCA3-FullNormDistanceDensities-SD0.15.png",distHistPlot,dpi=72)

distHistPlot <- ggplot()+stat_density(data=data.frame(x=(1-CA1CA3)/globalBaselineSD),aes(x=x),color="red",alpha=0.25,fill=NA,size=1.6)+
  stat_density(data=data.frame(x=(1-CA1CA1)/globalBaselineSD),aes(x=x),color=apicalColors["CA1"],alpha=0.25,fill=NA,size=1.6)+
  stat_density(data=data.frame(x=(1-CA3CA3)/globalBaselineSD),aes(x=x),color=apicalColors["CA3"],alpha=0.25,fill=NA,size=1.6)
ggsave(filename="/home/tgillett/figures/Pasta2/CA1vsCA3-ZScores.png",distHistPlot,dpi=72)

mean(CA1CA1)
sd(CA1CA1)
mean(CA1CA3)
sd(CA1CA3)
mean(CA3CA3)
sd(CA3CA3)

t.test(CA1CA1,CA3CA3)
t.test(CA1CA1,CA1CA3)
t.test(CA3CA3,CA1CA3)

#### Figure 2: Alignment space and pairwise clustering ####

# Figure 2a - Alignment space density and scatter plot colored by arbor type
# Relies on alignment space production of AllArbors in ClusterAnalysisAllData.r
#allArborsMDS <- read.csv(paste(globalAlignDataDir,"AllArbors_STLSpurLen2D_G3E1Identity_MDS14.csv",sep=""),row.names=1)
allArborsMDS <- read.csv(paste(globalAlignDataDir,"AllArbors_STLSpurLen2D_G3E1Identity_MDS10.csv",sep=""),row.names=1)
allArborsMDS <- allArborsMDS[!rownames(allArborsMDS)%in%flaggedNeurites,]
#allArborsMDS[,1] <- -allArborsMDS[,1]
#write.csv(allArborsMDS,paste(globalAlignDataDir,"AllArbors_STLSpurLen2D_G3E1Identity_MDS10_FlipX.csv",sep=""))

#pdf("")
#x11()
require(MASS)
dens2D <- kde2d(allArborsMDS[,1],allArborsMDS[,2])
dens2D_34 <- kde2d(allArborsMDS[,3],allArborsMDS[,4])

for (theta in 45*(0:6)){
  png(paste("/home/tgillett/figures/Pasta2/AllArborMDS_Density-1_2-Theta",theta,".png",sep=""),width=1000,height=600)
  kde2dplot(dens2D,theta=theta)
  dev.off()
}

png("/home/tgillett/figures/Pasta2/AllArborMDS_Density-3_4.png",width=800,height=500)
kde2dplot(dens2D_34)
dev.off()

logDens2D <- dens2D
logDens2D$z <- log(logDens2D$z-min(logDens2D$z)+1)
#logDens2D$z <- logDens2D$z-min(logDens2D$z)

png("/home/tgillett/figures/Pasta2/AllArborMDS_LogDensity-1_2-Theta45.png",width=1000,height=600)
kde2dplot(logDens2D,theta=45,zlab="log density")
dev.off()

#### Fig 2b ####

# All Arbor Scatterplots

neuriteArborTypes <- usedMetaData[rownames(allArborsMDS),"ArborType"]
myColors <- c(Axon="#DD00DD",ApicalDendrite="#00CC00",Dendrite="#0000EE")
#myColorsGrayed <- c(Axon="#CC99CC",ApicalDendrite="#92CC92",Dendrite="#8888CC")
myColorsGrayed <- c(Axon="#999999",ApicalDendrite="#999999",Dendrite="#999999")
myColorsCircles <- c(Axon="#EF00EF",ApicalDendrite="#00E000",Dendrite="#0000FF")

## All Arbor Scatterplots with cluster notation
specsByArborType <- list(
  Axon=list(ArborType="Axon",ExperCond="Control"),
  Apical=list(ArborType="ApicalDendrite",ExperCond="Control"),
  Dendrite=list(ArborType="Dendrite",ExperCond="Control"))
allArborSubsetObj <- loadSubsetSeqs(specsByArborType,usedMetaData,candidates=rownames(allArborsMDS))
allArborsMDS <- allArborsMDS[allArborSubsetObj$Sequences,]
allArborsEM <- Mclust(allArborsMDS[,1:5],G=4:12)

# Plot BIC
png("/home/tgillett/figures/Pasta2/AllArborsEM_BIC.png")
plot(allArborsEM,what="BIC")
dev.off()

# Determine associations between arbors types and clusters
groupNames <- paste(names(specsByArborType)," (",sapply(allArborSubsetObj$SeqsByGroup,length),")",sep="")
associations <- clusteringStatisticalAnalysis(allArborsMDS,allArborsEM,allArborSubsetObj,groupNames)

# GroupNum -> ArborType; ArborType -> clusters
arborTypeToClusterMap <- list(c(),c(),c())
clusterToArborTypeMap <- rep(0,allArborsEM$G)
for (i in 1:allArborsEM$G){
#  if (sum(associations$LogP[,i] > 4) == 1){
  if (sum(associations$LogP[,i] > 4) > 0){
    arborTypeToClusterMap[[which.max(associations$LogP[,i])]] <- append(arborTypeToClusterMap[[which.max(associations$LogP[,i])]],i)
    clusterToArborTypeMap[i] <- which.max(associations$LogP[,i])
  }
}

neuritesInAssocClust <- mapply(function(classification,groupId,groupClassMap){
                                 return(classification%in%groupClassMap[[groupId]])
                               },
                               allArborsEM$classification, allArborSubsetObj$GroupId, MoreArgs=list(arborTypeToClusterMap))

totalAccuracy <- mean(neuritesInAssocClust)

# Accuracy rate for each arbor type given by proportion of arbors of type in one of associated clusters
axonAccuracy <- mean(allArborsEM$classification[allArborSubsetObj$SeqsByGroup[[1]]]%in%arborTypeToClusterMap[[1]])
apicalAccuracy <- mean(allArborsEM$classification[allArborSubsetObj$SeqsByGroup[[2]]]%in%arborTypeToClusterMap[[2]])
dendAccuracy <- mean(allArborsEM$classification[allArborSubsetObj$SeqsByGroup[[3]]]%in%arborTypeToClusterMap[[3]])

# axonFalseNeg <- 1 - mean(neuritesInAssocClust[allArborSubsetObj$SeqsByGroup[[1]]])
# apicalFalseNeg <- 1 - mean(neuritesInAssocClust[allArborSubsetObj$SeqsByGroup[[2]]])
# dendFalseNeg <- 1 - mean(neuritesInAssocClust[allArborSubsetObj$SeqsByGroup[[3]]])

# False negative rate for each arbor type is the proportion of arbors of type in unassociated cluster
# (incorrrectly labeled as not-of-type)
axonFalseNeg <- 1 - axonAccuracy
apicalFalseNeg <- 1 - apicalAccuracy
dendFalseNeg <- 1 - dendAccuracy

# False positive rate for each arbor type is the proportion of neurites of wrong type in clusters assocated with given type
# (incorrectly labeled as of-type)
axonFalsePos <- 1 - mean(neuritesInAssocClust[names(allArborsEM$classification)[allArborsEM$classification %in% arborTypeToClusterMap[[1]]]])
apicalFalsePos <- 1 - mean(neuritesInAssocClust[names(allArborsEM$classification)[allArborsEM$classification %in% arborTypeToClusterMap[[2]]]])
dendFalsePos <- 1 - mean(neuritesInAssocClust[names(allArborsEM$classification)[allArborsEM$classification %in% arborTypeToClusterMap[[3]]]])



arborPlotColors <- myColors[allArborSubsetObj$GroupId]
arborPlotColors[!neuritesInAssocClust] <- myColorsGrayed[allArborSubsetObj$GroupId[!neuritesInAssocClust]]

#grp <- 1
#aggregate(allArborsEM$classification,list(allArborsEM$classification),length)
#aggregate(allArborsEM$classification[allArborSubsetObj$GroupId==grp],list(allArborsEM$classification[allArborSubsetObj$GroupId==grp]),length)

parms <- allArborsEM$parameters
dims <- c(1,2)

#require(car)
#png("/home/tgillett/ClusterAnalysis/AllArborsScatter-ByCluster&ArborType_1-2.png",width=800,height=800)
#png("/home/tgillett/figures/Pasta2/AllArborsScatter-ByCluster&ArborType_1-2.png",width=800,height=800)
#plot(allArborsMDS[,dims],col=arborPlotColors,bg=arborPlotColors,pch=21,cex=0.35)
#for (i in 1:allArborsEM$G){
#  car::ellipse(parms$mean[dims,i],shape=parms$variance$sigma[dims,dims,i],center.pch=F,radius=1,col=1)
#}
#dev.off()

require(ellipse)
#df_ell <- data.frame()
df_ell <- list()
lineSize <- 1.15
df_ell <- list() 
for(g in 1:allArborsEM$G){
  ellipseDF <- generateEllipse(parms,g)
  ellipseDF[,1] <- -ellipseDF[,1]
  
  tmpdf <- cbind(ellipseDF,data.frame(color=rep("#000000",nrow(ellipseDF)),size=lineSize+0.35))
  colnames(tmpdf) <- c("x","y","color","size")
  df_ell <- append(df_ell, list(tmpdf))
  tmpdf <- cbind(ellipseDF,data.frame(color=rep(myColors[clusterToArborTypeMap[g]],nrow(ellipseDF)),size=lineSize))
  colnames(tmpdf) <- c("x","y","color","size")
  df_ell <- append(df_ell, list(tmpdf))
}

#   #df_ell <- rbind(df_ell, cbind(as.data.frame(ellipse::ellipse(parms$variance$sigma[dims,dims,g], 
#   #                                            centre=parms$mean[dims,g])),color=myColors[clusterToArborTypeMap[g]]))
#   tmpdf <- as.data.frame(ellipse::ellipse(parms$variance$sigma[dims,dims,g], 
#                                           centre=parms$mean[dims,g]))
#   tmpdf <- cbind(tmpdf,color=rep(myColorsCircles[clusterToArborTypeMap[g]],nrow(tmpdf)))
#   colnames(tmpdf) <- c("x","y","color")
#   df_ell <- append(df_ell, list(tmpdf))
# }

clusteredArborsMDS <- cbind(allArborsMDS[,dims],arborPlotColors)
colnames(clusteredArborsMDS) <- c("x","y","color")
clusteredArborsMDS$x <- -clusteredArborsMDS$x
clusteredArborsMDS$size = .3

colorArray <- c(myColors,myColorsGrayed,myColorsCircles,"#000000")
names(colorArray) <- colorArray

allNeuritesClustFig <- ggplot(clusteredArborsMDS,aes(x=x,y=y,colour=color,fill=color,color=color))+
  geom_point(size=1.4)+
  scale_size(range=c(1.0,1.6))+
  scale_fill_manual(values=colorArray)+
  scale_colour_manual(values=colorArray)+
  paper2Theme+coord_fixed()+scale_x_continuous(breaks=c(-.5,-.3,-.1,.1,.3))
for (i in df_ell){
  allNeuritesClustFig <- allNeuritesClustFig + geom_path(aes(x=x,y=y,colour=color,size=size), data=i, linetype=1)
  #allNeuritesClustFig <- allNeuritesClustFig + geom_path(aes(x=x,y=y,colour=color), data=i, size=1, linetype=2)
}
ggsave("/home/tgillett/figures/Pasta2/AllArborsScatter-ByCluster&ArborType_1-2_012615.png",allNeuritesClustFig,width=8,height=8)
#ggsave("/home/tgillett/figures/Pasta2/AllArborsScatter-ByCluster&ArborType_3-2.png",allNeuritesClustFig,width=12,height=8)

#### Figure 2c - Apical clustering ####
apicalMDS <- read.csv(paste(globalAlignDataDir,"Apical_STL_G3E1Identity_MDS8.csv",sep=""),row.names=1)
apicalMDS[,1] <- -apicalMDS[,1]
#apicalMDS[,2] <- -apicalMDS[,2]

apicalSpecs <- AllApicalSpecificationsByHPNC
names(apicalSpecs) <- c("CA1","CA3","Neocortex")
allApicalSubsetObj <- loadSubsetSeqs(apicalSpecs,usedMetaData,candidates=rownames(apicalMDS))
#apicalMDS <- apicalMDS[subsetObj$Sequences,]
apicalEM <- Mclust(apicalMDS[,1:4],G=3,modelNames="EII")

# Determine associations between arbors types and clusters
groupNames <- paste(names(apicalSpecs)," (",sapply(allApicalSubsetObj$SeqsByGroup,length),")",sep="")
apicalAssociations <- clusteringStatisticalAnalysis(apicalMDS,apicalEM,allApicalSubsetObj,groupNames)

# GroupNum -> ArborType; ArborType -> clusters
apicalTypeToClusterMap <- list(c(),c(),c())
clusterToApicalTypeMap <- rep(0,apicalEM$G)
for (i in 1:apicalEM$G){
  if (sum(apicalAssociations$LogP[,i] > 4) == 1){
    apicalTypeToClusterMap[[which.max(apicalAssociations$LogP[,i])]] <- append(apicalTypeToClusterMap[[which.max(apicalAssociations$LogP[,i])]],i)
    clusterToApicalTypeMap[i] <- which.max(apicalAssociations$LogP[,i])
  }
}

neuritesInAssocClust <- mapply(function(classification,groupId,groupClassMap){
  return(classification%in%groupClassMap[[groupId]])
},
apicalEM$classification[allApicalSubsetObj$Sequences], allApicalSubsetObj$GroupId, MoreArgs=list(apicalTypeToClusterMap))

# CA1, CA3, NC
apicalColors <- c(CA1="#00B000",CA3="#839800",Neocortex="#00AAAB",
                  #CA1Gray="#99BB99",CA3Gray="#A0B688",NeocortexGray="#88DCDD",
                  CA1Gray="#99CC99",CA3Gray="#A8BB83",NeocortexGray="#92D8D9",
                  CA1Circle="#00B000",CA3Circle="#839800",NeocortexCircle="#00AAAB",Black="black")
                  #CA1Circle="#00CC00",CA3Circle="#A0B600",NeocrotexCricle="#00DCDD")

apicalGroup <- names(apicalSpecs)[allApicalSubsetObj$GroupId]
apicalGroup[!neuritesInAssocClust] <- paste(apicalGroup[!neuritesInAssocClust],"Gray",sep="")
#apicalColorsGrayed[allApicalSubsetObj$GroupId[!neuritesInAssocClust]]

#grp <- 1
#aggregate(allArborsEM$classification,list(allArborsEM$classification),length)
#aggregate(allArborsEM$classification[subsallApicalSubsetObjtObj$GroupId==grp],list(allArborsEM$classification[allApicalSubsetObj$GroupId==grp]),length)

parms <- apicalEM$parameters
dims <- c(1,2)

#require(ellipse)
df_ell <- list()
lineSize <- 1.6
for(g in 1:apicalEM$G){
#   tmpdf <- as.data.frame(ellipse::ellipse(parms$variance$sigma[dims,dims,g], 
#                                           centre=parms$mean[dims,g]))
#   #tmpdf <- cbind(tmpdf,color=rep(apicalColorsCircles[clusterToApicalTypeMap[g]],nrow(tmpdf)))
#   tmpdf <- cbind(tmpdf,group=rep(paste(names(apicalSpecs)[clusterToApicalTypeMap[g]],"Circle",sep=""),nrow(tmpdf)))
#   colnames(tmpdf) <- c("x","y","group")
#   df_ell <- append(df_ell, list(tmpdf))
  ellipseDF <- generateEllipse(parms,g,40)
#  ellipseDF[,1] <- -ellipseDF[,1]
  
  tmpdf <- cbind(ellipseDF,data.frame(group=rep("Black",nrow(ellipseDF)),size=lineSize+0.02))
  colnames(tmpdf) <- c("x","y","group","size")
  df_ell <- append(df_ell, list(tmpdf))
  tmpdf <- cbind(ellipseDF,data.frame(group=rep(names(apicalSpecs)[clusterToApicalTypeMap[g]],nrow(ellipseDF)),size=lineSize))
  colnames(tmpdf) <- c("x","y","group","size")
  df_ell <- append(df_ell, list(tmpdf))
}

clusteredApicalsMDS <- cbind(apicalMDS[allApicalSubsetObj$Sequences,dims],apicalGroup)
colnames(clusteredApicalsMDS) <- c("x","y","group")


apicalClustFig <- ggplot(clusteredApicalsMDS,aes(x=x,y=y,colour=group,fill=group))+
  geom_point(size=2.1)+
  scale_size(range=c(1.2,1.7))+
  scale_fill_manual(values=apicalColors)+scale_colour_manual(values=apicalColors)+
  paper2Theme+coord_fixed()
for (i in df_ell){
  apicalClustFig <- apicalClustFig + geom_path(aes(x=x,y=y,colour=group, size=size), data=i, linetype=1)
  #allNeuritesClustFig <- allNeuritesClustFig + geom_path(aes(x=x,y=y,colour=group), data=i, size=1, linetype=2)
}
ggsave("/home/tgillett/figures/Pasta2/ApicalScatter-ByCluster&Region1-2_012615.png",apicalClustFig,width=12,height=8)
#ggsave("/home/tgillett/figures/Pasta2/ApicalScatter-ByCluster&Region1-2.png",apicalClustFig,width=12,height=8)


#### Fig 2d-f - Cluster and Rand Index with AS & morphos ####

# Assumes 2 classes and 2 clusters only
ClusterAccuracy <- function(knownClassification,clusterClassification){
  score1 <- mean(clusterClassification==knownClassification)
  score2 <- mean(clusterClassification!=knownClassification)
  return(max(score1,score2))
}

#topologicalMetrics <- c("Branches","BranchOrder","PartitionAsymmetry","Caulescence")
topologicalMetrics <- c("BranchOrder","PartitionAsymmetry","Caulescence")

sharedSeqs <- intersect(rownames(morphometricsXformed)[!is.na(Morphometrics$Caulescence)],rownames(usedMetaData))
## First, do PCA on morphometrics (same PCA applies to all group comparisons/classifications)
morphoPCObj <- prcomp(morphometricsXformed[sharedSeqs,])
# PCs (SD)
# 1 (2.29): Overall size
# 2 (1.84): Branching density; higher density -> greater branch angles and asymmetry
# 3 (1.13): 2D extent (height & width) opposed to diameter (volume & surface area)
# 4 (1.01): Volume caulescence (-PKClassic)
# 5 (0.81): Flatness (-depth), associated with acaulescent arbors (but not necessarily symmetric)
# 6 (0.76): Narrow branching with more even child branch diameters (-RemoteBifAmpl,+PKClassic)
topoPCObj <- prcomp(morphometricsXformed[sharedSeqs,topologicalMetrics])
# PCs (SD)
# 1 (1.68): Size associated with asymmetry/caulescence (all 4 metrics)
# 2 (0.81): Number of branches against caulescence (fewer branches -> more caulescent?)
# 3 (0.64): Partition Asymmetry for few branches and low caulescence
# 4 (0.33)


apicalCA1Spec <- list(ArborType="ApicalDendrite",Class2="Pyramidal cell",ExperCond="Control",BrainRegion="Hippocampus",Region2="CA1")
apicalCA3Spec <- list(ArborType="ApicalDendrite",Class2="Pyramidal cell",ExperCond="Control",BrainRegion="Hippocampus",Region2="CA3")
specifications <- list(CA1=apicalCA1Spec,CA3=apicalCA3Spec)
arborType <- "Apical"
pairName <- "ApicalCA1vsCA3"
# axonMousePyrSpec <- list(ArborType="Axon",Class2="Pyramidal cell",ExperCond="Control",BrainRegion="Neocortex",Species="Mouse")
# axonUPN <- list(ArborType="Axon",Class2="Uniglomerular projection neuron",ExperCond="Control")
# specifications <- list(MousePyr=axonMousePyrSpec,FlyUPN=axonUPN)
# arborType <- "Axon"
arborName <- arborNamesByType[arborType]
alignmentSpace <- arborMDS[[arborType]]

sharedSeqs <- intersect(rownames(alignmentSpace),rownames(morphometricsXformed))

subsetObj <- loadSubsetSeqs(specifications,usedMetaData,candidates=sharedSeqs)
ASSub <- alignmentSpace[subsetObj$Sequences,]
morphoSub <- morphometricsXformed[subsetObj$Sequences,]
group1 <- subsetObj$SeqsByGroup[[1]]
group2 <- subsetObj$SeqsByGroup[[2]]

# Determine best combination of alignment space dimensions
bestScore <- 0
bestAccuracy <- 0
bestDims <- c()
permutations <- expand.grid(rep(list(0:1),length(1:ncol(ASSub))))
for (whichDimsInd in 2:nrow(permutations)){
  whichDims <- which(permutations[whichDimsInd,]==1)
  if (length(whichDims) == 1){
    featEM_ASCombo <- Mclust(ASSub[,whichDims,drop=F],G=2)
  }else{
    featEM_ASCombo <- Mclust(ASSub[,whichDims,drop=F],G=2,modelNames=ASOnlyModels)
  }
  riScore <- AdjustedRandIndex(featEM_ASCombo$classification,subsetObj$GroupIds)
  if (riScore > bestScore){
    bestScore <- riScore
    bestAccuracy <- ClusterAccuracy(subsetObj$GroupIds,featEM_ASCombo$classification)
    bestDims <- whichDims
  }
}

if (length(bestDims) == 1){
  featEM_ASBest <- Mclust(ASSub[,whichDims,drop=F],G=2)
}else{
  featEM_ASBest <- Mclust(ASSub[,whichDims,drop=F],G=2,modelNames=ASOnlyModels)
}
AdjustedRandIndex(featEM_ASBest$classification,subsetObj$GroupIds)


# Determine best combination of topological metrics
bestToposScore <- 0
bestToposAccuracy <- 0
bestTopos <- c()
permutations <- expand.grid(rep(list(0:1),length(1:length(topologicalMetrics))))
for (whichDimsInd in 2:nrow(permutations)){
  whichTopos <- topologicalMetrics[which(permutations[whichDimsInd,]==1)]
  featEM_TopoCombo <- Mclust(morphoSub[,whichTopos,drop=F],G=2)
  riScore <- AdjustedRandIndex(featEM_TopoCombo$classification,subsetObj$GroupIds)
  if (riScore > bestToposScore){
    bestToposScore <- riScore
    bestToposAccuracy <- ClusterAccuracy(subsetObj$GroupIds,featEM_TopoCombo$classification)
    bestTopos <- whichTopos
  }
}

featEM_ASTopoCombo <- Mclust(cbind(morphoSub[,bestTopos,drop=F],ASSub[,bestDims,drop=F]),G=2)
ASTopoComboScore <- AdjustedRandIndex(featEM_ASTopoCombo$classification,subsetObj$GroupIds)

## Classify by morphometrics

# Find how well each morphometric performs on its own
morphoRandInds <- c()
morphoModels <- c()
for (i in 1:ncol(morphoSub)){
  featEM_Morpho <- Mclust(morphoSub[,i,drop=F],G=2)
  morphoRandInds[i] <- AdjustedRandIndex(featEM_Morpho$classification,subsetObj$GroupIds)
  morphoModels[i] <- featEM_Morpho$modelName
}
bestMorpho <- colnames(morphoSub)[which.max(morphoRandInds)]
#bestMorpho
#max(morphoRandInds)
names(morphoRandInds) <- colnames(morphoSub)
morphoRandInds <- sort(morphoRandInds,decreasing=T)

# Cluster by best morphometrics
if (sum(morphoRandInds > .08) > 0){
  bestMorphos <- names(morphoRandInds)[morphoRandInds > .10]
  featEM_Best1Morpho <- Mclust(morphoSub[,bestMorphos[1]],G=2)
  
  featEM_BestMorphos <- Mclust(morphoSub[,bestMorphos],G=2)
  
  # Determine best morphometric combination
  bestScore <- 0
  bestMorphos <- c()
  morphoFeats <- names(morphoRandInds)[morphoRandInds > .15]
  permutations <- expand.grid(rep(list(0:1),length(morphoFeats)))
  for (whichDimsInd in 2:nrow(permutations)){
    selectMorphos <- morphoFeats[which(permutations[whichDimsInd,]==1)]
    featEM_Morphos <- Mclust(morphoSub[,selectMorphos,drop=F],G=2)
    riScore <- AdjustedRandIndex(featEM_Morphos$classification,subsetObj$GroupIds)
    if (riScore > bestScore){
      bestMorphosScore <- riScore
      bestMorphosAccuracy <- ClusterAccuracy(subsetObj$GroupIds,featEM_Morphos$classification)
      bestMorphos <- selectMorphos
    }
  }
  bestMorphosList[[rownames(riTable)[pairNum]]] <- bestMorphos
  
}


# Determine best combination of PCs of topological metrics
#morphoPCObj <- prcomp(morphometricsXformed)
topoPCs <- topoPCObj$x[rownames(morphoSub),]

bestTopoPCsScore <- 0
bestTopoPCs <- c()
permutations <- expand.grid(rep(list(0:1),length(1:length(topologicalMetrics))))
for (whichDimsInd in 2:nrow(permutations)){
  whichTopoPCs <- which(permutations[whichDimsInd,]==1)
  featEM_TopoPCCombo <- Mclust(topoPCs[,whichTopoPCs,drop=F],G=2)
  riScore <- AdjustedRandIndex(featEM_TopoPCCombo$classification,subsetObj$GroupIds)
  print(paste(paste(whichTopoPCs,collapse=","),riScore))
  if (riScore > bestTopoPCsScore){
    bestTopoPCsScore <- riScore
    bestTopoPCs <- whichTopoPCs
  }
}

featEM_ASTopoPCsCombo <- Mclust(cbind(topoPCs[,bestTopoPCs,drop=F],ASSub[,bestDims,drop=F]),G=2)
ASTopoPCsComboScore <- AdjustedRandIndex(featEM_ASTopoPCsCombo$classification,subsetObj$GroupIds)

# Plot best alignment space classification, Dims 1 & 2
plotdf <- cbind(cbind(as.data.frame(ASSub[,1:2]),cluster=factor(featEM_ASBest$classification)),group=names(specifications)[subsetObj$GroupIds])
colnames(plotdf) <- c("x","y","cluster","group")
apicalClassFig <- ggplot(plotdf,aes(x=x,y=y,colour=group,fill=group,shape=cluster))+
  geom_point(size=3)+scale_shape_manual(values=c(16,17)) +  
  #scale_shape_discrete(solid=T, guide="none")+
  scale_fill_manual(values=apicalColors)+scale_colour_manual(values=apicalColors)+
  paper2Theme+coord_fixed()
ggsave(paste(figDir,pairName,"Cluster-",featEM_ASBest$modelName,"2-AlignmentSpaceBestClassification-Dims12.png",sep=""),
       apicalClassFig,width=8,height=8)

# Plot best alignment space classification, Best two Dims
plotdf <- cbind(cbind(as.data.frame(ASSub[,bestDims[1:2]]),cluster=factor(featEM_ASBest$classification)),group=names(specifications)[subsetObj$GroupIds])
colnames(plotdf) <- c("x","y","cluster","group")
apicalClassFig <- ggplot(plotdf,aes(x=x,y=y,colour=group,fill=group,shape=cluster))+
  geom_point(size=3)+scale_shape_manual(values=c(16,17)) +  
  #scale_shape_discrete(solid=T, guide="none")+
  scale_fill_manual(values=apicalColors)+scale_colour_manual(values=apicalColors)+
  paper2Theme+coord_fixed()
ggsave(paste(figDir,pairName,"Cluster-",featEM_ASBest$modelName,"2-AlignmentSpaceBestClassification-Dims",paste(bestDims[1:2],collapse=""),".png",sep=""),
       apicalClassFig,width=8,height=8)

plotdf <- cbind(cbind(as.data.frame(ASSub[,bestDims[c(1,3)]]),cluster=factor(featEM_ASBest$classification)),group=names(specifications)[subsetObj$GroupIds])
colnames(plotdf) <- c("x","y","cluster","group")
apicalClassFig <- ggplot(plotdf,aes(x=x,y=y,colour=group,fill=group,shape=cluster))+
  geom_point(size=3)+scale_shape_manual(values=c(16,17)) +  
  #scale_shape_discrete(solid=T, guide="none")+
  scale_fill_manual(values=apicalColors)+scale_colour_manual(values=apicalColors)+
  paper2Theme+coord_fixed()
ggsave(paste(figDir,pairName,"Cluster-",featEM_ASBest$modelName,"2-AlignmentSpaceBestClassification-Dims",paste(bestDims[c(1,3)],collapse=""),".png",sep=""),
       apicalClassFig,width=8,height=8)

# Plot best morphometrics clustering (plot on raw Morphometrics)
plotdf <- cbind(cbind(as.data.frame(Morphometrics[rownames(morphoSub),bestMorphos[1:2]]),cluster=factor(featEM_BestMorphos$classification)),group=names(specifications)[subsetObj$GroupIds])
colnames(plotdf) <- c(bestMorphos[1:2],"cluster","group")
apicalClassFig <- ggplot(plotdf,aes_string(x=bestMorphos[1],y=bestMorphos[2],colour="group",fill="group",shape="cluster"))+
  geom_point(size=3.5)+scale_shape_discrete(solid=T, guide="none")+
  scale_fill_manual(values=apicalColors)+scale_colour_manual(values=apicalColors)+
  paper2Theme+
  theme(axis.text = element_text(size=17),
        axis.title=element_text(size=18),
        axis.ticks = element_line(size=1), 
        axis.ticks.length=unit(3.5,"mm"),
        axis.ticks.margin = unit(2.1, "mm"))
ggsave(paste(figDir,pairName,"Cluster-",featEM_BestMorphos$modelName,"2-BestMorphosClassification.png",sep=""),
       apicalClassFig,width=7,height=6) #,units="in",dpi="600")
#  ggsave(paste(figDir,"GroupPair",pairNum,"Cluster-",featEM_BestMorphos$modelName,"2-BestMorphosNormedClassification.png",sep=""),apicalClassFig+coord_fixed(),width=8,height=8)

# Cluster by best morphometrics and best alignment space dims
selectMorpho <- bestMorphos[1]
plotdf <- data.frame(Dim1=featMatSub[,1],Morphometrics[rownames(morphoSub),selectMorpho],cluster=factor(featEM_BestMorphosBestAS$classification),group=names(specifications)[subsetObj$GroupIds])
colnames(plotdf) <- c("Dimension 1", selectMorpho,"cluster","group")
apicalClassFig <- ggplot(plotdf,aes_string(x="Dimension 1",y=selectMorpho,colour="group",fill="group",shape="cluster"))+
  geom_point(size=3.5)+scale_shape_discrete(solid=T, guide="none")+
  scale_fill_manual(values=apicalColors)+scale_colour_manual(values=apicalColors)+
  paper2Theme+
  theme(axis.text = element_text(size=17),
        axis.title=element_text(size=18),
        axis.ticks = element_line(size=1), 
        axis.ticks.length=unit(3.5,"mm"),
        axis.ticks.margin = unit(2.1, "mm"))
ggsave(paste(figDir,pairName,"Cluster-",featEM_BestCombo$modelName,"2-BestComboClassification.png",sep=""),
       apicalClassFig,width=7,height=6) #,units="in",dpi="600")


# Plot best topological metrics clustering
featEM_TopoBest <- Mclust(morphoSub[,whichTopos,drop=F],G=2)

plotdf <- cbind(cbind(as.data.frame(Morphometrics[rownames(morphoSub),bestTopos[1:2]]),cluster=factor(featEM_TopoBest$classification)),group=names(specifications)[subsetObj$GroupIds])
#plotdf <- cbind(cbind(as.data.frame(morphoSub[,bestTopos[1:2]]),cluster=factor(featEM_Topos$classification)),group=names(specifications)[subsetObj$GroupIds])
colnames(plotdf) <- c(bestTopos[1:2],"cluster","group")
topoClassFig <- ggplot(plotdf,aes_string(x=bestTopos[1],y=bestTopos[2],colour="group",fill="group",shape="cluster"))+
  geom_point(size=3.5)+scale_shape_discrete(solid=T, guide="none")+
  #scale_fill_manual(values=apicalColors)+scale_colour_manual(values=apicalColors)+
  #scale_fill_manual(values=axonColors)+scale_colour_manual(values=axonColors)+
  paper2Theme+
  # For best Axon case of Pyramidal vs UPNs
  coord_cartesian(ylim = c(0, 35), xlim = c(0, 800))+
  theme(axis.text = element_text(size=17),
        axis.title=element_text(size=18),
        axis.ticks = element_line(size=1), 
        axis.ticks.length=unit(3.5,"mm"),
        axis.ticks.margin = unit(2.1, "mm"))
ggsave(paste(figDir,pairName,"Cluster-",featEM_Topos$modelName,"2-BestToposClassification-Raw.png",sep=""),
       topoClassFig,width=7,height=6) #,units="in",dpi="600")
ggsave(paste(figDir,pairName,"Cluster-",featEM_Topos$modelName,"2-BestToposClassification.png",sep=""),
       topoClassFig,width=7,height=6) #,units="in",dpi="600")

# Cluster by best topological metrics and best alignment space dims
selectMorpho <- bestTopos[1]
selectDim <- bestDims[1]
plotdf <- cbind(cbind(data.frame(Dim1=ASSub[,bestDims[1]],Morphometrics[rownames(morphoSub),selectMorpho]),cluster=factor(featEM_ASTopoCombo$classification)),group=names(specifications)[subsetObj$GroupIds])
#plotdf <- cbind(cbind(data.frame(Dim1=ASSub[,bestDims[1]],morphoSub[,selectMorpho]),cluster=factor(featEM_ASTopoCombo$classification)),group=names(specifications)[subsetObj$GroupIds])
colnames(plotdf) <- c(paste("Dimension",bestDims[1],sep=""), selectMorpho,"cluster","group")
apicalClassFig <- ggplot(plotdf,aes_string(x=paste("Dimension",bestDims[1],sep=""),y=selectMorpho,colour="group",fill="group",shape="cluster"))+
  geom_point(size=3.5)+scale_shape_discrete(solid=T, guide="none")+
  scale_fill_manual(values=apicalColors)+scale_colour_manual(values=apicalColors)+
  paper2Theme+
  # For best Axon case of Pyramidal vs UPNs
  #coord_cartesian(ylim = c(0, 800))+
  theme(axis.text = element_text(size=17),
        axis.title=element_text(size=18),
        axis.ticks = element_line(size=1), 
        axis.ticks.length=unit(3.5,"mm"),
        axis.ticks.margin = unit(2.1, "mm"))
ggsave(paste(figDir,pairName,"Cluster-",featEM_BestToposBestAS$modelName,"2-BestTopoRaw&DimsClassification.png",sep=""),
       apicalClassFig,width=7,height=6) #,units="in",dpi="600")
#ggsave(paste("/home/tgillett/figures/Pasta2/GroupPair",pairNum,"Cluster-",featEM_BestToposBestAS$modelName,"2-BestTopo&DimsClassification.png",sep=""),
#       apicalClassFig,width=7,height=6) #,units="in",dpi="600")

# Plot best topological PCs clustering
featEM_TopoPCsBest <- Mclust(topoPCs[,bestTopoPCs,drop=F],G=2)

plotdf <- cbind(cbind(as.data.frame(topoPCs[rownames(morphoSub),bestTopoPCs[1:2]]),cluster=factor(featEM_TopoPCsBest$classification)),group=names(specifications)[subsetObj$GroupIds])
colnames(plotdf) <- c(paste("PC",bestTopoPCs[1:2],sep=""),"cluster","group")
topoPCClassFig <- ggplot(plotdf,aes_string(x=paste("PC",bestTopoPCs[1],sep=""),y=paste("PC",bestTopoPCs[2],sep=""),colour="group",fill="group",shape="cluster"))+
  geom_point(size=3.5)+scale_shape_discrete(solid=T, guide="none")+
  scale_fill_manual(values=apicalColors)+scale_colour_manual(values=apicalColors)+
  paper2Theme
ggsave(paste(figDir,pairName,"Cluster-",featEM_TopoPCsBest$modelName,"2-BestTopoPCsClassification.png",sep=""),
       topoPCClassFig,width=7,height=6) #,units="in",dpi="600")

png(paste(figDir,"BestPCsClustering.png"))
plot(featEM_TopoPCsBest,what="classification")
dev.off()

klarASTopo <- stepclass(x=cbind(morphoSub[,topoFeats],ASSub),grouping=subsetObj$GroupIds,method="lda",direction="both",improvement=.01)
selectMorpho <- "BranchOrder"
asDim <- 1
#plotdf <- data.frame(Dim1=ASSub[,asDim],Morphometrics[rownames(morphoSub),selectMorpho],group=names(specifications)[subsetObj$GroupIds])
plotdf <- data.frame(Dim1=ASSub[,asDim],morphoSub[,selectMorpho],group=names(specifications)[subsetObj$GroupIds])
#colnames(plotdf) <- c(paste("Dimension",asDim,sep=""), selectMorpho,"cluster","group")
colnames(plotdf) <- c(paste("Dimension",asDim,sep=""), selectMorpho,"group")
apicalClassFig <- ggplot(plotdf,aes_string(x=paste("Dimension",asDim,sep=""),y=selectMorpho,colour="group",fill="group"))+
  geom_point(size=3.5)+scale_shape_discrete(solid=T, guide="none")+
  scale_fill_manual(values=apicalColors)+scale_colour_manual(values=apicalColors)+
  paper2Theme+
  # For best Axon case of Pyramidal vs UPNs
  #coord_cartesian(ylim = c(0, 800))+
  theme(axis.text = element_text(size=17),
        axis.title=element_text(size=18),
        axis.ticks = element_line(size=1), 
        axis.ticks.length=unit(3.5,"mm"),
        axis.ticks.margin = unit(2.1, "mm"))
#ggsave(paste("/home/tgillett/figures/Pasta2/",pairName,"Cluster-",featEM_BestToposBestAS$modelName,"2-BestTopo&DimsClass-Dim",asDim,"-vs-",selectMorpho,".png",sep=""),
#ggsave(paste("/home/tgillett/figures/Pasta2/",pairName,"-Dim",asDim,"-vs-",selectMorpho,"Raw.png",sep=""),
ggsave(paste("/home/tgillett/figures/Pasta2/",pairName,"-Dim",asDim,"-vs-",selectMorpho,".png",sep=""),
       apicalClassFig,width=7,height=6) #,units="in",dpi="600")

klarASs <- list()
klarTopoPCs <- list()
klarASTopoPCs <- list()
for (i in 1:10){
  klarAS <- stepclass(x=ASSub,grouping=subsetObj$GroupIds,method="lda",direction="both",improvement=.01)
  klarASs <- append(klarASs,list(klarAS))
  klarTopoPC <- stepclass(x=topoPCs,grouping=subsetObj$GroupIds,method="lda",direction="both",improvement=.01)
  klarTopoPCs <- append(klarTopoPCs,list(klarTopoPC))
  klarASTopoPC <- stepclass(x=cbind(topoPCs,ASSub),grouping=subsetObj$GroupIds,method="lda",direction="both",improvement=.01)
  klarASTopoPCs <- append(klarASTopoPCs,list(klarASTopoPC))
}
klarASScores <- as.numeric(sapply(klarASs,function(x){x$result.pm[1]}))
klarTopoPCScores <- as.numeric(sapply(klarTopoPCs,function(x){x$result.pm[1]}))
klarASTopoPCScores <- as.numeric(sapply(klarASTopoPCs,function(x){x$result.pm[1]}))
mean(klarASScores)
mean(klarTopoPCScores)
mean(klarASTopoPCScores)
median(klarASScores)
median(klarTopoPCScores)
median(klarASTopoPCScores)
max(klarASScores)
max(klarTopoPCScores)
max(klarASTopoPCScores)

selectMorpho <- "BranchOrder"
asDim <- 1
#plotdf <- data.frame(Dim1=ASSub[,asDim],Morphometrics[rownames(morphoSub),selectMorpho],group=names(specifications)[subsetObj$GroupIds])
plotdf <- data.frame(Dim1=ASSub[,asDim],morphoSub[,selectMorpho],group=names(specifications)[subsetObj$GroupIds])
#colnames(plotdf) <- c(paste("Dimension",asDim,sep=""), selectMorpho,"cluster","group")
colnames(plotdf) <- c(paste("Dimension",asDim,sep=""), selectMorpho,"group")
apicalClassFig <- ggplot(plotdf,aes_string(x=paste("Dimension",asDim,sep=""),y=selectMorpho,colour="group",fill="group"))+
  geom_point(size=3.5)+scale_shape_discrete(solid=T, guide="none")+
  scale_fill_manual(values=apicalColors)+scale_colour_manual(values=apicalColors)+
  paper2Theme+
  # For best Axon case of Pyramidal vs UPNs
  #coord_cartesian(ylim = c(0, 800))+
  theme(axis.text = element_text(size=17),
        axis.title=element_text(size=18),
        axis.ticks = element_line(size=1), 
        axis.ticks.length=unit(3.5,"mm"),
        axis.ticks.margin = unit(2.1, "mm"))
#ggsave(paste("/home/tgillett/figures/Pasta2/",pairName,"Cluster-",featEM_BestToposBestAS$modelName,"2-BestTopo&DimsClass-Dim",asDim,"-vs-",selectMorpho,".png",sep=""),
#ggsave(paste("/home/tgillett/figures/Pasta2/",pairName,"-Dim",asDim,"-vs-",selectMorpho,"Raw.png",sep=""),
ggsave(paste("/home/tgillett/figures/Pasta2/",pairName,"-Dim",asDim,"-vs-",selectMorpho,".png",sep=""),
       apicalClassFig,width=7,height=6) #,units="in",dpi="600")
#### Fig 2d-f Alt - Compare alignment distance and morphometric distances ####

arborMDS <- list()
arborMDS[["Axon"]] <- read.csv(paste(globalAlignDataDir,"Axon_STL_G3E1Identity_MDS7.csv",sep=""),row.names=1)
arborMDS[["Apical"]] <- read.csv(paste(globalAlignDataDir,"Apical_STL_G3E1Identity_MDS8.csv",sep=""),row.names=1)
arborMDS[["Dendrite"]] <- read.csv(paste(globalAlignDataDir,"Dendrite_STL_SpurLen2D_G3E1Identity_MDS7.csv",sep=""),row.names=1)
arborNamesByType <- c(Axon="Axon",ApicalDendrite="Apical",Dendrite="Dendrite")

arborDists <- list()
arborDists[["Axon"]] <- read.csv(paste(globalAlignDataDir,"AxonDistances_STL_G3E1Identity.csv",sep=""),row.names=1,check.names=F)
arborDists[["Apical"]] <- read.csv(paste(globalAlignDataDir,"ApicalDistances_STL_G3E1Identity.csv",sep=""),row.names=1,check.names=F)
arborDists[["Dendrite"]] <- read.csv(paste(globalAlignDataDir,"DendriteDistances_STL_SpurLen2D_G3E1Identity.csv",sep=""),row.names=1,check.names=F)

#read.csv(paste(globalAlignDataDir,arborName,"Distances_",travOrder,spurStr,"_",scoreParamStr,".csv",sep=""),row.names=1,check.names=F)
# Load and normalize morphometrics
Morphometrics <- read.csv(paste(clusterDir,"NM-Morphometrics_HWD.csv",sep=""),row.names=1)
Morphometrics <- Morphometrics[apply(Morphometrics,1,function(x){sum(is.na(x))})==0,]
# Drop "Stems"; Drop "Bifurcations", since it's almost exactly the same as "Branches"
Morphometrics <- Morphometrics[,!colnames(Morphometrics)%in%c("Stems","Bifurcations")]
# Remove FractalDimension, which currently is incorrect
Morphometrics <- Morphometrics[,!colnames(Morphometrics)%in%c("FractalDimension")]
# Eliminating metrics that tend to varry by Archive
Morphometrics <- Morphometrics[,!colnames(Morphometrics)%in%c("Diameter","Contraction","LocalBifurcationAmpl","Fragmentation")]
# Normalize
morphometricsXformed <- Morphometrics
morphometricsXformed$Depth[morphometricsXformed$Depth==0] <- 3
morphometricsXformed <- as.data.frame(apply(morphometricsXformed,2,logTransformData,threshold=.8))
morphometricsXformed <- scale(morphometricsXformed)

apicalNCSpec <- list(ArborType="ApicalDendrite",Class2="Pyramidal cell",ExperCond="Control",BrainRegion="Neocortex")
apicalCA3Spec <- list(ArborType="ApicalDendrite",Class2="Pyramidal cell",ExperCond="Control",BrainRegion="Hippocampus",Region2="CA3")
apicalCA1Spec <- list(ArborType="ApicalDendrite",Class2="Pyramidal cell",ExperCond="Control",BrainRegion="Hippocampus",Region2="CA1")
specifications <- list(CA1=apicalCA1Spec,CA3=apicalCA3Spec)
arborType <- "Apical"
arborName <- arborNamesByType[arborType]
alignmentSpace <- arborMDS[[arborType]]
alignmentSpace[,1] <- -alignmentSpace[,1]

sharedSeqs <- intersect(rownames(alignmentSpace),rownames(morphometricsXformed))

subsetObj <- loadSubsetSeqs(specifications,usedMetaData,candidates=sharedSeqs)
ASSub <- alignmentSpace[subsetObj$Sequences,]
morphoSub <- morphometricsXformed[subsetObj$Sequences,]
group1 <- subsetObj$SeqsByGroup[[1]]
group2 <- subsetObj$SeqsByGroup[[2]]

# Determine best combination of alignment space dimensions
bestScore <- 0
bestDims <- c()
permutations <- expand.grid(rep(list(0:1),length(1:ncol(ASSub))))
for (whichDimsInd in 2:nrow(permutations)){
  whichDims <- which(permutations[whichDimsInd,]==1)
  if (length(whichDims) == 1){
    featEM_ASCombo <- Mclust(ASSub[,whichDims,drop=F],G=2)
  }else{
    featEM_ASCombo <- Mclust(ASSub[,whichDims,drop=F],G=2,modelNames=ASOnlyModels)
  }
  riScore <- AdjustedRandIndex(featEM_ASCombo$classification,subsetObj$GroupIds)
  if (riScore > bestScore){
    bestScore <- riScore
    bestDims <- whichDims
  }
}
featEM_ASBest <- Mclust(ASSub[,bestDims,drop=F],G=2,modelNames=ASOnlyModels)
riScore <- AdjustedRandIndex(featEM_ASBest$classification,subsetObj$GroupIds)

# Plot best alignment space clustering on alignment space dims 1 and 2
dims <- c(1,2)
plotdf <- cbind(cbind(as.data.frame(ASSub[,dims]),cluster=factor(featEM_ASBest$classification)),group=names(specifications)[subsetObj$GroupIds])
colnames(plotdf) <- c("x","y","cluster","group")
apicalClassFig <- ggplot(plotdf,aes(x=x,y=y,colour=group,fill=group,shape=cluster))+
  geom_point(size=3)+scale_shape_manual(values=c(16,17)) +  
  #scale_shape_discrete(solid=T, guide="none")+
  scale_fill_manual(values=apicalColors)+scale_colour_manual(values=apicalColors)+
  paper2Theme+coord_fixed()

ggsave(paste("/home/tgillett/figures/Pasta2/GroupPair_",paste(names(specifications),collapse="-"),"_Cluster-",featEM_ASBest$modelName,"2-AlignmentSpaceBestClassification-Dims",paste(dims,collapse="",sep=""),".png",sep=""),
       apicalClassFig,width=8,height=8)

# Determine best combination of topological metrics
permutScores <- list()
bestScore <- 0
bestTopos <- c()
topoFeats <- c("Branches","BranchOrder","Caulescence","PartitionAsymmetry")
permutations <- expand.grid(rep(list(0:1),length(topoFeats)))
for (whichDimsInd in 2:nrow(permutations)){
  selectTopos <- topoFeats[which(permutations[whichDimsInd,]==1)]
  featEM_Topos <- Mclust(morphoSub[,selectTopos,drop=F],G=2)
  riScore <- AdjustedRandIndex(featEM_Topos$classification,subsetObj$GroupIds)
  permutScores[[paste(selectTopos,collapse="-")]] <- riScore
  if (riScore > bestScore){
    bestScore <- riScore
    bestTopos <- selectTopos
  }
}

# Cluster by best topometrics
featEM_BestTopos <- Mclust(morphoSub[,bestTopos],G=2)
riScore <- AdjustedRandIndex(featEM_BestTopos$classification,subsetObj$GroupIds)

#plotdf <- cbind(Morphometrics[rownames(morphoSub),bestMorphos[1]],cluster=factor(featEM_BestMorphos$classification),group=names(specifications)[subsetObj$GroupIds])
plotdf <- cbind(Morphometrics[rownames(morphoSub),bestTopos[1:2]],cluster=factor(featEM_BestMorphos$classification),group=names(specifications)[subsetObj$GroupIds])
#colnames(plotdf) <- c("Dimension 1", bestMorphos[1],"cluster","group")
colnames(plotdf) <- c(bestTopos[1:2],"cluster","group")
#apicalClassFig <- ggplot(plotdf,aes_string(x=bestMorphos[1],y=bestMorphos[2],colour="group",fill="group",shape="cluster"))+
apicalClassFig <- ggplot(plotdf,aes_string(x=bestTopos[1],y=bestTopos[2],colour="group",fill="group",shape="cluster"))+
  geom_point(size=3.5)+scale_shape_discrete(solid=T, guide="none")+
  scale_fill_manual(values=apicalColors)+scale_colour_manual(values=apicalColors)+
  paper2Theme+
  theme(axis.text = element_text(size=17),
        axis.title=element_text(size=18),
        axis.ticks = element_line(size=1), 
        axis.ticks.length=unit(3.5,"mm"),
        axis.ticks.margin = unit(2.1, "mm"))
ggsave(paste("/home/tgillett/figures/Pasta2/GroupPair_",paste(names(specifications),collapse="-"),"_Cluster-",featEM_BestMorphos$modelName,"2-BestTopoClassification.png",sep=""),
       apicalClassFig,width=7,height=6) #,units="in",dpi="600")


# Cluster by best morphometrics and best alignment space dims
#featEM_BestMorphosBestAS <- Mclust(as.data.frame(Dim1=ASSub[,bestDims],morphoSub[rownames(morphoSub),bestMorphos]),G=2)
featEM_BestToposBestAS <- Mclust(as.data.frame(Dim1=ASSub[,bestDims],morphoSub[rownames(morphoSub),bestTopos]),G=2)
riScore <- AdjustedRandIndex(featEM_BestToposBestAS$classification,subsetObj$GroupIds)

#plotdf <- cbind(cbind(as.data.frame(Dim1=ASSub[,1],Morphometrics[rownames(morphoSub),bestMorphos[1]]),cluster=factor(featEM_BestMorphosBestAS$classification)),group=names(specifications)[subsetObj$GroupIds])
plotdf <- cbind(cbind(data.frame(ASSub[,1],Morphometrics[rownames(morphoSub),bestTopos[1]]),cluster=factor(featEM_BestMorphosBestAS$classification)),group=names(specifications)[subsetObj$GroupIds])
#colnames(plotdf) <- c("Dimension 1", bestMorphos[1],"cluster","group")
colnames(plotdf) <- c("Dimension1", bestTopos[1],"cluster","group")
#apicalClassFig <- ggplot(plotdf,aes_string(x="Dimension 1",y=bestMorphos[1],colour="group",fill="group",shape="cluster"))+
apicalClassFig <- ggplot(plotdf,aes_string(x="Dimension1",y=bestTopos[1],colour="group",fill="group",shape="cluster"))+
  geom_point(size=3.5)+scale_shape_discrete(solid=T, guide="none")+
  scale_fill_manual(values=apicalColors)+scale_colour_manual(values=apicalColors)+
  paper2Theme+
  theme(axis.text = element_text(size=17),
        axis.title=element_text(size=18),
        axis.ticks = element_line(size=1), 
        axis.ticks.length=unit(3.5,"mm"),
        axis.ticks.margin = unit(2.1, "mm"))
ggsave(paste("/home/tgillett/figures/Pasta2/GroupPair_",paste(names(specifications),collapse="-"),"_Cluster-",featEM_BestMorphosBestAS$modelName,"2-BestASTopoComboClassification.png",sep=""),
       apicalClassFig,width=7,height=6) #,units="in",dpi="600")

## Older variants of above
# Plot best topological metrics clustering
plotdf <- cbind(cbind(as.data.frame(Morphometrics[rownames(morphoSub),bestTopos[1:2]]),cluster=factor(featEM_Topos$classification)),group=names(specifications)[subsetObj$GroupIds])
plotdf <- cbind(cbind(as.data.frame(morphoSub[,bestTopos[1:2]]),cluster=factor(featEM_Topos$classification)),group=names(specifications)[subsetObj$GroupIds])
colnames(plotdf) <- c(bestTopos[1:2],"cluster","group")
topoClassFig <- ggplot(plotdf,aes_string(x=bestTopos[1],y=bestTopos[2],colour="group",fill="group",shape="cluster"))+
  geom_point(size=3.5)+scale_shape_discrete(solid=T, guide="none")+
  #scale_fill_manual(values=apicalColors)+scale_colour_manual(values=apicalColors)+
  #scale_fill_manual(values=axonColors)+scale_colour_manual(values=axonColors)+
  paper2Theme+
  theme(axis.text = element_text(size=17),
        axis.title=element_text(size=18),
        axis.ticks = element_line(size=1), 
        axis.ticks.length=unit(3.5,"mm"),
        axis.ticks.margin = unit(2.1, "mm"))
ggsave(paste("/home/tgillett/figures/Pasta2/GroupPair",pairNum,"Cluster-",featEM_Topos$modelName,"2-BestToposClassification-Raw.png",sep=""),
       topoClassFig,width=7,height=6) #,units="in",dpi="600")
ggsave(paste("/home/tgillett/figures/Pasta2/GroupPair",pairNum,"Cluster-",featEM_Topos$modelName,"2-BestToposClassification.png",sep=""),
       topoClassFig,width=7,height=6) #,units="in",dpi="600")

# Cluster by best topological metrics and best alignment space dims
plotdf <- cbind(cbind(data.frame(Dim1=featMatSub[,bestDims[1]],morphoSub[,bestTopos[1]]),cluster=factor(featEM_BestToposBestAS$classification)),group=names(specifications)[subsetObj$GroupIds])
colnames(plotdf) <- c(paste("Dimension",bestDims[1],sep=""), bestTopos[1],"cluster","group")
apicalClassFig <- ggplot(plotdf,aes_string(x=paste("Dimension",bestDims[1],sep=""),y=bestTopos[1],colour="group",fill="group",shape="cluster"))+
  geom_point(size=3.5)+scale_shape_discrete(solid=T, guide="none")+
  #scale_fill_manual(values=apicalColors)+scale_colour_manual(values=apicalColors)+
  paper2Theme+
  theme(axis.text = element_text(size=17),
        axis.title=element_text(size=18),
        axis.ticks = element_line(size=1), 
        axis.ticks.length=unit(3.5,"mm"),
        axis.ticks.margin = unit(2.1, "mm"))
ggsave(paste("/home/tgillett/figures/Pasta2/GroupPair",pairNum,"Cluster-",featEM_BestToposBestAS$modelName,"2-BestTopo&DimsClassification.png",sep=""),
       apicalClassFig,width=7,height=6) #,units="in",dpi="600")

#### Fig 2d-f alternative - LDA models ####

# Apical CA1 vs CA3
apicalCA1Spec <- list(ArborType="ApicalDendrite",Class2="Pyramidal cell",ExperCond="Control",BrainRegion="Hippocampus",Region2="CA1")
apicalCA3Spec <- list(ArborType="ApicalDendrite",Class2="Pyramidal cell",ExperCond="Control",BrainRegion="Hippocampus",Region2="CA3")
specifications <- list(CA1=apicalCA1Spec,CA3=apicalCA3Spec)
arborType <- "Apical"
arborName <- arborNamesByType[arborType]
alignmentSpace <- arborMDS[[arborType]]

sharedSeqs <- intersect(rownames(alignmentSpace),rownames(morphometricsXformed))
subsetObj <- loadSubsetSeqs(specifications,usedMetaData,candidates=sharedSeqs)
ASSub <- alignmentSpace[subsetObj$Sequences,]
colnames(ASSub) <- 1:ncol(ASSub)
morphoSub <- morphometricsXformed[subsetObj$Sequences,]
group1 <- subsetObj$SeqsByGroup[[1]]
group2 <- subsetObj$SeqsByGroup[[2]]

klarAS <- stepclass(x=ASSub,grouping=subsetObj$GroupIds,method="lda",direction="both",improvement=.01)
klarMorpho <- stepclass(x=morphoSub,grouping=subsetObj$GroupIds,method="lda",direction="both",improvement=.01)
klarASMorpho <- stepclass(x=cbind(morphoSub,ASSub),grouping=subsetObj$GroupIds,method="lda",direction="both",improvement=.01)
klarTopo <- stepclass(x=morphoSub[,topoFeats],grouping=subsetObj$GroupIds,method="lda",direction="both",improvement=.01)
klarASTopo <- stepclass(x=cbind(morphoSub[,topoFeats],ASSub),grouping=subsetObj$GroupIds,method="lda",direction="both",improvement=.01)

topoPCs <- topoPCObj$x[rownames(morphoSub),]
klarTopoPC <- stepclass(x=topoPCs,grouping=subsetObj$GroupIds,method="lda",direction="both",improvement=.01)
klarASTopoPC <- stepclass(x=cbind(topoPCs,ASSub),grouping=subsetObj$GroupIds,method="lda",direction="both",improvement=.01)

featEM_ASTopoCombo <- Mclust(cbind(morphoSub[,bestTopos,drop=F],ASSub[,bestDims,drop=F]),G=2)
ASTopoComboScore <- AdjustedRandIndex(featEM_ASTopoCombo$classification,subsetObj$GroupIds)

# Plot best alignment space clustering on alignment space dims 1 and 2
dims <- c(1,2)
dims <- substr(klarAS$model[1:2,2],2,2)
#dims <- klarAS$model[1:2,2]
plotdf <- cbind(cbind(as.data.frame(ASSub[,dims]),cluster=factor(klarAS$classification)),group=names(specifications)[subsetObj$GroupIds])
colnames(plotdf) <- c("x","y","cluster","group")
apicalClassFig <- ggplot(plotdf,aes(x=x,y=y,colour=group,fill=group,shape=cluster))+
  geom_point(size=3)+scale_shape_manual(values=c(16,17)) +  
  #scale_shape_discrete(solid=T, guide="none")+
  scale_fill_manual(values=apicalColors)+scale_colour_manual(values=apicalColors)+
  paper2Theme+coord_fixed()

ggsave(paste("/home/tgillett/figures/Pasta2/GroupPair_",paste(names(specifications),collapse="-"),"_Cluster-",featEM_ASBest$modelName,"2-AlignmentSpaceBestClassification-Dims",paste(dims,collapse="",sep=""),".png",sep=""),
       apicalClassFig,width=8,height=8)

# Determine best combination of topological metrics
permutScores <- list()
bestScore <- 0
bestTopos <- c()
topoFeats <- c("Branches","BranchOrder","Caulescence","PartitionAsymmetry")
permutations <- expand.grid(rep(list(0:1),length(topoFeats)))
for (whichDimsInd in 2:nrow(permutations)){
  selectTopos <- topoFeats[which(permutations[whichDimsInd,]==1)]
  featEM_Topos <- Mclust(morphoSub[,selectTopos,drop=F],G=2)
  riScore <- AdjustedRandIndex(featEM_Topos$classification,subsetObj$GroupIds)
  permutScores[[paste(selectTopos,collapse="-")]] <- riScore
  if (riScore > bestScore){
    bestScore <- riScore
    bestTopos <- selectTopos
  }
}

# Cluster by best morphometrics
#featEM_BestMorphos <- Mclust(Morphometrics[rownames(morphoSub),bestMorphos],G=2)
featEM_BestMorphos <- Mclust(Morphometrics[rownames(morphoSub),bestTopos],G=2)
riScore <- AdjustedRandIndex(featEM_BestMorphos$classification,subsetObj$GroupIds)

#plotdf <- cbind(Morphometrics[rownames(morphoSub),bestMorphos[1]],cluster=factor(featEM_BestMorphos$classification),group=names(specifications)[subsetObj$GroupIds])
plotdf <- cbind(Morphometrics[rownames(morphoSub),bestTopos[1:2]],cluster=factor(featEM_BestMorphos$classification),group=names(specifications)[subsetObj$GroupIds])
#colnames(plotdf) <- c("Dimension 1", bestMorphos[1],"cluster","group")
colnames(plotdf) <- c(bestTopos[1:2],"cluster","group")
#apicalClassFig <- ggplot(plotdf,aes_string(x=bestMorphos[1],y=bestMorphos[2],colour="group",fill="group",shape="cluster"))+
apicalClassFig <- ggplot(plotdf,aes_string(x=bestTopos[1],y=bestTopos[2],colour="group",fill="group",shape="cluster"))+
  geom_point(size=3.5)+scale_shape_discrete(solid=T, guide="none")+
  scale_fill_manual(values=apicalColors)+scale_colour_manual(values=apicalColors)+
  paper2Theme+
  theme(axis.text = element_text(size=17),
        axis.title=element_text(size=18),
        axis.ticks = element_line(size=1), 
        axis.ticks.length=unit(3.5,"mm"),
        axis.ticks.margin = unit(2.1, "mm"))
ggsave(paste("/home/tgillett/figures/Pasta2/GroupPair_",paste(names(specifications),collapse="-"),"_Cluster-",featEM_BestMorphos$modelName,"2-BestTopoClassification.png",sep=""),
       apicalClassFig,width=7,height=6) #,units="in",dpi="600")


# Cluster by best morphometrics and best alignment space dims
#featEM_BestMorphosBestAS <- Mclust(as.data.frame(Dim1=ASSub[,bestDims],Morphometrics[rownames(morphoSub),bestMorphos]),G=2)
featEM_BestMorphosBestAS <- Mclust(as.data.frame(Dim1=ASSub[,bestDims],Morphometrics[rownames(morphoSub),bestTopos]),G=2)
riScore <- AdjustedRandIndex(featEM_BestMorphosBestAS$classification,subsetObj$GroupIds)

#plotdf <- cbind(cbind(as.data.frame(Dim1=ASSub[,1],Morphometrics[rownames(morphoSub),bestMorphos[1]]),cluster=factor(featEM_BestMorphosBestAS$classification)),group=names(specifications)[subsetObj$GroupIds])
plotdf <- cbind(cbind(data.frame(ASSub[,1],Morphometrics[rownames(morphoSub),bestTopos[1]]),cluster=factor(featEM_BestMorphosBestAS$classification)),group=names(specifications)[subsetObj$GroupIds])
#colnames(plotdf) <- c("Dimension 1", bestMorphos[1],"cluster","group")
colnames(plotdf) <- c("Dimension1", bestTopos[1],"cluster","group")
#apicalClassFig <- ggplot(plotdf,aes_string(x="Dimension 1",y=bestMorphos[1],colour="group",fill="group",shape="cluster"))+
apicalClassFig <- ggplot(plotdf,aes_string(x="Dimension1",y=bestTopos[1],colour="group",fill="group",shape="cluster"))+
  geom_point(size=3.5)+scale_shape_discrete(solid=T, guide="none")+
  scale_fill_manual(values=apicalColors)+scale_colour_manual(values=apicalColors)+
  paper2Theme+
  theme(axis.text = element_text(size=17),
        axis.title=element_text(size=18),
        axis.ticks = element_line(size=1), 
        axis.ticks.length=unit(3.5,"mm"),
        axis.ticks.margin = unit(2.1, "mm"))
ggsave(paste("/home/tgillett/figures/Pasta2/GroupPair_",paste(names(specifications),collapse="-"),"_Cluster-",featEM_BestMorphosBestAS$modelName,"2-BestASTopoComboClassification.png",sep=""),
       apicalClassFig,width=7,height=6) #,units="in",dpi="600")

## Older variants of above
# Plot best topological metrics clustering
plotdf <- cbind(cbind(as.data.frame(Morphometrics[rownames(morphoSub),bestTopos[1:2]]),cluster=factor(featEM_Topos$classification)),group=names(specifications)[subsetObj$GroupIds])
plotdf <- cbind(cbind(as.data.frame(morphoSub[,bestTopos[1:2]]),cluster=factor(featEM_Topos$classification)),group=names(specifications)[subsetObj$GroupIds])
colnames(plotdf) <- c(bestTopos[1:2],"cluster","group")
topoClassFig <- ggplot(plotdf,aes_string(x=bestTopos[1],y=bestTopos[2],colour="group",fill="group",shape="cluster"))+
  geom_point(size=3.5)+scale_shape_discrete(solid=T, guide="none")+
  #scale_fill_manual(values=apicalColors)+scale_colour_manual(values=apicalColors)+
  #scale_fill_manual(values=axonColors)+scale_colour_manual(values=axonColors)+
  paper2Theme+
  theme(axis.text = element_text(size=17),
        axis.title=element_text(size=18),
        axis.ticks = element_line(size=1), 
        axis.ticks.length=unit(3.5,"mm"),
        axis.ticks.margin = unit(2.1, "mm"))
ggsave(paste("/home/tgillett/figures/Pasta2/GroupPair",pairNum,"Cluster-",featEM_Topos$modelName,"2-BestToposClassification-Raw.png",sep=""),
       topoClassFig,width=7,height=6) #,units="in",dpi="600")
ggsave(paste("/home/tgillett/figures/Pasta2/GroupPair",pairNum,"Cluster-",featEM_Topos$modelName,"2-BestToposClassification.png",sep=""),
       topoClassFig,width=7,height=6) #,units="in",dpi="600")

# Cluster by best topological metrics and best alignment space dims
plotdf <- cbind(cbind(data.frame(Dim1=featMatSub[,bestDims[1]],morphoSub[,bestTopos[1]]),cluster=factor(featEM_BestToposBestAS$classification)),group=names(specifications)[subsetObj$GroupIds])
colnames(plotdf) <- c(paste("Dimension",bestDims[1],sep=""), bestTopos[1],"cluster","group")
apicalClassFig <- ggplot(plotdf,aes_string(x=paste("Dimension",bestDims[1],sep=""),y=bestTopos[1],colour="group",fill="group",shape="cluster"))+
  geom_point(size=3.5)+scale_shape_discrete(solid=T, guide="none")+
  #scale_fill_manual(values=apicalColors)+scale_colour_manual(values=apicalColors)+
  paper2Theme+
  theme(axis.text = element_text(size=17),
        axis.title=element_text(size=18),
        axis.ticks = element_line(size=1), 
        axis.ticks.length=unit(3.5,"mm"),
        axis.ticks.margin = unit(2.1, "mm"))
ggsave(paste("/home/tgillett/figures/Pasta2/GroupPair",pairNum,"Cluster-",featEM_BestToposBestAS$modelName,"2-BestTopo&DimsClassification.png",sep=""),
       apicalClassFig,width=7,height=6) #,units="in",dpi="600")

#### Table 1 primary - LDA models and classification accuracy ####

pairSpecifications <- list()

# Candidate group 1: Apical NC vs CA3
apicalNCSpec <- list(ArborType="ApicalDendrite",Class2="Pyramidal cell",ExperCond="Control",BrainRegion="Neocortex")
apicalCA3Spec <- list(ArborType="ApicalDendrite",Class2="Pyramidal cell",ExperCond="Control",BrainRegion="Hippocampus",Region2="CA3")
pairSpecifications <- append(pairSpecifications,list(
  list(Neocortex=apicalNCSpec,CA3=apicalCA3Spec)))
names(pairSpecifications)[length(pairSpecifications)] <- "Apical: CA3 vs NC"

# Candidate group 2: Apical CA1 vs CA3
apicalCA1Spec <- list(ArborType="ApicalDendrite",Class2="Pyramidal cell",ExperCond="Control",BrainRegion="Hippocampus",Region2="CA1")
pairSpecifications <- append(pairSpecifications,list(
  list(CA1=apicalCA1Spec,CA3=apicalCA3Spec)))
names(pairSpecifications)[length(pairSpecifications)] <- "Apical: CA1 vs CA3"

# Candidate group 3: Axon (Rodent Cortical) Interneuron vs Pyramidal
interneuronSpec <- list(ArborType="Axon",BrainRegion="Neocortex",Class1="Interneuron",ExperCond="Control",Order="Rodent")
rodentPyramidalSpec <- list(ArborType="Axon",BrainRegion="Neocortex",Class2="Pyramidal cell",ExperCond="Control")
pairSpecifications <- append(pairSpecifications,list(list(
  RodentInterneuron=interneuronSpec,RatPyramidal=rodentPyramidalSpec)))
names(pairSpecifications)[length(pairSpecifications)] <- "Axon (Rodent NC): Interneuron vs Pyramidal"

# Candidate group 4: Axon (Cortical) Rodent Interneuron vs Mouse Pyramidal
mousePyramidalSpec <- list(ArborType="Axon",BrainRegion="Neocortex",Class2="Pyramidal cell",ExperCond="Control",Species="Mouse")
pairSpecifications <- append(pairSpecifications,list(
  list(RodentInterneuron=interneuronSpec,MousePyramidal=mousePyramidalSpec)))
names(pairSpecifications)[length(pairSpecifications)] <- "Axon (NC): Rodent Interneuron vs Mouse Pyramidal"
# 
# Candidate group 5: Dendrite Rodent Cortical Pyramidal vs Interneuron
DendRNCPyr <- list(ArborType="Dendrite",Class2="Pyramidal cell",BrainRegion="Neocortex",Order="Rodent")
pairSpecifications <- append(pairSpecifications,list(
  list(DendRNCPyr=DendRNCPyr,
       DendRNCInt=AllDendriteSpecifications[["Rodent/Neocortex/Interneuron"]])))
names(pairSpecifications)[length(pairSpecifications)] <- "Dendrite (Rodent NC): Pyramidal vs Interneuron"

# Candidate group 6: Dendrite fly tangential vs Reintal Ganglion
pairSpecifications <- append(pairSpecifications,list(
  list(FlyTangential=AllDendriteSpecifications[["Fly/Tangential cell"]],
       RetinalGanglion=AllDendriteSpecifications[["Rodent/Retinal Ganglion cell"]])))
names(pairSpecifications)[length(pairSpecifications)] <- "Dendrite: Fly tangential vs retinal ganglion"

# Candidate group 7: Dendrite fly sensory DA2-4 vs Motoneuron
pairSpecifications <- append(pairSpecifications,list(
  #list(FlySensoryDA24=AllDendriteSpecifications[["Fly/Sensory DA2-4"]],
  list(FlySensoryDA24=AllDendriteSpecifications[["LarvalFly/Sensory neuron"]],
       Motoneuron=AllDendriteSpecifications[["Motoneuron"]])))
names(pairSpecifications)[length(pairSpecifications)] <- "Dendrite: Fly sensory (DA) vs Motoneuron"

# Candidate group 8: Axon fly olfactory vs Mouse Neocortical Pyramidal
pairSpecifications <- append(pairSpecifications,list(
  list(MouseCorticalPyramidal=AllAxonMajorSpecifications[["Mouse/Neocortex/Pyramidal cell"]],
       FlyOlfactory=AllAxonMajorSpecifications[["Drosophila/Olfactory bulb/Uniglomerular projection neuron"]])))
names(pairSpecifications)[length(pairSpecifications)] <- "Axon: Mouse Cort Pyr vs Fly Olf"

# Candidate group 9: Axon fly olfactory vs Rat Neocortical Pyramidal
pairSpecifications <- append(pairSpecifications,list(
  list(RatCorticalPyramidal=AllAxonMajorSpecifications[["Rat/Neocortex/Pyramidal cell"]],
       FlyOlfactory=AllAxonMajorSpecifications[["Drosophila/Olfactory bulb/Uniglomerular projection neuron"]])))
names(pairSpecifications)[length(pairSpecifications)] <- "Axon: Rat Cort Pyr vs Fly Olf"

# Former candidates, no good
# Axon (Cortical) Rat Pyramidal vs Mouse Pyramidal
pairSpecifications <- append(pairSpecifications,list(
 list(RatPyramidal=rodentPyramidalSpec,MousePyramidal=mousePyramidalSpec)))
names(pairSpecifications)[length(pairSpecifications)] <- "Axon: Rat Cort Pyr vs Mouse Cort Pyr"

# Dendrite (Cortical L2/3 Pyramidal): Primate vs Rodent pyramidal
pairSpecifications <- append(pairSpecifications,list(
  list(Primate=AllDendriteSpecifications[["Rodent/Pyramidal cell/Neocortex"]],
       Rodent=AllDendriteSpecifications[["Primate/Pyramidal cell/Neocortex/Layer2,3"]])))
names(pairSpecifications)[length(pairSpecifications)] <- "Axon: Rodent Cort Pyr vs Primate Cort Pyr"

## Multi-group (or pairwise) classification with feature selection using LDA

# Alternative packages to klaR::stepclass
# varSelRF: http://cran.r-project.org/web/packages/varSelRF/varSelRF.pdf
# FSelector: http://cran.r-project.org/web/packages/FSelector/FSelector.pdf

crossvalTable <- matrix(0,nrow=length(pairSpecifications),ncol=10)
# colnames(crossvalTable) <- c("ASDims","ASScore",
#                              "Morphometrics","MorphoScore","ASMorphoCombo","ASMorphoScore",
#                              "Topometrics","TopoScore","ASTopoCombo","ASTopoScore")
colnames(crossvalTable) <- c("ASDims","ASScore",
                             "Topometrics","TopoScore","ASTopoCombo","ASTopoScore",
                             "TopoPCs","TopoPCsScore","ASTopoPCsCombo","ASTopoPCsScore")
rownames(crossvalTable) <- names(pairSpecifications)
require(klaR)

topometrics <- morphometricsXformed[,topologicalMetrics]
topometrics <- topometrics[apply(topometrics,1,function(x){sum(is.na(x))})==0,]
topoPCObj <- prcomp(topometrics)
for (pairNum in 1:length(pairSpecifications)){
#for (pairNum in 10:11){
  specifications <- pairSpecifications[[pairNum]]
  arborType <- specifications[[1]]$ArborType
  if (arborType == "ApicalDendrite"){
    arborType <- "Apical"
  }
  arborName <- arborNamesByType[arborType]
  
  alignmentSpace <- arborMDS[[arborType]]
  
  sharedSeqs <- intersect(rownames(alignmentSpace),rownames(topometrics))
  
  subsetObj <- loadSubsetSeqs(specifications,usedMetaData,candidates=sharedSeqs)
  ASSub <- alignmentSpace[subsetObj$Sequences,]
  morphoSub <- morphometricsXformed[subsetObj$Sequences,]
  topoPCsSub <- topoPCObj$x[subsetObj$Sequences,]
  group1 <- subsetObj$SeqsByGroup[[1]]
  group2 <- subsetObj$SeqsByGroup[[2]]

  print(paste("Running ",rownames(crossvalTable)[pairNum],": ",length(group1),length(group2)))
  
  klarAS <- stepclass(x=ASSub,grouping=subsetObj$GroupIds,method="lda",direction="both",improvement=.01)
  crossvalTable[rownames(crossvalTable)[pairNum],"ASDims"] <- paste(klarAS$model$name,collapse=",")
  crossvalTable[rownames(crossvalTable)[pairNum],"ASScore"] <- klarAS$result.pm[1]
    
#   klarMorpho <- stepclass(x=morphoSub,grouping=subsetObj$GroupIds,method="lda",direction="both",improvement=.01)
#   crossvalTable[rownames(crossvalTable)[pairNum],"Morphometrics"] <- paste(klarMorpho$model$name,collapse=",")
#   crossvalTable[rownames(crossvalTable)[pairNum],"MorphoScore"] <- klarMorpho$result.pm[1]
# 
#   klarASMorpho <- stepclass(x=cbind(morphoSub,ASSub),grouping=subsetObj$GroupIds,method="lda",direction="both",improvement=.01)
#   crossvalTable[rownames(crossvalTable)[pairNum],"ASMorphoCombo"] <- paste(klarASMorpho$model$name,collapse=",")
#   crossvalTable[rownames(crossvalTable)[pairNum],"ASMorphoScore"] <- klarASMorpho$result.pm[1]

  klarTopo <- stepclass(x=morphoSub[,topologicalMetrics],grouping=subsetObj$GroupIds,method="lda",direction="both",improvement=.01)
  crossvalTable[rownames(crossvalTable)[pairNum],"Topometrics"] <- paste(klarTopo$model$name,collapse=",")
  crossvalTable[rownames(crossvalTable)[pairNum],"TopoScore"] <- klarTopo$result.pm[1]

  klarASTopo <- stepclass(x=cbind(morphoSub[,topologicalMetrics],ASSub),grouping=subsetObj$GroupIds,method="lda",direction="both",improvement=.01)
  crossvalTable[rownames(crossvalTable)[pairNum],"ASTopoCombo"] <- paste(klarASTopo$model$name,collapse=",")
  crossvalTable[rownames(crossvalTable)[pairNum],"ASTopoScore"] <- klarASTopo$result.pm[1]

  klarTopoPCs <- stepclass(x=topoPCsSub,grouping=subsetObj$GroupIds,method="lda",direction="both",improvement=.01)
  crossvalTable[rownames(crossvalTable)[pairNum],"TopoPCs"] <- paste(klarTopoPCs$model$name,collapse=",")
  crossvalTable[rownames(crossvalTable)[pairNum],"TopoPCsScore"] <- klarTopoPCs$result.pm[1]
  
  klarASTopoPCs <- stepclass(x=cbind(topoPCsSub,ASSub),grouping=subsetObj$GroupIds,method="lda",direction="both",improvement=.01)
  crossvalTable[rownames(crossvalTable)[pairNum],"ASTopoPCsCombo"] <- paste(klarASTopoPCs$model$name,collapse=",")
  crossvalTable[rownames(crossvalTable)[pairNum],"ASTopoPCsScore"] <- klarASTopoPCs$result.pm[1]
}

#write.csv(crossvalTable,paste(figDir,"GroupPair-LDA3.csv",sep=""))
write.csv(crossvalTable,paste(figDir,"GroupPair-LDA-NoNumBranches.csv",sep=""))



#### Table 1 variant 2 -  Mclust clustering and Rand Index for distinguishability ####
for (pairNum in 1:length(pairSpecifications)){
  specifications <- pairSpecifications[[pairNum]]
  arborType <- specifications[[1]]$ArborType
  if (arborType == "ApicalDendrite"){
    arborType <- "Apical"
  }
  arborName <- arborNamesByType[arborType]
  
  alignmentSpace <- arborMDS[[arborType]]
  
  sharedSeqs <- intersect(rownames(alignmentSpace),rownames(morphometricsXformed))
  
  subsetObj <- loadSubsetSeqs(specifications,usedMetaData,candidates=sharedSeqs)
  ASSub <- alignmentSpace[subsetObj$Sequences,]
  morphoSub <- morphometricsXformed[subsetObj$Sequences,]
  group1 <- subsetObj$SeqsByGroup[[1]]
  group2 <- subsetObj$SeqsByGroup[[2]]
  
  # Determine best combination of alignment space dimensions
  if (rownames(riTable)[pairNum]%in%names(bestDimsList)){
    bestDims <- bestDimsList[[rownames(riTable)[pairNum]]]
    # Cluster by alignment space best dims
    if (length(bestDims) == 1){
      featEM_ASBest <- Mclust(ASSub[,bestDims],G=2)
    }else{
      featEM_ASBest <- Mclust(ASSub[,bestDims],G=2,modelNames = ASOnlyModels)
    }
    bestScore <- AdjustedRandIndex(featEM_ASBest$classification,subsetObj$GroupIds)
  }else{
    bestScore <- 0
    bestDims <- c()
    permutations <- expand.grid(rep(list(0:1),length(1:ncol(ASSub))))
    for (whichDimsInd in 2:nrow(permutations)){
      whichDims <- which(permutations[whichDimsInd,]==1)
      if (length(whichDims) == 1){
        featEM_ASCombo <- Mclust(ASSub[,whichDims,drop=F],G=2)
      }else{
        featEM_ASCombo <- Mclust(ASSub[,whichDims,drop=F],G=2,modelNames=ASOnlyModels)
      }
      riScore <- AdjustedRandIndex(featEM_ASCombo$classification,subsetObj$GroupIds)
      if (riScore > bestScore){
        bestScore <- riScore
        bestDims <- whichDims
      }
    }
    bestDimsList[[rownames(riTable)[pairNum]]] <- bestDims
  }
  riTable[pairNum,"BestASDims"] <- paste(bestDims,collapse=",")
  
  riTable[pairNum,"BestAlignmentSpace"] <- bestScore
  
  # Cluster by each morphometric
  morphoRandInds <- c()
  morphoModels <- c()
  for (i in 1:ncol(morphoSub)){
    featEM_Morpho <- Mclust(morphoSub[,i,drop=F],G=2)
    morphoRandInds[i] <- AdjustedRandIndex(featEM_Morpho$classification,subsetObj$GroupIds)
    morphoModels[i] <- featEM_Morpho$modelName
  }
  bestMorpho <- colnames(morphoSub)[which.max(morphoRandInds)]
  #bestMorpho
  #max(morphoRandInds)
  names(morphoRandInds) <- colnames(morphoSub)
  morphoRandInds <- sort(morphoRandInds,decreasing=T)
  
  # Cluster by all morphometrics
  featEM_AllMorpho <- Mclust(morphoSub,G=2)
  riTable[pairNum,"AllMorphometrics"] <- 
    AdjustedRandIndex(featEM_AllMorpho$classification,subsetObj$GroupIds)
    
  # Cluster by best morphometrics
  if (sum(morphoRandInds > .08) > 0){
    bestMorphos <- names(morphoRandInds)[morphoRandInds > .15]
    featEM_Best1Morpho <- Mclust(morphoSub[,bestMorphos[1]],G=2)
    riTable[pairNum,"Best1Morphometric"] <- 
      AdjustedRandIndex(featEM_Best1Morpho$classification,subsetObj$GroupIds)
    
        featEM_BestMorphos <- Mclust(morphoSub[,bestMorphos],G=2)
        riTable[pairNum,"BestMorphosScore"] <- 
          AdjustedRandIndex(featEM_BestMorphos$classification,subsetObj$GroupIds)
        riTable[pairNum,"BestMorphos"] <- paste(bestMorphos,sep="",collapse=",")
        
    
    # Determine best morphometric combination
    if (rownames(riTable)[pairNum]%in%names(bestMorphosList)){
      bestMorphos <- bestMorphosList[[rownames(riTable)[pairNum]]]
      featEM_Morphos <- Mclust(morphoSub[,bestMorphos,drop=F],G=2)
      bestScore <- AdjustedRandIndex(featEM_Morphos$classification,subsetObj$GroupIds)
    }else{
      bestScore <- 0
      bestMorphos <- c()
      morphoFeats <- names(morphoRandInds)[morphoRandInds > .15]
      permutations <- expand.grid(rep(list(0:1),length(morphoFeats)))
      for (whichDimsInd in 2:nrow(permutations)){
        selectMorphos <- morphoFeats[which(permutations[whichDimsInd,]==1)]
        featEM_Morphos <- Mclust(morphoSub[,selectMorphos,drop=F],G=2)
        riScore <- AdjustedRandIndex(featEM_Morphos$classification,subsetObj$GroupIds)
        if (riScore > bestScore){
          bestScore <- riScore
          bestMorphos <- selectMorphos
        }
      }
      bestMorphosList[[rownames(riTable)[pairNum]]] <- bestMorphos
    }
    riTable[pairNum,"BestMorphos"] <- paste(bestMorphos,sep="",collapse=",")
    riTable[pairNum,"BestMorphosScore"] <- bestScore
    
    featEM_BestMorphosBestAS <- Mclust(
      cbind(ASSub[,bestDims,drop=F],morphoSub[,bestMorphos]),G=2)
    riTable[pairNum,"BestCombo"] <- 
      AdjustedRandIndex(featEM_BestMorphosBestAS$classification,subsetObj$GroupIds)
    
  }
  
  
  # Determine best topological morphometric combination
  if (rownames(riTable)[pairNum]%in%names(bestToposList)){
    bestTopos <- bestToposList[[rownames(riTable)[pairNum]]]
    featEM_Topos <- Mclust(morphoSub[,bestTopos,drop=F],G=2)
    bestScore <- AdjustedRandIndex(featEM_Topos$classification,subsetObj$GroupIds)
  }else{
    bestScore <- 0
    bestTopos <- c()
    topoFeats <- c("Branches","BranchOrder","Caulescence","PartitionAsymmetry")
    permutations <- expand.grid(rep(list(0:1),length(topoFeats)))
    for (whichDimsInd in 2:nrow(permutations)){
      selectTopos <- topoFeats[which(permutations[whichDimsInd,]==1)]
      featEM_Topos <- Mclust(morphoSub[,selectTopos,drop=F],G=2)
      riScore <- AdjustedRandIndex(featEM_Topos$classification,subsetObj$GroupIds)
      if (riScore > bestScore){
        bestScore <- riScore
        bestTopos <- selectTopos
      }
    }
    bestToposList[[rownames(riTable)[pairNum]]] <- bestTopos
  }
  riTable[pairNum,"BestTopos"] <- paste(bestTopos,sep="",collapse=",")
  riTable[pairNum,"BestToposScore"] <- bestScore
  
  featEM_BestToposBestAS <- Mclust(
    cbind(ASSub[,bestDims,drop=F],morphoSub[,bestTopos]),G=2)
  riTable[pairNum,"BestTopoCombo"] <- 
    AdjustedRandIndex(featEM_BestToposBestAS$classification,subsetObj$GroupIds)
  
  
}

write.csv(riTable,"/home/tgillett/figures/Pasta2/GroupPair-RIVals.csv")

# Plot all alignment space clustering
#png(paste("/home/tgillett/figures/Pasta2/GroupPair",pairNum,"Cluster-",featEM_AS$modelName,"2-AlignmentSpaceCompleteClassification.png",sep=""))
#plot(ASSub[,1:2],col=featEM_AS$classification,pch=subsetObj$GroupIds)
#dev.off()

 

# Cluster by best morphometric and alignment space dims 1-4
#featEM_BestComboAS14 <- Mclust(cbind(featMatSub[,1:4],morphoSub[,bestMorpho,drop=F]),G=2)
#AdjustedRandIndex(featEM_BestComboAS14$classification,subsetObj$GroupIds)
#png(paste("/home/tgillett/figures/Pasta2/GroupPair",pairNum,"Cluster-",featEM_BestCombo$modelName,"2-AS14&BestMorphoClassification_AS14.png",sep=""))
#plot(data.frame(AS1=featMatSub[,1],bestMorpho=morphoSub[,bestMorpho]),col=featEM_BestComboAS14$classification,pch=subsetObj$GroupIds)
#dev.off()
#png(paste("/home/tgillett/figures/Pasta2/GroupPair",pairNum,"Cluster-",featEM_BestCombo$modelName,"2-AS14&BestMorphoRawClassification_AS14.png",sep=""))
#plot(data.frame(AS1=featMatSub[,1],bestMorpho=Morphometrics[subsetObj$Sequences,bestMorpho]),col=featEM_BestComboAS14$classification,pch=subsetObj$GroupIds)
#dev.off()



#### Figure 3: Cluster analysis of dendrites ####
arborName <- "Dendrite"
arborType <- arborName
dendriteSpecs <- AllDendriteSpecifications

dendMat <- arborMDS[["Dendrite"]]
numDimsInit <- ncol(dendMat)
numDims <- 4

# Set the relevant groups and get rid of feature vectors associated with unrepresented neurites
dendriteSubset <- loadSubsetSeqs(dendriteSpecs,clusterMetaData,candidates=rownames(dendMat))
#dendMat <- dendMat[dendriteSubset$Sequences,]

nonSubsetSeqs = names(dendMat)[!rownames(dendMat)%in%dendriteSubset$Sequences]
groupIds <- dendriteSubset$GroupIds
allGroupIds <- rep(length(dendriteSpecs)+1,nrow(dendMat)) # Includes non-specified neurons by adding an extra group "All Others"
names(allGroupIds) <- rownames(dendMat)
allGroupIds[dendriteSubset$Sequences] <- dendriteSubset$GroupIds
dendriteSubset$AllGroupIds <- allGroupIds
numGroups <- length(dendriteSpecs)

#dendriteEM <- Mclust(dendMat[,1:numDims],G=1:(numGroups+2))
#dendriteEM <- Mclust(dendMat[,1:numDims],G=7,modelNames="VEV")
dendriteEM <- Mclust(dendMat[,1:numDims],G=6,modelNames="VEV")

groupNames <- paste(names(dendriteSpecs)," (",sapply(dendriteSubset$SeqsByGroup,length),")",sep="")
dendClusterResults <- clusteringStatisticalAnalysis(dendMat,dendriteEM,dendriteSubset,groupNames)

figDir <- "/home/tgillett/figures/Pasta2/"
filenameBase <- paste(figDir,"/DendriteSTL_SpurLen2D_V2_G3E1Identity_MDS",numDims,"of",numDimsInit,"G",dendriteEM$G,dendriteEM$modelName,sep="")
write.csv(dendClusterResults$Observed,paste(filenameBase,"_Observed.csv",sep=""))
write.csv(round(dendClusterResults$LogP,2),paste(filenameBase,"_LogPVals",".csv",sep=""))

# Set which neurites will be colored and how
#clustersToColor <- c(3,4,6)
clustersToColor <- c(3,4,5)
dims <- 1:2
#clustersToColor <- c(1,2,5)
clustersToColor <- c(1,2,6)
dims <- 3:4

dendriteColors <- c(Gray="#999999",Black="#000000",C1="#2222FF",C2="#00CC00",C3="#EE0000",C4="#00CCCC",C5="#9922FF",C6="#E68800",C7="#555555",PlainCircle="#333333")

#classification <- dendriteEM$classification[dendriteSubset$Sequences]
#dendriteGroup <- rep("Gray",length(dendriteSubset$Sequences))
dendriteGroup <- rep("Gray",length(dendriteEM$classification))
names(dendriteGroup) <- names(dendriteEM$classification)
dendriteGroup[dendriteEM$classification%in%clustersToColor] <- 
  paste("C",dendriteEM$classification[dendriteEM$classification%in%clustersToColor],sep="")
#dendriteGroup[dendriteSubset$SeqsByGroup[[5]]] <- "C5"
#clusteredDendriteMDS <- cbind(dendMat[dendriteSubset$Sequences,dims],dendriteGroup)
clusteredDendriteMDS <- cbind(dendMat[,dims],dendriteGroup)
colnames(clusteredDendriteMDS) <- c("x","y","group")
clusteredDendriteMDS <- rbind(subset(clusteredDendriteMDS,group=="Gray"),subset(clusteredDendriteMDS,group!="Gray"))

require(ellipse)
parms <- dendriteEM$parameters
df_ell <- list()
for(g in 1:dendriteEM$G){
  ellipseDF <- generateEllipse(parms,g)
  
  #ellipsedf <- as.data.frame(ellipse::ellipse(parms$variance$sigma[dims,dims,g], 
  #                                        centre=parms$mean[dims,g]))
  #tmpdf <- cbind(completeX,completeY)
  ellipseDF <- xy
               
  if (g %in% clustersToColor){
    lineSize <- 1.07
  }else{
    lineSize <- 0.65
#    tmpdf <- cbind(tmpdf,data.frame(group=rep("PlainCircle",nrow(tmpdf)),size=0.8))
    tmpdfP1 <- data.frame(group=rep(paste("C",g,sep=""),nrow(ellipseDF)),size=0.8)
    tmpdfP2 <- data.frame(group=rep("Black",nrow(ellipseDF)),size=0.85)
    tmpdf <- rbind(tmpdfP1,tmpdfP2)
    #tmpdf <- cbind(tmpdf,data.frame(group=rep(paste("C",g,sep=""),nrow(tmpdf)),size=0.8))
  }
  tmpdf <- cbind(ellipseDF,data.frame(group=rep("Black",nrow(tmpdf)),size=lineSize+0.18))
  colnames(tmpdf) <- c("x","y","group","size")
  df_ell <- append(df_ell, list(tmpdf))
  tmpdf <- cbind(ellipseDF,data.frame(group=rep(paste("C",g,sep=""),nrow(tmpdf)),size=lineSize))
  colnames(tmpdf) <- c("x","y","group","size")
  df_ell <- append(df_ell, list(tmpdf))
}

dendritesClustFig1 <- ggplot(clusteredDendriteMDS,aes(x=x,y=y,colour=group,fill=group))+
  geom_point(size=2.1,guide=F)+
  scale_fill_manual(values=dendriteColors,guide=F)+scale_colour_manual(values=dendriteColors,guide=F)+
  paper2Theme+coord_fixed()
for (i in df_ell){
  dendritesClustFig1 <- dendritesClustFig1 + geom_path(aes(x=x,y=y,colour=group,size=size), data=i, linetype=1)
}
selectDendrites <- highlightedDendrites[dendriteEM$classification[highlightedDendrites]%in%clustersToColor]
dendritesClustFig1 <- dendritesClustFig1 + scale_size(range=c(0.6, 2.1), guide=FALSE)
dendritesClustFig1 <- dendritesClustFig1 + geom_point(data=clusteredDendriteMDS[selectDendrites,],bg="#FFFFFF",colour="#FFFFFF",shape=21,size=4.3)
dendritesClustFig1 <- dendritesClustFig1 + geom_point(data=clusteredDendriteMDS[selectDendrites,],bg="black",colour="black",shape=21,size=3.1)
dendritesClustFig1 <- dendritesClustFig1 + geom_point(data=clusteredDendriteMDS[selectDendrites,],aes(color=group,bg=group),shape=21,size=1.5)
#dendritesClustFig1 <- dendritesClustFig1 + geom_point(data=clusteredDendriteMDS[selectDendrites,],bg="#FFFFFF",colour="#FFFFFF",shape=21,size=0.5)
#filename <- paste("/home/tgillett/figures/Pasta2/DendriteScatter-Select-ByCluster&Groups_DA1_Dims",paste(dims,collapse="-"),".png",sep="")
#filename <- paste("/home/tgillett/figures/Pasta2/DendriteScatter-Select-ByClusters",
filename <- paste("/home/tgillett/figures/Pasta2/DendriteScatterWithHighlights-Select-ByClusters",
                  paste(clustersToColor,collapse="-"),"_Dims",paste(dims,collapse="-"),"_G",dendriteEM$G,"-Ellipse.png",sep="")
filename <- paste("/home/tgillett/figures/Pasta2/DendriteScatterWithHighlights-Select-ByClusters",
                  paste(clustersToColor,collapse="-"),"_Dims",paste(dims,collapse="-"),"_G",dendriteEM$G,"-Sigma.png",sep="")
ggsave(filename,dendritesClustFig1,width=12,height=8)

# Test other dendrite clustering for better associations
groupNames <- paste(names(dendriteSpecs)," (",sapply(dendriteSubset$SeqsByGroup,length),")",sep="")
dendriteParams <- list(VVV=6,VEV=6:8,EEV=7:9)
for (modelName in names(dendriteParams)){
  for (numClusts in dendriteParams[[modelName]]){
    dendEMTest <- Mclust(dendMat[,1:numDims],G=numClusts,modelNames=modelName)
    arborClusterResults <- clusteringStatisticalAnalysis(dendMat,dendEMTest,dendriteSubset,groupNames)
    filenameBase <- paste(figDir,"/DendriteSTL_SpurLen2D_V2_G3E1Identity_MDS",numDims,"of",numDimsInit,"G",dendEMTest$G,dendEMTest$modelName,sep="")
#    write.csv(arborClusterResults$Observed,paste(filenameBase,"_Observed.csv",sep=""))
    write.csv(round(arborClusterResults$LogP,2),paste(filenameBase,"_LogPVals",".csv",sep=""))
  }
}

## Pick dendrites and generate morphologies and dedrograms (11 total)
# Calculate distance from cluster center for each case
names(dendriteSpecs)
#arborClusterResults$Observed
highlightedDendrites <- c()

# Cluster 1
dendC1s <- names(dendriteEM$classification)[dendriteEM$classification == 1]
#   Rodent NC Pyramidal(7)
dendC1Assoc <- dendC1s[dendC1s%in%dendriteSubset$SeqsByGroup[[7]]]
sort(apply(dendMat[dendC1Assoc,1:4],1,function(x,center){return(sqrt(sum((x-center)^2)))},dendriteEM$parameters$mean[,1]))[1:10]
#highlightedDendrites <- append(highlightedDendrites,"070316-exp2-zB-BasalDendrite")
highlightedDendrites <- append(highlightedDendrites,"070502-exp1-zB-BasalDendrite") # RERUN WITH THIS ONE

# Cluster 2
dendC2s <- names(dendriteEM$classification)[dendriteEM$classification == 2]
#   Adult Fly Tangential(3)
dendC2Assoc <- dendC2s[dendC2s%in%dendriteSubset$SeqsByGroup[[3]]]
sort(apply(dendMat[dendC2Assoc,1:4],1,function(x,center){return(sqrt(sum((x-center)^2)))},dendriteEM$parameters$mean[,2]))[1:10]
highlightedDendrites <- append(highlightedDendrites,"HSN-fluoro09-Dendrite")
#   Larval Fly Sensory(4)
dendC2Assoc <- dendC2s[dendC2s%in%dendriteSubset$SeqsByGroup[[4]]]
sort(apply(dendMat[dendC2Assoc,1:4],1,function(x,center){return(sqrt(sum((x-center)^2)))},dendriteEM$parameters$mean[,2]))[1:10]
highlightedDendrites <- append(highlightedDendrites,"wt-MARCM-C3-21-Dendrite")

# Cluster 3
dendC3s <- names(dendriteEM$classification)[dendriteEM$classification == 3]
#   Rodent Retinal Ganglion(2)
dendC3Assoc <- dendC3s[dendC3s%in%dendriteSubset$SeqsByGroup[[2]]]
sort(apply(dendMat[dendC3Assoc,1:4],1,function(x,center){return(sqrt(sum((x-center)^2)))},dendriteEM$parameters$mean[,3]))[1:10]
#highlightedDendrites <- append(highlightedDendrites,"030110-Dendrite")
highlightedDendrites <- append(highlightedDendrites,"121341-Dendrite") # RERUN WITH THIS ONE
#   Adult Fly Tangential(3)
dendC3Assoc <- dendC3s[dendC3s%in%dendriteSubset$SeqsByGroup[[3]]]
sort(apply(dendMat[dendC3Assoc,1:4],1,function(x,center){return(sqrt(sum((x-center)^2)))},dendriteEM$parameters$mean[,3]))[1:10]
highlightedDendrites <- append(highlightedDendrites,"VS4-fluoro05-Dendrite")
#   Larval Fly Sensory(4)
dendC3Assoc <- dendC3s[dendC3s%in%dendriteSubset$SeqsByGroup[[4]]]
sort(apply(dendMat[dendC3Assoc,1:4],1,function(x,center){return(sqrt(sum((x-center)^2)))},dendriteEM$parameters$mean[,3]))[1:10]
highlightedDendrites <- append(highlightedDendrites,"Class4-3-noddaE-Dendrite")

# Cluster 4
dendC4s <- names(dendriteEM$classification)[dendriteEM$classification == 4]
#   Primate NC L23 Pyramidal(6)
dendC4Assoc <- dendC4s[dendC4s%in%dendriteSubset$SeqsByGroup[[6]]]
sort(apply(dendMat[dendC4Assoc,1:4],1,function(x,center){return(sqrt(sum((x-center)^2)))},dendriteEM$parameters$mean[,4]))[1:10]
highlightedDendrites <- append(highlightedDendrites,"143-1-7-BasalDendrite")

# Cluster 5
dendC5s <- names(dendriteEM$classification)[dendriteEM$classification == 5]
#   Motoneuron(1)
dendC5Assoc <- dendC5s[dendC5s%in%dendriteSubset$SeqsByGroup[[1]]]
sort(apply(dendMat[dendC5Assoc,1:4],1,function(x,center){return(sqrt(sum((x-center)^2)))},dendriteEM$parameters$mean[,5]))[1:10]
highlightedDendrites <- append(highlightedDendrites,"AK3N3-Dendrite")
#   Rodent Retinal Ganglion(2)
dendC5Assoc <- dendC5s[dendC5s%in%dendriteSubset$SeqsByGroup[[2]]]
sort(apply(dendMat[dendC5Assoc,1:4],1,function(x,center){return(sqrt(sum((x-center)^2)))},dendriteEM$parameters$mean[,5]))[1:10]
#highlightedDendrites <- append(highlightedDendrites,"cell-205-trace-Dendrite")
highlightedDendrites <- append(highlightedDendrites,"cell-151-trace-Dendrite")

# Cluster 6
dendC6s <- names(dendriteEM$classification)[dendriteEM$classification == 6]
#   Rodent NC Pyramidal(7)
dendC6Assoc <- dendC6s[dendC6s%in%dendriteSubset$SeqsByGroup[[7]]]
sort(apply(dendMat[dendC6Assoc,1:4],1,function(x,center){return(sqrt(sum((x-center)^2)))},dendriteEM$parameters$mean[,6]))[1:10]
highlightedDendrites <- append(highlightedDendrites,"2005-01-27-A1-BasalDendrite")
#   Rodent NC Interneuron(8)
dendC6Assoc <- dendC6s[dendC6s%in%dendriteSubset$SeqsByGroup[[8]]]
sort(apply(dendMat[dendC6Assoc,1:4],1,function(x,center){return(sqrt(sum((x-center)^2)))},dendriteEM$parameters$mean[,6]))[1:10]
#highlightedDendrites <- append(highlightedDendrites,"C110300A1-Dendrite")
highlightedDendrites <- append(highlightedDendrites,"AAC-H0320-Dendrite")

# highlightedDendrites <- c("070316-exp2-zB-BasalDendrite",
#                           "HSN-fluoro09-Dendrite","wt-MARCM-C3-21-Dendrite",
#                           "030110-Dendrite","VS4-fluoro05-Dendrite","Class4-3-noddaE-Dendrite",
#                           "143-1-7-BasalDendrite",
#                           "AK3N3-Dendrite","cell-205-trace-Dendrite",
#                           "2005-01-27-A1-BasalDendrite","C110300A1-Dendrite")
highlightedDendrites <- c("143-1-7-BasalDendrite",
  "070502-exp1-zB-BasalDendrite",
  "AAC-H0320-Dendrite",
  "AK3N3-Dendrite",
  "cell-151-trace-Dendrite",
  "121341-Dendrite",
  "wt-MARCM-C3-21-Dendrite",
  "HSN-fluoro09-Dendrite")

usedMetaData[highlightedDendrites,"Archive"]

# Generate Newick files and then dendrograms
#java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/krieger/CNG version/070502-exp1-zB.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 3 -L NONE -b &
#java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/borst/CNG version/HSN-fluoro09.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 3 -L NONE -b &
#java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/cox/CNG version/wt-MARCM-C3-21.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 3 -L NONE -b &
#java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/masland/CNG version/121341.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 3 -L NONE -b &
#java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/borst/CNG version/VS4-fluoro05.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 3 -L NONE -b &
#java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/cox/CNG version/Class4-3-noddaE.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 3 -L NONE -b &
#java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/jacobs/CNG version/143-1-7.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 3 -L NONE -b &
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/cameron/CNG version/AK3N3.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 3 -L NONE -b &
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/chalupa/CNG version/cell-151-trace.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 3 -L NONE -b &
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/meyer/CNG version/2005-01-27-A1.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 3 -L NONE -b &
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/kawaguchi/CNG version/AAC-H0320.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 3 -L NONE -b &

for (neurite in highlightedDendrites){
  filename <- paste(figDir,gsub("-(Basal)?Dendrite",".CNG-Dendrite.newick",neurite),sep="")
  clusterNum <- dendriteEM$classification[neurite]
  dendColor <- dendriteColors[paste("C",clusterNum,sep="")]
  png(paste(figDir,"Fig3c_Clust",clusterNum,"Dendrograms_",neurite,".png",sep=""),width=600,height=500,units="px")
  tree <- read.tree(filename)
  plotTree(tree,treePalette=dendColor,colorByType=F,edge.width=3,no.margin=T)
  dev.off()
}

# Replot cluster plots with highlighting of examples using code above

#### Figure 4: Cluster analysis of axons ####
arborName <- "Axon"
arborType <- arborName
axonSpecs <- AllAxonMajorSpecifications
#axonSpecs <- AllAxonSpecifications

axonMat <- arborMDS[["Axon"]]
numDimsInit <- ncol(axonMat)
numDims <- 2

# Set the relevant groups and get rid of feature vectors associated with unrepresented neurites
axonSubset <- loadSubsetSeqs(axonSpecs,clusterMetaData,candidates=rownames(axonMat))
#axonMat <- axonMat[axonSubset$Sequences,]

nonSubsetSeqs = names(axonMat)[!rownames(axonMat)%in%axonSubset$Sequences]
groupIds <- axonSubset$GroupIds
allGroupIds <- rep(length(axonSpecs)+1,nrow(axonMat)) # Includes non-specified neurons by adding an extra group "All Others"
names(allGroupIds) <- rownames(axonMat)
allGroupIds[axonSubset$Sequences] <- axonSubset$GroupIds
axonSubset$AllGroupIds <- allGroupIds
numGroups <- length(axonSpecs)

axonEM <- Mclust(axonMat[,1:numDims],G=4,modelNames="VII")

groupNames <- paste(names(axonSpecs)," (",sapply(axonSubset$SeqsByGroup,length),")",sep="")
axonClusterResults <- clusteringStatisticalAnalysis(axonMat,axonEM,axonSubset,groupNames)

#filenameBase <- paste(clusterDir,"/AxonSTL_G3E1Identity_MDS",numDims,"of",numDimsInit,"G",axonEM$G,axonEM$modelName,sep="")
#write.csv(axonClusterResults$Observed,paste(filenameBase,"_Observed.csv",sep=""))
#write.csv(round(axonClusterResults$LogP,2),paste(filenameBase,"_LogPVals",".csv",sep=""))

# Set which neurites will be colored and how
clustersToColor <- c(1:4)
dims <- 1:2

axonColors <- c(Gray="#999999",C1="#3333FF",C2="#00CC00",C3="#EE0000",C4="#00CCCC",C5="#DD00DD",C6="#E68800",C7="#555555",PlainCircle="#333333",White="#FFFFFF",Black="#000000")

# For when only 1 group should associate with each cluster
# axonTypeToClusterMap <- list(c(),c(),c(),c(),c())
# clusterToAxonTypeMap <- rep(0,axonEM$G)
# for (i in 1:axonEM$G){
#   if (sum(axonClusterResults$LogP[,i] > 3) == 1){
#     axonTypeToClusterMap[[which.max(axonClusterResults$LogP[,i])]] <- append(axonTypeToClusterMap[[which.max(axonClusterResults$LogP[,i])]],i)
#     clusterToAxonTypeMap[i] <- which.max(axonClusterResults$LogP[,i])
#   }
# }
# 

# Generating group/cluster mapping by hand, based on clustering results
# Include "Not reported" interneurons
axonTypeToClusterMap <- list(c(1),c(4),c(3),c(1),c(),c(2),c())
clusterToAxonTypeMap <- list(c(1,4),c(6),c(3),c(2))
# WIthout "Not reported" interneurons
axonTypeToClusterMap <- list(c(1),c(4),c(3),c(1),c(2),c())
clusterToAxonTypeMap <- list(c(1,4),c(5),c(3),c(2))

axonsInAssocClust <- mapply(function(classification,groupId,groupClassMap){
 return(classification%in%groupClassMap[[groupId]])
},
axonEM$classification, axonSubset$AllGroupId, MoreArgs=list(axonTypeToClusterMap))


#classification <- axonEM$classification[axonSubset$Sequences]
#axonGroup <- rep("Gray",length(axonSubset$Sequences))
axonGroup <- rep("Gray",length(axonEM$classification))
names(axonGroup) <- names(axonEM$classification)
axonsToColor <- axonEM$classification%in%clustersToColor & axonsInAssocClust
axonGroup[axonsToColor] <- 
  paste("C",axonEM$classification[axonsToColor],sep="")
pointFill <- axonGroup
#axonGroup2[axonSubset$AllGroupId==5] <- "White"
pointFill[axonSubset$AllGroupId==max(axonSubset$AllGroupId)] <- "White"
axonShape <- factor(allGroupIds,levels=1:6)

axonSize <- rep(1.35,length(allGroupIds))
axonSize[axonSubset$AllGroupId==max(axonSubset$AllGroupId)] <- 1.1
#axonSize <- factor(rep(1,length(allGroupIds)),levels=1:6)
#axonSize[axonSubset$AllGroupId==max(axonSubset$AllGroupId)] <- 2

#axonGroup[axonSubset$SeqsByGroup[[5]]] <- "C5"
#clusteredAxonMDS <- cbind(axonMat[axonSubset$Sequences,dims],axonGroup)
clusteredAxonMDS <- cbind(axonMat[,dims],data.frame(axonGroup,pointFill,axonShape,axonSize))
colnames(clusteredAxonMDS) <- c("x","y","group","fill","shape","size")
clusteredAxonMDS <- rbind(subset(clusteredAxonMDS,group=="Gray"),subset(clusteredAxonMDS,group!="Gray"))

require(ellipse)
parms <- axonEM$parameters
df_ell <- list()
for(g in 1:axonEM$G){
  ellipseDF <- generateEllipse(parms,g)

  if (g %in% clustersToColor){
    lineSize <- 1.07
  }else{
    lineSize <- 0.65
    #    tmpdf <- cbind(tmpdf,data.frame(group=rep("PlainCircle",nrow(tmpdf)),size=0.8))
    tmpdfP1 <- data.frame(group=rep(paste("C",g,sep=""),nrow(ellipseDF)),size=0.8)
    tmpdfP2 <- data.frame(group=rep("Black",nrow(ellipseDF)),size=0.85)
    tmpdf <- rbind(tmpdfP1,tmpdfP2)
    #tmpdf <- cbind(tmpdf,data.frame(group=rep(paste("C",g,sep=""),nrow(tmpdf)),size=0.8))
  }
  tmpdf <- cbind(ellipseDF,data.frame(group=rep("Black",nrow(tmpdf)),size=lineSize+0.13))
  colnames(tmpdf) <- c("x","y","group","size")
  df_ell <- append(df_ell, list(tmpdf))
  tmpdf <- cbind(ellipseDF,data.frame(group=rep(paste("C",g,sep=""),nrow(tmpdf)),size=lineSize))
  colnames(tmpdf) <- c("x","y","group","size")
  df_ell <- append(df_ell, list(tmpdf))
  
#   tmpdf <- as.data.frame(ellipse::ellipse(parms$variance$sigma[dims,dims,g], 
#                                           centre=parms$mean[dims,g]))
#   if (g %in% clustersToColor){
#     tmpdf <- cbind(tmpdf,data.frame(group=rep(paste("C",g,sep=""),nrow(tmpdf)),size=factor(3,1:4)))
#   }else{
#     #    tmpdf <- cbind(tmpdf,data.frame(group=rep("PlainCircle",nrow(tmpdf)),size=0.8))
#     tmpdf <- cbind(tmpdf,data.frame(group=rep(paste("C",g,sep=""),nrow(tmpdf)),size=factor(4,1:4)))
#   }
#   colnames(tmpdf) <- c("x","y","group","size")
#   df_ell <- append(df_ell, list(tmpdf))
}

#filename <- paste("/home/tgillett/figures/Pasta2/AxonScatter-Select-ByClusters",
#                  paste(clustersToColor,collapse="-"),"_Dims",paste(dims,collapse="-"),".png",sep="")

#filename <- paste("/home/tgillett/figures/Pasta2/Fig4a-AxonScatter-Select-ByAssocClusters",
#                  paste(clustersToColor,collapse="-"),"_Dims",paste(dims,collapse="-"),".png",sep="")
filename <- paste("/home/tgillett/figures/Pasta2/Fig4a-AxonScatter-Select-ByAssocClusters",
                  paste(clustersToColor,collapse="-"),"_Dims",paste(dims,collapse="-"),"-Sigma.png",sep="")
shapeAssignments <- c(21,21,21,23,21,21,21)
names(shapeAssignments) <- 1:length(shapeAssignments)

axonsClustFig1 <- ggplot(clusteredAxonMDS,aes(x=x,y=y,colour=group,fill=fill,shape=shape,size=size))+
#  geom_point(size=2.1,shape=21)+
  geom_point()+
  scale_size(range=c(1.4,4.5))+
  #scale_size_manual(values=c(2.1,1.4,1.6,0.8))+
  #scale_size_manual(values=c(2.1,1.4,3.2,3.6))+
  scale_fill_manual(values=axonColors)+
  scale_colour_manual(values=axonColors)+
  #scale_shape_discrete(solid=F)+  
  scale_shape_manual(values=shapeAssignments)+
  scale_x_continuous(breaks=c(-.8,-.6,-.4,-.2,0,.2,.4,.6))+
  #scale_shape_manual(values=c(1:5)) +  
  paper2Theme+coord_fixed()
for (i in df_ell){
#  axonsClustFig1 <- axonsClustFig1 + geom_path(aes(x=x,y=y,colour=group,size=size,shape=NULL,fill=NULL), data=i, linetype=2)
  axonsClustFig1 <- axonsClustFig1 + geom_path(aes(x=x,y=y,colour=group,size=size,fill=NULL,shape=NULL), data=i, linetype=1)
}
#axonsClustFig1 <- axonsClustFig1 + scale_size(range=c(0.8, 2.7), guide=FALSE)
axonsClustFig1 <- axonsClustFig1 + geom_point(data=clusteredAxonMDS[highlightedAxons,],bg="#FFFFFF",colour="#FFFFFF",aes(shape=shape),size=4.5)
axonsClustFig1 <- axonsClustFig1 + geom_point(data=clusteredAxonMDS[highlightedAxons,],bg="black",colour="black",aes(shape=shape),size=3.7)
axonsClustFig1 <- axonsClustFig1 + geom_point(data=clusteredAxonMDS[highlightedAxons,],aes(color=group,bg=group,shape=shape),size=1.75)
ggsave(filename,axonsClustFig1,width=12,height=8)

#axonsClustFig1 <- axonsClustFig1 + geom_point(data=clusteredAxonMDS[highlightedAxons,],bg="black",colour="black",aes(shape=shape),size=2.7)
#axonsClustFig1 <- axonsClustFig1 + geom_point(data=clusteredAxonMDS[highlightedAxons,],aes(color=group,bg=group,shape=shape),size=1.4)

## Pick dendrites and generate morphologies and dedrograms (11 total)
names(axonSpecs)
axonClusterResults$LogP
axonClusterResults$Observed
highlightedAxons <- c()
# Cluster 1
axonC1s <- names(axonEM$classification)[axonEM$classification == 1]
#   Mouse NC Pyramidal(1)
axonC1Assoc <- axonC1s[axonC1s%in%axonSubset$SeqsByGroup[[1]]]
sort(apply(axonMat[axonC1Assoc,1:2],1,function(x,center){return(sqrt(sum((x-center)^2)))},axonEM$parameters$mean[,1]))[1:10]
#axonC1s[axonC1s%in%axonSubset$SeqsByGroup[[1]]]
#highlightedAxons <- append(highlightedAxons,"VN042105-0-Axon")
highlightedAxons <- append(highlightedAxons,"M060405SL1-Axon")
#   Rodent Interneuron-Dendritic Targeting (4)
axonC1Assoc <- axonC1s[axonC1s%in%axonSubset$SeqsByGroup[[4]]]
sort(apply(axonMat[axonC1Assoc,1:2],1,function(x,center){return(sqrt(sum((x-center)^2)))},axonEM$parameters$mean[,1]))[1:10]
#axonC1s[axonC1s%in%axonSubset$SeqsByGroup[[4]]]
#highlightedAxons <- append(highlightedAxons,"C140898A-I1-Axon")
highlightedAxons <- append(highlightedAxons,"C260199A-I3-Axon")

# Cluster 2
axonC2s <- names(axonEM$classification)[axonEM$classification == 2]
#   Adult Fly Olfactory Uniglomerular (5)
axonC2Assoc <- axonC2s[axonC2s%in%axonSubset$SeqsByGroup[[5]]]
sort(apply(axonMat[axonC2Assoc,1:2],1,function(x,center){return(sqrt(sum((x-center)^2)))},axonEM$parameters$mean[,2]))[1:10]
#axonC2s[axonC2s%in%axonSubset$SeqsByGroup[[5]]]
#highlightedAxons <- append(highlightedAxons,"EBT53L1-Axon")
highlightedAxons <- append(highlightedAxons,"EBT83L1-Axon")

# Cluster 3
axonC3s <- names(axonEM$classification)[axonEM$classification == 3]
#   Rodent Interneuron-Perisomatic Targeting (3)
axonC3s[axonC3s%in%axonSubset$SeqsByGroup[[3]]]
highlightedAxons <- append(highlightedAxons,"C281199A-IN-Axon")

# Cluster 4
axonC4s <- names(axonEM$classification)[axonEM$classification == 4]
#   Rat NC Pyramidal (2)
axonC4Assoc <- axonC4s[axonC4s%in%axonSubset$SeqsByGroup[[2]]]
sort(apply(axonMat[axonC4Assoc,1:2],1,function(x,center){return(sqrt(sum((x-center)^2)))},axonEM$parameters$mean[,4]))[1:10]
#axonC4s[axonC4s%in%axonSubset$SeqsByGroup[[2]]]
#highlightedAxons <- append(highlightedAxons,"C310897B-P3-Axon")
highlightedAxons <- append(highlightedAxons,"C220797A-P4-Axon")

highlightedAxons <- c("M060405SL1-Axon","C260199A-I3-Axon",
                      "EBT83L1-Axon",
                      "C281199A-IN-Axon",
                      "C220797A-P4-Axon")
usedMetaData[highlightedAxons,"Archive"]

# Generate Newick files and then dendrograms
#java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/yuste/CNG version/M060405SL1.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 1 -L NONE -b &
#java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/markram/CNG version/C260199A-I3.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 1 -L NONE -b &
#java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/jefferis/CNG version/EBT83L1.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 1 -L NONE -b &
#java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/markram/CNG version/C281199A-IN.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 1 -L NONE -b &
#java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/swcFiles/NM/markram/CNG version/C220797A-P4.CNG.swc" -o "/home/tgillett/figures/Pasta2/" -a 1 -L NONE -b &

for (neurite in highlightedAxons){
  filename <- paste(figDir,gsub("-Axon",".CNG-Axon.newick",neurite),sep="")
  clusterNum <- axonEM$classification[neurite]
  axonColor <- axonColors[paste("C",clusterNum,sep="")]
  png(paste(figDir,"Fig4c_Clust",clusterNum,"Dendrograms_",neurite,".png",sep=""),width=600,height=500,units="px")
  tree <- read.tree(filename)
  plotTree(tree,treePalette=axonColor,colorByType=F,edge.width=3,no.margin=T)
  dev.off()
}

# Replot cluster plots with highlighting of examples using code above

#### Fig 5a: Scatterplot highlighting chosen cluster ####

# Version 1: CA1 pyramidal apical
arborType <- "Apical"
selectCluster <- 3
modelName <- "EII"
numClusts <- 3
clusterColors <- axonColors #using the same colors for clustering for now
# Version 2: Neocortical pyramidal axon
arborType <- "Axon"
#selectCluster <- 4
selectCluster <- 1
modelName <- "VII"
numClusts <- 4
plotColors <- clusterColors

featMat <- arborMDS[[arborType]]
numDims <- numDimsList[[arborType]]
featEM <- Mclust(featMat[,1:numDims],G=numClusts,modelNames=modelName)

# Make plot data with colors and sizes
dims <- 1:2
pointColors <- rep("Gray",nrow(featMat))
pointSizes <- rep(0.7,nrow(featMat))
pointColors[featEM$classification == selectCluster] <- paste("C",selectCluster,sep="")
pointSizes[featEM$classification == selectCluster] <- 0.9
plotdf <- cbind(featMat[,dims],data.frame(color=pointColors,size=pointSizes))
colnames(plotdf)[1:2] <- c("x","y")

# Make cluster ellipses
require(ellipse)
parms <- featEM$parameters
#for(g in 1:featEM$G){
g <- selectCluster
selectEllipse <- as.data.frame(ellipse::ellipse(parms$variance$sigma[dims,dims,g], 
                                                centre=parms$mean[dims,g]))
selectEllipse <- cbind(selectEllipse,data.frame(group=rep(paste("C",g,sep=""),nrow(selectEllipse)),size=.6))
colnames(selectEllipse) <- c("x","y","group","size")

filename <- paste(figDir,"/MSA-Cluster_Fig5a_",arborType,"-",selectCluster,".png",sep="")
clustforMSA <- ggplot(plotdf,aes(x=x,y=y, color=color,size=size))+
  geom_point() + 
  scale_fill_manual(values=clusterColors)+
  scale_colour_manual(values=clusterColors)+
  scale_x_continuous(breaks=c(-.6,-.4,-.2,0,.2,.4))+ # for axons
  #scale_x_continuous(breaks=c(-.8,-.6,-.4,-.2,0,.2,.4,.6))+ 
  #scale_shape_manual(values=c(1:5)) +  
  paper2Theme+coord_fixed()+scale_size(range=c(1.5, 2.5), guide=FALSE)+
  geom_path(aes(x=x,y=y,colour=group,size=size,shape=NULL,fill=NULL), data=selectEllipse, linetype=2)
ggsave(filename,clustforMSA,width=12,height=8)

highlightCell <- "C210797A-I" #Axon
highlightCell <- "n181" #Apical

highlightNeurite <- paste(highlightCell,arborType,sep="-")
filename <- paste(figDir,"/MSA-Cluster_Fig5a_",arborType,"-",selectCluster,"_Highlight-",highlightCell,".png",sep="")
clustforMSA <- ggplot(plotdf,aes(x=x,y=y, color=color,size=size))+
  geom_point() + 
  scale_fill_manual(values=axonColors)+
  scale_colour_manual(values=axonColors)+
  scale_x_continuous(breaks=c(-.6,-.4,-.2,0,.2,.4))+ # for axons
  #scale_x_continuous(breaks=c(-.8,-.6,-.4,-.2,0,.2,.4,.6))+ 
  #scale_shape_manual(values=c(1:5)) +  
  paper2Theme+coord_fixed()+scale_size(range=c(1.5, 2.5), guide=FALSE)+
  geom_path(aes(x=x,y=y,colour=group,size=size,shape=NULL,fill=NULL), data=selectEllipse, linetype=2)+
  geom_point(x=featMat[highlightNeurite,1],y=featMat[highlightNeurite,2],color="#00FFFF")
ggsave(filename,clustforMSA,width=12,height=8)

## Create whole-MSA view via HTML, snippet inserted directly into Powerpoint (An example for axon and apical)
# Removed extra long sequence from SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust4-Zoom-DistLim1.fasta to make SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust4-Zoom-DistLim1-shorter.fasta
# Removed 2 long sequences from SeqList-ApicalNC-G3E1Identity-Dims4of8-EII3-Clust3-Zoom-DistLim1.fasta to make SeqList-ApicalNC-G3E1Identity-Dims4of8-EII3-Clust3-Zoom-DistLim1-shorter.fasta
#   Included MDE798 and 02-02-09-ClassIV-1 (a DA Class IV neuron that happened to be split into apical and basal)

# Create MSA (then convert to HTML)
#python3 penne.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust4-Zoom-DistLim1-shorter.fasta -build /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust4-Zoom-DistLim1_T0.5Sub120I-shorter.xml --threshold 0.5 --subsample 120 --iterate 0 -n 10
#python3 penne.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-ApicalNC-G3E1Identity-Dims4of8-EII3-Clust3-Zoom-DistLim1-shorter.fasta -build /home/tgillett/Domains/consensuses/MSA-ApicalNC-G3E1-Dims4of8-EII3-Clust3-Zoom-DistLim1_T0.5Sub100I-shorter.xml --threshold 0.5 --subsample 100 --iterate 0 -n 10
#python3 penne.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1-shorter.fasta -build /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120I-shorter.xml --threshold 0.5 --subsample 120 --iterate 0 -n 10

# Copy xml to html
#cp /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120I-shorter.xml /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120I-shorter.html
# Manually move composite to end and consensus to 2nd-to-end and remove extraneous xml
# Run sed commands to convert
sed -i 's/ *<sequence\( name=".*"\)*>//g' MSA-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120I-shorter.html
sed -i 's/ *<\/sequence>//g' MSA-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120I-shorter.html
sed -i 's/-/ /g' MSA-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120I-shorter.html
sed -i 's/\(AA*\)/<b>\1<\/b>/g' MSA-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120I-shorter.html
sed -i 's/\(CC*\)/<i>\1<\/i>/g' MSA-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120I-shorter.html
sed -i 's/\(TT*\)/<u>\1<\/u>/g' MSA-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120I-shorter.html
# Insert rest of html before and after main content


# Generate consensus for modified version
#python3 capellini.py -build /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust4-Zoom-DistLim1_T0.5Sub120I-shorter.xml -newick /home/tgillett/Domains/consensuses/Consensus-Axon-G3E1-Dims2of7-VII4-Clust4-Zoom-DistLim1_T0.5Sub120I-shorter.newick
#python3 capellini.py -build /home/tgillett/Domains/consensuses/MSA-ApicalNC-G3E1-Dims4of8-EII3-Clust3-Zoom-DistLim1_T0.5Sub100I-shorter.xml -newick /home/tgillett/Domains/consensuses/Consensus-ApicalNC-G3E1-Dims4of8-EII3-Clust3-Zoom-DistLim1_T0.5Sub100I-shorter.newick


#### Fig 5b-c: Consensus Dendrograms and Morphologies ####

## Create whole-MSA for chosen clusters
# Axon Cluster 1
#python3 penne.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1.fasta -build SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1.fasta --threshold 0.5 --subsample 120 --iterate 0 -n 10
# Axon Cluster 3
#python3 penne.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1.fasta -build SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1.fasta --threshold 0.5 --subsample 120 --iterate 0 -n 10

# Create consensus newick files
#python3 capellini.py -build /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120I.xml -newick /home/tgillett/Domains/consensuses/Consensus-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120I.newick
#python3 capellini.py -build /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120I.xml -newick /home/tgillett/Domains/consensuses/Consensus-Axon-G3E1-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120I.newick

# Read out/calculate conservation and relative length values at the given thresholds
#python3 capellini.py -build /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120I.xml -scores /home/tgillett/Domains/consensuses/MSA-ScoreVsThreshold-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_Sub120I.csv
#python3 capellini.py -build /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120I.xml -scores /home/tgillett/Domains/consensuses/MSA-ScoreVsThreshold-Axon-G3E1-Dims2of7-VII4-Clust3-Zoom-DistLim1_Sub120I.csv
#grep 0.5 /home/tgillett/Domains/consensuses/MSA-ScoreVsThreshold-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_Sub120I.csv
#grep 0.5 /home/tgillett/Domains/consensuses/MSA-ScoreVsThreshold-Axon-G3E1-Dims2of7-VII4-Clust3-Zoom-DistLim1_Sub120I.csv


## Consensus Dendrograms ##

consensusDir <- "/home/tgillett/Domains/consensuses/"

consensusName <- "Consensus-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120I"
consensusTitle <- "Axon Cluster 1 Threshold 50%"
conservedColor <- clusterColors["C1"]
consensusName <- "Consensus-Axon-G3E1-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120I"
consensusTitle <- "Axon Cluster 3 Threshold 50%"
conservedColor <- clusterColors["C3"]

consensusName <- "Consensus-Axon-G3E1-Dims2of7-VII4-Clust2-Zoom-DistLim1_T0.5Sub120I"
consensusTitle <- "Axon Cluster 2 Threshold 50%"
conservedColor <- clusterColors["C2"]
consensusName <- "Consensus-Axon-G3E1-Dims2of7-VII4-Clust4-Zoom-DistLim1_T0.5Sub120I"
consensusTitle <- "Axon Cluster 4 Threshold 50%"
conservedColor <- clusterColors["C4"]

# Branch widths for conservation (rescaled), no color, "color" down
consensusTree <- read.tree(paste(consensusDir,consensusName,".newick",sep=""))
png(paste(figDir,consensusName,"-ClusterColor-CD-BranchWidth.png",sep=""),width=800,height=600)
par(mar=margins)
plotConsensusDendrogram(consensusTree,primaryColor=conservedColor,secondaryColor=conservedColor,colorDown=T,modulateEdgeWidth=T,rescaleConservation=T,main=consensusTitle,edge.width=4,colorLegend=F)
dev.off()

# Branch widths for conservation (rescaled) with legend, no color, "color" down
consensusTree <- read.tree(paste(consensusDir,consensusName,".newick",sep=""))
png(paste(figDir,consensusName,"-ClusterColor-CD-BranchWidthScaledWLegend.png",sep=""),width=800,height=600)
par(mar=margins)
plotConsensusDendrogram(consensusTree,primaryColor=conservedColor,secondaryColor=conservedColor,colorDown=T,modulateEdgeWidth=T,rescaleConservation=T,main=consensusTitle,edge.width=4,colorLegend=F,widthLegend=T)
dev.off()

## Branch partition asymmetry -> color


# Test of subtrees function by labeling by subtree size
subtreeList <- subtrees(tree)
tree$node.label <- sapply(subtreeList,function(x){return(x$Nnode)})
png("/home/tgillett/figures/Pasta2/TestSubtrees.png")
plot.phylo(tree,show.node.label=T)
dev.off()

#### Fig 5 companion: Cluster asymmetry comparison ####

axonCluster1Seqs <- read.csv(paste(consensusCompareDir,"SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1.txt",sep=""),stringsAsFactor=F)[,1]
axonCluster3Seqs <- read.csv(paste(consensusCompareDir,"SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1.txt",sep=""),stringsAsFactor=F)[,1]
mean(Morphometrics[axonCluster1Seqs,"PartitionAsymmetry"],na.rm=T)
sem(Morphometrics[axonCluster1Seqs,"PartitionAsymmetry"],na.rm=T)
mean(Morphometrics[axonCluster3Seqs[1:120],"PartitionAsymmetry"],na.rm=T)
sem(Morphometrics[axonCluster3Seqs[1:120],"PartitionAsymmetry"],na.rm=T)
wilcox.test(
  Morphometrics[axonCluster1Seqs,"PartitionAsymmetry"],
  Morphometrics[axonCluster3Seqs[1:120],"PartitionAsymmetry"])

#### Fig 5 companion: Length/Convex hull density ####
require(geometry)

# Need to remove extra spaces from files to make readable with read.csv
#find . -type f -iname '*.swc' -exec sed -i 's/^[ \t]*//g' {} +
#find . -type f -iname '*.swc' -exec sed -i 's/  */ /g' {} +
#find . -type f -iname '*.swc' -exec sed -i 's/ *$//g' {} +

AllMorphometrics <- read.csv(paste(clusterDir,"NM-Morphometrics_HWD.csv",sep=""),row.names=1)

# Calculate convex hull volume if not already calculated
if (!"CHullVolume"%in%colnames(AllMorphometrics)){
  AllMorphometrics$CHullVolume <- rep(NA,nrow(AllMorphometrics))
  swcDir <- "/home/tgillett/swcFiles/NM/"
  typeIdFromArborType <- c(Axon=2,Dendrite=3,Apical=4)
  nameExtFromArborType <- list(Axon="Axon",Dendrite=c("Dendrite","BasalDendrite"),Apical="ApicalDendrite")
  # Read all swc files, stripping comments and just getting X,Y,Z coordinates
  files <- list.files(path=swcDir,pattern = "\\.swc$",recursive=T)
  for (file in files){
    # Get neuron name from filename and proceed if it's in clusterMetaData
    neuronName <- gsub(".*\\/([^\\/\\.]+)(\\.CNG)?\\.swc","\\1",file)
    needsToRun  <- F
    for (arborType in names(typeIdFromArborType)){
      neuriteNames <- paste(neuronName,nameExtFromArborType[[arborType]],sep="-")
      if (sum(neuriteNames%in%rownames(AllMorphometrics)) > 0 && 
            is.na(AllMorphometrics[neuriteNames[neuriteNames%in%rownames(AllMorphometrics)][1],"CHullVolume"])){
        needsToRun <- T
      }
    }
    if (!neuronName %in% flaggedNeurons && needsToRun){
      # Read file
      file <- paste(swcDir,file,sep="")
      positions <- read.csv(file,sep=" ",comment.char = "#",header = F,row.names=1)[,-5]
      colnames(positions) <- c("type","x","y","z","parent")
      for (arborType in names(typeIdFromArborType)){
#        arborType <- "Dendrite"
        typeId = typeIdFromArborType[arborType]
        positionsOfType <- subset(positions,type==typeId&parent!=-1)
        neuriteNames <- paste(neuronName,nameExtFromArborType[[arborType]],sep="-")
        if (nrow(positionsOfType) > 2 && sum(neuriteNames%in%rownames(AllMorphometrics)) > 0){
          neuriteName <- neuriteNames[neuriteNames%in%rownames(AllMorphometrics)][1]

           tryCatch(
             { neuritehull <- convhulln(positionsOfType[,c("x","y","z")], "FA")
               AllMorphometrics[neuriteName,"CHullVolume"] <- neuritehull$vol
             },
             error = function(e){ 
               neuritehull <- NA
               AllMorphometrics[neuriteName,"CHullVolume"] <- NA
               print(paste("Failure on",neuriteName))
             },
             finally = { })

#           ## Calculate which segments compose the central 95%, and eliminate the most extreme 5%
#           # Generate new set of points using parent position and segment length
#           segmentCenters <- (positionsOfType[,c("x","y","z")]+positions[positionsOfType$parent,c("x","y","z")])/2
#           segmentLengths <- sqrt(apply((positionsOfType[,c("x","y","z")]-positions[positionsOfType$parent,c("x","y","z")])^2,1,sum))
#           
#           # Calculate center of "mass" and distances
#           center <- apply(segmentCenters * segmentLengths,2,mean)
#           segmentDistanceVectors <- sweep(segmentCenters,2,center)
#           distances <- sqrt(apply(segmentDistanceVectors,1,function(x){sum(x^2)}))
#           
#           #sumOfWeightedDistance <- sum(distances*segmentLengths)
#           #cumsumDistanceProp <- cumsum(sumOfWeightedDistance[order(distances)])/sumOfWeightedDistance
#           #segmentsToKeep <- order(distances)[cumsumDistanceByDistProp < 0.95]
#           cumsumLengthByDistProp <- cumsum(segmentLengths[order(distances)])/sum(segmentLengths)
#           segmentsToKeep <- order(distances)[cumsumLengthByDistProp < 0.95]
#           
#           positionsOfType <- positionsOfType[segmentsToKeep,c("x","y","z")]
#           
#           tryCatch(
#             { neuritehull <- convhulln(positionsOfType, "FA")
#               AllMorphometrics[neuriteName,"CHullVolume95"] <- neuritehull$vol
#             },
#             error = function(e){ 
#               neuritehull <- NA
#               AllMorphometrics[neuriteName,"CHullVolume95"] <- NA
#               print(paste("Failure on",neuriteName))
#             },
#             finally = { })
        }
      }
    }
  }
  write.csv(AllMorphometrics,paste(clusterDir,"NM-Morphometrics_HWD.csv",sep=""))
}

# Calculate length/convex hull density
AllMorphometrics$LengthHullDensity95 <- AllMorphometrics$Length/AllMorphometrics$CHullVolume95

## Compare clusters
summary(AllMorphometrics[axonCluster1Seqs,"LengthHullDensity95"])
summary(AllMorphometrics[axonCluster3Seqs,"LengthHullDensity95"])
wilcox.test(AllMorphometrics[axonCluster1Seqs,"LengthHullDensity95"],AllMorphometrics[axonCluster3Seqs,"LengthHullDensity95"])
ks.test(AllMorphometrics[axonCluster1Seqs,"LengthHullDensity95"],AllMorphometrics[axonCluster3Seqs,"LengthHullDensity95"])
ks.test(AllMorphometrics[axonCluster1Seqs,"SecondMoment"],AllMorphometrics[axonCluster3Seqs,"SecondMoment"])

rodentNCPyrAxons <- rownames(subset(usedMetaData,Order=="Rodent"&BrainRegion=="Neocortex"&Class2=="Pyramidal cell"&ExperCond=="Control"))
mouseNCPyrAxons <- rownames(subset(usedMetaData[rodentNCPyrAxons,],Species=="Mouse"))
ratNCPyrAxons <- rownames(subset(usedMetaData[rodentNCPyrAxons,],Species=="Rat"))
perisomaticTargetingSeqs <- rownames(subset(usedMetaData,Order=="Rodent"&BrainRegion=="Neocortex"&Class2%in%perisomaticTargetingInterneuronClasses&ExperCond=="Control"))
dendriticTargetingSeqs <- rownames(subset(usedMetaData,Order=="Rodent"&BrainRegion=="Neocortex"&Class2%in%dendriticTargetingInterneuronClasses&ExperCond=="Control"))

summary(AllMorphometrics[mouseNCPyrAxons,"LengthHullDensity95"])
summary(AllMorphometrics[dendriticTargetingSeqs,"LengthHullDensity95"])
summary(AllMorphometrics[perisomaticTargetingSeqs,"LengthHullDensity95"])

summary(AllMorphometrics[mouseNCPyrAxons,"CHullVolume95"])
summary(AllMorphometrics[dendriticTargetingSeqs,"CHullVolume95"])
summary(AllMorphometrics[perisomaticTargetingSeqs,"CHullVolume95"])

wilcox.test(AllMorphometrics[perisomaticTargetingSeqs,"LengthHullDensity95"],AllMorphometrics[dendriticTargetingSeqs,"LengthHullDensity95"])
wilcox.test(AllMorphometrics[mouseNCPyrAxons,"LengthHullDensity95"],AllMorphometrics[dendriticTargetingSeqs,"LengthHullDensity95"])
wilcox.test(AllMorphometrics[mouseNCPyrAxons,"LengthHullDensity95"],AllMorphometrics[perisomaticTargetingSeqs,"LengthHullDensity95"])
ks.test(AllMorphometrics[perisomaticTargetingSeqs,"LengthHullDensity95"],AllMorphometrics[dendriticTargetingSeqs,"LengthHullDensity95"])
ks.test(AllMorphometrics[mouseNCPyrAxons,"LengthHullDensity95"],AllMorphometrics[dendriticTargetingSeqs,"LengthHullDensity95"])
ks.test(AllMorphometrics[mouseNCPyrAxons,"LengthHullDensity95"],AllMorphometrics[perisomaticTargetingSeqs,"LengthHullDensity95"])

## ?See if there is a relationship between density and how well a sequence aligns to the consensus?

#### Fig 5 companion: 2nd moment to determine spatial variability ####
# If not done already, need to remove extra spaces from files to make readable with read.csv
#find . -type f -iname '*.swc' -exec sed -i 's/^[ \t]*//g' {} +
#find . -type f -iname '*.swc' -exec sed -i 's/  */ /g' {} +
#find . -type f -iname '*.swc' -exec sed -i 's/ *$//g' {} +

AllMorphometrics <- read.csv(paste(clusterDir,"NM-Morphometrics_HWD.csv",sep=""),row.names=1)
AllMorphometricsTmp <- read.csv(paste(clusterDir,"NM-Morphometrics_HWD.csv",sep=""),row.names=1)

# Calculate convex hull volume if not already calculated
AllMorphometrics$SecondMoment <- rep(0,nrow(AllMorphometrics))
AllMorphometrics$ThirdMoment <- rep(0,nrow(AllMorphometrics))
AllMorphometrics$FourthMoment <- rep(0,nrow(AllMorphometrics))
swcDir <- "/home/tgillett/swcFiles/NM/"
typeIdFromArborType <- c(Axon=2,Dendrite=3,Apical=4)
nameExtFromArborType <- list(Axon="Axon",Dendrite=c("Dendrite","BasalDendrite"),Apical="ApicalDendrite")
# Read all swc files, stripping comments and just getting X,Y,Z coordinates
files <- list.files(path=swcDir,pattern = "\\.swc$",recursive=T)
for (file in files){
#for (neuronName in neuronsToRun){
#   archive <- NA
#   for (arborType in names(typeIdFromArborType)){
#     if (paste(neuronName,arborType,sep="-")%in%rownames(usedMetaData)){
#       archive <- tolower(usedMetaData[paste(neuronName,arborType,sep="-"),"Archive"])
#     }
#   }
#   file <- paste(archive,"/CNG version/",neuronName,".CNG.swc",sep="")
  # Get neuron name from filename and proceed if it's in clusterMetaData
  neuronName <- gsub(".*\\/([^\\/\\.]+)(\\.CNG)?\\.swc","\\1",file)
  if (!neuronName %in% flaggedNeurons){
#  if (!is.na(archive) && !neuronName %in% flaggedNeurons){
    # Read file
    file <- paste(swcDir,file,sep="")
    positions <- read.csv(file,sep=" ",comment.char = "#",header = F,row.names=1)[,-5]
    colnames(positions) <- c("type","x","y","z","parent")
    for (arborType in names(typeIdFromArborType)){
#       arborType <- "Dendrite"
      typeId = typeIdFromArborType[arborType]
      positionsOfType <- subset(positions,type==typeId)
      positionsOfType <- positionsOfType[positionsOfType$parent!=-1,]
      neuriteNames <- paste(neuronName,nameExtFromArborType[[arborType]],sep="-")
      if (nrow(positionsOfType) > 2 && sum(neuriteNames%in%rownames(AllMorphometrics)) > 0){
        neuriteName <- neuriteNames[neuriteNames%in%rownames(AllMorphometrics)][1]

        # Generate new set of points using parent position and segment length
        segmentCenters <- (positionsOfType[,c("x","y","z")]+positions[positionsOfType$parent,c("x","y","z")])/2
        segmentLengths <- sqrt(apply((positionsOfType[,c("x","y","z")]-positions[positionsOfType$parent,c("x","y","z")])^2,1,sum))

        # Calculate center of "mass" and distances
        center <- apply(segmentCenters * segmentLengths,2,mean)
        segmentDistanceVectors <- sweep(segmentCenters,2,center)
        distances <- sqrt(apply(segmentDistanceVectors,1,function(x){sum(x^2)}))
        weightedDistances <- segmentLengths*distances
        
        # Calculate 2nd and 4th moments
        AllMorphometrics[neuriteName,"SecondMoment"] <- var(weightedDistances)
        AllMorphometrics[neuriteName,"ThirdMoment"] <- standardizedMoment(weightedDistances,3)
        AllMorphometrics[neuriteName,"FourthMoment"] <- standardizedMoment(weightedDistances,4)
      }
    }
  }
}
write.csv(AllMorphometrics[,c("SecondMoment","ThirdMoment","FourthMoment")],paste(clusterDir,"NM-Moments.csv",sep=""))

# Compare
summary(sqrt(AllMorphometrics[axonCluster1Seqs,"SecondMoment"]))
summary(sqrt(AllMorphometrics[axonCluster3Seqs,"SecondMoment"]))
wilcox.test(sqrt(AllMorphometrics[axonCluster1Seqs,"SecondMoment"]),sqrt(AllMorphometrics[axonCluster3Seqs,"SecondMoment"]))

summary(AllMorphometrics[axonCluster1Seqs,"ThirdMoment"])
summary(AllMorphometrics[axonCluster3Seqs,"ThirdMoment"])
wilcox.test(AllMorphometrics[axonCluster1Seqs,"ThirdMoment"],AllMorphometrics[axonCluster3Seqs,"ThirdMoment"])

summary(AllMorphometrics[axonCluster1Seqs,"FourthMoment"])
summary(AllMorphometrics[axonCluster3Seqs,"FourthMoment"])
wilcox.test(AllMorphometrics[axonCluster1Seqs,"FourthMoment"],AllMorphometrics[axonCluster3Seqs,"FourthMoment"])

summary(AllMorphometrics[axonCluster1Seqs,"Length"])
summary(AllMorphometrics[axonCluster3Seqs,"Length"])
wilcox.test(AllMorphometrics[axonCluster1Seqs,"Length"],AllMorphometrics[axonCluster3Seqs,"Length"])

rodentNCPyrAxons <- rownames(subset(usedMetaData,Order=="Rodent"&BrainRegion=="Neocortex"&Class2=="Pyramidal cell"&ExperCond=="Control"))
mouseNCPyrAxons <- rownames(subset(usedMetaData[rodentNCPyrAxons,],Species=="Mouse"))
ratNCPyrAxons <- rownames(subset(usedMetaData[rodentNCPyrAxons,],Species=="Rat"))
perisomaticTargetingSeqs <- rownames(subset(usedMetaData,Order=="Rodent"&BrainRegion=="Neocortex"&Class2%in%perisomaticTargetingInterneuronClasses&ExperCond=="Control"))
dendriticTargetingSeqs <- rownames(subset(usedMetaData,Order=="Rodent"&BrainRegion=="Neocortex"&Class2%in%dendriticTargetingInterneuronClasses&ExperCond=="Control"))

summary(AllMorphometrics[ratNCPyrAxons,"Length"])
summary(AllMorphometrics[mouseNCPyrAxons,"Length"])
summary(AllMorphometrics[dendriticTargetingSeqs,"Length"])
summary(AllMorphometrics[perisomaticTargetingSeqs,"Length"])
wilcox.test(AllMorphometrics[mouseNCPyrAxons,"Length"],AllMorphometrics[dendriticTargetingSeqs,"Length"])
wilcox.test(AllMorphometrics[perisomaticTargetingSeqs,"Length"],AllMorphometrics[dendriticTargetingSeqs,"Length"])

summary(AllMorphometrics[ratNCPyrAxons,"CHullVolume"])
summary(AllMorphometrics[mouseNCPyrAxons,"CHullVolume"])
summary(AllMorphometrics[dendriticTargetingSeqs,"CHullVolume"])
summary(AllMorphometrics[perisomaticTargetingSeqs,"CHullVolume"])
wilcox.test(AllMorphometrics[mouseNCPyrAxons,"CHullVolume"],AllMorphometrics[dendriticTargetingSeqs,"CHullVolume"])
wilcox.test(AllMorphometrics[perisomaticTargetingSeqs,"CHullVolume"],AllMorphometrics[dendriticTargetingSeqs,"CHullVolume"])

summary(AllMorphometrics[mouseNCPyrAxons,"Height"]*AllMorphometrics[mouseNCPyrAxons,"Width"])
summary(AllMorphometrics[dendriticTargetingSeqs,"Height"]*AllMorphometrics[dendriticTargetingSeqs,"Width"])
summary(AllMorphometrics[perisomaticTargetingSeqs,"Height"]*AllMorphometrics[perisomaticTargetingSeqs,"Width"])

#### Mouse vs Rat size check ####

rodentNCPyrAxons <- rownames(subset(usedMetaData,Order=="Rodent"&BrainRegion=="Neocortex"&Class2=="Pyramidal cell"&ExperCond=="Control"))
mouseNCPyrAxons <- rownames(subset(usedMetaData[rodentNCPyrAxons,],Species=="Mouse"))
ratNCPyrAxons <- rownames(subset(usedMetaData[rodentNCPyrAxons,],Species=="Rat"))

summary(AllMorphometrics[ratNCPyrAxons,"Height"]*AllMorphometrics[ratNCPyrAxons,"Width"])
summary(AllMorphometrics[mouseNCPyrAxons,"Height"]*AllMorphometrics[mouseNCPyrAxons,"Width"])


#### Fig 5: Comparing representativeness of consensuses ####

# Create consensus fasta files from MSAs of real and surrogate sets -> /home/tgillett/Domains/Consensus-<rest of filename>.fasta
head -n 10 MSA-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120I.xml | tail -n 1
head -n 10 MSA-Axon-G3E1-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120I.xml | tail -n 1

# Run cluster set fasta file against surrogate consensus fasta file, and vice versa
## GAP OPEN = 0
# Cluster 1 vs 1
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G 0 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust1-vs-Clust1ConsT0.5Sub120.tab
# Cluster 3 vs 3
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G 0 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust3-vs-Clust3ConsT0.5Sub120.tab
# Cluster 1 vs 3
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G 0 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust1-vs-Clust3ConsT0.5Sub120.tab
# Cluster 3 vs 1
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G 0 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust3-vs-Clust1ConsT0.5Sub120.tab

## GAP OPEN = -3
# Cluster 1 vs 1
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G -3 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G3E1Identity_Dims2of7-VII4-Clust1-vs-Clust1ConsT0.5Sub120.tab
# Cluster 3 vs 3
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G -3 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G3E1Identity_Dims2of7-VII4-Clust3-vs-Clust3ConsT0.5Sub120.tab
# Cluster 1 vs 3
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G -3 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G3E1Identity_Dims2of7-VII4-Clust1-vs-Clust3ConsT0.5Sub120.tab
# Cluster 3 vs 1
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G -3 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G3E1Identity_Dims2of7-VII4-Clust3-vs-Clust1ConsT0.5Sub120.tab

#source(paste(clusterGlobalDir,"BaselineHandler.r",sep=""))
#baselineKey <- paste("G3E1_e3",travOrder)
#baselineKey <- paste("G0E1_e3",travOrder)
#loadBaselineAlignmentStats("G3E1_e3 STL",directory=paste(baselineDir,"GapOpen3_1000/",sep=""),travOrder=travOrder)
#loadBaselineAlignmentStats("G0E1_e3 STL",directory=paste(baselineDir,"GapOpen0_1000/",sep=""),travOrder=travOrder)

clust1Cons <- "AATACCCTACTCCCCTACATCCTAACTCATCCTACCCTCACCTATATCCCCT"
clust2Cons <- "ATCACTCACCCTACTCACTCATCCCT"
clust3Cons <- "AAACTACTATCCTATAATCCTACCTACTCCATCTAAACTCATCTAATCCCTACCTATCATCCCTAAATCCTACCTATCCCTACACTCACTCCTAATATCCTACCTACCTACTATCCT"
clust4Cons <- "ATAACTCATCTACCTAATCCTATACTATCT"

arborType <- "Axon"
arborParams <- "Dims2of7-VII4"
consNum <- 1
clustNum <- 1

threshold <- "0.5"
subsetSize <- 120
#gapPenalty <- 0

if (!exists("consensusLengths")) consensusLengths <- list()
consensusLengths[["clust1"]] <- nchar(clust1Cons)
consensusLengths[["clust3"]] <- nchar(clust3Cons)
consensusLengths[["clust2"]] <- nchar(clust2Cons)
consensusLengths[["clust4"]] <- nchar(clust4Cons)

loadBaselineAlignmentStats("G3E1_e5 STL","STL",directory=paste(baselineDir,"GapOpen3_e5/",sep=""),gapOpenCost=-3)
loadBaselineAlignmentStats("G0E1_e3 STL","STL",directory=paste(baselineDir,"GapOpen0_1000/",sep=""),gapOpenCost=-3)

rawScores <- list()
zScores <- list()
for (gapPenalty in c(0,3)){
  if (gapPenalty == 3){
    bkey <- "G3E1_e5 STL"
  }else{
    bkey <- paste("G",gapPenalty,"E1_e3 STL",sep="")
  }
  gapPenaltyStr <- as.character(gapPenalty)

  for (clustNum in c(1,3)){
    for (consNum in c(1,3)){
      compareKey <- paste("Clust",clustNum,"-",consNum," G",gapPenalty,sep="")
      consensusLength <- consensusLengths[[paste("clust",consNum,sep="")]]
      filename <- paste(consensusCompareDir,"ScoreMat_",arborType,"STL_G",gapPenalty,"E1Identity_",arborParams,"-Clust",clustNum,"-vs-Clust",consNum,"ConsT",threshold,"Sub",subsetSize,".tab",sep="")
      scores <- read.csv(filename,row.names=1,sep="\t")
      # Eliminate sequences that are way too big
      scores <- scores[usedMetaData[rownames(scores),"SeqLen"] <= 800,,drop=F]
      rawScores[[compareKey]] <- scores
      perCharScores <- mapply(perCharScore,scores[,1],usedMetaData[rownames(scores),"SeqLen"],consensusLength,list(gapPenalty=gapPenalty))
      names(perCharScores) <- rownames(scores)
      zscores <- mapply(getBaselineZScore,bkey,perCharScores,usedMetaData[rownames(scores),"SeqLen"],consensusLength)
      names(zscores) <- rownames(scores)
      zScores[[compareKey]] <- zscores
    }
  }
}

sapply(zScores,mean)
sapply(zScores,sem)

sapply(zScores,function(x){mean(x[usedMetaData[names(x),"SeqLen"]<500])})

## Extract and Normalize scores explicitly
scoresG0Axon1VsCons1T50 <- read.csv("/home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust1-vs-Clust1ConsT0.5Sub120.tab",row.names=1,sep="\t")
# Eliminate sequences that are way too big
scoresG0Axon1VsCons1T50 <- scoresG0Axon1VsCons1T50[usedMetaData[rownames(scoresG0Axon1VsCons1T50),"SeqLen"] <= 800,,drop=F]
perCharScoresG0Axon1VsCons1T50 <- mapply(perCharScore,scoresG0Axon1VsCons1T50[,1],usedMetaData[rownames(scoresG0Axon1VsCons1T50),"SeqLen"],cons1T50Length,list(gapPenalty=0))
names(perCharScoresG0Axon1VsCons1T50) <- rownames(scoresG0Axon1VsCons1T50)
zscoresG0Axon1VsCons1T50 <- mapply(getBaselineZScore,"G0E1_e3 STL",perCharScoresG0Axon1VsCons1T50,usedMetaData[rownames(scoresG0Axon1VsCons1T50),"SeqLen"],cons1T50Length)
names(zscoresG0Axon1VsCons1T50) <- rownames(scoresG0Axon1VsCons1T50)

scoresG0Axon1VsCons3T50 <- read.csv("/home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust1-vs-Clust3ConsT0.5Sub120.tab",row.names=1,sep="\t")
# Eliminate sequences that are way too big
scoresG0Axon1VsCons3T50 <- scoresG0Axon1VsCons3T50[usedMetaData[rownames(scoresG0Axon1VsCons3T50),"SeqLen"] <= 800,,drop=F]
perCharScoresG0Axon1VsCons3T50 <- mapply(perCharScore,scoresG0Axon1VsCons3T50[,1],usedMetaData[rownames(scoresG0Axon1VsCons3T50),"SeqLen"],cons3T50Length,list(gapOpen=0))
names(perCharScoresG0Axon1VsCons3T50) <- rownames(scoresG0Axon1VsCons3T50)
zscoresG0Axon1VsCons3T50 <- mapply(getBaselineZScore,"G0E1_e3 STL",perCharScoresG0Axon1VsCons3T50,usedMetaData[rownames(scoresG0Axon1VsCons3T50),"SeqLen"],cons3T50Length)
names(zscoresG0Axon1VsCons3T50) <- rownames(scoresG0Axon1VsCons3T50)

scoresG0Axon3VsCons1T50 <- read.csv("/home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust3-vs-Clust1ConsT0.5Sub120.tab",row.names=1,sep="\t")
# Eliminate those used to construct the consensus
# Eliminate sequences that are way too big
scoresG0Axon3VsCons1T50 <- scoresG0Axon3VsCons1T50[usedMetaData[rownames(scoresG0Axon3VsCons1T50),"SeqLen"] <= 800,,drop=F]
perCharScoresG0Axon3VsCons1T50 <- mapply(perCharScore,scoresG0Axon3VsCons1T50[,1],usedMetaData[rownames(scoresG0Axon3VsCons1T50),"SeqLen"],cons1T50Length,list(gapOpen=0))
names(perCharScoresG0Axon3VsCons1T50) <- rownames(scoresG0Axon3VsCons1T50)
zscoresG0Axon3VsCons1T50 <- mapply(getBaselineZScore,"G0E1_e3 STL",perCharScoresG0Axon3VsCons1T50,usedMetaData[rownames(scoresG0Axon3VsCons1T50),"SeqLen"],cons1T50Length)
names(zscoresG0Axon3VsCons1T50) <- rownames(scoresG0Axon3VsCons1T50)

scoresG0Axon3VsCons3T50 <- read.csv("/home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust3-vs-Clust3ConsT0.5Sub120.tab",row.names=1,sep="\t")
# Eliminate sequences that are way too big
scoresG0Axon3VsCons3T50 <- scoresG0Axon3VsCons3T50[usedMetaData[rownames(scoresG0Axon3VsCons3T50),"SeqLen"] <= 800,,drop=F]
perCharScoresG0Axon3VsCons3T50 <- mapply(perCharScore,scoresG0Axon3VsCons3T50[,1],usedMetaData[rownames(scoresG0Axon3VsCons3T50),"SeqLen"],cons3T50Length,list(gapOpen=0))
names(perCharScoresG0Axon3VsCons3T50) <- rownames(scoresG0Axon3VsCons3T50)
zscoresG0Axon3VsCons3T50 <- mapply(getBaselineZScore,"G0E1_e3 STL",perCharScoresG0Axon3VsCons3T50,usedMetaData[rownames(scoresG0Axon3VsCons3T50),"SeqLen"],cons3T50Length)
names(zscoresG0Axon3VsCons3T50) <- rownames(scoresG0Axon3VsCons3T50)

mean(zscoresG0Axon1VsCons1T50)
mean(zscoresG0Axon3VsCons3T50)
mean(zscoresG0Axon1VsCons3T50)
mean(zscoresG0Axon3VsCons1T50)

mean(zscoresG0Axon1VsCons1T50[usedMetaData[names(zscoresG0Axon1VsCons1T50),"SeqLen"]<500])
mean(zscoresG0Axon3VsCons3T50[usedMetaData[names(zscoresG0Axon3VsCons3T50),"SeqLen"]<500])
mean(zscoresG0Axon1VsCons3T50[usedMetaData[names(zscoresG0Axon1VsCons3T50),"SeqLen"]<500])
mean(zscoresG0Axon3VsCons1T50[usedMetaData[names(zscoresG0Axon3VsCons1T50),"SeqLen"]<500])

# Eliminate those used to construct the consensus
mean(zscoresG0Axon1VsCons1T50[121:length(zscoresG0Axon1VsCons1T50)])
mean(zscoresG0Axon1VsCons3T50)
mean(zscoresG0Axon3VsCons1T50)
mean(zscoresG0Axon3VsCons3T50[121:length(zscoresG0Axon3VsCons3T50)])


#### Fig 5c&e: Select neurites for consensus-matching coloring ####

## Select neurites
## Pick dendrites and generate morphologies and dedrograms (11 total)
names(axonSpecs)
axonClusterResults$LogP
axonClusterResults$Observed
fig5Clust1HighlightedAxons <- c()
fig5Clust3HighlightedAxons <- c()

# Cluster 1
axonC1s <- names(axonEM$classification)[axonEM$classification == 1]
# Pick neurites of approximately the same size
# (preferably assoc class MousePyramidal & RodentDendriticTargetingInterneuron?)
summary(usedMetaData[axonC1s,c("SeqLen")])
subset(usedMetaData[axonC1s,],SeqLen >= 90 & SeqLen <= 120)[,c("SeqLen","Class2","Species"),drop=F]
subset(usedMetaData[axonC3s,],SeqLen >= 90 & SeqLen <= 120 & Species=="Mouse" & Class2=="Pyramidal cell")[,"SeqLen",drop=F]

fig5Clust1HighlightedAxons <- append(fig5Clust1HighlightedAxons,"C290500C-I4-Axon") # Interneuron:Martinotti, len 146
fig5Clust1HighlightedAxons <- append(fig5Clust1HighlightedAxons,"C290500B-I3-Axon") # Interneuron:Martinotti, len 129
fig5Clust1HighlightedAxons <- append(fig5Clust1HighlightedAxons,"Vn05312006-1-B-Axon") # Interneuron:SOM+, len 125
fig5Clust1HighlightedAxons <- append(fig5Clust1HighlightedAxons,"TG073axEndLayers-Axon") # Interneuron:Bipolar, len 144
fig5Clust1HighlightedAxons <- append(fig5Clust1HighlightedAxons,"AM107-4-3-Axon") # Mouse Pyramidal, len 98
fig5Clust1HighlightedAxons <- append(fig5Clust1HighlightedAxons,"MJ042605-1-Axon") # Mouse Pyramidal, len 123
fig5Clust1HighlightedAxons <- append(fig5Clust1HighlightedAxons,"AM60-1-Axon") # Mouse Pyramidal, len 103

# Not in first 120
#fig5HighlightedAxons <- append(fig5HighlightedAxons,"C251197A-I1-Axon") # Martinotti, len 111
#fig5HighlightedAxons <- append(fig5HighlightedAxons,"C200199B-I3-Axon") # Bitufted, len 113
#fig5HighlightedAxons <- append(fig5HighlightedAxons,"AM107-4-2-Axon") # Mouse Pyramidal, len 106
#fig5HighlightedAxons <- append(fig5HighlightedAxons,"JM012704-40-1-Axon") # Mouse Pyramidal, len 108

# Cluster 3 (for Fig 6)
axonC3s <- names(axonEM$classification)[axonEM$classification == 3]
# Pick neurites of approximately the same size (preferably assoc class Rodent-PerisomaticTargetingInterneuron?)
summary(usedMetaData[axonC3s,"SeqLen"])
subset(usedMetaData[axonC3s,],SeqLen >= 160 & SeqLen <= 200 & Order=="Rodent")[,"SeqLen",drop=F]

fig5Clust3HighlightedAxons <- append(fig5Clust3HighlightedAxons,"C200897C-I1-Axon") # Rat Basket, len 178
fig5Clust3HighlightedAxons <- append(fig5Clust3HighlightedAxons,"BE12D-Axon") # Rat Basket, len 181
fig5Clust3HighlightedAxons <- append(fig5Clust3HighlightedAxons,"BE83F-Axon") # Rat Basket, len 182
fig5Clust3HighlightedAxons <- append(fig5Clust3HighlightedAxons,"C180298A-I4-Axon") # Rat Basket, len 192
fig5Clust3HighlightedAxons <- append(fig5Clust3HighlightedAxons,"AP041910E-1-Axon") # Mouse Basket, len 190

# Same for Axon clusters 1 and 3 if need to use other threshold
#python3 capellini.py -build /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120I.xml --threshold 0.5 -newbuild /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120I.xml
#python3 capellini.py -build /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120I.xml --threshold 0.5 -newbuild /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120I.xml

## Generate morphologies retyped by alignment with consensus
# java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.ColorNeuriteByGlobalAlignment

## Generate newick files from retyped morphologies

# Figure 5 Cluster 1
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/figures/Pasta2/Alignment_AxonCluster1ForFig5-T50_vs_C290500C-I4-Axon-STL.swc" -a 0 -L NODE_TYPE -b &
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/figures/Pasta2/Alignment_AxonCluster1ForFig5-T50_vs_MJ042605-1-Axon-STL.swc" -a 0 -L NODE_TYPE -b &

# Figure 5 Cluster 3
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/figures/Pasta2/Alignment_AxonCluster3ForFig5-T50_vs_BE12D-Axon-STL.swc" -a 0 -L NODE_TYPE -b &
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/figures/Pasta2/Alignment_AxonCluster3ForFig5-T50_vs_C180298A-I4-Axon-STL.swc" -a 0 -L NODE_TYPE -b &

# Old ones
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/figures/Pasta2/C290500B-I3-Axon-STL_AlignmentVs_AxonCluster1ForFig6-T50.swc" -a 0 -L NODE_TYPE -b &
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/figures/Pasta2/Vn05312006-1-B-Axon-STL_AlignmentVs_AxonCluster1ForFig6-T50.swc" -a 0 -L NODE_TYPE -b &
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/figures/Pasta2/TG073axEndLayers-Axon-STL_AlignmentVs_AxonCluster1ForFig6-T50.swc" -a 0 -L NODE_TYPE -b &
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/figures/Pasta2/AM107-4-3-Axon-STL_AlignmentVs_AxonCluster1ForFig6-T50.swc" -a 0 -L NODE_TYPE -b &
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/figures/Pasta2/AM60-1-Axon-STL_AlignmentVs_AxonCluster1ForFig6-T50.swc" -a 0 -L NODE_TYPE -b &
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/figures/Pasta2/C200897C-I1-Axon-STL_AlignmentVs_AxonCluster3ForFig6-T50.swc" -a 0 -L NODE_TYPE -b &
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/figures/Pasta2/BE83F-Axon-STL_AlignmentVs_AxonCluster3ForFig6-T50.swc" -a 0 -L NODE_TYPE -b &
# java -classpath .:lib/JSAP-2.1.jar org.cn3.morpho.data.SwcToNewick -i "/home/tgillett/figures/Pasta2/AP041910E-1-Axon-STL_AlignmentVs_AxonCluster3ForFig6-T50.swc" -a 0 -L NODE_TYPE -b &

# Specifically chosen axons for Figure 5:
fig5Clust1HighlightedAxons <- c("MJ042605-1-Axon","C290500C-I4-Axon")
fig5Clust3HighlightedAxons <- c("BE12D-Axon","C180298A-I4-Axon")

consensusName <- "AxonCluster1ForFig5-T50"
setPalette <- c("","","","","#888888",axonColors["C1"],"black","","")

for (neurite in fig5Clust1HighlightedAxons){
  treeName <- paste("Alignment_",consensusName,"_vs_",neurite,"-STL",sep="")
  tree <- read.tree(paste(figDir,treeName,"-BinaryLabelType.newick",sep=""))
  png(paste(figDir,treeName,"-Dendrogram-ColDown.png",sep=""),width=800,height=600)
  par(mar=margins)
  consensusTitle <- paste(consensusName,": ",neurite,sep="")
  plotTree(tree,main=consensusTitle,edge.width=3,treePalette=setPalette,colorDown=T)
  dev.off()
}

consensusName <- "AxonCluster3ForFig5-T50"
setPalette <- c("","","","","#888888",axonColors["C3"],"black","","")

for (neurite in fig5Clust3HighlightedAxons){
  treeName <- paste("Alignment_",consensusName,"_vs_",neurite,"-STL",sep="")
  tree <- read.tree(paste(figDir,treeName,"-BinaryLabelType.newick",sep=""))
  png(paste(figDir,treeName,"-Dendrogram-ColDown.png",sep=""),width=800,height=600)
  par(mar=margins)
  consensusTitle <- paste(consensusName,": ",neurite,sep="")
  plotTree(tree,main=consensusTitle,edge.width=3,treePalette=setPalette,colorDown=T)
  dev.off()
}

## Manually produce morphology visualization

# Rerun cluster figure using chosen neurites?

#### Figure 6: Kmer-constrained surrogate MSAs ####

## Generate graph of certain kmer PRs in particular clusters
# Load motif functions and data
source(paste(motifCodeDir,'PRAnalysisFunctions.r',sep=''))
source(paste(motifCodeDir,'RawKmerFreqsFunctions.r',sep=''))
dataset <- "NM_SpurLen2D"
defaultOrder <- "STL"
defaultMethod <- 8
defaultThreshold <- "0.0"
defaultThreshType <- "Whole"

prDataB8 <- list()
prDataB8Long <- list()
kmersPerK <- list()
for (k in 1:3){
  prFilename <- getPRFilename(k,dataset=dataset,sets=numSets,surrogateSet=surrogateSetNum,order="STL",baselineMethod=8,threshold="0.0",thresholdTypeString="Whole",directory=prDataDirectory,sourceTraversal="STL")
  prDataB8[[k]] <- read.csv(prFilename,stringsAsFactors=F,row.names=1)
  prDataB8[[k]] <- prDataB8[[k]][,colnames(prDataB8[[k]])!="N",drop=F]
  prDataB8[[k]] <- prDataB8[[k]][!is.na(prDataB8[[k]][,1]),,drop=F]
  
  if (k >= 3){
    # Read in LTS k-mers
    prDataB8LTS <- read.csv(getPRFilename(k,dataset=dataset,sets=numSets,surrogateSet=surrogateSetNum,order="LTS",baselineMethod=8,threshold="0.0",thresholdTypeString="Whole",directory=prDataDirectory,sourceTraversal="STL"),stringsAsFactors=F,row.names=1)
    prDataB8LTS <- prDataB8LTS[,colnames(prDataB8LTS)!="N",drop=F]
    # For merging, get rid of LTS k-mers that have an STL counterpart
    inds <- grep("[ACT]C+[ACT]",colnames(prDataB8LTS),perl=T)
    colnames(prDataB8LTS) <- paste("L-",colnames(prDataB8LTS),sep="")
    STLNames <- colnames(prDataB8[[k]])
    prDataB8[[k]] <- cbind(prDataB8[[k]],prDataB8LTS[,-inds])
    colnames(prDataB8[[k]]) <- c(STLNames,colnames(prDataB8LTS)[-inds])
    
    # Remove fully constrained (ATT related) kmers
    prDataB8[[k]] <- prDataB8[[k]][,removeATT("STL",colnames(prDataB8[[k]]))]
  }
  
  prDataB8[[k]] <- prDataB8[[k]][union(rownames(arborMDS[["Dendrite"]]),union(rownames(arborMDS[["Axon"]]),rownames(arborMDS[["Apical"]]))),,drop=F]
  prDataB8[[k]][,"cluster"] <- factor("C1",levels=paste("C",1:6,sep=""))
  for (arborType in names(emResults)){
    prDataB8[[k]][names(emResults[[arborType]]$classification),"cluster"] <- factor(paste("C",emResults[[arborType]]$classification,sep=""),levels=paste("C",1:6,sep=""))
  }
  kmers <- colnames(prDataB8[[k]])[colnames(prDataB8[[k]])!="cluster"]
  prDataB8Long[[k]] <- reshape(prDataB8[[k]],direction="long",varying=list(kmers),times=kmers,idvar="SeqName",ids=rownames(prDataB8[[k]]),timevar="Kmer",v.names="PercentileRank")
}

lowsem <- function(x){ return(mean(x) - sem(x)) }
highsem <- function(x){ return(mean(x) + sem(x)) }

# Get subsets by cluster and compare motif PRs
barColors <- c("#4545EE","#43C043","#DF5454","#44CCCC","#D033D0","#E08040","#555555")
names(barColors) <- paste("C",1:7,sep="")
#arborType <- "ApicalDendrite"
arborType <- "Axon"
arborName <- arborNamesByType[arborType]
# Complete plots of PR from k 1-3 (for review and possibly supplemenal figs)
for (k in 1:3){
  prArborData <- subset(prDataB8Long[[k]],SeqName%in%rownames(getSubset2(prDataB8[[k]],list(ArborType=arborType,ExperCond="Control"),usedMetaData)))
  # Update cluster numbers for figure
  if (arborType=="Axon")
    prArborData$cluster <- factor(as.character(prArborData$cluster),levels=paste("C",c(2,4,1,3),sep=""))
  
  # Filter, taking only kmers that show substantial difference between clusters
  if (k > 1){
    groupMeans <- aggregate(prArborData$PercentileRank,by=list(Cluster=prArborData$cluster,Kmer=prArborData$Kmer),mean,na.rm=T)
    groupSDs <- aggregate(prArborData$PercentileRank,by=list(Cluster=prArborData$cluster,Kmer=prArborData$Kmer),sd,na.rm=T)
    for (kmer in unique(prArborData$Kmer)){
      diff(range(subset(groupMeans,Kmer==kmer)$x))
    }
  }
  
  #filename <- paste(figDir,"/MotifsPerCluster-",arborType,"-K",k,".png",sep="")
  filename <- paste(figDir,"/MotifsPerCluster-",arborType,"-UpdatedClusterNums-K",k,".png",sep="")
#  prArborData$cluster <- factor(emResults[[arborType]]$classification[prArborData$SeqName])
  if (k == 1){
    barWidth <- 0.7
    dodgeWidth <- 0.9
  }else{
    barWidth <- 0.6
    dodgeWidth <- 0.7
  }
  motifPlot <- ggplot(prArborData,aes(x=Kmer,y=PercentileRank,fill=cluster,group=cluster,width=barWidth))+
    stat_summary(fun.y=mean, geom="bar",colour="white",position=position_dodge(width = dodgeWidth))+
    stat_summary(fun.y=mean, fun.ymin=lowsem, fun.ymax=highsem,position=position_dodge(width = dodgeWidth), geom="linerange",color = 'black')
  motifPlot <- motifPlot + scale_fill_manual(values=barColors)+
    paper2Theme + theme(panel.grid.major.y=element_line(size=0.7, colour="#f0f0f0", linetype="solid"))
  ggsave(filename,motifPlot,width=12,height=8)
}

## Get %C and k-mer PRs of consensuses
# 1. Generate consensus fasta file: /home/tgillett/fastaFromSwc/Consensus/Arborization/Mapping2/Size/ConsensusArborSizeSTLMap2.fasta
# 2. Calculate %C
#    java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.motif.analysis.KmerFrequencyCalculator -D Consensus -o STL --startK 1 --endK 1 --outputDir ~/results/ --sourceTraversal STL
# 3. Generate surrogates
#    java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.motif.generation.ConstrainedSurrogateGenerator -D Consensus -n 100 -o STL --startConstraint 0 -M 3 -s 1 -b 8 &
# 4. Generate PRs
#    java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.motif.analysis.CompleteConstraintAnalyzer -D Consensus -o STL -C 2 -s 100 -m 8 --surrogateSet 1 --outputDir ~/results/PercentileRanks/
#    java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.motif.analysis.CompleteConstraintAnalyzer -D Consensus -o LTS --sourceTraversal STL --startK 2 -C 2 -s 100 -m 8 --surrogateSet 1 --outputDir ~/results/PercentileRanks/

clusterOrder <- c(2,4,1,3)

consensusPCs <- read.csv(getKmerFreqsFilename(1,dataset="Consensus"),row.names=1)

surrogateSetNum <- 1
numSets <- 100
consensusPRs_STL <- list()
consensusPRs_LTS <- list()
consensusPRs_Combo <- list()
consensusPRs_Long <- list()
for (k in 2:3){
  prFilename <- getPRFilename(k,dataset="Consensus",sets=numSets,surrogateSet=surrogateSetNum,order="STL",baselineMethod=8,threshold="0.0",thresholdTypeString="Whole",directory=prDataDirectory,sourceTraversal="STL")
  consensusPRs_STL[[k]] <- read.csv(prFilename,stringsAsFactors=F,row.names=1)
  consensusPRs_STL[[k]] <- consensusPRs_STL[[k]][,colnames(consensusPRs_STL[[k]])!="N",drop=F]
  
  consensusPRs_Combo[[k]] <- consensusPRs_STL[[k]]
  
  if (k >= 3){
    # Read in LTS k-mers
    consensusPRs_LTS[[k]] <- read.csv(getPRFilename(k,dataset="Consensus",sets=numSets,surrogateSet=surrogateSetNum,order="LTS",baselineMethod=8,threshold="0.0",thresholdTypeString="Whole",directory=prDataDirectory,sourceTraversal="STL"),stringsAsFactors=F,row.names=1)
    consensusPRs_LTS[[k]] <- consensusPRs_LTS[[k]][,colnames(consensusPRs_LTS[[k]])!="N",drop=F]
    tmp <- consensusPRs_LTS[[k]]
    # For merging, get rid of LTS k-mers that have an STL counterpart
    inds <- grep("[ACT]C+[ACT]",colnames(tmp),perl=T)
    colnames(tmp) <- paste("L-",colnames(tmp),sep="")
    consensusPRs_Combo[[k]] <- cbind(consensusPRs_STL[[k]],tmp[,-inds])
    colnames(consensusPRs_Combo[[k]]) <- c(colnames(consensusPRs_STL[[k]]),colnames(tmp)[-inds])
    
    # Remove fully constrained (ATT related) kmers
    consensusPRs_Combo[[k]] <- consensusPRs_Combo[[k]][,removeATT("STL",colnames(consensusPRs_Combo[[k]]))]
  }
  
  consensusPRs_Combo[[k]]$cluster <- factor(paste("C",gsub("Cluster([0-9]+)T[0-9]+","\\1",rownames(consensusPRs_Combo[[k]])),sep=""),levels=paste("C",clusterOrder,sep=""))
  consensusPRs_Combo[[k]]$threshold <- gsub("Cluster[0-9]+T([0-9]+)","\\1",rownames(consensusPRs_Combo[[k]]))
  kmers <- colnames(consensusPRs_Combo[[k]])[!colnames(consensusPRs_Combo[[k]])%in%c("cluster","threshold")]
  consensusPRs_Long[[k]] <- reshape(consensusPRs_Combo[[k]],direction="long",varying=list(kmers),times=kmers,idvar="SeqName",ids=rownames(consensusPRs_Combo[[k]]),timevar="Kmer",v.names="PercentileRank")
}
if (exists("tmp")) rm(tmp)

consenususPCT50 <- c(0.481,0.5,0.393,0.30)
consenususPCT25 <- c(0.475,0.596,0.458,0.415)

consensusPC.frame <- data.frame(
  cluster=factor(paste("C",1:4,sep=""),levels=paste("C",clusterOrder,sep="")),
  T25PercentC=consenususPCT25,T50PercentC=consenususPCT50,kmer=rep("C",4))


## Plot raw %C
pcFilename <- getKmerFreqsFilename(1,dataset=dataset,travOrder="STL",threshold="0.0",thresholdTypeString="Whole",directory=kmerFreqDirectory,sourceTraversal="STL")
percentC <- read.csv(pcFilename,stringsAsFactors=F,row.names=1)
percentC$cluster <- factor("C1",levels=paste("C",clusterOrder,sep=""))
for (arborType in names(emResults)){
  percentC[names(emResults[[arborType]]$classification),"cluster"] <- factor(paste("C",emResults[[arborType]]$classification,sep=""),levels=paste("C",c(2,4,1,3),sep=""))
}
percentC$kmer <- "C"
percentC <- subset(percentC,!is.na(cluster))
axonPercentC <- percentC[rownames(percentC)%in%rownames(subset(usedMetaData,ArborType=="Axon"&ExperCond=="Control"&Protocol!="Acute culture"&Protocol!="Culture")),]

arborType <- "Axon"
#filename <- paste(figDir,"/PercentCPerCluster-",arborType,"-UpdatedClusterNums-K.png",sep="")
filename <- paste(figDir,"/PercentCPerCluster-",arborType,"-WithConsensusVals-UpdatedClusterNums.png",sep="")
barWidth <- 0.7
dodgeWidth <- 0.9
motifPlot <- ggplot(axonPercentC,aes(x=kmer,y=C,fill=cluster,group=cluster,width=barWidth))+
  stat_summary(fun.y=mean, geom="bar",colour="white",position=position_dodge(width = dodgeWidth))+
  stat_summary(fun.y=mean, fun.ymin=lowsem, fun.ymax=highsem,position=position_dodge(width = dodgeWidth), geom="linerange",color = 'black')+
  scale_fill_manual(values=barColors)+
  paper2Theme + theme(panel.grid.major.y=element_line(size=0.7, colour="#f0f0f0", linetype="solid"))
# Add in consensus values
motifPlot <- motifPlot + geom_bar(aes(x=kmer,y=PercentC,group=cluster,fill=cluster),data=consensusPC.frame,position=position_dodge(width = dodgeWidth),stat="identity")
ggsave(filename,motifPlot,width=12,height=8)

#filename <- paste(figDir,"/PercentCPerCluster-Zoom-",arborType,"-UpdatedClusterNums.png",sep="")
filename <- paste(figDir,"/PercentCPerCluster-Zoom-",arborType,"-WithConsensusVals-UpdatedClusterNums.png",sep="")
barWidth <- 0.7
dodgeWidth <- 0.9
motifPlot <- ggplot(axonPercentC,aes(x=kmer,y=C,fill=cluster,group=cluster,width=barWidth))+
  stat_summary(fun.y=mean, geom="bar",colour="white",position=position_dodge(width = dodgeWidth))+
  stat_summary(fun.y=mean, fun.ymin=lowsem, fun.ymax=highsem,position=position_dodge(width = dodgeWidth), geom="linerange",color = 'black')+
  scale_fill_manual(values=barColors)+
#  coord_cartesian(ylim = c(.3, .62))+
  coord_cartesian(ylim = c(.2, .62))+
  paper2Theme + theme(panel.grid.major.y=element_line(size=0.7, colour="#f0f0f0", linetype="solid"))
# Add in consensus values
motifPlot <- motifPlot + geom_bar(aes(x=kmer,y=T25PercentC,group=cluster,fill=NA,colour=NA),colour="black",data=consensusPC.frame,width=barWidth-.02,position=position_dodge(width = dodgeWidth),stat="identity",size=1,lty=1)
motifPlot <- motifPlot + geom_bar(aes(x=kmer,y=T50PercentC,group=cluster,fill=NA,colour=NA),colour="black",data=consensusPC.frame,width=barWidth-.32,position=position_dodge(width = dodgeWidth),stat="identity",size=0.4,lty=1)
ggsave(filename,motifPlot,width=12,height=8)

# Select Dimers
arborType <- "Axon"
arborName <- arborNamesByType[arborType]
# For just cluster 1 and 3 (3 and 4 in figure)
veryDifferentDimers <- c("AA","CA")
#selectTrimers <- veryDifferentTrimers
selectDimers <- c("AA","TA","CA")
selectClusters <- c(1,3)
k <- 2

barWidth <- 0.6
dodgeWidth <- 0.7

prArborData <- subset(prDataB8Long[[k]],SeqName%in%rownames(getSubset2(prDataB8[[k]],list(ArborType=arborType,ExperCond="Control"),usedMetaData)))
prArborData$cluster <- factor(prArborData$cluster,levels=paste("C",clusterOrder,sep=""))
if (!is.factor(selectClusters)) selectClusters <- factor(paste("C",selectClusters,sep=""),levels=levels(prArborData$cluster))
prArborData <- subset(prArborData,Kmer%in%selectDimers&cluster%in%selectClusters)
filename <- paste(figDir,"/MotifsPerCluster_SelectiveByClusterAndDimer-",arborName,"-K",k,".png",sep="")

#filename <- paste(figDir,"/MotifsPerCluster_Clusters",paste(selectClusters,collapse="-"),"_StoryDimers-",arborName,"-K",k,".png",sep="")
#filename <- paste(figDir,"/MotifsPerCluster_Clusters",paste(selectClusters,collapse="-"),"_Dimers",arborName,".png",sep="")
filename <- paste(figDir,"/MotifsPerCluster_WithConsensus_Clusters",paste(selectClusters,collapse="-"),"_StoryDimers-",arborName,"-K",k,".png",sep="")
prArborData <- subset(prArborData,Kmer%in%selectDimers&cluster%in%selectClusters)
#filename <- paste(figDir,"/MotifsPerCluster_SelectiveByClusterAndStoryDimers-",arborName,"-K",k,".png",sep="")
motifPlot <- ggplot(prArborData,aes(x=Kmer,y=PercentileRank,fill=cluster,group=cluster,width=barWidth))+
  stat_summary(fun.y=mean, geom="bar",colour="white",position=position_dodge(width = dodgeWidth))+
  stat_summary(fun.y=mean, fun.ymin=lowsem, fun.ymax=highsem,position=position_dodge(width = dodgeWidth), geom="linerange",color = 'black')+
  scale_fill_manual(values=barColors)+
  paper2Theme + theme(axis.text=element_text(size=13),panel.grid.major.y=element_line(size=0.7, colour="#f0f0f0", linetype="solid"))
# Add in consensus values
motifPlot <- motifPlot + geom_bar(aes(x=Kmer,y=PercentileRank,fill=NA,group=cluster),colour="black",data=subset(consensusPRs_Long[[2]],threshold==50),width=barWidth-.02,position=position_dodge(width = dodgeWidth),stat="identity",size=1,lty=1)
motifPlot <- motifPlot + geom_bar(aes(x=Kmer,y=PercentileRank,fill=NA,group=cluster),colour="black",data=subset(consensusPRs_Long[[2]],threshold==25),width=barWidth-.32,position=position_dodge(width = dodgeWidth),stat="identity",size=0.4,lty=1)
ggsave(filename,motifPlot,width=12,height=8)


# Select trimers
arborType <- "Axon"
arborName <- arborNamesByType[arborType]
# For all 4 clusters
allClusterTrimers <- c("AAA","AAT","ACA","ACC","ACT","CAA","CAT","CTC","L-CAC","L-TAA","L-TAC","L-TAT","TCC","TCT")
# For just cluster 1 and 3 (3 and 4 in figure)
slightlyDifferentTrimers <- c("CCC","CCT","TCC") # Also L-TAC
moderatelyDifferentTrimers <- c("AAA","AAC","ACA","ACT","ATA","ATC","CTA","CTC","TCA") # Also L-CAT, L-TAT
veryDifferentTrimers <- c("AAT","ACC","CAA","CAT","CCA","L-CAC","L-TAA","TCT") # Also L-CAA, 

storyTrimers <- c("AAT","CAT","CAA","L-CAC","ACA","ACC")
#selectTrimers <- storyTrimers
selectTrimers <- c(moderatelyDifferentTrimers,veryDifferentTrimers)

selectClusters <- c(1,3)
k <- 3
prArborData <- subset(prDataB8Long[[k]],SeqName%in%rownames(getSubset2(prDataB8[[k]],list(ArborType=arborType,ExperCond="Control"),usedMetaData)))
if (!is.factor(selectClusters)) selectClusters <- factor(paste("C",selectClusters,sep=""),levels=levels(prArborData$cluster))
prArborData$cluster <- factor(prArborData$cluster,levels=c("C2","C4","C1","C3"))

prArborData <- subset(prArborData,Kmer%in%selectTrimers&cluster%in%selectClusters)
#filename <- paste(figDir,"/MotifsPerCluster_SelectiveByClusterAndTrimer-",arborName,"-K",k,".png",sep="")
#filename <- paste(figDir,"/MotifsPerCluster_Clusters",paste(selectClusters,collapse=""),"-SelectiveByTrimer-",arborName,"-K",k,".png",sep="")
filename <- paste(figDir,"/MotifsPerCluster_MostTrimers-",arborName,"-K",k,".png",sep="")
#filename <- paste(figDir,"/MotifsPerCluster_SelectiveByClusterAndStoryTrimers-",arborName,"-K",k,".png",sep="")
#prArborData <- subset(prArborData,cluster%in%selectClusters)
#filename <- paste(figDir,"/MotifsPerCluster_SelectiveByCluster-",arborName,"-K",k,".png",sep="")
#  prArborData$cluster <- factor(emResults[[arborType]]$classification[prArborData$SeqName])
dodgeWidth <- 0.8
motifPlot <- ggplot(prArborData,aes(x=Kmer,y=PercentileRank,fill=cluster,group=cluster,width=barWidth))+
  stat_summary(fun.y=mean, geom="bar",colour="white",position=position_dodge(width = dodgeWidth))+
  stat_summary(fun.y=mean, fun.ymin=lowsem, fun.ymax=highsem,position=position_dodge(width = dodgeWidth), geom="linerange",color = 'black')+
  scale_fill_manual(values=barColors)+
  paper2Theme + theme(axis.text=element_text(size=12),panel.grid.major.y=element_line(size=0.7, colour="#f0f0f0", linetype="solid"))
ggsave(filename,motifPlot,width=12,height=8)

## Alternate with Cluster 3 and All Axons
# create dataset copy and duplicate all rows from cluster 3, giving them class of "All Arbors"
if (!"All Arbors"%in%names(barColors)){
  barColors <- c(barColors,"#555555")
  names(barColors)[length(barColors)] <- "All Arbors"
}

k <- 2
prArborData <- subset(prDataB8Long[[k]],SeqName%in%rownames(getSubset2(prDataB8[[k]],list(ArborType=arborType,ExperCond="Control"),usedMetaData)))
prArborData$cluster <- factor(as.character(prArborData$cluster),levels=c(levels(prArborData$cluster),"All Arbors"))
prArborData[prArborData$cluster!="C3","cluster"] <- "All Arbors"
prArborClust3Copy <- subset(prArborData,cluster=="C3")
prArborClust3Copy$cluster <- rep("All Arbors",nrow(prArborClust3Copy))
prArborData <- rbind(prArborData,prArborClust3Copy)

# Dimers
barWidth <- 0.6
dodgeWidth <- 0.7

prArborData <- subset(prDataB8Long[[k]],SeqName%in%rownames(getSubset2(prDataB8[[k]],list(ArborType=arborType,ExperCond="Control"),usedMetaData)))
prArborData <- subset(prArborData,Kmer%in%selectDimers)
motifPlot <- ggplot(prArborData,aes(x=Kmer,y=PercentileRank,fill=cluster,group=cluster,width=barWidth))+
  stat_summary(fun.y=mean, geom="bar",colour="white",position=position_dodge(width = dodgeWidth))+
  stat_summary(fun.y=mean, fun.ymin=lowsem, fun.ymax=highsem,position=position_dodge(width = dodgeWidth), geom="linerange",color = 'black')+
  scale_fill_manual(values=barColors)+
  paper2Theme + theme(panel.grid.major.y=element_line(size=0.7, colour="#f0f0f0", linetype="solid"))
filename <- paste(figDir,"/MotifsPerCluster_Cluster3vsAllAxons-StoryDimers-K",k,".png",sep="")
ggsave(filename,motifPlot,width=12,height=8)

# ## Dimers of clusters 1 and 4 (2 and 3 originally)
# prArborData <- subset(prDataB8Long[[k]],SeqName%in%rownames(getSubset2(prDataB8[[k]],list(ArborType=arborType,ExperCond="Control"),usedMetaData)))
# prArborData$cluster <- factor(as.character(prArborData$cluster),levels=c(levels(prArborData$cluster),"All Arbors"))
# prArborData <- subset(prArborData,cluster%in%c("C2","C3"))

## Dimers of clusters 1, 3, and 4 (2, 1, and 3 originally)
prArborData <- subset(prDataB8Long[[2]],!is.na(PercentileRank)&SeqName%in%rownames(getSubset2(prDataB8[[2]],list(ArborType=arborType,ExperCond="Control"),usedMetaData)))
prArborData$cluster <- factor(as.character(prArborData$cluster),levels=c("C2","C4","C1","C3","All Arbors"))
prArborData <- subset(prArborData,cluster%in%c("C2", "C1", "C3"))

# Order kmers by mean PR
kmerMeans <- with(prArborData,aggregate(PercentileRank,by=list(Kmer=Kmer),mean,na.rm=T))
prArborData$Kmer <- factor(prArborData$Kmer,levels=kmerMeans$Kmer[order(kmerMeans$x)])

# Dimers
barWidth <- 0.6
dodgeWidth <- 0.7

motifPlot <- ggplot(prArborData,aes(x=Kmer,y=PercentileRank,fill=cluster,group=cluster,width=barWidth))+
  stat_summary(fun.y=mean, geom="bar",colour="white",position=position_dodge(width = dodgeWidth))+
  stat_summary(fun.y=mean, fun.ymin=lowsem, fun.ymax=highsem,position=position_dodge(width = dodgeWidth), geom="linerange",color = 'black')+
  scale_fill_manual(values=barColors)+
  paper2Theme + theme(panel.grid.major.y=element_line(size=0.7, colour="#f0f0f0", linetype="solid"))
filename <- paste(figDir,"/MotifsPerCluster_Clusters12and3-Dimers.png",sep="")
ggsave(filename,motifPlot,width=12,height=8)


# Trimers
k <- 3
prArborData <- subset(prDataB8Long[[k]],SeqName%in%rownames(getSubset2(prDataB8[[k]],list(ArborType=arborType,ExperCond="Control"),usedMetaData)))
prArborData$cluster <- factor(as.character(prArborData$cluster),levels=c(levels(prArborData$cluster),"All Arbors"))
prArborData[prArborData$cluster!="C3","cluster"] <- "All Arbors"
prArborClust3Copy <- subset(prArborData,cluster=="C3")
prArborClust3Copy$cluster <- rep("All Arbors",nrow(prArborClust3Copy))
prArborData <- rbind(prArborData,prArborClust3Copy)

prArborData <- subset(prArborData,Kmer%in%selectTrimers)
motifPlot <- ggplot(prArborData,aes(x=Kmer,y=PercentileRank,fill=cluster,group=cluster,width=barWidth))+
  stat_summary(fun.y=mean, geom="bar",colour="white",position=position_dodge(width = dodgeWidth))+
  stat_summary(fun.y=mean, fun.ymin=lowsem, fun.ymax=highsem,position=position_dodge(width = dodgeWidth), geom="linerange",color = 'black')+
  scale_fill_manual(values=barColors)+
  paper2Theme + theme(panel.grid.major.y=element_line(size=0.7, colour="#f0f0f0", linetype="solid"))
filename <- paste(figDir,"/MotifsPerCluster_Cluster3vsAllAxons-StoryTrimers-K",k,".png",sep="")
ggsave(filename,motifPlot,width=12,height=8)

## Dimers and Trimers - Show lack of consensus motif match with cluster motifs
jointKmers <- rbind(
  subset(prDataB8Long[[2]],SeqName%in%rownames(getSubset2(prDataB8[[2]],list(ArborType=arborType,ExperCond="Control"),usedMetaData))),
  subset(prDataB8Long[[3]],SeqName%in%rownames(getSubset2(prDataB8[[3]],list(ArborType=arborType,ExperCond="Control"),usedMetaData))))

# Run stats to get means and SEMs
kmerMeansByCluster <- with(jointKmers,aggregate(PercentileRank,by=list(cluster=cluster,Kmer=Kmer),mean))
colnames(kmerMeansByCluster)[3] <- "NeuritesPR"
kmerMeansByCluster[,"ConsensusPR"] <- rep(0,nrow(kmerMeansByCluster))
rownames(kmerMeansByCluster) <- paste(as.character(kmerMeansByCluster$cluster),as.character(kmerMeansByCluster$Kmer))

t50Kmers <- rbind(subset(consensusPRs_Long[[2]],threshold=="50"),
                  subset(consensusPRs_Long[[3]],threshold=="50"))
rownames(t50Kmers) <- paste(as.character(t50Kmers$cluster),as.character(t50Kmers$Kmer))

kmerMeansByCluster[,"ConsensusPR"] <- t50Kmers[rownames(kmerMeansByCluster),"PercentileRank"]

#neurConsPRfit <- lm(ConsensusPR ~ NeuritesPR*cluster,kmerMeansByCluster)
neurConsPRfit <- lm(ConsensusPR ~ NeuritesPR,kmerMeansByCluster)
neurConsPRfit1 <- lm(ConsensusPR ~ NeuritesPR,subset(kmerMeansByCluster,cluster=="C1"))
neurConsPRfit2 <- lm(ConsensusPR ~ NeuritesPR,subset(kmerMeansByCluster,cluster=="C2"))
neurConsPRfit3 <- lm(ConsensusPR ~ NeuritesPR,subset(kmerMeansByCluster,cluster=="C3"))
neurConsPRfit4 <- lm(ConsensusPR ~ NeuritesPR,subset(kmerMeansByCluster,cluster=="C4"))
neurConsPRfitNot1 <- lm(ConsensusPR ~ NeuritesPR,subset(kmerMeansByCluster,cluster!="C1"))

scatplot <- ggplot(kmerMeansByCluster,aes(x=NeuritesPR,y=ConsensusPR,group=cluster,colour=cluster))+geom_point()+
  geom_abline(intercept = neurConsPRfit$coefficients[1],
              slope = neurConsPRfit$coefficients[2])+
  scale_colour_manual(values=axonColors)+paper2Theme+
  theme(panel.grid.major.x = element_line(colour = "grey70",linetype=2),
        panel.grid.major.y = element_line(colour = "grey70",linetype=2))
scatplot <- scatplot + geom_abline(colour=axonColors["C1"],intercept = neurConsPRfit1$coefficients[1],slope = neurConsPRfit1$coefficients[2])
scatplot <- scatplot + geom_abline(colour=axonColors["C2"],intercept = neurConsPRfit2$coefficients[1],slope = neurConsPRfit2$coefficients[2])
scatplot <- scatplot + geom_abline(colour=axonColors["C3"],intercept = neurConsPRfit3$coefficients[1],slope = neurConsPRfit3$coefficients[2])
scatplot <- scatplot + geom_abline(colour=axonColors["C4"],intercept = neurConsPRfit4$coefficients[1],slope = neurConsPRfit4$coefficients[2])
ggsave(paste(figDir,"ClusterKmers-NeuritesVsConsensuses-FitEach.png",sep=""),scatplot)

neurConsDimerPRfit <- lm(ConsensusPR ~ NeuritesPR,subset(kmerMeansByCluster,nchar(Kmer)==2))

scatplot <- ggplot(subset(kmerMeansByCluster,nchar(Kmer)==2),aes(x=NeuritesPR,y=ConsensusPR,group=cluster,colour=cluster))+geom_point()+
  geom_abline(intercept = neurConsPRfit$coefficients[1],
              slope = neurConsPRfit$coefficients[2])+
  scale_colour_manual(values=axonColors)+paper2Theme+
  theme(panel.grid.major.x = element_line(colour = "grey70",linetype=2),
        panel.grid.major.y = element_line(colour = "grey70",linetype=2))
scatplot <- scatplot + geom_abline(colour=axonColors["C1"],intercept = neurConsDimerPRfit$coefficients[1],slope = neurConsDimerPRfit$coefficients[2])
scatplot <- scatplot + geom_abline(colour=axonColors["C2"],intercept = neurConsDimerPRfit$coefficients[1],slope = neurConsDimerPRfit$coefficients[2])
scatplot <- scatplot + geom_abline(colour=axonColors["C3"],intercept = neurConsDimerPRfit$coefficients[1],slope = neurConsDimerPRfit$coefficients[2])
scatplot <- scatplot + geom_abline(colour=axonColors["C4"],intercept = neurConsDimerPRfit$coefficients[1],slope = neurConsDimerPRfit$coefficients[2])
ggsave(paste(figDir,"ClusterDimers-NeuritesVsConsensuses-FitEach.png",sep=""),scatplot)

# Test overall correlation and correlation for each cluster
cor.test(kmerMeansByCluster$NeuritesPR,kmerMeansByCluster$ConsensusPR)
with(subset(kmerMeansByCluster,cluster=="C1"),cor.test(NeuritesPR,ConsensusPR))
with(subset(kmerMeansByCluster,cluster=="C2"),cor.test(NeuritesPR,ConsensusPR))
with(subset(kmerMeansByCluster,cluster=="C3"),cor.test(NeuritesPR,ConsensusPR))
with(subset(kmerMeansByCluster,cluster=="C4"),cor.test(NeuritesPR,ConsensusPR))
with(subset(kmerMeansByCluster,cluster!="C1"),cor.test(NeuritesPR,ConsensusPR))

## Dimers and Trimers - Show correlation of clusters across kmers

# Test correlations between clusters
kmers <- unique(kmerMeansByCluster$Kmer)
cor.test(kmerMeansByCluster[paste("C2",kmers),"NeuritesPR"],kmerMeansByCluster[paste("C3",kmers),"NeuritesPR"])
cor.test(kmerMeansByCluster[paste("C2",kmers),"NeuritesPR"],kmerMeansByCluster[paste("C4",kmers),"NeuritesPR"])
cor.test(kmerMeansByCluster[paste("C3",kmers),"NeuritesPR"],kmerMeansByCluster[paste("C4",kmers),"NeuritesPR"])




## Generate consensus stats and dendrograms of target surrogate clusters
# Script generating 20 surrogate sets per cluster, then running MSA on each, producing stats on all and newick file for 1

#python3 capellini.py -build /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_SurrogateConstraint0-1of100_T0.5Sub120I.xml -newick /home/tgillett/Domains/consensuses/Consensus-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_SurrogateConstraint0-1of100_T0.5Sub120I.newick --threshold 0.5
#python3 capellini.py -build /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust3-Zoom-DistLim1_SurrogateConstraint0-1of100_T0.5Sub120I.xml -newick /home/tgillett/Domains/consensuses/Consensus-Axon-G3E1-Dims2of7-VII4-Clust3-Zoom-DistLim1_SurrogateConstraint0-1of100_T0.5Sub120I.newick --threshold 0.5
#python3 capellini.py -build /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_SurrogateConstraint3-1of100_T0.5Sub120I.xml -newick /home/tgillett/Domains/consensuses/Consensus-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_SurrogateConstraint3-1of100_T0.5Sub120I.newick --threshold 0.5
#python3 capellini.py -build /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust3-Zoom-DistLim1_SurrogateConstraint3-1of100_T0.5Sub120I.xml -newick /home/tgillett/Domains/consensuses/Consensus-Axon-G3E1-Dims2of7-VII4-Clust3-Zoom-DistLim1_SurrogateConstraint3-1of100_T0.5Sub120I.newick --threshold 0.5

conservedColor <- clusterColors["C1"]

consensusName <- "Consensus-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_SurrogateConstraint0-1of100_T0.5Sub120I"
shortName <- "Consensus-Axon-Clust1_SurrogateC0-1of100_T0.5Sub120I"
consensusTitle <- "Axon Cluster 1 0-Surrogate Threshold 50%"

consensusName <- "Consensus-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_SurrogateConstraint3-1of100_T0.5Sub120I"
shortName <- "Consensus-Axon-Clust1_SurrogateC3-1of100_T0.5Sub120I"
consensusTitle <- "Axon Cluster 1 3-Surrogate Threshold 50%"


conservedColor <- clusterColors["C3"]

consensusName <- "Consensus-Axon-G3E1-Dims2of7-VII4-Clust3-Zoom-DistLim1_SurrogateConstraint0-1of100_T0.5Sub120I"
shortName <- "Consensus-Axon-Clust3_SurrogateC0-1of100_T0.5Sub120I"
consensusTitle <- "Axon Cluster 3 0-Surrogate Threshold 50%"

consensusName <- "Consensus-Axon-G3E1-Dims2of7-VII4-Clust3-Zoom-DistLim1_SurrogateConstraint3-1of100_T0.5Sub120I"
shortName <- "Consensus-Axon-Clust3_SurrogateC3-1of100_T0.5Sub120I"
consensusTitle <- "Axon Cluster 3 3-Surrogate Threshold 50%"


conservedColor <- clusterColors["C2"]
consensusName <- "Consensus-Axon-G3E1-Dims2of7-VII4-Clust2-Zoom-DistLim1_SurrogateConstraint3-1of100_T0.5Sub120I"
shortName <- "Consensus-Axon-Clust2_SurrogateC3-1of100_T0.5Sub120I"
consensusTitle <- "Axon Cluster 2 3-Surrogate Threshold 50%"

conservedColor <- clusterColors["C4"]
consensusName <- "Consensus-Axon-G3E1-Dims2of7-VII4-Clust4-Zoom-DistLim1_SurrogateConstraint3-1of100_T0.5Sub120I"
shortName <- "Consensus-Axon-Clust4_SurrogateC3-1of100_T0.5Sub120I"
consensusTitle <- "Axon Cluster 4 3-Surrogate Threshold 50%"

consensusTree <- read.tree(paste(consensusDir,consensusName,".newick",sep=""))

# Color Down
png(paste(figDir,shortName,"-CD.png",sep=""),width=800,height=600)
par(mar=margins)
plotConsensusDendrogram(consensusTree,treePalette=conservedColor,colorDown=T,main=consensusTitle,edge.width=3)
dev.off()

# Color applied to branch above bifurcation (original)
#png(paste(figDir,shortName,".png",sep=""),width=800,height=600)
png(paste(figDir,shortName,".png",sep=""),width=800,height=600)
par(mar=margins)
plotConsensusDendrogram(consensusTree,treePalette=conservedColor,colorDown=F,main=consensusTitle,edge.width=4)
dev.off()

# Branch widths for conservation, no color, "color" down
consensusTree <- read.tree(paste(consensusDir,consensusName,".newick",sep=""))
png(paste(figDir,shortName,"-ClusterColor-CD-BranchWidthScaled.png",sep=""),width=800,height=600)
par(mar=margins)
plotConsensusDendrogram(consensusTree,primaryColor=conservedColor,secondaryColor=conservedColor,colorDown=T,modulateEdgeWidth=T,main=consensusTitle,edge.width=4,colorLegend=F,rescaleConservation=T)
dev.off()

#### Figure 6-associated: global alignment against consensus ####

# Create surrogate sets and then run MSA on them
java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.CreateGroupFastaFile -f "/home/tgillett/fastaFromSwc/NM_SpurLen2D/Random/Method8/Mapping2/surrogatesSTL-100sets_B8_Set1.dat" -g "/home/tgillett/Domains/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1.txt" -o "/home/tgillett/Domains/ClusterLists/" --surrogateNum 1 --constraint 0 --numSurrogates 100
java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.CreateGroupFastaFile -f "/home/tgillett/fastaFromSwc/NM_SpurLen2D/Random/Method8/Mapping2/surrogatesSTL-100sets_B8_Set1.dat" -g "/home/tgillett/Domains/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1.txt" -o "/home/tgillett/Domains/ClusterLists/" --surrogateNum 1 --constraint 3 --numSurrogates 100
java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.CreateGroupFastaFile -f "/home/tgillett/fastaFromSwc/NM_SpurLen2D/Random/Method8/Mapping2/surrogatesSTL-100sets_B8_Set1.dat" -g "/home/tgillett/Domains/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust2-Zoom-DistLim1.txt" -o "/home/tgillett/Domains/ClusterLists/" --surrogateNum 1 --constraint 0 --numSurrogates 100 &
java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.CreateGroupFastaFile -f "/home/tgillett/fastaFromSwc/NM_SpurLen2D/Random/Method8/Mapping2/surrogatesSTL-100sets_B8_Set1.dat" -g "/home/tgillett/Domains/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust2-Zoom-DistLim1.txt" -o "/home/tgillett/Domains/ClusterLists/" --surrogateNum 1 --constraint 3 --numSurrogates 100 &
java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.CreateGroupFastaFile -f "/home/tgillett/fastaFromSwc/NM_SpurLen2D/Random/Method8/Mapping2/surrogatesSTL-100sets_B8_Set1.dat" -g "/home/tgillett/Domains/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1.txt" -o "/home/tgillett/Domains/ClusterLists/" --surrogateNum 1 --constraint 0 --numSurrogates 100
java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.CreateGroupFastaFile -f "/home/tgillett/fastaFromSwc/NM_SpurLen2D/Random/Method8/Mapping2/surrogatesSTL-100sets_B8_Set1.dat" -g "/home/tgillett/Domains/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1.txt" -o "/home/tgillett/Domains/ClusterLists/" --surrogateNum 1 --constraint 3 --numSurrogates 100
java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.CreateGroupFastaFile -f "/home/tgillett/fastaFromSwc/NM_SpurLen2D/Random/Method8/Mapping2/surrogatesSTL-100sets_B8_Set1.dat" -g "/home/tgillett/Domains/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust4-Zoom-DistLim1.txt" -o "/home/tgillett/Domains/ClusterLists/" --surrogateNum 1 --constraint 0 --numSurrogates 100 &
java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.CreateGroupFastaFile -f "/home/tgillett/fastaFromSwc/NM_SpurLen2D/Random/Method8/Mapping2/surrogatesSTL-100sets_B8_Set1.dat" -g "/home/tgillett/Domains/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust4-Zoom-DistLim1.txt" -o "/home/tgillett/Domains/ClusterLists/" --surrogateNum 1 --constraint 3 --numSurrogates 100 &

python3 penne.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1_SurrogateConstraint0-1of100.fasta -build /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120I_SurrogateConstraint0-1of100.xml --threshold 0.5 --subsample 120 --iterate 0 -n 6 &
python3 penne.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1_SurrogateConstraint3-1of100.fasta -build /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120I_SurrogateConstraint3-1of100.xml --threshold 0.5 --subsample 120 --iterate 0 -n 6 &
python3 penne.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust2-Zoom-DistLim1_SurrogateConstraint0-1of100.fasta -build /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust2-Zoom-DistLim1_T0.5Sub120I_SurrogateConstraint0-1of100.xml --threshold 0.5 --subsample 120 --iterate 0 -n 6 &
python3 penne.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust2-Zoom-DistLim1_SurrogateConstraint3-1of100.fasta -build /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust2-Zoom-DistLim1_T0.5Sub120I_SurrogateConstraint3-1of100.xml --threshold 0.5 --subsample 120 --iterate 0 -n 6 &
python3 penne.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1_SurrogateConstraint0-1of100.fasta -build /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120I_SurrogateConstraint0-1of100.xml --threshold 0.5 --subsample 120 --iterate 0 -n 6 &
python3 penne.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1_SurrogateConstraint3-1of100.fasta -build /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120I_SurrogateConstraint3-1of100.xml --threshold 0.5 --subsample 120 --iterate 0 -n 6 &
python3 penne.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust4-Zoom-DistLim1_SurrogateConstraint0-1of100.fasta -build /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust4-Zoom-DistLim1_T0.5Sub120I_SurrogateConstraint0-1of100.xml --threshold 0.5 --subsample 120 --iterate 0 -n 6 &
python3 penne.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust4-Zoom-DistLim1_SurrogateConstraint3-1of100.fasta -build /home/tgillett/Domains/consensuses/MSA-Axon-G3E1-Dims2of7-VII4-Clust4-Zoom-DistLim1_T0.5Sub120I_SurrogateConstraint3-1of100.xml --threshold 0.5 --subsample 120 --iterate 0 -n 6 &

# Create consensus fasta files from MSAs of real and surrogate sets -> /home/tgillett/Domains/Consensus-<rest of filename>.fasta
head -n 10 MSA-Axon-G3E1-Dims2of7-VII4-Clust2-Zoom-DistLim1_T0.5Sub120I_SurrogateConstraint0-1of100.xml | tail -n 1
head -n 10 MSA-Axon-G3E1-Dims2of7-VII4-Clust2-Zoom-DistLim1_T0.5Sub120I_SurrogateConstraint3-1of100.xml | tail -n 1
head -n 10 MSA-Axon-G3E1-Dims2of7-VII4-Clust4-Zoom-DistLim1_T0.5Sub120I_SurrogateConstraint0-1of100.xml | tail -n 1
head -n 10 MSA-Axon-G3E1-Dims2of7-VII4-Clust4-Zoom-DistLim1_T0.5Sub120I_SurrogateConstraint3-1of100.xml | tail -n 1

# Run cluster set fasta file against surrogate consensus fasta file, and vice versa
# # Cluster 1 actual vs trimer surrogates, within and between; GAP OPEN = 0
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G 0 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust1-vs-Clust1ConsT0.5Sub120.tab
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1_SurrogateConstraint3-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G 0 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust1TrimerSurrogates-vs-Clust1ConsT0.5Sub120.tab
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint3-1of100.fasta -E -1 -G 0 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust1-vs-Clust1TrimerSurrogatesConsT0.5Sub120.tab
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1_SurrogateConstraint3-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint3-1of100.fasta -E -1 -G 0 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust1TrimerSurrogates-vs-Clust1TrimerSurrogatesConsT0.5Sub120.tab
# # Cluster 1 actual vs trimer surrogates, within and between; GAP OPEN = -3
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G -3 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G3E1Identity_Dims2of7-VII4-Clust1-vs-Clust1ConsT0.5Sub120.tab
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1_SurrogateConstraint3-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G -3 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G3E1Identity_Dims2of7-VII4-Clust1TrimerSurrogates-vs-Clust1ConsT0.5Sub120.tab
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint3-1of100.fasta -E -1 -G -3 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G3E1Identity_Dims2of7-VII4-Clust1-vs-Clust1TrimerSurrogatesConsT0.5Sub120.tab
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1_SurrogateConstraint3-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint3-1of100.fasta -E -1 -G -3 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G3E1Identity_Dims2of7-VII4-Clust1TrimerSurrogates-vs-Clust1TrimerSurrogatesConsT0.5Sub120.tab
# # Cluster 1 actual vs length surrogates, within and between; GAP OPEN = 0
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G 0 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust1-vs-Clust1ConsT0.5Sub120.tab
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1_SurrogateConstraint0-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G 0 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust1LengthSurrogates-vs-Clust1ConsT0.5Sub120.tab
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint0-1of100.fasta -E -1 -G 0 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust1-vs-Clust1LengthSurrogatesConsT0.5Sub120.tab
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1_SurrogateConstraint0-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint0-1of100.fasta -E -1 -G 0 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust1LengthSurrogates-vs-Clust1LengthSurrogatesConsT0.5Sub120.tab
# # Cluster 1 actual vs length surrogates, within and between; GAP OPEN = -3
python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G -3 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G3E1Identity_Dims2of7-VII4-Clust1-vs-Clust1ConsT0.5Sub120.tab
python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1_SurrogateConstraint0-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G -3 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G3E1Identity_Dims2of7-VII4-Clust1LengthSurrogates-vs-Clust1ConsT0.5Sub120.tab
python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint0-1of100.fasta -E -1 -G -3 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G3E1Identity_Dims2of7-VII4-Clust1-vs-Clust1LengthSurrogatesConsT0.5Sub120.tab
python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1_SurrogateConstraint0-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust1-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint0-1of100.fasta -E -1 -G -3 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G3E1Identity_Dims2of7-VII4-Clust1LengthSurrogates-vs-Clust1LengthSurrogatesConsT0.5Sub120.tab

# # Cluster 3 actual vs trimer surrogates, within and between; GAP OPEN = 0
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G 0 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust3-vs-Clust3ConsT0.5Sub120.tab
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1_SurrogateConstraint3-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G 0 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust3TrimerSurrogates-vs-Clust3ConsT0.5Sub120.tab
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint3-1of100.fasta -E -1 -G 0 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust3-vs-Clust3TrimerSurrogatesConsT0.5Sub120.tab
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1_SurrogateConstraint3-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint3-1of100.fasta -E -1 -G 0 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust3TrimerSurrogates-vs-Clust3TrimerSurrogatesConsT0.5Sub120.tab
# # Cluster 3 actual vs trimer surrogates, within and between; GAP OPEN = -3
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G -3 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G3E1Identity_Dims2of7-VII4-Clust3-vs-Clust3ConsT0.5Sub120.tab
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1_SurrogateConstraint3-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G -3 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G3E1Identity_Dims2of7-VII4-Clust3TrimerSurrogates-vs-Clust3ConsT0.5Sub120.tab
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint3-1of100.fasta -E -1 -G -3 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G3E1Identity_Dims2of7-VII4-Clust3-vs-Clust3TrimerSurrogatesConsT0.5Sub120.tab
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1_SurrogateConstraint3-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint3-1of100.fasta -E -1 -G -3 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G3E1Identity_Dims2of7-VII4-Clust3TrimerSurrogates-vs-Clust3TrimerSurrogatesConsT0.5Sub120.tab
# # Cluster 3 actual vs length surrogates, within and between; GAP OPEN = 0
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G 0 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust3-vs-Clust3ConsT0.5Sub120.tab
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1_SurrogateConstraint0-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G 0 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust3LengthSurrogates-vs-Clust3ConsT0.5Sub120.tab
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint0-1of100.fasta -E -1 -G 0 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust3-vs-Clust3LengthSurrogatesConsT0.5Sub120.tab
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1_SurrogateConstraint0-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint0-1of100.fasta -E -1 -G 0 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust3LengthSurrogates-vs-Clust3LengthSurrogatesConsT0.5Sub120.tab
# # Cluster 3 actual vs length surrogates, within and between; GAP OPEN = -3
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G -3 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G3E1Identity_Dims2of7-VII4-Clust3-vs-Clust3ConsT0.5Sub120.tab
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1_SurrogateConstraint0-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G -3 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G3E1Identity_Dims2of7-VII4-Clust3LengthSurrogates-vs-Clust3ConsT0.5Sub120.tab
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint0-1of100.fasta -E -1 -G -3 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G3E1Identity_Dims2of7-VII4-Clust3-vs-Clust3LengthSurrogatesConsT0.5Sub120.tab
# python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1_SurrogateConstraint0-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust3-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint0-1of100.fasta -E -1 -G -3 -n 12 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G3E1Identity_Dims2of7-VII4-Clust3LengthSurrogates-vs-Clust3LengthSurrogatesConsT0.5Sub120.tab

# Cluster 2 actual vs length surrogates, within and between; GAP OPEN = 0
python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust2-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust2-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G 0 -n 6 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust2-vs-Clust2ConsT0.5Sub120.tab
python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust2-Zoom-DistLim1_SurrogateConstraint0-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust2-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G 0 -n 6 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust2LengthSurrogates-vs-Clust2ConsT0.5Sub120.tab
python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust2-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust2-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint3-1of100.fasta -E -1 -G 0 -n 6 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust2-vs-Clust2LengthSurrogatesConsT0.5Sub120.tab
python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust2-Zoom-DistLim1_SurrogateConstraint0-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust2-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint0-1of100.fasta -E -1 -G 0 -n 6 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust2LengthSurrogates-vs-Clust2LengthSurrogatesConsT0.5Sub120.tab
# Cluster 2 actual vs trimer surrogates, within and between; GAP OPEN = 0
python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust2-Zoom-DistLim1_SurrogateConstraint3-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust2-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G 0 -n 6 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust2TrimerSurrogates-vs-Clust2ConsT0.5Sub120.tab
python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust2-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust2-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint3-1of100.fasta -E -1 -G 0 -n 6 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust2-vs-Clust2TrimerSurrogatesConsT0.5Sub120.tab
python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust2-Zoom-DistLim1_SurrogateConstraint3-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust2-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint3-1of100.fasta -E -1 -G 0 -n 6 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust2TrimerSurrogates-vs-Clust2TrimerSurrogatesConsT0.5Sub120.tab
# Cluster 4 actual vs length surrogates, within and between; GAP OPEN = 0
python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust4-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust4-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G 0 -n 6 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust4-vs-Clust4ConsT0.5Sub120.tab
python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust4-Zoom-DistLim1_SurrogateConstraint0-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust4-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G 0 -n 6 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust4LengthSurrogates-vs-Clust4ConsT0.5Sub120.tab
python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust4-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust4-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint3-1of100.fasta -E -1 -G 0 -n 6 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust4-vs-Clust4LengthSurrogatesConsT0.5Sub120.tab
python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust4-Zoom-DistLim1_SurrogateConstraint0-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust4-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint0-1of100.fasta -E -1 -G 0 -n 6 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust4LengthSurrogates-vs-Clust4LengthSurrogatesConsT0.5Sub120.tab
# Cluster 4 actual vs trimer surrogates, within and between; GAP OPEN = 0
python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust4-Zoom-DistLim1_SurrogateConstraint3-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust4-Zoom-DistLim1_T0.5Sub120.fasta -E -1 -G 0 -n 6 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust4TrimerSurrogates-vs-Clust4ConsT0.5Sub120.tab
python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust4-Zoom-DistLim1.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust4-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint3-1of100.fasta -E -1 -G 0 -n 6 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust4-vs-Clust4TrimerSurrogatesConsT0.5Sub120.tab
python3 spaghetti.py --custom /home/tgillett/PairwiseAlignment/identity.tab -f /home/tgillett/Domains/ClusterLists/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust4-Zoom-DistLim1_SurrogateConstraint3-1of100.fasta -f2 /home/tgillett/Domains/Consensus-Axon-Dims2of7-VII4-Clust4-Zoom-DistLim1_T0.5Sub120_SurrogateConstraint3-1of100.fasta -E -1 -G 0 -n 6 -o /home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust4TrimerSurrogates-vs-Clust4TrimerSurrogatesConsT0.5Sub120.tab

#source(paste(clusterGlobalDir,"BaselineHandler.r",sep=""))
#baselineKey <- paste("G3E1_e3",travOrder)
#baselineKey <- paste("G3E1_e5",travOrder)
#loadBaselineAlignmentStats("G3E1_e5 STL",directory=paste(baselineDir,"GapOpen3_e5/",sep=""),travOrder=travOrder)
#loadBaselineAlignmentStats("G0E1_e3 STL",directory=paste(baselineDir,"GapOpen0_1000/",sep=""),travOrder=travOrder)

clust1Cons <- "AATACCCTACTCCCCTACATCCTAACTCATCCTACCCTCACCTATATCCCCT"
clust1LengthSurrogateCons <- "CACCTCACACTCATCTCCAATCTCATAATCATCTCCACTATCATCCT"
clust1TrimerSurrogateCons <- "AACCTACCTATCCCCTCAATCACTCCCTCAATCCTACCCCTCACCTCCCCCT"

clust3Cons <- "AAACTACTATCCTATAATCCTACCTACTCCATCTAAACTCATCTAATCCCTACCTATCATCCCTAAATCCTACCTATCCCTACACTCACTCCTAATATCCTACCTACCTACTATCCT"
clust3LengthSurrogateCons <- "ATCACCCCTATACCACCATCTATCACCTCACTCATATCTCACTACATCCTCACTCACACCTCACCTCATCCTATCACATATCCTCATACATCCTCACCTATATATATCCT"
clust3TrimerSurrogateCons <- "AATAATATCCTACCTCACCTCCATCCTACCTAACATCCTAATCTCACTATCCTCAACCTACTATCCCTACACTATCCTAATCCTACATCCTACTCACTCCCCT"

clust2Cons <- "ATCACTCACCCTACTCACTCATCCCT"
clust2LengthSurrogateCons <- "ATAATCATTCATAATATCTCACTATCATCT"
clust2TrimerSurrogateCons <- "CATACATCCCCTCACCCTCACTCATCCCCT"

clust4Cons <- "ATAACTCATCTACCTAATCCTATACTATCT"
clust4LengthSurrogateCons <- "CATACATCTCCAATCTCACCTCATACTCCT"
clust4TrimerSurrogateCons <- "AATATCCTAATCCTAATCCTCACTATCCT"

arborType <- "Axon"
arborParams <- "Dims2of7-VII4"
consNum <- 1
clustNum <- 1

threshold <- "0.5"
subsetSize <- 120
#gapPenalty <- 0
consensusCompareDir <- "/home/tgillett/Domains/"

consensusLengths <- list()
consensusLengths[["clust1"]] <- nchar(clust1Cons)
consensusLengths[["clust1 0"]] <- nchar(clust1LengthSurrogateCons)
consensusLengths[["clust1 3"]] <- nchar(clust1TrimerSurrogateCons)
consensusLengths[["clust3"]] <- nchar(clust3Cons)
consensusLengths[["clust3 0"]] <- nchar(clust3LengthSurrogateCons)
consensusLengths[["clust3 3"]] <- nchar(clust3TrimerSurrogateCons)
consensusLengths[["clust2"]] <- nchar(clust2Cons)
consensusLengths[["clust2 0"]] <- nchar(clust2LengthSurrogateCons)
consensusLengths[["clust2 3"]] <- nchar(clust2TrimerSurrogateCons)
consensusLengths[["clust4"]] <- nchar(clust4Cons)
consensusLengths[["clust4 0"]] <- nchar(clust4LengthSurrogateCons)
consensusLengths[["clust4 3"]] <- nchar(clust4TrimerSurrogateCons)

constraintLabels <- list()
constraintLabels[["0"]] <- "LengthSurrogates"
constraintLabels[["3"]] <- "TrimerSurrogates"

loadBaselineAlignmentStats("G3E1_e5","STL",directory=paste(baselineDir,"GapOpen3_e5/",sep=""),gapOpenCost=-3)
loadBaselineAlignmentStats("G0E1_e3","STL",directory=paste(baselineDir,"GapOpen0_1000/",sep=""),gapOpenCost=-3)

rawScores <- list()
zScores <- list()
for (gapPenalty in c(0,3)){
  if (gapPenalty == 3){
    bkey <- "G3E1_e5"
  }else{
    bkey <- paste("G",gapPenalty,"E1_e3",sep="")
  }
  gapPenaltyStr <- as.character(gapPenalty)
  for (surrogateConstraint in c("0","3")){
    constraintLabel <- constraintLabels[[surrogateConstraint]]
    # Cluster vs cluster consensus
    compareKey <- paste("Clust",clustNum,"-",consNum," G",gapPenalty," Constraint",surrogateConstraint, " Neurites vs NeuriteConsensus",sep="")
    consensusLength <- consensusLengths[[paste("clust",clustNum,sep="")]]
    filename <- paste(consensusCompareDir,"ScoreMat_",arborType,"STL_G",gapPenalty,"E1Identity_",arborParams,"-Clust",clustNum,"-vs-Clust",consNum,"ConsT",threshold,"Sub",subsetSize,".tab",sep="")
    scores <- read.csv(filename,row.names=1,sep="\t")
    # Eliminate sequences that are way too big
    scores <- scores[usedMetaData[rownames(scores),"SeqLen"] <= 800,,drop=F]
    rawScores[[compareKey]] <- scores
    perCharScores <- mapply(perCharScore,scores[,1],usedMetaData[rownames(scores),"SeqLen"],consensusLength,list(gapPenalty=gapPenalty))
    names(perCharScores) <- rownames(scores)
    zscores <- mapply(getBaselineZScore,bkey,perCharScores,usedMetaData[rownames(scores),"SeqLen"],consensusLength)
    names(zscores) <- rownames(scores)
    zScores[[compareKey]] <- zscores
    
    # Surrogates vs cluster consensus
    filename <- paste(consensusCompareDir,"ScoreMat_",arborType,"STL_G",gapPenalty,"E1Identity_",arborParams,"-Clust",clustNum,constraintLabel,"-vs-Clust",consNum,"ConsT",threshold,"Sub",subsetSize,".tab",sep="")
    scores <- read.csv(filename,row.names=1,sep="\t")
    compareKey <- paste("Clust",clustNum,"-",consNum," G",gapPenalty," Constraint",surrogateConstraint," Surrogates vs NeuriteConsensus",sep="")
    # Eliminate sequences that are way too big
    scores <- scores[usedMetaData[rownames(scores),"SeqLen"] <= 800,,drop=F]
    rawScores[[compareKey]] <- scores
    perCharScores <- mapply(perCharScore,scores[,1],usedMetaData[rownames(scores),"SeqLen"],consensusLength,list(gapPenalty=gapPenalty))
    names(perCharScores) <- rownames(scores)
    zscores <- mapply(getBaselineZScore,bkey,perCharScores,usedMetaData[rownames(scores),"SeqLen"],consensusLength)
    names(zscores) <- rownames(scores)
    zScores[[compareKey]] <- zscores
    
    # Cluster vs surrogate consensus
    filename <- paste(consensusCompareDir,"ScoreMat_",arborType,"STL_G",gapPenalty,"E1Identity_",arborParams,"-Clust",clustNum,"-vs-Clust",consNum,constraintLabel,"ConsT",threshold,"Sub",subsetSize,".tab",sep="")
    scores <- read.csv(filename,row.names=1,sep="\t")
    compareKey <- paste("Clust",clustNum,"-",consNum," G",gapPenalty," Constraint",surrogateConstraint," Neurites vs SurrogateConsensus",sep="")
    consensusLength <- consensusLengths[[paste("clust",clustNum," ",surrogateConstraint,sep="")]]
    # Eliminate sequences that are way too big
    scores <- scores[usedMetaData[rownames(scores),"SeqLen"] <= 800,,drop=F]
    rawScores[[compareKey]] <- scores
    perCharScores <- mapply(perCharScore,scores[,1],usedMetaData[rownames(scores),"SeqLen"],consensusLength,list(gapPenalty=gapPenalty))
    names(perCharScores) <- rownames(scores)
    zscores <- mapply(getBaselineZScore,bkey,perCharScores,usedMetaData[rownames(scores),"SeqLen"],consensusLength)
    names(zscores) <- rownames(scores)
    zScores[[compareKey]] <- zscores
    
    # Surrogates vs surrogate consensus
    filename <- paste(consensusCompareDir,"ScoreMat_",arborType,"STL_G",gapPenalty,"E1Identity_",arborParams,"-Clust",clustNum,constraintLabel,"-vs-Clust",consNum,constraintLabel,"ConsT",threshold,"Sub",subsetSize,".tab",sep="")
    scores <- read.csv(filename,row.names=1,sep="\t")
    compareKey <- paste("Clust",clustNum,"-",consNum," G",gapPenalty," Constraint",surrogateConstraint," Surrogates vs SurrogateConsensus",sep="")
    # Eliminate sequences that are way too big
    scores <- scores[usedMetaData[rownames(scores),"SeqLen"] <= 800,,drop=F]
    rawScores[[compareKey]] <- scores
    perCharScores <- mapply(perCharScore,scores[,1],usedMetaData[rownames(scores),"SeqLen"],consensusLength,list(gapPenalty=gapPenalty))
    names(perCharScores) <- rownames(scores)
    zscores <- mapply(getBaselineZScore,bkey,perCharScores,usedMetaData[rownames(scores),"SeqLen"],consensusLength)
    names(zscores) <- rownames(scores)
    zScores[[compareKey]] <- zscores
  }
}

sapply(zScores,mean)
sapply(zScores,sem)

# Extract and Normalize scores
scoresG0Axon1VsCons1T50 <- read.csv("/home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust1-vs-Clust1ConsT0.5Sub120.tab",row.names=1,sep="\t")
# Eliminate sequences that are way too big
scoresG0Axon1VsCons1T50 <- scoresG0Axon1VsCons1T50[usedMetaData[rownames(scoresG0Axon1VsCons1T50),"SeqLen"] <= 800,,drop=F]
perCharScoresG0Axon1VsCons1T50 <- mapply(perCharScore,scoresG0Axon1VsCons1T50[,1],usedMetaData[rownames(scoresG0Axon1VsCons1T50),"SeqLen"],cons1T50Length,list(gapPenalty=0))
names(perCharScoresG0Axon1VsCons1T50) <- rownames(scoresG0Axon1VsCons1T50)
zscoresG0Axon1VsCons1T50 <- mapply(getBaselineZScore,"G0E1_e3",perCharScoresG0Axon1VsCons1T50,usedMetaData[rownames(scoresG0Axon1VsCons1T50),"SeqLen"],cons1T50Length)
names(zscoresG0Axon1VsCons1T50) <- rownames(scoresG0Axon1VsCons1T50)

scoresG0Axon1VsCons3T50 <- read.csv("/home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust1-vs-Clust3ConsT0.5Sub120.tab",row.names=1,sep="\t")
# Eliminate sequences that are way too big
scoresG0Axon1VsCons3T50 <- scoresG0Axon1VsCons3T50[usedMetaData[rownames(scoresG0Axon1VsCons3T50),"SeqLen"] <= 800,,drop=F]
perCharScoresG0Axon1VsCons3T50 <- mapply(perCharScore,scoresG0Axon1VsCons3T50[,1],usedMetaData[rownames(scoresG0Axon1VsCons3T50),"SeqLen"],cons3T50Length,list(gapOpen=0))
names(perCharScoresG0Axon1VsCons3T50) <- rownames(scoresG0Axon1VsCons3T50)
zscoresG0Axon1VsCons3T50 <- mapply(getBaselineZScore,"G0E1_e3",perCharScoresG0Axon1VsCons3T50,usedMetaData[rownames(scoresG0Axon1VsCons3T50),"SeqLen"],cons3T50Length)
names(zscoresG0Axon1VsCons3T50) <- rownames(scoresG0Axon1VsCons3T50)

scoresG0Axon3VsCons1T50 <- read.csv("/home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust3-vs-Clust1ConsT0.5Sub120.tab",row.names=1,sep="\t")
# Eliminate those used to construct the consensus
# Eliminate sequences that are way too big
scoresG0Axon3VsCons1T50 <- scoresG0Axon3VsCons1T50[usedMetaData[rownames(scoresG0Axon3VsCons1T50),"SeqLen"] <= 800,,drop=F]
perCharScoresG0Axon3VsCons1T50 <- mapply(perCharScore,scoresG0Axon3VsCons1T50[,1],usedMetaData[rownames(scoresG0Axon3VsCons1T50),"SeqLen"],cons1T50Length,list(gapOpen=0))
names(perCharScoresG0Axon3VsCons1T50) <- rownames(scoresG0Axon3VsCons1T50)
zscoresG0Axon3VsCons1T50 <- mapply(getBaselineZScore,"G0E1_e3",perCharScoresG0Axon3VsCons1T50,usedMetaData[rownames(scoresG0Axon3VsCons1T50),"SeqLen"],cons1T50Length)
names(zscoresG0Axon3VsCons1T50) <- rownames(scoresG0Axon3VsCons1T50)

scoresG0Axon3VsCons3T50 <- read.csv("/home/tgillett/Domains/ScoreMat_AxonSTL_G0E1Identity_Dims2of7-VII4-Clust3-vs-Clust3ConsT0.5Sub120.tab",row.names=1,sep="\t")
# Eliminate sequences that are way too big
scoresG0Axon3VsCons3T50 <- scoresG0Axon3VsCons3T50[usedMetaData[rownames(scoresG0Axon3VsCons3T50),"SeqLen"] <= 800,,drop=F]
perCharScoresG0Axon3VsCons3T50 <- mapply(perCharScore,scoresG0Axon3VsCons3T50[,1],usedMetaData[rownames(scoresG0Axon3VsCons3T50),"SeqLen"],cons3T50Length,list(gapOpen=0))
names(perCharScoresG0Axon3VsCons3T50) <- rownames(scoresG0Axon3VsCons3T50)
zscoresG0Axon3VsCons3T50 <- mapply(getBaselineZScore,"G0E1_e3",perCharScoresG0Axon3VsCons3T50,usedMetaData[rownames(scoresG0Axon3VsCons3T50),"SeqLen"],cons3T50Length)
names(zscoresG0Axon3VsCons3T50) <- rownames(scoresG0Axon3VsCons3T50)

mean(zscoresG0Axon1VsCons1T50[usedMetaData[names(zscoresG0Axon1VsCons1T50),"SeqLen"]<500])
mean(zscoresG0Axon1VsCons3T50[usedMetaData[names(zscoresG0Axon1VsCons3T50),"SeqLen"]<500])
mean(zscoresG0Axon3VsCons1T50[usedMetaData[names(zscoresG0Axon3VsCons1T50),"SeqLen"]<500])
mean(zscoresG0Axon3VsCons3T50[usedMetaData[names(zscoresG0Axon3VsCons3T50),"SeqLen"]<500])

# Eliminate those used to construct the consensus
mean(zscoresG0Axon1VsCons1T50[121:length(zscoresG0Axon1VsCons1T50)])
mean(zscoresG0Axon1VsCons3T50)
mean(zscoresG0Axon3VsCons1T50)
mean(zscoresG0Axon3VsCons3T50[121:length(zscoresG0Axon3VsCons3T50)])

## Get classification setup if not already
if (!exists("emResults")){
  emResults <- list()
}
for (arborType in names(arborMDS)){
  if (!arborType%in%names(emResults)){
    emResults[[arborType]] <- Mclust(arborMDS[[arborType]],G=clusterParams[[arborType]]$G,modelNames=clusterParams[[arborType]]$modelName)
  }
}

#### MSA Consensuses ####
myColor <- "#660016"
margins <- c(0.1,0.05,2.4,0.05)
consensusDir <- "/home/tgillett/Domains/consensuses/"

# Cluster 1
threshold <- "0.5"

arborType <- "Apical"
setName <- "ApicalNC-G3E1-Dims4of8-EII3"
subsetSize <- 100

arborType <- "Axon"
setName <- "Axon-G3E1-Dims2of7-VII4"
subsetSize <- 120

clusterNum <- 1
surrogateString <- ""
surrogateString <- "_SurrogateConstraint3-1of100"
consensusName <- paste("Consensus-",setName,"-Clust",clusterNum,"-Zoom-DistLim1",surrogateString,"_T",threshold,"Sub",subsetSize,"I",sep="")
if (surrogateString == ""){
  consensusTitle <- paste(arborType," Cluster ",clusterNum," Threshold ",threshold,sep="")
}else{
  consensusTitle <- paste(arborType," Surrogates Cluster ",clusterNum," Threshold ",threshold,sep="")
}
consensusTree <- read.tree(paste(consensusDir,consensusName,".newick",sep=""))

png(paste(consensusDir,consensusName,".png",sep=""),width=800,height=600)
par(mar=margins)
plotConsensusDendrogram(consensusTree,treePalette=myColor,main=consensusTitle,edge.width=3)
dev.off()

#### Fig 6 or Supp. fig: Cluster 3(4) SeqLen Distr ####

## Add sequence length distributions to %C vs SeqLen plot
seqLenDF <- motifMetaData[rownames(axonCFreq),c("ArborType","SeqLen"),drop=F]
seqLenDF[axonCluster3Seqs,"ArborType"] <- "Cluster3"


gg_seqLen <- ggplot(subset(seqLenDF,SeqLen<=500),aes(SeqLen,alpha=.05,group=ArborType,fill=ArborType))+geom_density(linetype=0)+
  scale_fill_manual(values=c(Axon="grey",Cluster3="red"))+
  scale_y_continuous(breaks=c(0,.05),minor_breaks=0.023)
ggsave(gg_seqLen,file=paste(figDir,"AxonCluster4_SeqLenDist.png",sep=""))

#### Fig 6 or Supp. fig: Cluster 3(4) % Captured ####

currOrder <- "STL"
kRange <- 1:5
myResults <- matrix(0,nrow=2,ncol=length(kRange))
colnames(myResults) <- kRange
rownames(myResults) <- 1:nrow(myResults)
diffSEs <- matrix(0,nrow=length(specifications),ncol=length(kRange))
colnames(diffSEs) <- kRange
rownames(diffSEs) <- 1:nrow(diffSEs)
resultOffset <- 0 # 8
minSize <- 20
maxSize <- NaN
removeATTFlag <- TRUE
ltsOnly <- FALSE
useAlternate <- F

# Set data source
myData <- prDataB8Combo
mySurrogateData <- prSurrogateDataB8Combo
resultTitle <- "With LTS"

#myData <- prDataB8STL
#mySurrogateData <- prSurrogateDataB8STL
#resultTitle <- ""

for (k in kRange){
  sharedSeqs <- intersect(rownames(myData[[k]]),rownames(mySurrogateData[[k]]))
  myData[[k]] <- myData[[k]][sharedSeqs,,drop=F]
  mySurrogateData[[k]] <- mySurrogateData[[k]][sharedSeqs,,drop=F]
}

#for (i in 1:length(specifications)){
#specification <- specifications[[i]]
#specification <- specifications[[1]] # All Arbors
realMeans <- kRange
names(realMeans) <- kRange
surrogateMeans <- kRange
realSDs <- kRange
surrogateSDs <- kRange
diffSE <- kRange
# Get the spec name
specName <- "Axon Cluster 4"

for (k in kRange){
  # Get the data subset
  subData <- myData[[k]][rownames(myData[[k]])%in%axonCluster3Seqs,,drop=F]
  subSurrogateData <- mySurrogateData[[k]][rownames(mySurrogateData[[k]])%in%axonCluster3Seqs,,drop=F]
  
  # Remove ATT-related kmers
  if (removeATTFlag && k >= 3){
    subData <- subData[,removeATT(currOrder,colnames(subData)),drop=F]
    subSurrogateData <- subSurrogateData[,removeATT(currOrder,colnames(subSurrogateData)),drop=F]
  }
  # Determine which columns to run
  if (ltsOnly){
    inds <- grep("L-",colnames(subData))
    subData <- subData[,inds,drop=F]
    subSurrogateData <- subSurrogateData[,inds,drop=F]
  }
  
  #outlierData <- calculatePROutlierRatio(subData,k=k,alpha=0.05)
  #surrogateOutlierData <- calculatePROutlierRatio(subSurrogateData,k=k,alpha=0.05)
  captured <- apply(subData,1,function(x){mean(x > .025 & x < .975)})
  capturedSurrogate <- apply(subSurrogateData,1,function(x){mean(x >= .025 & x <= .975)})
  
  kInd <- k-min(kRange)+1
  realMeans[kInd] <- mean(captured)
  realSDs[kInd] <- sd(captured)
  
  surrogateMeans[kInd] <- mean(capturedSurrogate)
  surrogateSDs[kInd] <- sd(capturedSurrogate)
  diffSE[kInd] = sqrt((var(captured)+var(capturedSurrogate)+cov(captured,capturedSurrogate))/length(captured))
}

myResults[1,] <- realMeans
rownames(myResults)[1] <- paste(resultTitle,specName,"Real")

myResults[2,] <- surrogateMeans-realMeans
rownames(myResults)[2] <- paste(resultTitle,specName,"Diff")

diffSEs[1,] <- diffSE
rownames(diffSEs)[1] <- paste(resultTitle,specName,"Diff SEs")

realSEs <- realSDs/sqrt(nrow(subData))
surrogateSEs <- surrogateSDs/sqrt(nrow(subData))

# Bar plot of results
title <- "Axon Cluster 3(4)"
if (useAlternate){
  realFlip <- 1-realMeans
  surrFlip <- 1-surrogateMeans
  vals <- realFlip/surrFlip
  barplot(vals,main=title,xlab="k",ylab="Real-Baseline captured ratio")
  realSE <- realSDs/sqrt(nrow(subData))
  surrSE <- surrogateSDs/sqrt(nrow(subData))
  errs <- sqrt((surrSE*realFlip/surrFlip^2)^2 + (realSE/surrFlip)^2)
  #errs <- sqrt(realSDs^2/nrow(subData) + surrogateSDs^2/nrow(subData))
  error.bar(1.2*(kRange-min(kRange))+.7,vals,errs)
}else{
  #offset <- floor(20*min(realMeans)-.5)/20
  offset <- offsets[i]
  png(paste(figDir,"/PercentCaptured_AxonCluster3_SurrogateN100_Min",offset,".png",sep=""),width=800,height=800)
  barplot(realMeans-offset,ylim=c(offset,1),offset=offset,main=title,xlab="k",ylab="Average proportion of k-mers captured by Baseline")
  error.bar(1.2*(kRange-min(kRange))+.7,realMeans,realSDs/sqrt(nrow(subData)))
  barplot(1-surrogateMeans,offset=surrogateMeans,col="gray50",add=T)
  error.bar(1.2*(kRange-min(kRange))+.7,surrogateMeans,surrogateSDs/sqrt(nrow(subData)))
  dev.off()
}

#### Slicing effects ####

resliceCandidates <- subset(clusterMetaData,ArborType=="Axon"&Order=="Rodent"&BrainRegion=="Neocortex")
aggregate(resliceCandidates[,1],by=list(Species=resliceCandidates$Species,Protocol=resliceCandidates$Protocol),length)
#resliceCandidates <- subset(clusterMetaData,ArborType=="Axon"&Species=="Mouse"&BrainRegion=="Neocortex"&Protocol=="In vivo")

# Selected neurons: ca3b-N2 (Ascoli+Buzsaki; Rat HC pyramidal), 980804axden (Hirsch; Cat NC pyramidal), oi15rpy4-1 (Kisvarday Cat NC pyramidal) 

# Slice neurites
#java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.generation.SwcZDistanceCutter -s "/home/tgillett/swcFiles/NM/ascoli+buzsaki/CNG version/ca3b-N2.CNG.swc" -a axon -o /home/tgillett/zCuts/ -c 4
#java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.generation.SwcZDistanceCutter -s "/home/tgillett/swcFiles/NM/hirsch/CNG version/980804axden.CNG.swc" -a axon -o /home/tgillett/zCuts/ -c 4
#java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.generation.SwcZDistanceCutter -s "/home/tgillett/swcFiles/NM/kisvarday/CNG version/oi15rpy4-1.CNG.swc" -a axon -o /home/tgillett/zCuts/ -c 4

# Extract original fasta (in zCuts directory)
#tail -n 2 ZCuts-4_ca3b-N2-Axon.fasta > ca3b-N2-Axon.fasta
#tail -n 2 ZCuts-4_980804axden-Axon.fasta > 980804axden-Axon.fasta
#tail -n 2 ZCuts-4_oi15rpy4-1-Axon.fasta > oi15rpy4-1-Axon.fasta

# Run spaghetti.py on these files
#python3 spaghetti.py -f /home/tgillett/zCuts/ca3b-N2-Axon.fasta -f2 /home/tgillett/zCuts/ZCuts-4_ca3b-N2-Axon.fasta -E -1 -G -3 --custom identity.tab -n 5 -o /home/tgillett/zCuts/ScoreMat_ca3b-N2-Axon_ZCuts-4_G3E1.tab -a /home/tgillett/zCuts/Alignments_ca3b-N2-Axon_ZCuts-4_G3E1.tab.txt
#python3 spaghetti.py -f /home/tgillett/zCuts/ca3b-N2-Axon.fasta -f2 /home/tgillett/zCuts/ZCuts-4_ca3b-N2-Axon.fasta -E -1 -G -1 --custom identity.tab -n 5 -o /home/tgillett/zCuts/ScoreMat_ca3b-N2-Axon_ZCuts-4_G1E1.tab -a /home/tgillett/zCuts/Alignments_ca3b-N2-Axon_ZCuts-4_G1E1.tab.txt
#python3 spaghetti.py -f /home/tgillett/zCuts/980804axden-Axon.fasta -f2 /home/tgillett/zCuts/ZCuts-4_980804axden-Axon.fasta -E -1 -G -3 --custom identity.tab -n 5 -o /home/tgillett/zCuts/ScoreMat_980804axden-Axon_ZCuts-4_G3E1.tab -a /home/tgillett/zCuts/Alignments_980804axden-Axon_ZCuts-4_G3E1.tab.txt
#python3 spaghetti.py -f /home/tgillett/zCuts/980804axden-Axon.fasta -f2 /home/tgillett/zCuts/ZCuts-4_980804axden-Axon.fasta -E -1 -G -1 --custom identity.tab -n 5 -o /home/tgillett/zCuts/ScoreMat_980804axden-Axon_ZCuts-4_G1E1.tab -a /home/tgillett/zCuts/Alignments_980804axden-Axon_ZCuts-4_G1E1.tab.txt
#python3 spaghetti.py -f /home/tgillett/zCuts/oi15rpy4-1-Axon.fasta -f2 /home/tgillett/zCuts/ZCuts-4_oi15rpy4-1-Axon.fasta -E -1 -G -3 --custom identity.tab -n 5 -o /home/tgillett/zCuts/ScoreMat_oi15rpy4-1-Axon_ZCuts-4_G3E1.tab -a /home/tgillett/zCuts/Alignments_oi15rpy4-1-Axon_ZCuts-4_G3E1.tab.txt
#python3 spaghetti.py -f /home/tgillett/zCuts/oi15rpy4-1-Axon.fasta -f2 /home/tgillett/zCuts/ZCuts-4_oi15rpy4-1-Axon.fasta -E -1 -G -1 --custom identity.tab -n 5 -o /home/tgillett/zCuts/ScoreMat_oi15rpy4-1-Axon_ZCuts-4_G1E1.tab -a /home/tgillett/zCuts/Alignments_oi15rpy4-1-Axon_ZCuts-4_G1E1.tab.txt

# Convert to per-character scores
#python3 spaghetti.py --mode calculate_percharacter -f /home/tgillett/zCuts/ca3b-N2-Axon.fasta -f2 /home/tgillett/zCuts/ZCuts-4_ca3b-N2-Axon.fasta -E -1 -G -3 --custom identity.tab -n 5 -i /home/tgillett/zCuts/ScoreMat_ca3b-N2-Axon_ZCuts-4_G3E1.tab -o /home/tgillett/zCuts/PerCharScores_ca3b-N2-Axon_ZCuts-4_G3E1.tab
#python3 spaghetti.py --mode calculate_percharacter -f /home/tgillett/zCuts/ca3b-N2-Axon.fasta -f2 /home/tgillett/zCuts/ZCuts-4_ca3b-N2-Axon.fasta -E -1 -G -1 --custom identity.tab -n 5 -i /home/tgillett/zCuts/ScoreMat_ca3b-N2-Axon_ZCuts-4_G1E1.tab -o /home/tgillett/zCuts/PerCharScores_ca3b-N2-Axon_ZCuts-4_G1E1.tab
#python3 spaghetti.py --mode calculate_percharacter -f /home/tgillett/zCuts/980804axden-Axon.fasta -f2 /home/tgillett/zCuts/ZCuts-4_980804axden-Axon.fasta -E -1 -G -3 --custom identity.tab -n 5 -i /home/tgillett/zCuts/ScoreMat_980804axden-Axon_ZCuts-4_G3E1.tab -o /home/tgillett/zCuts/PerCharScores_980804axden-Axon_ZCuts-4_G3E1.tab
#python3 spaghetti.py --mode calculate_percharacter -f /home/tgillett/zCuts/980804axden-Axon.fasta -f2 /home/tgillett/zCuts/ZCuts-4_980804axden-Axon.fasta -E -1 -G -1 --custom identity.tab -n 5 -i /home/tgillett/zCuts/ScoreMat_980804axden-Axon_ZCuts-4_G1E1.tab -o /home/tgillett/zCuts/PerCharScores_980804axden-Axon_ZCuts-4_G1E1.tab
#python3 spaghetti.py --mode calculate_percharacter -f /home/tgillett/zCuts/oi15rpy4-1-Axon.fasta -f2 /home/tgillett/zCuts/ZCuts-4_oi15rpy4-1-Axon.fasta -E -1 -G -3 --custom identity.tab -n 5 -i /home/tgillett/zCuts/ScoreMat_oi15rpy4-1-Axon_ZCuts-4_G3E1.tab -o /home/tgillett/zCuts/PerCharScores_oi15rpy4-1-Axon_ZCuts-4_G3E1.tab
#python3 spaghetti.py --mode calculate_percharacter -f /home/tgillett/zCuts/oi15rpy4-1-Axon.fasta -f2 /home/tgillett/zCuts/ZCuts-4_oi15rpy4-1-Axon.fasta -E -1 -G -1 --custom identity.tab -n 5 -i /home/tgillett/zCuts/ScoreMat_oi15rpy4-1-Axon_ZCuts-4_G1E1.tab -o /home/tgillett/zCuts/PerCharScores_oi15rpy4-1-Axon_ZCuts-4_G1E1.tab

# Read in and normalize
normScoreMatFilename <- paste(globalAlignDataDir,"/NormScore_",arborType,travOrder,spurStr,"_",scoreParamStr,matrixExt,sep="")
normScoreMatDF <- read.csv(normScoreMatFilename,sep="\t",check.names=F,row.names=rownamesVal,na.strings="None",stringsAsFactors=F)

PCScoresG3_ca3b_N2 <- read.csv(paste(homeDir,"/zCuts/PerCharScores_ca3b-N2-Axon_ZCuts-4_G3E1.tab",sep=""),sep="\t",check.names=F,row.names=1,na.strings="None",stringsAsFactors=F)
PCScoresG1_ca3b_N2 <- read.csv(paste(homeDir,"/zCuts/PerCharScores_ca3b-N2-Axon_ZCuts-4_G1E1.tab",sep=""),sep="\t",check.names=F,row.names=1,na.strings="None",stringsAsFactors=F)
PCScoresG3_980804axden <- read.csv(paste(homeDir,"/zCuts/PerCharScores_980804axden-Axon_ZCuts-4_G3E1.tab",sep=""),sep="\t",check.names=F,row.names=1,na.strings="None",stringsAsFactors=F)
PCScoresG1_980804axden <- read.csv(paste(homeDir,"/zCuts/PerCharScores_980804axden-Axon_ZCuts-4_G1E1.tab",sep=""),sep="\t",check.names=F,row.names=1,na.strings="None",stringsAsFactors=F)
PCScoresG3_oi15rpy4_1 <- read.csv(paste(homeDir,"/zCuts/PerCharScores_oi15rpy4-1-Axon_ZCuts-4_G3E1.tab",sep=""),sep="\t",check.names=F,row.names=1,na.strings="None",stringsAsFactors=F)
PCScoresG1_oi15rpy4_1 <- read.csv(paste(homeDir,"/zCuts/PerCharScores_oi15rpy4-1-Axon_ZCuts-4_G1E1.tab",sep=""),sep="\t",check.names=F,row.names=1,na.strings="None",stringsAsFactors=F)

zCutNeurons <- c("ca3b-N2","980804axden","oi15rpy4-1")
# Get sequence lengths from fasta files
ZCutLengths <- list()
for (neuron in zCutNeurons){
  zCutFastas <- read.table(paste(homeDir,"/zCuts/ZCuts-4_",neuron,"-Axon.fasta",sep=""),header=F,stringsAsFactors=F,sep=",")
  numLines <- nrow(zCutFastas)/2
  for (i in 0:(numLines-1)){
    neuronName <- gsub("> ","",zCutFastas[i*2+1,1])
    ZCutLengths[[neuronName]] <- nchar(zCutFastas[i*2+2,1])
  }
}

ZCutProportions <- list()
for (seq in names(ZCutLengths)){
  parentSeq <- gsub("(.+\\-Axon)_ZCut-(\\d+)","\\1",seq)
  parentLength <- ZCutLengths[[parentSeq]]
  ZCutProportions[[seq]] <- ZCutLengths[[seq]]/parentLength
}

g3Key <- "G3E1_e3 STL"
g1Key <- "G1E1_e3 STL"
source(paste(clusterGlobalDir,"BaselineHandler.r",sep=""))
loadBaselineAlignmentStats(g3Key, directory=paste(baselineDir,"GapOpen3_1000/",sep=""),travOrder="STL")
loadBaselineAlignmentStats(g1Key, directory=paste(baselineDir,"GapOpen1_1000/",sep=""),travOrder="STL")

NormScoresG3_ca3b_N2 <- PCScoresG3_ca3b_N2
NormScoresG1_ca3b_N2<- PCScoresG1_ca3b_N2
NormScoresG3_980804axden <- PCScoresG3_980804axden
NormScoresG1_980804axden <- PCScoresG1_980804axden
NormScoresG3_oi15rpy4_1 <- PCScoresG3_oi15rpy4_1
NormScoresG1_oi15rpy4_1 <- PCScoresG1_oi15rpy4_1
for (i in 1:nrow(NormScoresG3_ca3b_N2)){
  NormScoresG3_ca3b_N2[i,1] <- getBaselineNormalizedDistance(g3Key,PCScoresG3_ca3b_N2[i,1],ZCutLengths[[rownames(PCScoresG3_ca3b_N2)[i]]],ZCutLengths[["ca3b-N2-Axon"]])
  NormScoresG1_ca3b_N2[i,1] <- getBaselineNormalizedDistance(g1Key,PCScoresG1_ca3b_N2[i,1],ZCutLengths[[rownames(PCScoresG1_ca3b_N2)[i]]],ZCutLengths[["ca3b-N2-Axon"]])
  NormScoresG3_980804axden[i,1] <- getBaselineNormalizedDistance(g3Key,PCScoresG3_980804axden[i,1],ZCutLengths[[rownames(PCScoresG3_980804axden)[i]]],ZCutLengths[["980804axden-Axon"]])
  NormScoresG1_980804axden[i,1] <- getBaselineNormalizedDistance(g1Key,PCScoresG1_980804axden[i,1],ZCutLengths[[rownames(PCScoresG1_980804axden)[i]]],ZCutLengths[["980804axden-Axon"]])
  NormScoresG3_oi15rpy4_1[i,1] <- getBaselineNormalizedDistance(g3Key,PCScoresG3_oi15rpy4_1[i,1],ZCutLengths[[rownames(PCScoresG3_oi15rpy4_1)[i]]],ZCutLengths[["oi15rpy4-1-Axon"]])
  NormScoresG1_oi15rpy4_1[i,1] <- getBaselineNormalizedDistance(g1Key,PCScoresG1_oi15rpy4_1[i,1],ZCutLengths[[rownames(PCScoresG1_oi15rpy4_1)[i]]],ZCutLengths[["oi15rpy4-1-Axon"]])
}

ZScoresG3_ca3b_N2 <- PCScoresG3_ca3b_N2
ZScoresG3_ca3b_N2[,c("Length","Proportion","G")] <- NA
ZScoresG1_ca3b_N2<- PCScoresG1_ca3b_N2
ZScoresG1_ca3b_N2[,c("Length","Proportion","G")] <- NA
ZScoresG3_980804axden <- PCScoresG3_980804axden
ZScoresG3_980804axden[,c("Length","Proportion,"G"")] <- NA
ZScoresG1_980804axden <- PCScoresG1_980804axden
ZScoresG1_980804axden[,c("Length","Proportion,"G"")] <- NA
ZScoresG3_oi15rpy4_1 <- PCScoresG3_oi15rpy4_1
ZScoresG3_oi15rpy4_1[,c("Length","Proportion","G")] <- NA
ZScoresG1_oi15rpy4_1 <- PCScoresG1_oi15rpy4_1
ZScoresG1_oi15rpy4_1[,c("Length","Proportion","G")] <- NA

for (i in 1:nrow(ZScoresG3_ca3b_N2)){
  ZScoresG3_ca3b_N2[i,"Length"] <- ZCutLengths[[rownames(PCScoresG3_ca3b_N2)[i]]]
  ZScoresG3_ca3b_N2[i,"Proportion"] <- ZCutProportions[[rownames(PCScoresG3_ca3b_N2)[i]]]
  ZScoresG3_ca3b_N2[i,1] <- getBaselineZScore(g3Key,PCScoresG3_ca3b_N2[i,1],ZScoresG3_ca3b_N2[i,"Length"],ZCutLengths[["ca3b-N2-Axon"]])
  ZScoresG3_ca3b_N2[i,"G"] <- 3
  
  ZScoresG1_ca3b_N2[i,"Length"] <- ZCutLengths[[rownames(PCScoresG1_ca3b_N2)[i]]]
  ZScoresG1_ca3b_N2[i,"Proportion"] <- ZCutProportions[[rownames(PCScoresG1_ca3b_N2)[i]]]
  ZScoresG1_ca3b_N2[i,1] <- getBaselineZScore(g1Key,PCScoresG1_ca3b_N2[i,1],ZScoresG1_ca3b_N2[i,"Length"],ZCutLengths[["ca3b-N2-Axon"]])
  ZScoresG1_ca3b_N2[i,"G"] <- 1
  
  ZScoresG3_980804axden[i,"Length"] <- ZCutLengths[[rownames(ZScoresG3_980804axden)[i]]]
  ZScoresG3_980804axden[i,"Proportion"] <- ZCutProportions[[rownames(ZScoresG3_980804axden)[i]]]
  ZScoresG3_980804axden[i,1] <- getBaselineZScore(g3Key,PCScoresG3_980804axden[i,1],ZScoresG3_980804axden[i,"Length"],ZCutLengths[["980804axden-Axon"]])
  ZScoresG3_980804axden[i,"G"] <- 3

  ZScoresG1_980804axden[i,"Length"] <- ZCutLengths[[rownames(ZScoresG1_980804axden)[i]]]
  ZScoresG1_980804axden[i,"Proportion"] <- ZCutProportions[[rownames(ZScoresG1_980804axden)[i]]]
  ZScoresG1_980804axden[i,1] <- getBaselineZScore(g1Key,PCScoresG1_980804axden[i,1],ZScoresG1_980804axden[i,"Length"],ZCutLengths[["980804axden-Axon"]])
  ZScoresG1_980804axden[i,"G"] <- 1
  
  ZScoresG3_oi15rpy4_1[i,"Length"] <- ZCutLengths[[rownames(ZScoresG3_oi15rpy4_1)[i]]]
  ZScoresG3_oi15rpy4_1[i,"Proportion"] <- ZCutProportions[[rownames(ZScoresG3_oi15rpy4_1)[i]]]
  ZScoresG3_oi15rpy4_1[i,1] <- getBaselineZScore(g3Key,PCScoresG3_oi15rpy4_1[i,1],ZScoresG3_oi15rpy4_1[i,"Length"],ZCutLengths[["oi15rpy4-1-Axon"]])
  ZScoresG3_oi15rpy4_1[i,"G"] <- 3
    
  ZScoresG1_oi15rpy4_1[i,"Length"] <- ZCutLengths[[rownames(ZScoresG1_oi15rpy4_1)[i]]]
  ZScoresG1_oi15rpy4_1[i,"Proportion"] <- ZCutProportions[[rownames(ZScoresG1_oi15rpy4_1)[i]]]
  ZScoresG1_oi15rpy4_1[i,1] <- getBaselineZScore(g1Key,PCScoresG1_oi15rpy4_1[i,1],ZScoresG1_oi15rpy4_1[i,"Length"],ZCutLengths[["oi15rpy4-1-Axon"]])
  ZScoresG1_oi15rpy4_1[i,"G"] <- 1
}

colnames(ZScoresG3_ca3b_N2)[1] <- "ZScore"
colnames(ZScoresG1_ca3b_N2)[1] <- "ZScore"
colnames(ZScoresG3_980804axden)[1] <- "ZScore"
colnames(ZScoresG1_980804axden)[1] <- "ZScore"
colnames(ZScoresG3_oi15rpy4_1)[1] <- "ZScore"
colnames(ZScoresG1_oi15rpy4_1)[1] <- "ZScore"

ZCutZScores <- rbind(ZScoresG3_ca3b_N2,ZScoresG1_ca3b_N2)
ZCutZScores <- rbind(ZCutZScores,ZScoresG3_980804axden)
ZCutZScores <- rbind(ZCutZScores,ZScoresG1_980804axden)
ZCutZScores <- rbind(ZCutZScores,ZScoresG3_oi15rpy4_1)
ZCutZScores <- rbind(ZCutZScores,ZScoresG1_oi15rpy4_1)

write.csv(ZCutZScores,paste(homeDir,"/zCuts/ZCuts-4_ZScores.csv",sep=""))

#### Symmetry risk calculation ####

# Produce the data
#java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.analysis.SubtreeSizesAndTopologyComparer -D NM_SpurLen2D -T 6 -o /home/tgillett/results/

# Read in the results for analysis
symmetryRiskData <- read.csv("/home/tgillett/results/SubtreeSizeData_NM_SpurLen2D_Threshold6.0.csv")
# Add arbor type
symmetryRiskData$ArborType <- usedMetaData[symmetryRiskData$Neurite,"ArborType"]
symmetryRiskData$SeqLen <- usedMetaData[symmetryRiskData$Neurite,"SeqLen"]
symmetryRiskData$RelSize <- symmetryRiskData$SubtreeSize/symmetryRiskData$SeqLen

# Bifurcations with 10 or more bifurcations in their children (degree = 11)
axonBifurcations <- 68771
apicalBifurcations <- 9702
dendriteBifurcations <- 48221

# Histogram?

# Worst case at each size group
plotObj <- ggplot(symmetryRiskData,aes(x = SubtreeSize, y = AsymmetryDiff, colour = ArborType, group=ArborType)) + 
  stat_summary(fun.y=max, geom="line")+
  scale_colour_manual(values=myColors,guide=F)+
  scale_fill_manual(values=myColors,guide=F)
ggsave(paste(figDir,"/SymmetryRisk-WorstCaseBySize.png",sep=""),plotObj)

# Median case at each size group
plotObj <- ggplot(symmetryRiskData,aes(x = SubtreeSize, y = AsymmetryDiff, colour = ArborType, group=ArborType)) + 
  stat_summary(fun.y=median, geom="line")+
#  stat_smooth(method = "median", span=5, se = TRUE,level = 0.95, aes(fill = Group),size=1)+
  scale_colour_manual(values=myColors,guide=F)+
  scale_fill_manual(values=myColors,guide=F)
ggsave(paste(figDir,"/SymmetryRisk-MedianBySize.png",sep=""),plotObj)

# Mean case at each size group
plotObj <- ggplot(symmetryRiskData,aes(x = SubtreeSize, y = AsymmetryDiff, colour = ArborType, group=ArborType)) + 
  stat_summary(fun.y=mean, geom="line")+
  #  stat_smooth(method = "median", span=5, se = TRUE,level = 0.95, aes(fill = Group),size=1)+
  scale_colour_manual(values=myColors,guide=F)+
  scale_fill_manual(values=myColors,guide=F)
ggsave(paste(figDir,"/SymmetryRisk-MeanBySize.png",sep=""),plotObj)

plotObj <- ggplot(symmetryRiskData,aes(x = SubtreeSize, y = CaulescenceDiff, colour = ArborType, group=ArborType)) + 
  stat_summary(fun.y=mean, geom="line")+
  #  stat_smooth(method = "median", span=5, se = TRUE,level = 0.95, aes(fill = Group),size=1)+
  scale_colour_manual(values=myColors,guide=F)+
  scale_fill_manual(values=myColors,guide=F)
ggsave(paste(figDir,"/SymmetryRisk-MeanCaulDiffBySize.png",sep=""),plotObj)

# Mean smooth
plotObj <- ggplot(symmetryRiskData,aes(x = SubtreeSize, y = AsymmetryDiff, colour = ArborType, group=ArborType)) + 
  stat_smooth(aes(fill=ArborType))+
  scale_colour_manual(values=myColors,guide=F)+
  scale_fill_manual(values=myColors,guide=F)
ggsave(paste(figDir,"/SymmetryRisk-MeanSmoothDefaults.png",sep=""),plotObj)

plotObj <- ggplot(symmetryRiskData,aes(x = SubtreeSize, y = CaulescenceDiff, colour = ArborType, group=ArborType)) + 
  stat_smooth(aes(fill=ArborType))+
  scale_colour_manual(values=myColors,guide=F)+
  scale_fill_manual(values=myColors,guide=F)
ggsave(paste(figDir,"/SymmetryRisk-CaulDiffMeanSmoothDefaults.png",sep=""),plotObj)

# Median case by size difference, limited to subtree size of <=25
plotObj <- ggplot(subset(symmetryRiskData,SubtreeSize<=25),aes(x = SizeDifference, y = AsymmetryDiff, colour = ArborType, group=ArborType)) + 
stat_summary(fun.y=mean, geom="line")+
  #  stat_smooth(method = "median", span=5, se = TRUE,level = 0.95, aes(fill = Group),size=1)+
  scale_colour_manual(values=myColors,guide=F)+
  scale_fill_manual(values=myColors,guide=F)
ggsave(paste(figDir,"/SymmetryRisk-MedianAsymDiffBySizeDiff.png",sep=""),plotObj)

# Bin by size and size difference
plotObj <- ggplot(symmetryRiskData,aes(x = SubtreeSize, y = SizeDifference, z = AsymmetryDiff)) + stat_summary2d()
ggsave(paste(figDir,"/SymmetryRisk-AsymDiffBySizeAndSizeDiff.png",sep=""),plotObj)

plotObj <- ggplot(subset(symmetryRiskData,SubtreeSize<=100),aes(x = SubtreeSize, y = SizeDifference, z = AsymmetryDiff)) + stat_summary2d()
ggsave(paste(figDir,"/SymmetryRisk-AsymDiffBySizeAndSizeDiff_Under100.png",sep=""),plotObj)

plotObj <- ggplot(subset(symmetryRiskData,SubtreeSize<=100),aes(x = SubtreeSize, y = SizeDifference, z = AsymmetryDiff)) + stat_summary2d(fun=max)
ggsave(paste(figDir,"/SymmetryRisk-AsymDiffWorstCaseBySizeAndSizeDiff_Under100.png",sep=""),plotObj)

plotObj <- ggplot(symmetryRiskData,aes(x = SubtreeSize, y = SizeDifference, z = CaulescenceDiff)) + stat_summary2d()
ggsave(paste(figDir,"/SymmetryRisk-CaulDiffBySizeAndSizeDiff.png",sep=""),plotObj)

plotObj <- ggplot(subset(symmetryRiskData,SubtreeSize<=100),aes(x = SubtreeSize, y = SizeDifference, z = CaulescenceDiff)) + stat_summary2d()
ggsave(paste(figDir,"/SymmetryRisk-CaulDiffBySizeAndSizeDiff_Under100.png",sep=""),plotObj)

plotObj <- ggplot(subset(symmetryRiskData,SubtreeSize<=100),aes(x = SubtreeSize, y = SizeDifference, z = CaulescenceDiff)) + stat_summary2d(fun=max)
ggsave(paste(figDir,"/SymmetryRisk-CaulDiffWorstCaseBySizeAndSizeDiff_Under100.png",sep=""),plotObj)

# Bin of proportion with asymmetry difference > .2
plotObj <- ggplot(symmetryRiskData,aes(x = SubtreeSize, y = SizeDifference, z = AsymmetryDiff)) + stat_summary2d(fun=function(x){return(mean(x>.2))})
ggsave(paste(figDir,"/SymmetryRisk-AsymPercentWorseThan0.2.png",sep=""),plotObj)

# Bin of proportion with caulescence difference > .2
plotObj <- ggplot(symmetryRiskData,aes(x = SubtreeSize, y = SizeDifference, z = CaulescenceDiff)) + stat_summary2d(fun=function(x){return(mean(x>.2))})
ggsave(paste(figDir,"/SymmetryRisk-CaulPercentWorseThan0.2.png",sep=""),plotObj)

symmetryRiskData$LargerDiff <- apply(symmetryRiskData[,c("AsymmetryDiff","CaulescenceDiff")],1,max)
plotObj <- ggplot(subset(symmetryRiskData,SubtreeSize<=100),aes(x = SubtreeSize, y = SizeDifference, z = LargerDiff)) + stat_summary2d(fun=function(x){return(mean(x>.2))})
ggsave(paste(figDir,"/SymmetryRisk-CaulOrAsymPercentWorseThan0.2.png",sep=""),plotObj)

## Find worst case examples
symmetryRiskData[which(symmetryRiskData$AsymmetryDiff==max(subset(symmetryRiskData,SubtreeSize<=20)$AsymmetryDiff)),]
symmetryRiskData[which(symmetryRiskData$AsymmetryDiff==max(subset(symmetryRiskData,SubtreeSize>20)$AsymmetryDiff)),]
symmetryRiskData[which(symmetryRiskData$AsymmetryDiff==max(subset(symmetryRiskData,SubtreeSize<=20)$CaulescenceDiff)),]
symmetryRiskData[which(symmetryRiskData$AsymmetryDiff==max(subset(symmetryRiskData,SubtreeSize>20)$CaulescenceDiff)),]

#### Other calculations ####

# Figure out number of pairs that had distances < 0.2
dmSize <- nrow(distMats[[arborType]])
numPairs <- (dmSize*dmSize-dmSize)
sum(distMats[[arborType]]>0&distMats[[arborType]]<0.2)/numPairs
which(distMats[[arborType]]>0&distMats[[arborType]]<0.2,inds=T)[1:20,]
rownames(distMats[[arborType]])[c(1008,1213)]


numAxons <- nrow(distMats[["Axon"]])
numDends <- nrow(distMats[["Dendrite"]])
numApics <- nrow(distMats[["Apical"]])
numArbors <- nrow(distMats[["AllArbors"]])

tmpFunc <- function(distMat){
  dmSize <- nrow(distMat)
  numPairs <- (dmSize*dmSize-dmSize)
  return (sum(distMat>0&distMat<0.2)/numPairs)
}
tmpFunc(distMats[["Axon"]])
tmpFunc(distMats[["Dendrite"]])
tmpFunc(distMats[["Apical"]])
tmpFunc(distMats[["AllArbors"]])
